#define MAIN_
#include "checkmark.hpp"

/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to correct data flow for data slip and block labels\n\n"
	 "[USAGE]: %s [options] <file(s) to process>\n\n"
	 "[OPTIONS]:\n"
	 "-d, --path   <path>           - path to the output file(s), default - the same\n"
	 "                                where are the source files\n"
	 "--ext        <string>         - auxiliary filename extension given to the output\n"	 
	 "                                file(s), default - .out\n"
	 "--out        <filename>       - name of file for output data. If --ext is given\n"
	 "                                then full name is filename + extension\n"
	 "                                default - input file + .out\n"
	 "--header     <size>           - header in bytes of input file(s), default = 0\n"
	 "--dummy                       - set dummy mode. Output file(s) won't be created,\n"
	 "                                but one can see statitics for each file\n"
	 "-q, --quite                   - turn on quite output, skip printing spinning marker\n"
	 "-h, --help                    - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"path", required_argument, 0, 'd'},
				   {"ext", required_argument, 0, 10},
				   {"header", required_argument, 0, 11},
				   {"out", required_argument, 0, 12},
				   {"dummy", no_argument, 0, 13},
				   {"quite", no_argument, 0, 'q'},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "hd:q", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 10:
	  if( AuxExt ) {
	    AuxExt = 0;
	    free(AuxExt);
	                 }
	  if( !( AuxExt = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(AuxExt, optarg, (int)strlen(optarg));
      break;
      
      case 11:
       Header = atoi(optarg);
       if (Header < 0) Header = 0;
      break;
      
      case 12:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
      
      case 13:
        dummy = true;
      break;

      case 'q':
       is_quite = true;
      break;
      
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
}

/*--- ,       ----*/

char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/*---      
      true -    ,  - false
---*/
int analyze_data (TYPE* x, long no) {
 long checksum = 0,
      true_checksum = 0;  //      
 for (long k=no; k<=256+no; ++k) { checksum += x[(k-no)*(4096+1)]; true_checksum += (k >= 256 ? k-256 : k); }
 if (checksum == true_checksum) return 1;
  else return 0;
}

/*---------------- m a i n --------------------------*/

int main ( int argc, char *argv[] ) {

   struct stat info;
   int next, file, outfile;

   if (argc == 1) Help(argv[0]);

   if (!dummy) {
    AuxExt = new char[MaxString];
    OutFile = new char[MaxString];
    OutputPath = new char[MaxString];
    if (!AuxExt || !OutFile || !OutputPath) { printf("Not enough memory!\n"); exit(1); }
    memset(AuxExt, 0, MaxString);
    memset(OutFile, 0, MaxString);
    memset(OutputPath, 0, MaxString);
    sprintf(AuxExt, "%s", ".out");
    sprintf(OutFile, "%s", "\0");
    sprintf(OutputPath, "%s", "\0");
    
    outfilename = new char[MaxString];
   }

   long output_info[7][1]; // 0 -  4-  ()
                           // 1 -      
			   // 2 -   
			   // 3 -   -
			   // 4 -    
			   // 5 -   -    
			   // 6 -       			   
   
   next = ParseCmdLine(argc, argv);
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }   

   long Size;  //    
   long blksize = (1 + 4096) * 256 + 1;  //    ,     +   
   long cur_pointer;  //       
   long cur_marker_number;  //   
   char vis_bar[4] = {'\\', '-', '/', '|'};
   unsigned char vis_bar_count;

   indata  = new TYPE[blksize];
   if (!indata) { printf("Not enough memory!\n"); exit(1); }

printf("\nProcessing...\n");

   for (int i=next; i<argc; ++i) {

   printf(" [%3d]\t%s ... ", i-next, argv[i]);
   fflush(NULL);

   if (!dummy) {
    //    
    memset(outfilename, 0, MaxString);
    if (!strcmp(OutputPath, "\0")) { if (strcmp(OutFile, "\0") && !strcmp(AuxExt, ".out")) sprintf(outfilename, "%s", OutFile);
     else sprintf(outfilename, "%s%s", (!strcmp(OutFile, "\0") ? argv[i] : OutFile), AuxExt);
    } else { if (strcmp(OutFile, "\0") && !strcmp(AuxExt, ".out")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
              else sprintf(outfilename, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(argv[i]) : OutFile), AuxExt);
      }
   } // if (!dummy)

   if ( stat(argv[i], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[i]); exit(1); }

   Size = (info.st_size - Header)/sizeof(TYPE);
   if (Size <= 0) { printf("\nError:\nSize of header is too much!\n"); exit(1); }

   /*-- open file for reading --*/
   if ( (file = open(argv[i], O_RDONLY | O_LARGEFILE)) == -1 )
    { printf("\nError:\nCouldn't open \"%s\" for reading!\n", argv[i]); exit(1);  }

   /*-- skip header --*/
   if (lseek(file, Header, SEEK_SET) == -1 )
    { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }

   if (!dummy) {
    if ( (outfile = open(outfilename, O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
       { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
         exit(1);
        }
   } // if (!dummy)

   cur_pointer = 0;
   cur_marker_number = 0;
   for (int q=0; q<7; q++) output_info[q][0] = 0;
   vis_bar_count = 0;

   //    
   while (cur_pointer + blksize <= Size) {  //  ,      256 4k-

    if (!is_quite) { printf ("%c", vis_bar[(++vis_bar_count%4)]); fflush(NULL); }
   
    memset(indata,  0, blksize*sizeof(TYPE));
    read (file, indata, blksize*sizeof(TYPE));
    //    
    if (analyze_data(indata, cur_marker_number)) {
     cur_pointer += blksize - 1;
     //    
     if (lseek(file, Header + cur_pointer*sizeof(TYPE), SEEK_SET) == -1 )
      { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }
      //        
      if (!dummy) { for (int k=0; k<256; ++k) write (outfile, indata+k*(4096+1) + 1, 4096*sizeof(TYPE)); }
      output_info[0][0] += 256;
      output_info[2][0] += 256;
      
    } else { //      
      //      
      output_info[2][0]++;
      for (int k=0; k<=256; ++k) {
       if (indata[k*(4096+1)] != (k+cur_marker_number >= 256 ? k+cur_marker_number-256 : k+cur_marker_number) && k == 0) {
	  printf ("\n-- warning -- bad start mark (%d)!\n", indata[0]);
	  output_info[4][0]++;
	  output_info[2][0]--;
       }
       if (indata[k*(4096+1)] != (k+cur_marker_number >= 256 ? k+cur_marker_number-256 : k+cur_marker_number) && k != 0) {
        //       4096-
	//   ,     ,     
	long pos = 0;
        for (long h=k*(4096+1)-1; h>=(k-1)*(4096+1)+1; --h) {
	 if (indata[h] == (k+cur_marker_number >= 256 ? k+cur_marker_number-256 : k+cur_marker_number)) { pos = h; break; }
	}
	 if (pos == 0) {
	   //   , ..   
	   // :      , ..   , 
	   //     ,         
	   //     1-2 ,   
	   //     64    256*4k-
	   //     slip-events
	   pos = 0;
	    for (long a=k*(4096+1)+1; a<=(k+64 >= 256 ? 256 : k+64)*(4096+1)-1; ++a)
	     if (indata[a] == (k+cur_marker_number == 256 ? 0 : k+cur_marker_number)) { pos = a; break; }
	     
	   if (pos != 0 && indata[pos-(4096+1)] == (k-1+cur_marker_number >= 256 ? k-1+cur_marker_number-256 : k-1+cur_marker_number)) {
	     output_info[2][0] += k-2;
	     output_info[6][0]++;
             //     
             cur_pointer += pos-(4096+1);
	     cur_marker_number = (k-1+cur_marker_number >= 256 ? k-1+cur_marker_number-256 : k-1+cur_marker_number); 
             if (lseek(file, Header + cur_pointer*sizeof(TYPE), SEEK_SET) == -1 )
              { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }
             //      
             if (!dummy) { for (int m=0; m<k-1; ++m) write (outfile, indata+m*(4096+1) + 1, 4096*sizeof(TYPE)); }
	     output_info[0][0] += k-1;
	     break;
	   }
	   else {

           //      slip-events,     
	   //   64    256*4k-
	   long u;
	   pos = 0;
	   for (u=k; u<=255; ++u) {
	    for (long h=u*(4096+1)+1; h<=(u+64 >= 256 ? 256 : u+64)*(4096+1)-1; ++h) {
	     if (indata[h] == (u+1+cur_marker_number >= 256 ? u+1+cur_marker_number-256 : u+1+cur_marker_number)) { pos = h; break; }
	    }
	    if (pos != 0) break;
	   }
	     if (pos == 0) {
	      if (k == 0) { printf("\nError: can't find block marks in current \"256*4k\"-block position after %d number!\n", k); exit(1); }
	       //       , 
	       //        256*4k-
	       else {
                 cur_pointer += (k-1)*(4096+1);
  	         cur_marker_number = (k-1+cur_marker_number >= 256 ? k-1+cur_marker_number-256 : k-1+cur_marker_number); 
	         output_info[2][0] += k-2;
                 if (lseek(file, Header + cur_pointer*sizeof(TYPE), SEEK_SET) == -1 )
                  { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }
                 //      
                 if (!dummy) { for (int m=0; m<k-1; ++m) write (outfile, indata+m*(4096+1) + 1, 4096*sizeof(TYPE)); }
	         output_info[0][0] += k-1;
	         break;
	       }
	     }
	     output_info[4][0] += u+1-k;
	     output_info[2][0] += k-1;
             //     
             cur_pointer += pos;
             cur_marker_number = (u+1+cur_marker_number >= 256 ? u+1+cur_marker_number-256 : u+1+cur_marker_number);
             if (lseek(file, Header + cur_pointer*sizeof(TYPE), SEEK_SET) == -1 )
              { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }
	     //     indata    65535 (  1  , ..     +3)
     	     if (!dummy) { for (int l=k*(4096+1); l<(u+1)*(4096+1); ++l) indata[l] = 65535; }
	     output_info[5][0] += (u+1-k)*4096;
             //    +    
             if (!dummy) { for (int m=0; m<(u+1); ++m) write (outfile, indata+m*(4096+1) + 1, 4096*sizeof(TYPE)); }
	     output_info[0][0] += u+1;
	     break;
	     
	   } // if (pos != 0 && indata[pos-(4096+1)] == (k-1+cur_marker_number >= 256 ? k-1+cur_marker_number-256 : k-1+cur_marker_number)) 
	  }
	  else {
	   //   , ..      
           cur_pointer += pos;
           cur_marker_number = (k+cur_marker_number >= 256 ? k+cur_marker_number-256 : k+cur_marker_number); 
           //     
           if (lseek(file, Header + cur_pointer*sizeof(TYPE), SEEK_SET) == -1 )
            { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }
	   //     indata    65535 (  1  , ..     +3)
	   if (!dummy) { for (int l=pos; l<k*(4096+1); ++l) indata[l] = 65535; }
	   output_info[3][0] += k*(4096+1) - pos;
           output_info[2][0] += k-1;
           //    +    
           if (!dummy) { for (int m=0; m<k; ++m) write (outfile, indata+m*(4096+1) + 1, 4096*sizeof(TYPE)); }
	   output_info[0][0] += k;
	  }
	 break;
       }
      }
    } // else
    if (!is_quite) { printf("\b"); fflush(NULL); }
   } // while
   
   memset(indata,  0, blksize*sizeof(TYPE));
   ssize_t bytes_read; 
   //     
   for (long f=0; cur_pointer < Size; ++f) {
    if (!is_quite) { printf ("%c", vis_bar[(++vis_bar_count%4)]); fflush(NULL); }
    memset (indata, 0, (4096+1+1)*sizeof(TYPE));
    bytes_read = read (file, indata, (4096+1+1)*sizeof(TYPE));
    if (bytes_read == (4096+1+1)*sizeof(TYPE)) {
     if (indata[4097] == (f+1+cur_marker_number >= 256 ? f+1+cur_marker_number-256 : f+1+cur_marker_number)) {
      //    
      cur_pointer += 4096+1;
      if (lseek(file, Header + cur_pointer*sizeof(TYPE), SEEK_SET) == -1 )
       { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }
      if (!dummy) write (outfile, indata+1, 4096*sizeof(TYPE));
      output_info[0][0]++;
      output_info[2][0]++;
     } else {
        //   
	long pos = 0;
        for (long h=4096; h>=1; --h)
	 if (indata[h] == (f+1+cur_marker_number >= 256 ? f+1+cur_marker_number-256 : f+1+cur_marker_number)) { pos = h; break; }
	 if (pos == 0) { printf("\nError: no block mark in current position after %d number!\n", f); exit(1); }
           cur_pointer += pos;
           //     
           if (lseek(file, Header + cur_pointer*sizeof(TYPE), SEEK_SET) == -1 )
            { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", argv[i]); exit(1); }
	   //     indata    65535 (  1  , ..     +3)
	   if (!dummy) { for (int l=pos; l<4097; ++l) indata[l] = 65535; }
	   output_info[3][0] += 4097-pos;
           //      
           if (!dummy) write (outfile, indata+1, 4096*sizeof(TYPE));
	   output_info[0][0]++;
	   output_info[2][0]++;	   
     }
    } // if bytes_read ==
    else { if (!dummy) write (outfile, indata+1, bytes_read - sizeof(TYPE));
           output_info[1][0]++;
	   output_info[0][0]++;
	   output_info[2][0]++;
	   break;
          }
    if (!is_quite) { printf("\b"); fflush(NULL); }
   }

   /*-- close file --*/
   close (file);
   if (!dummy) close (outfile);
   
  if (!is_quite) printf("\b - done\n");
   else printf(" - done\n");

  printf ("\n---------------- statistics ------------------\n");
  printf ("no. of processed 4K-blocks:   %d\n", output_info[0][0]);
  printf ("   with %d not full block of size %d words\n", output_info[1][0], (bytes_read - sizeof(TYPE))/2);
  printf ("no. of markers found:   %d\n", output_info[2][0]);
  printf ("no. of added pseudo-words:   %d\n", output_info[3][0]);
  printf ("no. of absent markers:   %d\n", output_info[4][0]);
  printf ("no. of added pseudo-words in absent markers:   %d\n", output_info[5][0]);
  printf ("no. of slip events:   %d\n", output_info[6][0]);
  printf ("----------------------------------------------\n");
 } // for through input files

  delete (indata);
  if (!dummy) {
   delete (AuxExt);
   delete (OutFile);
   delete (OutputPath);
   delete (outfilename);
  }
 return 0;
}

