#include "header.hpp"

#include <string.h>
#include <math.h>
#include <gmp.h> // high-precision math library

// constructor
Header::Header (char *m, char type, off_t p, double t) { 
  init (m, type, p, t);
}

// initialize
void Header::init (char *m, char type, off_t p, double t) {
  start_mjd = m; 
  pos = p; 
  res = t; 
  pulse_type = type;

  mpf_t m1, temp, temp2;
  mpf_init2 (m1, 512); mpf_init2 (temp, 512); mpf_init2 (temp2, 512);
  mpf_set_str (m1, get_start_mjd(), 10);
  mpf_set_ui (temp, (unsigned int)pos);
  mpf_set_d (temp2, res);
  mpf_mul (temp, temp, temp2);
  mpf_set_str (temp2, "86400000000000", 10);
  mpf_div (temp, temp, temp2);
  mpf_add (temp, temp, m1);
  mpf_clear (m1); mpf_clear (temp2);
  long exponent = 5;
  mjd = mpf_get_str (NULL, &exponent, 10, mjd_string_size, temp);
  mpf_clear (temp);
}

// return start mjd in usual format like 53528.4567...
// show only 16 digits after the radix point
// 1 ns -- it's 14'th digit of the fraction of day
char* Header::get_start_mjd (char *m) {
 string cur;
 cur = start_mjd.substr(0, 5) + "." + start_mjd.substr(5, 16);
 memcpy (m, cur.c_str(), mjd_string_size);
 return m;
}

// return start mjd in usual format like 53528.4567...
// show only 16 digits after the radix point
// 1 ns -- it's 14'th digit of the fraction of day
const char* Header::get_start_mjd () {
 string cur;
 cur = start_mjd.substr(0, 5) + "." + start_mjd.substr(5, 16);
 return cur.c_str();
}

// get mjd of the GP
char* Header::get_mjd (char *m) {
 string cur;
 cur = mjd.substr(0, 5) + "." + mjd.substr(5, 16);
 memcpy (m, cur.c_str(), mjd_string_size);
 return m;
}

// get mjd of the GP
const char* Header::get_mjd () {
 string cur;
 cur = mjd.substr(0, 5) + "." + mjd.substr(5, 16);
 return cur.c_str();
}

// get hour, min and sec of GP
void Header::getTime (int& hour, int& min, double& sec) {
 string cur;
 cur = "0." + mjd.substr(5, 15);
 mpf_t temp, temp2;
 mpf_init2 (temp, 512); mpf_init2 (temp2, 512);
 mpf_set_str (temp, cur.c_str(), 10);
 mpf_set_ui (temp2, 24);
 mpf_mul (temp, temp, temp2);
 hour = (int)floor(mpf_get_d (temp));
 mpf_sub_ui (temp, temp, (unsigned long)hour);
 mpf_set_ui (temp2, 60);
 mpf_mul (temp, temp, temp2);
 min = (int)floor(mpf_get_d (temp));
 mpf_sub_ui (temp, temp, (unsigned long)min);
 mpf_mul (temp, temp, temp2);
 sec = mpf_get_d (temp);
 mpf_clear (temp); mpf_clear (temp2);
}
