#include "cut2ccf.hpp"

// observing conf
ObsConf obs;

// cut-size is 548317, so we just ignore the 24029 counts in dedispersion
off_t size = 524288;

// number of iterations
off_t niter = 100;

double fraction = 0.;

/*--------------------------------------------------------------------------------------
 *                               M   A   I   N
 * -------------------------------------------------------------------------------------*/

int main (int argc, char *argv[]) {

 if (argc == 1) help (argv[0]);
 // initialize wisdom
 initialize_wisdom ();

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;
        
 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE, &r);
 /*------------------------------------------*/

 off_t size4 = 4*size;
 off_t size8 = 8*size;

 int next = parse_command_line (argc, argv);
 if (argc < next+1) { printf ("Not point the files to process!\n"); exit (1); }
 
 off_t cutsize = size + bitstat_points;
 off_t left = 0;

 signed char *src;
 src = new signed char[cutsize];
 if (!src) { perror("new"); exit(1); } 
 memset (src, 0, cutsize);

 float *srcf;
 srcf = new float[size4];
 if (!srcf) { perror("new"); exit(1); } 

 float *meanccf;
 meanccf = new float[size4];
 if (!meanccf) { perror("new"); exit (1); }
 memset (meanccf, 0, size4 * sizeof(float));

 fftwf_complex *part1, *part2, *destlcp, *destrcp;
 part1 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * 2 * size);
 part2 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * 2 * size);
 destlcp = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * 2 * size4);
 destrcp = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * 2 * size4);
 if (!part1 || !part2 || !destlcp || !destrcp) { perror("fftwf_malloc"); exit(1); }
 fftwf_plan plan1, plan2, backplan;
 plan1 = fftwf_plan_dft_1d (2 * size, part1, part1, FFTW_FORWARD, FFTW_EXHAUSTIVE);
 plan2 = fftwf_plan_dft_1d (2 * size, part2, part2, FFTW_FORWARD, FFTW_EXHAUSTIVE);
 backplan = fftwf_plan_dft_1d (2 * size4, destlcp, destlcp, FFTW_BACKWARD, FFTW_EXHAUSTIVE);

  /* read correction functions */
  float *bandlcp, *bandrcp;
  bandlcp = new float[size4/2+1];
  bandrcp = new float[size4/2+1];
  if (!bandlcp || !bandrcp) { perror("new"); exit (1); }
  memset (bandlcp, 0, (size4/2+1)*sizeof(float));
  memset (bandrcp, 0, (size4/2+1)*sizeof(float));
  int bandfile;
  if ((bandfile = open ("lcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
  lseek (bandfile, 0, SEEK_SET);
  memset (bandlcp, 0, (size4/2+1)*sizeof(float));
  read (bandfile, bandlcp, (size4/2+1)*sizeof(float));
  close (bandfile);
  if ((bandfile = open ("rcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
  lseek (bandfile, 0, SEEK_SET);
  memset (bandrcp, 0, (size4/2+1)*sizeof(float));
  read (bandfile, bandrcp, (size4/2+1)*sizeof(float));
  close (bandfile);
  /* - */

 printf ("processing: "); fflush (NULL);
 for (off_t iter=0; iter<niter; ++iter) {

 printf ("%ld,", (unsigned long)iter); fflush (NULL);
 left += size;
	
 memset (part1, 0, 2 * size*sizeof(fftwf_complex));
 memset (part2, 0, 2 * size*sizeof(fftwf_complex));

 memset (destlcp, 0, size8*sizeof(fftwf_complex));

 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next]);
 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+1]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next+2]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+3]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (destlcp, part1, part2, 2*size);

 memset (part1, 0, 2 * size*sizeof(fftwf_complex));
 memset (part2, 0, 2 * size*sizeof(fftwf_complex));

 memset (destrcp, 0, size8*sizeof(fftwf_complex));

 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next+4]);
 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+5]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next+6]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+7]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (destrcp, part1, part2, 2*size);

 for (off_t i=0; i<size4/2+1; ++i) {
  destlcp[i][0] *= bandlcp[i];
  destlcp[i][1] *= bandlcp[i];
  destrcp[i][0] *= bandrcp[i];
  destrcp[i][1] *= bandrcp[i];
 }
// double lacf = get_acf0 (destlcp, size8);
// double racf = get_acf0 (destrcp, size8);
// double lracf = sqrt (lacf * racf);
 double lacf1 = get_acf (destlcp, size8);
 double racf1 = get_acf (destrcp, size8);
 double lracf1 = sqrt (lacf1 * racf1);
// printf ("lacf : %lf  racf : %lf  lracf : %lf\n", lacf, racf, lracf);
// printf ("lacf1: %lf  racf1: %lf  lracf1: %lf\n", lacf1, racf1, lracf1);

 double z;
 float re, im;
 for (off_t i=0; i<size4/2+1; ++i) {
  z = ((-1. * M_PI * fraction) * ((double)i))/((double)(size4));
  re = destlcp[i][0] * cos(z) - destlcp[i][1] * sin(z);
  im = destlcp[i][0] * sin(z) + destlcp[i][1] * cos(z);
  destlcp[i][0] = re * destrcp[i][0] + im * destrcp[i][1];
  destlcp[i][1] = im * destrcp[i][0] - re * destrcp[i][1];
 }
 for (off_t i=size4/2+1; i<size8; ++i) { destlcp[i][0] = 0.; destrcp[i][0] = 0.; }

 fftwf_execute (backplan);
 for (off_t i=0, j=size4 + size4/2 + 1; j<2*size4; ++i, ++j) { meanccf[i] += ((double)destlcp[j][0] * destlcp[j][0] + (double)destlcp[j][1] * destlcp[j][1]) / lracf1; }
 for (off_t i=size4/2-1, j=0; j<size4/2+1; ++i, ++j) { meanccf[i] += ((double)destlcp[j][0] * destlcp[j][0] + (double)destlcp[j][1] * destlcp[j][1]) / lracf1; }
}

 for (off_t i=0; i<size4; ++i) meanccf[i] /= niter;
 char *outname;
 outname = new char[255];
 if (!outname) { perror("new"); exit (1); }
 memset (outname, 0, 255);
 sprintf (outname, "mean.ccf.shiftes%.2lf", fraction);
 write2file (meanccf, size4, outname);
 printf ("done\n");

 delete (outname);
 delete (bandlcp);
 delete (bandrcp);
 delete (srcf);
 delete (src);
 fftwf_free (part1);
 fftwf_free (part2);
 fftwf_free (backplan);
 fftwf_free (destlcp);
 fftwf_free (destrcp);
 fftwf_destroy_plan (plan1);
 fftwf_destroy_plan (plan2);
 delete (meanccf);

 return 0;
}
