#include <stdio.h>
#include <stdlib.h>

#include "obsconf.hpp"
#include "constant.hpp"

ObsConf::ObsConf () {
 center_freq = 2084.;
 ref_freq = 2100.;
 bandwidth = 64.;
 band_direct = true;
 timeres = 7.8125;
 dm = 71.0255;
}

ObsConf::ObsConf (double cen, double ref, double bw, bool bd, double tres, double disp) {
 center_freq = cen;
 ref_freq = ref;
 bandwidth = bw;
 band_direct = bd;
 timeres = tres;
 dm = disp;
}

/* get total number of spoiled points */
unsigned long ObsConf::get_spoiled_points () {
 double tbad, tdelay;
 double f1 = center_freq - bandwidth/2.;
 double f2 = center_freq + bandwidth/2.;
 tbad = dm * ALPHA2 * ( 1./(f1 * f1) - 1./(f2 * f2) );
 tdelay = (tbad * 0.001)/timeres;
 return (unsigned long)ceil(fabs(tdelay));
}

/* get total number of spoiled points */
/* f1 - lower frequency, f2 - upper frequency */
/* to get spoiled points at the beginning: f1 = ref freq, f2 = highest freq of the band */
/* to get spoiled points at the end: f1 = lowest freq of the band, f2 = ref freq */
unsigned long ObsConf::get_spoiled_points (double f1, double f2) {
 double tbad, tdelay;
 tbad = dm * ALPHA2 * ( 1./(f1 * f1) - 1./(f2 * f2) );
 tdelay = (tbad * 0.001)/timeres;
 return (unsigned long)ceil(fabs(tdelay));
}

/* get number of spoiled points in the beginning */
unsigned long ObsConf::get_start_spoiled_points () {
 double tbad, tdelay;
 double f1 = ref_freq;
 double f2 = center_freq + bandwidth/2.;
 tbad = dm * ALPHA2 * ( 1./(f1 * f1) - 1./(f2 * f2) );
 tdelay = (tbad * 0.001)/timeres;
 return (unsigned long)ceil(fabs(tdelay));
}

/* get number of spoiled points in the end */
unsigned long ObsConf::get_end_spoiled_points () {
 double tbad, tdelay;
 double f1 = center_freq - bandwidth/2.;
 double f2 = ref_freq;
 tbad = dm * ALPHA2 * ( 1./(f1 * f1) - 1./(f2 * f2) );
 tdelay = (tbad * 0.001)/timeres;
 return (unsigned long)ceil(fabs(tdelay));
}

/* print out info */
void ObsConf::print_info () {
 printf ("\n");
 printf ("ref freq = %lg\n", ref_freq);
 printf ("center freq = %lg\n", center_freq);
 printf ("bandwidth = %lg [%s]\n", bandwidth, band_direct ? "direct" : "inverse");
 printf ("timeres = %lg\n", timeres);
 printf ("dm = %lg\n", dm);
 printf ("spoiled points: start -> %ld   end -> %ld   total -> %ld [%+ld]\n", get_start_spoiled_points(), get_end_spoiled_points(), get_spoiled_points(), get_start_spoiled_points()+get_end_spoiled_points()-get_spoiled_points());
 printf ("\n");
}
