#include "cmd.hpp"

extern ObsConf obs;
extern mpf_t mjd_t;
extern char pulse_type;

/* help */ 
void help (char *s) {
 printf("\nPROGRAMM %s\n"
	"To reconstruct raw cut-files in total band\n"
        "[USAGE]: %s [options] <cut.lcp.W> <cut.lcp.X> <cut.lcp.Y> <cut.lcp.Z> <cut.rcp.W> <cut.rcp.X> <cut.rcp.Y> <cut.rcp.Z>\n\n"
        "[OPTIONS]:\n"
	"-t, --timeres <value>         - time resolution (ns) in total band, default = 7.8125\n"
	"--dm <value>                  - dispersion measure (pc/cm^3), default = 71.0255\n"
	"--bw <value>                  - bandwidth (MHz), default = 64\n"
	"--reffreq <value>             - reference frequency (MHz), default = 2100\n"
	"--centerfreq <value>          - center frequency (MHz) of total band, default = 2084\n"
	"--direct                      - to set the band as direct, band is direct by default\n"
	"--inverse                     - to set the band as inverse, band is direct by default\n"
	"--basename   <name>           - base name for output files: basename + \".[lcp|rcp]\" + \".cut.det\"\n"
	"--mjd <value>                 - MJD of the beginning of the cut-files, default = 0\n"
	"--type <2|1|0>                - type of the GP: 2 - false event, 1 - main pulse, 0 - interpulse\n"
	"-d, --path   <path>           - path for output files\n"
	"-h, --help                    - help\n\n", s, s);
 exit(0);
}

/* parse command line */
int parse_command_line (int argc, char *argv[]) {
  int op, tm;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"timeres", required_argument, 0, 't'},
				   {"dm", required_argument, 0, 10},
				   {"bw", required_argument, 0, 11},
				   {"reffreq", required_argument, 0, 12},
				   {"centerfreq", required_argument, 0, 13},
                                   {"direct", no_argument, 0, 14},
				   {"inverse", no_argument, 0, 15},
				   {"basename", required_argument, 0, 16},
				   {"path", required_argument, 0, 'd'},
				   {"mjd", required_argument, 0, 17},
				   {"type", required_argument, 0, 18},
				   {0, 0, 0, 0}
                                  };

  optind = 0; optarg = 0;
  while((op = getopt_long(argc, argv, "ht:d:", long_options, 0)) != EOF)
    switch(op){
      
      case 'h':
        help (argv[0]);
      break;
      
      case 't':
        obs.set_time_resolution (atof(optarg));
      break;
      
      case 10:
        obs.set_dispersion_measure (atof(optarg));
      break;
      
      case 11:
        obs.set_bandwidth (atof(optarg));
      break;
      
      case 12:
        obs.set_ref_freq (atof(optarg));
      break;
      
      case 13:
        obs.set_center_freq (atof(optarg));
      break;
      
      case 14:
        obs.set_band_direction (true);
      break;

      case 15:
        obs.set_band_direction (false);
      break;

      case 16:
        if (base) { base = 0; free (base); }
	if (!(base = (char *)calloc((int)strlen(optarg), sizeof(char *)))) { perror("new"); exit (1); }
	memcpy (base, optarg, (int)strlen(optarg));
      break;

      case 'd':
        if (output_path) { output_path = 0; free (output_path); }
	if (!(output_path = (char *)calloc((int)strlen(optarg), sizeof(char *)))) { perror("new"); exit (1); }
	memcpy (output_path, optarg, (int)strlen(optarg));
      break;

      case 17:
        mpf_set_str (mjd_t, optarg, 10);
      break;

      case 18:
        int t = atoi (optarg);
	switch (t) {
	 case 0: pulse_type = 'I';
		 break;
	 case 1: pulse_type = 'M';
		 break;
	 case 2: pulse_type = 'F';
		 break;
	 default: pulse_type = 'F';
		 break;
	}
      break;
      
      case '?':
        help (argv[0]);
      break;
    }
  return(optind);
}
