#ifndef _DISP_HPP
#define _DISP_HPP

#include <string.h>
#include <math.h>
#include <stdlib.h>

#include <fftw3.h>

#include "constant.hpp"
#include "obsconf.hpp"

/* calculate phase due to dispersion */ 
double get_disp_phase (double f);

/* calculate phase due to dispersion for separate channel */
double get_channel_disp_phase (double f, double frefi);

/* give delay between fcentr and separate channle */
/* if delay > 0 so we have to shift data in direction to the end */
/* if delay < 0 so we have to move data to the beginning */
signed long get_channel_delay (double frefi);

/* compute dedispersion matrix */
fftwf_complex* get_matrix (fftwf_complex *matrix, off_t size);

/* compute dedispersion matrix for individual channel */
fftwf_complex* get_channel_matrix (fftwf_complex *matrix, off_t size, double fcentri, double frefi);

/* ship FFT data out in two complex arrays */
/* part1 originally has packed completely, part2 is empty */
void ship_out2complex (fftwf_complex *part1, fftwf_complex *part2, off_t size);

/* ship FFT data from the GBT to arrange in total band */
/* dest is 4*n size, part1 & part is n size */
fftwf_complex* ship_out2gbt_total (fftwf_complex *dest, fftwf_complex *part1, fftwf_complex *part2, off_t size);

/* making the dedispersion */
fftwf_complex* dedispersion (fftwf_complex *ampl_sp, fftwf_complex *matrix, float *band, fftwf_plan plan, off_t size, bool do_fraction, bool do_zero = true);

/* get ACF0 value */
double get_acf0 (fftwf_complex *x, off_t size);

/*get ACF0 value calculating the ACF itself */
double get_acf (fftwf_complex *x, off_t size);

/* to dedisperse the data in separate channels and create filterbank in the output */
/* to present as 2D-plot in pgplot */
float* filterbank (float *bank, float *aligned, fftwf_complex *ampl_sp, float *band, off_t size, off_t ts, bool do_fraction, bool do_zero);

#endif //#ifndef _DISP_HPP
