#ifndef _OBSCONF_HPP
#define _OBSCONF_HPP

class ObsConf {
   double center_freq;
   double ref_freq;
   double bandwidth;
   bool band_direct;
   double timeres;
   double dm;
   int channel_number;

  public:
   ObsConf ();
   ObsConf (double cen, double ref, double bw, bool bd, double tres, double disp, int nch);
   virtual ~ObsConf () {}

   inline void set_center_freq (double c) { center_freq = c; }
   inline void set_ref_freq (double r) { ref_freq = r; }
   inline void set_bandwidth (double b) { bandwidth = b; }
   inline void set_band_direction (bool is) { band_direct = is; }
   inline void set_time_resolution (double t) { timeres = t; }
   inline void set_dispersion_measure (double d) { dm = d; }
   inline void set_channel_number (int n) { channel_number = n; }

   inline double get_center_freq () { return center_freq; }
   inline double get_ref_freq () { return ref_freq; }
   inline double get_bandwidth () { return bandwidth; }
   inline bool is_band_direct () { return band_direct; }
   inline double get_time_resolution () { return timeres; }
   inline double get_dispersion_measure () { return dm; }
   inline int get_channel_number () { return channel_number; }

   /* print out info */
   void print_info ();

   /* get total number of spoiled points */
   inline unsigned long get_spoiled_points ();
   inline unsigned long get_spoiled_points (double f1, double f2);
   /* to get the maximum spoiled points among all channels */
   unsigned long get_channel_spoiled_points ();

   /* get number of spoiled points in the beginning */
   unsigned long get_start_spoiled_points ();

   /* get number of spoiled points in the end */
   unsigned long get_end_spoiled_points ();
};

#endif //#ifndef _OBSCONF_HPP
