#include "cut2poldegree.hpp"

// observing conf
ObsConf obs;

// cut-size is 548317, so we just ignore the 24029 counts in dedispersion
off_t size = 524288;

// high precision value for mjd
mpf_t mjd_t;
char *mjd;

// type of the GP type: 'M' - main pulse, 'I' - interpulse, 'F' - false event
char pulse_type = 'F';

double sefd = 12.; // sefd
double tres = 7.8125;  // time resolution

/*--------------------------------------------------------------------------------------
 *                               M   A   I   N
 * -------------------------------------------------------------------------------------*/

int main (int argc, char *argv[]) {

 if (argc == 1) help (argv[0]);
 // initialize wisdom
 initialize_wisdom ();
 // initialize mjd
 mpf_init2 (mjd_t, 512);

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;
        
 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE, &r);
 /*------------------------------------------*/

 off_t size4 = 4*size;

 output_path  = new char[255];
 base = new char[255];
 if (!base || !output_path) { perror("new"); exit (1); }
 memset (output_path, 0, 255);
 memset (base, 0, 255);
 sprintf (base, "%s", "dest");
 sprintf (output_path, "%s", ".");

 int next = parse_command_line (argc, argv);
 if (argc < next+1) { printf ("Not point the files to process!\n"); exit (1); }
 
 struct stat info;
 if (stat(argv[next], &info) != 0) { perror("stat"); exit(1); }
 off_t cutsize = info.st_size;
 off_t leftborder = cutsize - size;

 unsigned long begpoints = obs.get_start_spoiled_points();
 unsigned long endpoints = obs.get_end_spoiled_points();
// obs.print_info ();
// printf ("cutsize: %ld  fft size: %ld\n", (unsigned long)cutsize, (unsigned long)size);

 signed char *src;
 src = new signed char[cutsize];
 if (!src) { perror("new"); exit(1); } 
 memset (src, 0, cutsize);

 float *srcfI, *srcfV, *srcfP;
 srcfI = new float[size4];
 srcfV = new float[size4];
 srcfP = new float[size4];
 if (!srcfI || !srcfV || !srcfP) { perror("new"); exit(1); } 

 fftwf_complex *matrix;
 matrix = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * (size4/2 + 1));
 if (!matrix) { perror("fftwf_malloc"); exit (1); }
 memset (matrix, 0, (size4/2+1)*sizeof(fftwf_complex));
 matrix = get_matrix (matrix, size4);

 fftwf_complex *part1, *part2, *destlcp, *destrcp;
 part1 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 part2 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 destlcp = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size4);
 destrcp = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size4);
 if (!part1 || !part2 || !destlcp || !destrcp) { perror("fftwf_malloc"); exit(1); }
 fftwf_plan plan1, plan2, backplan1, backplan2;
 plan1 = fftwf_plan_dft_1d (size, part1, part1, FFTW_FORWARD, FFTW_EXHAUSTIVE);
 plan2 = fftwf_plan_dft_1d (size, part2, part2, FFTW_FORWARD, FFTW_EXHAUSTIVE);
 backplan1 = fftwf_plan_dft_1d (size4, destlcp, destlcp, FFTW_BACKWARD, FFTW_EXHAUSTIVE);
 backplan2 = fftwf_plan_dft_1d (size4, destrcp, destrcp, FFTW_BACKWARD, FFTW_EXHAUSTIVE);
 memset (part1, 0, size*sizeof(fftwf_complex));
 memset (part2, 0, size*sizeof(fftwf_complex));
 memset (destlcp, 0, size4*sizeof(fftwf_complex));
 memset (destrcp, 0, size4*sizeof(fftwf_complex));

 /* read correction functions */
 float *bandlcp, *bandrcp;
 bandlcp = new float[size4/2+1];
 bandrcp = new float[size4/2+1];
 if (!bandlcp || !bandrcp) { perror("new"); exit (1); }
 memset (bandlcp, 0, (size4/2+1)*sizeof(float));
 memset (bandrcp, 0, (size4/2+1)*sizeof(float));
 int bandfile;
 if ((bandfile = open ("lcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 lseek (bandfile, 0, SEEK_SET);
 memset (bandlcp, 0, (size4/2+1)*sizeof(float));
 read (bandfile, bandlcp, (size4/2+1)*sizeof(float));
 close (bandfile);
 if ((bandfile = open ("rcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 lseek (bandfile, 0, SEEK_SET);
 memset (bandrcp, 0, (size4/2+1)*sizeof(float));
 read (bandfile, bandrcp, (size4/2+1)*sizeof(float));
 close (bandfile);
 /* - */

 read_correct_n_fill (src, srcfI, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next]);
 read_correct_n_fill (src, srcfI, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+1]);
 read_correct_n_fill (src, srcfI, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next+2]);
 read_correct_n_fill (src, srcfI, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+3]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (destlcp, part1, part2, size);

 destlcp = dedispersion (destlcp, matrix, bandlcp, backplan1, size4, true, false);

 memset (part1, 0, size*sizeof(fftwf_complex));
 memset (part2, 0, size*sizeof(fftwf_complex));

 read_correct_n_fill (src, srcfI, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next+4]);
 read_correct_n_fill (src, srcfI, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+5]);
 read_correct_n_fill (src, srcfI, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next+6]);
 read_correct_n_fill (src, srcfI, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+7]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (destrcp, part1, part2, size);

 destrcp = dedispersion (destrcp, matrix, bandrcp, backplan2, size4, false, false);

 mjd = new char[255];
 if (!mjd) { perror("new"); exit (1); } 
 memset (mjd, 0, 255);
 mjd = getMJD (mjd_t, leftborder, begpoints, obs.get_time_resolution(), mjd);
// printf ("mjd = %s\n", mjd);

 for (off_t i=begpoints; i<size4-endpoints; ++i) { destlcp[i][0] /= size4; destlcp[i][1] /= size4; destrcp[i][0] /= size4; destrcp[i][1] /= size4; }

 /*--- calculate I  ---*/ 
 memset (srcfI, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcfI[j] = destlcp[i][0]*destlcp[i][0] + destlcp[i][1]*destlcp[i][1] + destrcp[i][0]*destrcp[i][0] + destrcp[i][1]*destrcp[i][1]; }
 /*-----------------------------------------------*/

 double ymax, aver, sigma;
 off_t lowedge, upedge;
 off_t xmax = getmax (srcfI, size4-begpoints-endpoints, ymax);
 get_energy (srcfI, size4-begpoints-endpoints, xmax, aver, sigma, lowedge, upedge);
 off_t w90 = upedge - lowedge;
	       
 double energy = 0.;
 double energy_jymcs = 0.; 
 for (off_t i=lowedge; i<=upedge; ++i) {
  energy += ( (double)srcfI[i] - aver );
  energy_jymcs += ( ((((double)srcfI[i] - aver)/sigma)*sefd) * (tres / 1000.) );
 }
 double level = (ymax - aver) / sigma;
 double max_jy = level * sefd;
 double weff = energy / (ymax - aver);
 double weff_mcs = energy_jymcs / max_jy;

 /*--- calculate V  ---*/
 memset (srcfV, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcfV[j] = destrcp[i][0]*destrcp[i][0] + destrcp[i][1]*destrcp[i][1] - destlcp[i][0]*destlcp[i][0] - destlcp[i][1]*destlcp[i][1]; }
 /*-----------------------------------------------*/

 double averV, sigmaV;
 off_t lowedgeV, upedgeV;
 double Vmax1, Vmax2, Vmin1;
 off_t xmaxV = getmax (srcfV, size4-begpoints-endpoints, Vmax1);
 off_t xminV = getmin (srcfV, size4-begpoints-endpoints, Vmin1);
 if (fabs(Vmin1) > fabs(Vmax1)) { xmaxV = xminV; Vmax1 = Vmin1; }
 get_energy (srcfV, size4-begpoints-endpoints, xmaxV, averV, sigmaV, lowedgeV, upedgeV);
 Vmax1 = (Vmax1 - averV) / (srcfI[xmaxV] - aver);
 Vmax2 = (srcfV[xmax] - averV) / (ymax - aver);
 double Vmean = 0.;
 for (off_t i=lowedge; i<=upedge; ++i) Vmean += (srcfV[i] - averV) / (srcfI[i] - aver);
 Vmean /= (upedge - lowedge + 1);
			    
 /*--- calculate P  --*/
 memset (srcfP, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcfP[j] = sqrt( (destrcp[i][0]*destrcp[i][0] + destrcp[i][1]*destrcp[i][1]) * (destlcp[i][0]*destlcp[i][0] + destlcp[i][1]*destlcp[i][1]) ); }
 /*-----------------------------------------------*/

 double averP, sigmaP;
 off_t lowedgeP, upedgeP;
 double Pmax1, Pmax2;
 off_t xmaxP = getmax (srcfP, size4-begpoints-endpoints, Pmax1);
 get_energy (srcfP, size4-begpoints-endpoints, xmaxP, averP, sigmaP, lowedgeP, upedgeP);
 Pmax1 = (Pmax1 - averP) / (srcfI[xmaxP] - aver);
 Pmax2 = (srcfP[xmax] - averP)  / (ymax - aver);
 double Pmean = 0.;
 for (off_t i=lowedge; i<=upedge; ++i) Pmean += (srcfP[i] -averP) / (srcfI[i] - aver);
 Pmean /= (upedge - lowedge + 1);

// printf ("# name mjd pulse_type level max_jy xmax lowedge upedge w90 weff weff_mcs energy_jymcs *** xmaxV Vmax1 Vmax2 Vmean *** xmaxP Pmax1 Pmax2 Pmean\n");
 printf ("%s\t%s\t%cP\t%lf\t%.3lf\t%ld\t%ld\t%ld\t%ld\t%.3lf\t%.3lf\t%.3lf *** %ld\t%lf\t%lf\t%lf *** %ld\t%lf\t%lf\t%lf\n", base, mjd, pulse_type, level, max_jy, (unsigned long)xmax, (unsigned long)lowedge, (unsigned long)upedge, (unsigned long)w90, weff, weff_mcs, energy_jymcs, (unsigned long)xmaxV, Vmax1, Vmax2, Vmean, (unsigned long)xmaxP, Pmax1, Pmax2, Pmean);

 delete (mjd);
 mpf_clear (mjd_t);
 free (output_path);
 free (base);
 delete (bandlcp);
 delete (bandrcp);
 delete (srcfI);
 delete (srcfV);
 delete (srcfP);
 delete (src);
 fftwf_free (matrix);
 fftwf_free (part1);
 fftwf_free (part2);
 fftwf_free (destlcp);
 fftwf_free (destrcp);
 fftwf_destroy_plan (plan1);
 fftwf_destroy_plan (plan2);
 fftwf_destroy_plan (backplan1);
 fftwf_destroy_plan (backplan2);

 return 0;
}
