#include "cut2sp.hpp"

// observing conf
ObsConf obs;

// cut-size is 548317, so we just ignore the 24029 counts in dedispersion
off_t size = 524288;

// number of iterations
off_t niter = 1;

/*--------------------------------------------------------------------------------------
 *                               M   A   I   N
 * -------------------------------------------------------------------------------------*/

int main (int argc, char *argv[]) {

 if (argc == 1) help (argv[0]);
 // initialize wisdom
 initialize_wisdom ();

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;
        
 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE, &r);
 /*------------------------------------------*/

 off_t size4 = 4*size;

 int next = parse_command_line (argc, argv);
 if (argc < next+1) { printf ("Not point the files to process!\n"); exit (1); }
 
 off_t cutsize = size + bitstat_points;
 off_t left = 0;

 signed char *src;
 src = new signed char[cutsize];
 if (!src) { perror("new"); exit(1); } 
 memset (src, 0, cutsize);

 float *srcf;
 srcf = new float[size4];
 if (!srcf) { perror("new"); exit(1); } 

 float *meanlcp, *meanrcp;
 meanlcp = new float[size4/2 + 1];
 meanrcp = new float[size4/2 + 1];
 if (!meanlcp || !meanrcp) { perror("new"); exit (1); }
 memset (meanlcp, 0, (size4/2 + 1)*sizeof(float));
 memset (meanrcp, 0, (size4/2 + 1)*sizeof(float));

 fftwf_complex *part1, *part2, *dest;
 part1 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 part2 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 dest = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size4);
 if (!part1 || !part2 || !dest) { perror("fftwf_malloc"); exit(1); }
 fftwf_plan plan1, plan2;
 plan1 = fftwf_plan_dft_1d (size, part1, part1, FFTW_FORWARD, FFTW_EXHAUSTIVE);
 plan2 = fftwf_plan_dft_1d (size, part2, part2, FFTW_FORWARD, FFTW_EXHAUSTIVE);

  /* read correction functions */
  float *bandlcp, *bandrcp;
  bandlcp = new float[size4/2+1];
  bandrcp = new float[size4/2+1];
  if (!bandlcp || !bandrcp) { perror("new"); exit (1); }
  memset (bandlcp, 0, (size4/2+1)*sizeof(float)); 
  memset (bandrcp, 0, (size4/2+1)*sizeof(float));
  int bandfile;
  if ((bandfile = open ("lcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
  lseek (bandfile, 0, SEEK_SET);
  memset (bandlcp, 0, (size4/2+1)*sizeof(float));
  read (bandfile, bandlcp, (size4/2+1)*sizeof(float));
  close (bandfile);
  if ((bandfile = open ("rcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
  lseek (bandfile, 0, SEEK_SET);
  memset (bandrcp, 0, (size4/2+1)*sizeof(float));
  read (bandfile, bandrcp, (size4/2+1)*sizeof(float));
  close (bandfile);
  /* - */

 printf ("processing: "); fflush (NULL);
 for (off_t iter=0; iter<niter; ++iter) {

 printf ("%ld,", (unsigned long)iter); fflush (NULL);
 left += size;
	
 memset (part1, 0, size*sizeof(fftwf_complex));
 memset (part2, 0, size*sizeof(fftwf_complex));
 memset (dest, 0, size4*sizeof(fftwf_complex));

 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next]);
 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+1]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next+2]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+3]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (dest, part1, part2, size);

 for (off_t i=0; i<size4/2+1; ++i) { meanlcp[i] += ((dest[i][0] / size4) * (dest[i][0] / size4) + (dest[i][1] / size4) * (dest[i][1] / size4)) * bandlcp[i] * bandlcp[i]; }

 memset (part1, 0, size*sizeof(fftwf_complex));
 memset (part2, 0, size*sizeof(fftwf_complex));
 memset (dest, 0, size4*sizeof(fftwf_complex));

 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next+4]);
 read_correct_n_fill (src, srcf, part1, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+5]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 0, argv[next+6]);
 read_correct_n_fill (src, srcf, part2, size, left, cutsize, size4, bitstat_points, bitstat_points, 1, argv[next+7]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (dest, part1, part2, size);

 for (off_t i=0; i<size4/2+1; ++i) { meanrcp[i] += ((dest[i][0] / size4) * (dest[i][0] / size4) + (dest[i][1] / size4) * (dest[i][1] / size4)) * bandrcp[i] * bandrcp[i]; }

 }

 for (off_t i=0; i<size4/2+1; ++i) { meanlcp[i] /= niter; meanrcp[i] /= niter; }
 write2file (meanlcp, size4/2+1, "spectrum.lcp.sp2");
 write2file (meanrcp, size4/2+1, "spectrum.rcp.sp2");
 printf ("done\n");

 delete (bandlcp);
 delete (bandrcp);
 delete (srcf);
 delete (src);
 fftwf_free (part1);
 fftwf_free (part2);
 fftwf_free (dest);
 fftwf_destroy_plan (plan1);
 fftwf_destroy_plan (plan2);
 delete (meanlcp);
 delete (meanrcp);

 return 0;
}
