#include "cut2stoks.hpp"

// observing conf
ObsConf obs;

// cut-size is 548317, so we just ignore the 24029 counts in dedispersion
off_t size = 524288;

// high precision value for mjd
mpf_t mjd_t;

// type of the GP type: 'M' - main pulse, 'I' - interpulse, 'F' - false event
char pulse_type = 'F';

/*--------------------------------------------------------------------------------------
 *                               M   A   I   N
 * -------------------------------------------------------------------------------------*/

int main (int argc, char *argv[]) {

 if (argc == 1) help (argv[0]);
 // initialize wisdom
 initialize_wisdom ();
 // initialize mjd
 mpf_init2 (mjd_t, 512);

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;
        
 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE, &r);
 /*------------------------------------------*/

 off_t size4 = 4*size;

 char *outname;
 outname  = new char[255];
 output_path  = new char[255];
 base = new char[255];
 if (!outname || !base || !output_path) { perror("new"); exit (1); }
 memset (output_path, 0, 255);
 memset (outname, 0, 255);
 memset (base, 0, 255);
 sprintf (base, "%s", "dest");
 sprintf (output_path, "%s", ".");

 int next = parse_command_line (argc, argv);
 if (argc < next+1) { printf ("Not point the files to process!\n"); exit (1); }
 
 struct stat info;
 if (stat(argv[next], &info) != 0) { perror("stat"); exit(1); }
 off_t cutsize = info.st_size;
 off_t leftborder = cutsize - size;

 unsigned long begpoints = obs.get_start_spoiled_points();
 unsigned long endpoints = obs.get_end_spoiled_points();
 obs.print_info ();
 printf ("cutsize: %ld  fft size: %ld\n", (unsigned long)cutsize, (unsigned long)size);

 signed char *src;
 src = new signed char[cutsize];
 if (!src) { perror("new"); exit(1); } 
 memset (src, 0, cutsize);

 float *srcf;
 srcf = new float[size4];
 if (!srcf) { perror("new"); exit(1); } 

 fftwf_complex *matrix;
 matrix = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * (size4/2 + 1));
 if (!matrix) { perror("fftwf_malloc"); exit (1); }
 memset (matrix, 0, (size4/2+1)*sizeof(fftwf_complex));
 matrix = get_matrix (matrix, size4);

 fftwf_complex *part1, *part2, *destlcp, *destrcp;
 part1 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 part2 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 destlcp = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size4);
 destrcp = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size4);
 if (!part1 || !part2 || !destlcp || !destrcp) { perror("fftwf_malloc"); exit(1); }
 fftwf_plan plan1, plan2, backplan1, backplan2;
 plan1 = fftwf_plan_dft_1d (size, part1, part1, FFTW_FORWARD, FFTW_EXHAUSTIVE);
 plan2 = fftwf_plan_dft_1d (size, part2, part2, FFTW_FORWARD, FFTW_EXHAUSTIVE);
 backplan1 = fftwf_plan_dft_1d (size4, destlcp, destlcp, FFTW_BACKWARD, FFTW_EXHAUSTIVE);
 backplan2 = fftwf_plan_dft_1d (size4, destrcp, destrcp, FFTW_BACKWARD, FFTW_EXHAUSTIVE);
 memset (part1, 0, size*sizeof(fftwf_complex));
 memset (part2, 0, size*sizeof(fftwf_complex));
 memset (destlcp, 0, size4*sizeof(fftwf_complex));
 memset (destrcp, 0, size4*sizeof(fftwf_complex));

 /* read correction functions */
 float *bandlcp, *bandrcp;
 bandlcp = new float[size4/2+1];
 bandrcp = new float[size4/2+1];
 if (!bandlcp || !bandrcp) { perror("new"); exit (1); }
 memset (bandlcp, 0, (size4/2+1)*sizeof(float));
 memset (bandrcp, 0, (size4/2+1)*sizeof(float));
 int bandfile;
 if ((bandfile = open ("lcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 lseek (bandfile, 0, SEEK_SET);
 memset (bandlcp, 0, (size4/2+1)*sizeof(float));
 read (bandfile, bandlcp, (size4/2+1)*sizeof(float));
 close (bandfile);
 if ((bandfile = open ("rcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 lseek (bandfile, 0, SEEK_SET);
 memset (bandrcp, 0, (size4/2+1)*sizeof(float));
 read (bandfile, bandrcp, (size4/2+1)*sizeof(float));
 close (bandfile);
 /* - */

 read_correct_n_fill (src, srcf, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next]);
 read_correct_n_fill (src, srcf, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+1]);
 read_correct_n_fill (src, srcf, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next+2]);
 read_correct_n_fill (src, srcf, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+3]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (destlcp, part1, part2, size);

 destlcp = dedispersion (destlcp, matrix, bandlcp, backplan1, size4, true, false);

 memset (part1, 0, size*sizeof(fftwf_complex));
 memset (part2, 0, size*sizeof(fftwf_complex));

 read_correct_n_fill (src, srcf, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next+4]);
 read_correct_n_fill (src, srcf, part1, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+5]);
 read_correct_n_fill (src, srcf, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 0, argv[next+6]);
 read_correct_n_fill (src, srcf, part2, size, 0, cutsize, size4, bitstat_points, leftborder, 1, argv[next+7]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (destrcp, part1, part2, size);

 destrcp = dedispersion (destrcp, matrix, bandrcp, backplan2, size4, false, false);

 char *mjd;
 mjd = new char[255];
 if (!mjd) { perror("new"); exit (1); } 
 memset (mjd, 0, 255);
 mjd = getMJD (mjd_t, leftborder, begpoints, obs.get_time_resolution(), mjd);
 printf ("mjd = %s\n", mjd);

 for (off_t i=begpoints; i<size4-endpoints; ++i) { destlcp[i][0] /= size4; destlcp[i][1] /= size4; destrcp[i][0] /= size4; destrcp[i][1] /= size4; }

 /*--- calculate I  ---*/ 
 memset (outname, 0, 255);
 sprintf (outname, "%s/%s%s", output_path, base, ".cut.I");
 memset (srcf, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcf[j] = destlcp[i][0]*destlcp[i][0] + destlcp[i][1]*destlcp[i][1] + destrcp[i][0]*destrcp[i][0] + destrcp[i][1]*destrcp[i][1]; }
 write2file (srcf, mjd, pulse_type, size4-begpoints-endpoints, outname);
 /*-----------------------------------------------*/
	       
 /*--- calculate V  ---*/
 memset (outname, 0, 255);
 sprintf (outname, "%s/%s%s", output_path, base, ".cut.V");
 memset (srcf, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcf[j] = destrcp[i][0]*destrcp[i][0] + destrcp[i][1]*destrcp[i][1] - destlcp[i][0]*destlcp[i][0] - destlcp[i][1]*destlcp[i][1]; }
 write2file (srcf, mjd, pulse_type, size4-begpoints-endpoints, outname);
 /*-----------------------------------------------*/
			    
 /*--- calculate Q  ---*/
 memset (outname, 0, 255);
 sprintf (outname, "%s/%s%s", output_path, base, ".cut.Q");
 memset (srcf, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcf[j] = destrcp[i][0]*destlcp[i][0] + destrcp[i][1]*destlcp[i][1]; }
 write2file (srcf, mjd, pulse_type, size4-begpoints-endpoints, outname);
 /*-----------------------------------------------*/
				       
 /*--- calculate U  --*/
 memset (outname, 0, 255);
 sprintf (outname, "%s/%s%s", output_path, base, ".cut.U");
 memset (srcf, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcf[j] = destrcp[i][1]*destlcp[i][0] - destrcp[i][0]*destlcp[i][1]; }
 write2file (srcf, mjd, pulse_type, size4-begpoints-endpoints, outname);
 /*-----------------------------------------------*/
						  
 /*--- calculate P  --*/
 memset (outname, 0, 255);
 sprintf (outname, "%s/%s%s", output_path, base, ".cut.P");
 memset (srcf, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcf[j] = sqrt( (destrcp[i][0]*destrcp[i][0] + destrcp[i][1]*destrcp[i][1]) * (destlcp[i][0]*destlcp[i][0] + destlcp[i][1]*destlcp[i][1]) ); }
 write2file (srcf, mjd, pulse_type, size4-begpoints-endpoints, outname);
 /*-----------------------------------------------*/

 /*--- calculate A  в градусах --*/
 memset (outname, 0, 255);
 sprintf (outname, "%s/%s%s", output_path, base, ".cut.A");
 memset (srcf, 0, size4*sizeof(float));
 if (!phase_straigh) { for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcf[j] = (180./M_PI)*atan2( destrcp[i][1]*destlcp[i][0] - destrcp[i][0]*destlcp[i][1], destrcp[i][0]*destlcp[i][0] + destrcp[i][1]*destlcp[i][1]); } }
  else { // продолжение фазы    
         double add = 0., prev = 0., cur, test;
	 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) {
          cur = add + (180./M_PI)*atan2( destrcp[i][1]*destlcp[i][0] - destrcp[i][0]*destlcp[i][1], destrcp[i][0]*destlcp[i][0] + destrcp[i][1]*destlcp[i][1]);
          test = cur - prev;
          if (fabs(test) < 180.) ; else {
           if (test < 0.) { add += 360.; cur += 360.; } else { add -= 360.; cur -= 360.; }
          }
	  prev = cur;
	  srcf[j] = (float)cur;
	 } // for     
        } // else
 write2file (srcf, mjd, pulse_type, size4-begpoints-endpoints, outname);
 /*-----------------------------------------------*/

 delete (mjd);
 mpf_clear (mjd_t);
 free (output_path);
 delete (outname);
 free (base);
 delete (bandlcp);
 delete (bandrcp);
 delete (srcf);
 delete (src);
 fftwf_free (matrix);
 fftwf_free (part1);
 fftwf_free (part2);
 fftwf_free (destlcp);
 fftwf_free (destrcp);
 fftwf_destroy_plan (plan1);
 fftwf_destroy_plan (plan2);
 fftwf_destroy_plan (backplan1);
 fftwf_destroy_plan (backplan2);

 return 0;
}
