#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>

#include <inttypes.h>

#include <fftw3.h>




#define ALPHA1 2.607131e16
#define ALPHA2 (ALPHA1 / (2 * M_PI))


struct pomexInfo {  char type;                /*-- RFI type: "m" (mono-) or "p" (poly-) chromate --*/
                    long start_pos,           /*-- Start point for RFI (for poly) or RFI itself (for mono-) --*/
                         end_pos,             /*-- End point for RFI, in case of mono- RFI the other field are not necessary --*/
                         sigma_start_pos,     /*-- Start edge of the window for rms estimation --*/
                         sigma_size;          /*-- Window size for rms estimation --*/
                  };

extern struct pomexInfo *pomArray;

void PomexCorrectionFunc ( fftwf_complex *, long );
void PomexFileRead ( char * );
void PomexFileCorrect ( long, long );


size_t size = 524288, badpoints, truesize;
fftwf_complex *matrix, *x, *y, *z;
float Level, *det;
fftwf_plan pfor, prev;
char bandext[4] = { 'Z', 'Y', 'X', 'W' };


// dedispersion
void DeDispersion (/*int e*/) {
 
 fftwf_execute (pfor);
 
 PomexCorrectionFunc (x, size/2 + 1);

 float re, im;
 for (size_t i=0; i<size/2 + 1; ++i) {
  re = x[i][0]; im = x[i][1];
  x[i][0] = re * matrix[i][0] - matrix[i][1] * im;
  x[i][1] = re * matrix[i][1] + matrix[i][0] * im; 
 }
}

// level calculation
void LevelCalc () {

 float sum = 0.;
 for (size_t i=0; i<size; ++i) sum += x[i][0];
 Level = sum/size;
}

// level elimination
void NoLevel () { 
 for (size_t i=0; i<size; ++i) x[i][0] -= Level;
}



// --------------- M A I N ------------------

int main (int argc, char* argv[]) {
 if (argc == 1) {
  printf ("Usage: %s <matrix> <RFI file> <size of band> <template first> <template second>\n", argv[0]);
  printf (" <template first> - output file prefix (first part of the name)\n");
  printf (" <template second> - output file suffix (the rest part of the name)\n");
  exit (0);
 }
 if (argc < 6) {
  printf ("Not all the parameters!\n");
  exit (1);
 }

 matrix = (fftwf_complex *) fftwf_malloc ((size/2 + 1) * sizeof(fftwf_complex));
 if (!matrix) { printf ("Not enough memory!\n"); exit(1); }

 int file_matrix;

 x = (fftwf_complex *) fftwf_malloc (size * sizeof(fftwf_complex));
 y = (fftwf_complex *) fftwf_malloc (4*size * sizeof(fftwf_complex));
 z = (fftwf_complex *) fftwf_malloc (4*size * sizeof(fftwf_complex));
 if (!x || !y || !z) { printf ("Not enough memory!\n"); exit(1); }

 float* data;
 data = new float[size];
 if (!data) { printf ("Not enough memory!\n"); exit(1); }

 char *outname, *basename, *matrixname, *interfname;
 outname = new char[255];
 basename = new char[255];
 matrixname = new char[255];
 interfname = new char[255];
 if (!outname || !basename || !matrixname || !interfname) { printf("Not enough memory!\n"); exit(1); }
 memset (outname, 0, 255);
 memset (basename, 0, 255);
 sprintf (outname, "%s%s.det", argv[4], argv[5]);
 sprintf (basename, "%s.%c%s", argv[4], bandext[0], argv[5]);

 struct stat info;
 if ( stat(basename, &info) != 0 ) { perror("stat"); exit (1); }

 int in, out;
 ssize_t bytesread;
 uintmax_t left = 0, filesize = info.st_size / sizeof (float);

 truesize = 4*size;

 det = new float[truesize];
 if (!det) { printf("Not enough memory!\n"); exit(1); }

 /* output some information */
 printf ("\n");
 printf ("ALPHA1: %g\n", ALPHA1);
 printf ("ALPHA2: %g\n", ALPHA2);
 printf ("filesize: %.0lf\n", (double)filesize);
 printf ("size: %lu\n", size);
 printf ("matrix: %s    interf: %s\n", argv[1], argv[2]);
 printf ("\nProcessing ...");
 fflush (NULL);

 pfor = fftwf_plan_dft_1d (size, x, x, FFTW_FORWARD, FFTW_ESTIMATE);
 prev = fftwf_plan_dft_1d (4*size, z, y, FFTW_BACKWARD, FFTW_ESTIMATE);

 memset (det, 0, truesize * sizeof(float));
 
 // at first remove existing file
 unlink (outname);
 /* open for append */
 if ( (out = open (outname, O_WRONLY | O_CREAT | O_APPEND, 0644)) < 0 ) {
  perror("open output file: ");
  exit(1);
 }
 lseek (out, 0, SEEK_SET);

 /* cycle on bands */
 for (int b=0; b<4; ++b) {

 memset (matrixname, 0, 255);
 sprintf (matrixname, "%s.%c", argv[1], bandext[b]);
 memset (interfname, 0, 255);
 sprintf (interfname, "%s.%c", argv[2], bandext[b]);
 memset (basename, 0, 255);
 sprintf (basename, "%s.%c%s", argv[4], bandext[b], argv[5]);

 if ( (file_matrix = open(matrixname, O_RDONLY)) == -1 )
  { printf("\nError:\nCouldn't open matrix \"%s\" for reading!\n", matrixname); exit(1);  }
  read(file_matrix, matrix, (size/2 + 1) * sizeof(fftwf_complex));
  close(file_matrix);

 PomexFileRead (interfname);
 PomexFileCorrect (atol(argv[3]), size/2 + 1);

  /* read input file */
  if ( (in = open (basename, O_RDONLY)) < 0 ) { perror("open input file"); exit(1); }
   lseek (in, 0, SEEK_SET);
   memset (data, 0, size * sizeof(float));
   bytesread = read (in, data, size * sizeof(float));
   if (bytesread != size * sizeof (float)) { close(in); break; }
  close (in);
  for (size_t i=0; i<size; ++i) { x[i][0] = data[i]; x[i][1] = 0.; }

  LevelCalc(); 
  printf (" Level = %g ", Level); fflush (NULL);
  NoLevel();
  DeDispersion();
   
  if (b == 0) { for (size_t t=0; t<size/2; ++t) { z[t][0] = x[t][0]; z[t][1] = x[t][1]; } }
  if (b == 1) { for (size_t t=0; t<size/2 + 1; ++t) { z[t+size/2][0] = x[t][0]; z[t+size/2][1] = x[t][1]; } }
  if (b == 2) { for (size_t t=0; t<size/2; ++t) { z[t+size+1][0] = x[t][0]; z[t+size+1][1] = x[t][1]; } }
  if (b == 3) { for (size_t t=0; t<size/2; ++t) { z[t+size+1+size/2][0] = x[t][0]; z[b+size+1+size/2][1] = x[t][1]; } }


 } // cycle on bands

  for (size_t i=2*size + 1; i<4*size; ++i) { z[i][0] = 0.; z[i][1] = 0.; }
   
  // reverse
  fftwf_execute (prev);

  // normalize
  for (size_t i=0; i<truesize; ++i) { det[i] = 4 * ((y[i][0] / (4*size)) * (y[i][0] / (4*size)) + (y[i][1] / (4*size)) * (y[i][1] / (4*size))); }

  if (truesize * sizeof(float) != write(out, det, truesize * sizeof(float))) {
     perror("write out: ");
     exit(1);
  }
   
 close (out);

 fftwf_destroy_plan (pfor);
 fftwf_destroy_plan (prev);
 fftwf_free (matrix);
 fftwf_free (x);
 fftwf_free (y);
 fftwf_free (z);
 delete (outname);
 delete (basename);
 delete (matrixname);
 delete (interfname);
 delete (det);
 delete (data);

 return 0;
}
