#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>

#include <fftw3.h>




#define ALPHA1 2.607131e16
#define ALPHA2 (ALPHA1 / (2 * M_PI))

long size;
float *matrix, *band;

double ind  = 1.0e6, // multiplier
       sign,         // +1 for direct band , -1 for inverse band 
       freq,   // lower frequency for direct band (in MHz), upper freq for inverse band (in MHz)
       DeltaF = 0., // not in use now       
       timeres,     // time resolution (in ns)
       DM,       // dispersion measure
       T;

double Calc ( long j ) {
  return  ( ( ( ( ALPHA1 * DM ) / ( freq + sign * (j/T)) ) / (T * T * freq * freq * ind) ) * ((double)j) ) * ((double)j) + ( ALPHA1 * DM * DeltaF ) / freq / (freq - DeltaF) / ind;
}

/*------------ c o m p u t e   m a t r i x ---------------*/

void ComputeMatrix () {
 double z;
 
 for (long i=0, j=0; i < size + 2; i+=2, ++j) {
   z = Calc (j);
   matrix[i]   = (float)cos(z) * band[j];
   matrix[i+1] = -1. * sign * (float)sin(z) * band[j]; 
 }
}


/*---- Get the nearest size of power 2 downwards from the actual size -------*/

long resize (long sz) {
 long n = 0;
 sz += 1;
 while ( (1<<(++n)) < sz) ;
 return (1<<(--n));
}

/*--------------- Band interpolation ------------------*/

void BandInterpolation ( char * bandfile, float * x, long rsize, long nsize )
{
  if ( 2*(rsize-1) != resize(2*(rsize-1)) ) {
   printf("\nError:\nSize of file \"%s\" is expected to be equal to N/2 + 1 harmonics,\nwhere N is power if 2 !\n", bandfile);
   exit(1);
  }
  if (rsize < 2) {
   printf("\nError\nSize of file \"%s\" must be more then 1 harmonic!\n", bandfile);
   exit(1);
  }

  int index = ( rsize < nsize ? (nsize-1)/(rsize-1) : (rsize-1)/(nsize-1) );
  char dir = (rsize < nsize ? 1 : 0);

  switch (dir) {
   case 1:  for (long i=rsize-1;i>0;i--) {
             x[i*index] = x[i];
             for (int j=1;j<index;j++)
              x[i*index-j] = x[i] + (x[i-1] - x[i])*((float)(j/index));
                                          }
            break;
   case 0:  for (long i=1;i<nsize;i++) {
             for (int j=1;j<index;j++) x[i] += x[i+j];
              x[i] /= index;
                                       }
            break;
   default: break;
                }
}

/*------------- m a i n ---------------*/

int main (int argc, char* argv[]) {

 if (argc == 1) {
  printf ("Usage: %s <band> <outfile>\n", argv[0]);
  exit (0);
 }
 if (argc < 3) {
  printf ("Not all the parameters!\n");
  exit (1);
 }
 printf ("size: "); scanf ("%ld", &size);
 printf ("band (1 for direct, -1 for inverse): "); scanf ("%lf", &sign);
 printf ("freq (MHz): "); scanf ("%lf", &freq);
 printf ("timeres (ns): "); scanf ("%lf", &timeres);
 printf ("DM: "); scanf ("%lf", &DM);

 T = (timeres * size)/1000.;
 printf ("size: %ld  sign: %.1lf   freq: %lf  timeres:  %lf   DM:  %lf  T: %lf\n", size, sign, freq, timeres, DM, T);

 matrix = new float[size + 2];
 if (!matrix) { printf("Not enough memory!\n"); exit(1); }
 memset(matrix, 0, (size + 2)*sizeof(float));

 struct stat inform;
 if ( stat(argv[1], &inform) != 0 ) { perror("stat band file"); exit(1); }
 long bs = inform.st_size/sizeof(float);

 band = new float[(bs > size/2 + 1 ? bs : size/2 + 1)];
 if (!band) { printf("Not enough memory!\n"); exit(1); }
 memset (band, 0, (bs > size/2 + 1 ? bs : size/2 + 1)*sizeof(float));

 /* read file with band */
 int in;
 if ( (in = open (argv[1], O_RDONLY)) < 0 ) { perror("open band file"); exit(1); }
  lseek(in, 0, SEEK_SET);
  read(in, band, inform.st_size);
 close(in);

 if (bs != size/2 + 1) {
  printf("\n- Warning ! -\n"
          "Size = %ld of band correction function and size = %ld\n"
          "of dispersion matrix don't match !\n"
          "...interpolation begin...", bs, size/2 + 1);
      fflush(NULL);
      BandInterpolation(argv[1], band, bs, size/2 + 1);
      printf("done\n\n");
 }

 ComputeMatrix();

 int out;

 if ( (out = open(argv[2], O_TRUNC | O_CREAT | O_WRONLY, 0644)) == -1 )
  { printf("\nError: Couldn't open file %s for writing!\n", argv[2]);
    exit(1);
   } else { lseek(out, 0, SEEK_SET);
            write(out, matrix, (size + 2)*sizeof(float));
            close(out);
           }

 delete (matrix);
 delete (band);
 return 0;
}
