#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>

#include <inttypes.h>

#include <fftw3.h>




#define ALPHA1 2.607131e16
#define ALPHA2 (ALPHA1 / (2 * M_PI))


struct pomexInfo {  char type;                /*-- RFI type: "m" (mono-) or "p" (poly-) chromate --*/
                    long start_pos,           /*-- Start point for RFI (for poly) or RFI itself (for mono-) --*/
                         end_pos,             /*-- End point for RFI, in case of mono- RFI the other field are not necessary --*/
                         sigma_start_pos,     /*-- Start edge of the window for rms estimation --*/
                         sigma_size;          /*-- Window size for rms estimation --*/
                  };

extern struct pomexInfo *pomArray;

void PomexCorrectionFunc ( fftwf_complex *, long );
void PomexFileRead ( char * );
void PomexFileCorrect ( long, long );


size_t size, badpoints, truesize;
fftwf_complex *matrix, *x, *y;
float Level, *det, *real, *as;
fftwf_plan pfor, prev;

double freq,   // lower freq for direct band (in MHz), upper freq for inverse band (in MHz)
       bandwidth,   // bandwidth in MHz
       timeres,     // time resolution (in ns)
       DM;       // dispersion measure
bool inverse_band = true; 

void BadPoints () {
 double f = freq - bandwidth,
        tbad = ( DM * ALPHA2 * ( 1./(f * f) - 1./(freq * freq) ) * 0.001 ) / timeres;

 badpoints = (size_t)ceil(fabs(tbad));
}

// dedispersion
void DeDispersion () {
 
 fftwf_execute (pfor);
 
 PomexCorrectionFunc (y, size/2 + 1);

 for (size_t i=0; i<size/2 + 1; ++i) {
  x[i][0] = y[i][0] * matrix[i][0] - matrix[i][1] * y[i][1];
  x[i][1] = y[i][0] * matrix[i][1] + matrix[i][0] * y[i][1]; 
 }
 for (size_t i=size/2 + 1; i<size; ++i) { x[i][0] = 0.; x[i][1] = 0.; }

 // reverse
 fftwf_execute (prev);

 // normalize
 for (size_t i=0; i<truesize; ++i) { det[i] = 4 * ((y[i][0] / size) * (y[i][0] / size) + (y[i][1] / size) * (y[i][1] / size)); }
 for (size_t i=0; i<truesize; ++i) { real[i] = 2 * (y[i][0] / size ); }
// for (size_t i=0, j=0; i<truesize; ++i, j+=2) { as[j] = 2 * (y[i][0] / size ); as[j+1] = 2 * (y[i][1] / size ); }

}

// level calculation
void LevelCalc () {

 float sum = 0.;
 for (size_t i=0; i<size; ++i) sum += x[i][0];
 Level = sum/size;
}

// level elimination
void NoLevel () { 
 for (size_t i=0; i<size; ++i) x[i][0] -= Level;
}



// --------------- M A I N ------------------

int main (int argc, char* argv[]) {
 if (argc == 1) {
  printf ("Usage: %s <matrix> <pomex> <size of band> <size> <freq> <bandwidth> <timeres> <DM> <banddir: 1 or 0> <infile>\n", argv[0]);
  exit (0);
 }
 if (argc < 11) {
  printf ("Not all the parameters!\n");
  exit (1);
 }

 unsigned char sig;
 size = atol (argv[4]);
 freq = atof (argv[5]);
 bandwidth = atof (argv[6]);
 timeres = atof (argv[7]);
 DM = atof (argv[8]);
 sig = (unsigned char)atoi (argv[9]);

 if (sig) inverse_band = false; else inverse_band = true;

 printf ("size: %ld  freq: %lf  bandwidth: %lf  timeres: %lf  DM: %lf  direct: %d\n", size, freq, bandwidth, timeres, DM, sig);

 matrix = (fftwf_complex *) fftwf_malloc ((size/2 + 1) * sizeof(fftwf_complex));
 if (!matrix) { printf ("Not enough memory!\n"); exit(1); }

 int file_matrix;
 if ( (file_matrix = open(argv[1], O_RDONLY)) == -1 )
  { printf("\nError:\nCouldn't open matrix \"%s\" for reading!\n", argv[1]); exit(1);  }
  read(file_matrix, matrix, (size/2 + 1) * sizeof(fftwf_complex));
  close(file_matrix);

 PomexFileRead (argv[2]);
 PomexFileCorrect (atol(argv[3]), size/2 + 1);

 x = (fftwf_complex *) fftwf_malloc (size * sizeof(fftwf_complex));
 y = (fftwf_complex *) fftwf_malloc (size * sizeof(fftwf_complex));
 if (!x || !y) { printf ("Not enough memory!\n"); exit(1); }

// signed char* data;
 float* data;
// data = new signed char[size];
 data = new float[size];
 if (!data) { printf ("Not enough memory!\n"); exit(1); }

 char *outname, *outnamereal; //, *outnameas;
 outname = new char[255];
 outnamereal = new char[255];
// outnameas = new char[255];
// if (!outname || !outnameas) { printf("Not enough memory!\n"); exit(1); }
 if (!outname || !outnamereal) { printf("Not enough memory!\n"); exit(1); }

 struct stat info;
 if ( stat(argv[10], &info) != 0 ) { perror("stat"); exit (1); }

 int in, out, outreal; //, outas;
 unsigned long niter; 
 ssize_t bytesread;
 uintmax_t left, filesize = info.st_size / sizeof (float);

 BadPoints();
 truesize = size - badpoints;

 det = new float[truesize];
 if (!det) { printf("Not enough memory!\n"); exit(1); }

 real = new float[truesize];
 if (!real) { printf("Not enough memory!\n"); exit(1); }

// as = new float[2 * truesize];
// if (!as) { printf("Not enough memory!\n"); exit(1); }

 niter = (unsigned long)(((double)filesize)/(size - badpoints));

 /* output some information */
 printf ("\n");
 printf ("ALPHA1: %g\n", ALPHA1);
 printf ("ALPHA2: %g\n", ALPHA2);
 printf ("filesize: %.0lf\n", (double)filesize);
 printf ("size: %lu   badpoints: %lu  -> truesize: %lu\n", size, badpoints, truesize);
 printf ("freq: %g\nbandwidth: %g [%s]\ntimeres: %g\nDM = %g\n", freq, bandwidth, (inverse_band ? "inverse" : "direct"), timeres, DM);
 printf ("matrix: %s\n", argv[1]);
 printf ("\nProcessing [%4lu] ...", niter);
 fflush (NULL);

 pfor = fftwf_plan_dft_1d (size, x, y, FFTW_FORWARD, FFTW_ESTIMATE);
 prev = fftwf_plan_dft_1d (size, x, y, FFTW_BACKWARD, FFTW_ESTIMATE);

 for (int fcount=10; fcount<argc; ++fcount) {

 printf ("\n[%2d - %s] ", fcount-10, argv[fcount]); fflush (NULL);
 memset (outname, 0, 255);
 sprintf (outname, "%s.det", argv[fcount]);
 memset (outnamereal, 0, 255);
 sprintf (outnamereal, "%s.real", argv[fcount]);
// memset (outnameas, 0, 255);
// sprintf (outnameas, "%s.as", argv[fcount]);

 memset (det, 0, truesize * sizeof(float));
 memset (real, 0, truesize * sizeof(float));
// memset (as, 0, 2 * truesize * sizeof(float));
 
 // at first remove existing file
 unlink (outname);
 unlink (outnamereal);
// unlink (outnameas);
 /* open for append */
 if ( (out = open (outname, O_WRONLY | O_CREAT | O_APPEND, 0644)) < 0 ) {
  perror("open output file: ");
  exit(1);
 }
 lseek (out, 0, SEEK_SET);
 /* open for append */
// if ( (outas = open (outnameas, O_WRONLY | O_CREAT | O_APPEND, 0644)) < 0 ) {
//  perror("open output (as) file: ");
//  exit(1);
// }
// lseek (outas, 0, SEEK_SET);

 if ( (outreal = open (outnamereal, O_WRONLY | O_CREAT | O_APPEND, 0644)) < 0 ) {
  perror("open output (real) file: ");
  exit(1);
 }
 lseek (outreal, 0, SEEK_SET);

  for (int iter=0; iter<niter; ++iter) {
 
  printf (".%5lu", iter+1); fflush (NULL);
  left = ((uintmax_t)(size - badpoints)) * ((uintmax_t)iter);
  if (filesize - left < (uintmax_t)(size)) left = filesize - (uintmax_t)(size);

  /* read input file */
  if ( (in = open (argv[fcount], O_RDONLY)) < 0 ) { perror("open input file"); exit(1); }
   lseek (in, (off_t)left * sizeof(float), SEEK_SET);
//   memset (data, 0, size);
   memset (data, 0, size * sizeof(float));
//   bytesread = read (in, data, size);
   bytesread = read (in, data, size * sizeof(float));
//   if (bytesread != size) { close(in); break; }
   if (bytesread != size * sizeof (float)) { close(in); break; }
  close (in);
//  for (size_t i=0; i<size; ++i) { x[i][0] = (float)data[i]; x[i][1] = 0.; }
  for (size_t i=0; i<size; ++i) { x[i][0] = data[i]; x[i][1] = 0.; }

  if (iter == 0) { LevelCalc(); printf ("\b\b\b\b\b Level = %g      ", Level); fflush (NULL); }
  NoLevel();
  DeDispersion();

  if (truesize * sizeof(float) != write(out, det, truesize * sizeof(float))) {
     perror("write out: ");
     exit(1);
  }
   
  if (truesize * sizeof(float) != write(outreal, real, truesize * sizeof(float))) {
     perror("write outreal: ");
     exit(1);
  }
   
//  if (2 * truesize * sizeof(float) != write(outas, as, 2 * truesize * sizeof(float))) {
//     perror("write outas: ");
//     exit(1);
//  }

  printf ("\b\b\b\b\b"); fflush (NULL);
 } /* iter cycle */
  
 close (out);
 close (outreal);
// close (outas);

} 
 
 fftwf_destroy_plan (pfor);
 fftwf_destroy_plan (prev);
 fftwf_free (matrix);
 fftwf_free (x);
 fftwf_free (y);
 delete (outname);
 delete (outnamereal);
// delete (outnameas);
 delete (det);
 delete (real);
// delete (as);
 delete (data);

 return 0;
}
