#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <time.h>

#include <fftw3.h>

#define MaxString 255

struct pomexInfo {  char type;                /*-- RFI type: "m" (mono-) or "p" (poly-) chromate --*/
                    long start_pos,           /*-- Start point for RFI (for poly) or RFI itself (for mono-) --*/
                         end_pos,             /*-- End point for RFI, in case of mono- RFI the other field are not necessary --*/
                         sigma_start_pos,     /*-- Start edge of the window for rms estimation --*/
                         sigma_size;          /*-- Window size for rms estimation --*/
                  };

int pomNumber;
struct pomexInfo *pomArray;

/*-------- Determine the number of valuable raws in RFI file, and hence
           the size of pomexInfo array -----------*/
 
int PomexNumber ( char * pomexfile )
 {
  int n = 0;
  FILE * pom;
    if ( (pom = fopen(pomexfile, "rt")) == NULL ) { 
      printf("\nError:\nCouldn't open interference file \"%s\" !\n", pomexfile);
      exit(1);
	                                                       }
                fseek(pom, 0, SEEK_SET);
		
  char *p, curline[MaxString+1];

  while( fgets(curline, MaxString+1, pom)  ) {  
	p = curline;
	curline[(int)strlen(curline) -1] = 0;
	while(*p && isascii(*p) && isspace(*p)) ++p;
	if(!*p || *p == '#') continue;
	n++;
	                                      }
      fclose(pom);
  return n;    
 }

/*------ Get the number from the raw ------------*/

char * GetNumber( char * dest, char * p )
{
  int n = 0;
  char *p1 = p;
  while(*p1 && isascii(*p1) && !isspace(*p1)) { p1++; n++; }
  strncat(dest, p, n);
  return dest;
}

/*--------- Analyze read raw ------------------*/

void AnalyzeLine( char *p, struct pomexInfo *tmp, char * pomexfile )
 {
   int l = 1, num;
   void *entry[] = { &tmp->start_pos, &tmp->end_pos, &tmp->sigma_start_pos, &tmp->sigma_size };
   char *dest;
         dest = new char[MaxString+1];
	 if (!dest) { printf("Not enough memory!\n"); exit(1); }
   if ( !strncasecmp(p, "m", l) || !strncasecmp(p, "p", l) )
    {
      tmp->type = *p;
      p += l;
      num = (tmp->type == 'm' || tmp->type == 'M' ? 1 : 4);
       for (int k=0;k<num;k++) {
        while(*p && isascii(*p) && isspace(*p)) p++;
	if (*p) { memset(dest, 0, MaxString+1);
	          dest = GetNumber(dest, p);
	          *(long *)entry[k] = atol(dest);
	          p += (int)strlen(dest);
	         } else { printf("\nError:\nBad line in interference file \"%s\" : not full !\n", pomexfile);
		          exit(1);
		         }
                                }      
     }
    else { printf("\nError:\nBad line in interference file \"%s\" : %s !\n", pomexfile, p);
           exit(1);
          }
    delete(dest);	  
 }

/*--------------- Read RFI file --------------*/
 
void PomexFileRead ( char * pomexfile )
 {
  pomNumber = PomexNumber(pomexfile);

  FILE * pom;
         if ( (pom = fopen(pomexfile, "rt")) == NULL ) {
	    printf("\nError:\nCouldn't open interference file \"%s\" !\n", pomexfile);
	    exit(1);
	                                                            }
                     fseek(pom, 0, SEEK_SET);
		     
  pomArray = new (struct pomexInfo)[pomNumber];
  if (!pomArray) { printf("Not enough memory!\n"); exit(1); }
		    
  char *p, curline[MaxString+1];
  struct pomexInfo *tmp = pomArray;

  while( fgets(curline, MaxString+1, pom)  ) {
	p = curline;
	curline[(int)strlen(curline) -1] = 0;
	while(*p && isascii(*p) && isspace(*p)) ++p;
	if(!*p || *p == '#') continue;
	AnalyzeLine(p, tmp, pomexfile);
	tmp++;
	                                      }
      fclose(pom);						       
 }

/*--------------- Get random gauss distributed number ------------*/

void NormRand ( fftwf_complex * y, long q, float rs, float is )
{
 long rn1, rn2;
 double yj, zj, pj;

 do { rn1 = random(); rn2 = random();
      yj = 2.*(((double)rn1)/((double)(RAND_MAX+1.0))) - 1.;
      zj = 2.*(((double)rn2)/((double)(RAND_MAX+1.0))) - 1.;
      pj = yj*yj + zj*zj;
     } while ( pj >= 1. );

 y[q][0] = rs * yj * sqrt ((-2.*log(pj))/pj);
 y[q][1] = is * zj * sqrt ((-2.*log(pj))/pj);
}


/*---------- RFI elimination -------------*/

void PomexCorrectionFunc ( fftwf_complex * y, long size )
{
  if (pomNumber == 0) return;
   struct pomexInfo *tmp = pomArray;
   long ind, s1, si, beg, end;
   float sum_r, sum_i, sum2_r, sum2_i, sigma_r, sigma_i;  
   
   for (int i=0; i<pomNumber; i++, tmp++) {
   
   char tip = (tmp->type == 'm' || tmp->type == 'M' ? 1 : 4);
   void *entry[] = { &tmp->start_pos, &tmp->end_pos, &tmp->sigma_start_pos, &tmp->sigma_size };
   for (char j=0; j<tip; j++) {
    if ( *(long *)entry[j] < 0 || *(long *)entry[j] > size - 1 ) {
     printf("\n- warning !.. - interference N%d has bad value and will be ignored!\n\n", i);
     tip = 10;
     break;
    }
   }
     srandom((unsigned long)time(NULL));
     switch (tip) {
       case 1:  ind = tmp->start_pos;
                y[ind][0] = (ind == 0 ? (y[ind+2][0]-y[ind+1][0])/2 : (ind == size-1 ? (y[ind-2][0]-y[ind-1][0])/2 : (y[ind+1][0]-y[ind-1][0])/2 )) * cos(y[ind][0]);
                y[ind][1] = (ind == 0 ? (y[ind+2][1]-y[ind+1][1])/2 : (ind == size-1 ? (y[ind-2][1]-y[ind-1][1])/2 : (y[ind+1][1]-y[ind-1][1])/2 )) * sin(y[ind][1]);
                break;
       case 4:  s1 = tmp->sigma_start_pos; si = tmp->sigma_size;
                sum_r = 0.; sum_i = 0.; sum2_r = 0.; sum2_i = 0.; sigma_r; sigma_i;
		for (long w=s1; w<s1+si; w++) {
		 sum_r += y[w][0]; sum2_r += y[w][0]*y[w][0];
		 sum_i += y[w][1]; sum2_i += y[w][1]*y[w][1];
		}
		sum_r /= si; sum_i /= si;
		sum2_r /= si; sum2_i /= si;
		sigma_r = sqrt(sum2_r - sum_r*sum_r);
		sigma_i = sqrt(sum2_i - sum_i*sum_i);
		beg = tmp->start_pos; end = tmp->end_pos;
		for (long q=beg;q<=end;q++) NormRand (y, q, sigma_r, sigma_i);
                break;
       case 10: break;		
       default: break;		
     }
   }
}

/*--------- Multiplication and dividing by number of long-values in pomexInfo structure ----------*/ 

void pomexLocChangeFF ( struct pomexInfo *tmp, long index )
{
  void *entry[] = { &tmp->start_pos, &tmp->end_pos, &tmp->sigma_start_pos, &tmp->sigma_size };
  int num = (tmp->type == 'm' ? 1 : 4);

  for (int k=0;k<num;k++) *(long *)entry[k] *= index;
}

void pomexLocChangeRV ( struct pomexInfo *tmp, long index )
{
  void *entry[] = { &tmp->start_pos, &tmp->end_pos, &tmp->sigma_start_pos, &tmp->sigma_size };
  int num = (tmp->type == 'm' ? 1 : 4);

  for (int k=0;k<num;k++) *(long *)entry[k] = (long)ceil((double)((*(long *)entry[k])/index));
}

/*---------- Correction of harmonic numbers having the RFIs -----------*/

void PomexFileCorrect ( long rsize, long nsize )
{
  int index = ( rsize < nsize ? (nsize-1)/(rsize-1) : (rsize-1)/(nsize-1) );
  char dir = (rsize < nsize ? 1 : 0);

  struct pomexInfo *tmp = pomArray;

  switch (dir) {
   case 1:  for (int i=0;i<pomNumber;i++, tmp++) pomexLocChangeFF(tmp, index);
            break;
   case 0:  for (int i=0;i<pomNumber;i++, tmp++) pomexLocChangeRV(tmp, index);
            break;
   default: break;
                }
}

