#include "bitstat.hpp"

/* get recovered value */
float recovered_value (signed char value, double part) {

 double x = inverf(part);
 double ts = x*sqrt((double)2.);
 double st = ((double)1.)/ts;
 static double pi2 = sqrt((double)2./(double)M_PI);

 double x2 = ((double)-1.)*x*x;
 double y = (value == 1 || value == -1 ? ((double)value)*st*sqrt( (double)1. - ts*pi2*(exp(x2)/part) ) : ((double)value/(double)3.)*st*sqrt( (double)1. + ts*pi2*(exp(x2)/((double)1. - part)) ) );

 return (float)y;
}  

/* correct for bit-statistics */
/*
 * usually we set the number equal to number spoiling points due to cycling convolution
 * during correction for dispersion. Spoiling points for direct band (when we setting the values
 * to lowest frequency) are in the beginning of the array. 
 * It's ease to correct values in the beginning of the array with the same statistics -- anyway
 * we have to ignore these values after the dedispersion!!! But if the spoiling points are in the
 * end of the array, we have to change slithly this code !!!
 */
void bits_correction_begin (signed char *x, float *y, off_t size, long number) {
 
 if (number > size) { 
   printf ("warning - number of counts for bits-statistics is larger than file size\n"); 
   printf ("setting number equal to the file size ... %d\n", size);
   number = size;
 }
 double sum = 0.;
 for (off_t i=0; i<number; ++i) sum += (x[i] == 1 || x[i] == -1 ? 1. : 0.);
 for (off_t i=0; i<number; ++i) y[i] = recovered_value (x[i], sum/number);
 for (off_t i=number, j=0; i<size; ++i, ++j) {
  sum -= (x[j] == 1 || x[j] == -1 ? 1. : 0.);
  sum += (x[i] == 1 || x[i] == -1 ? 1. : 0.);
  y[i] = recovered_value (x[i], sum/number);
 }
}

/* correct for bit-statistics */
/* in this function 'number' of points with the same statistics are in the end
 * of the array
 */
void bits_correction_end (signed char *x, float *y, off_t size, long number) {
 
 if (number > size) { 
   printf ("warning - number of counts for bits-statistics is larger than file size\n"); 
   printf ("setting number equal to the file size ... %d\n", size);
   number = size;
 }
 double sum = 0.;
 for (off_t i=size-1; i>=size-number; --i) sum += (x[i] == 1 || x[i] == -1 ? 1. : 0.);
 for (off_t i=size-1; i>=size-number; --i) y[i] = recovered_value (x[i], sum/number);
 for (off_t i=size-number-1, j=size-1; i>=0; --i, --j) {
  sum -= (x[j] == 1 || x[j] == -1 ? 1. : 0.);
  sum += (x[i] == 1 || x[i] == -1 ? 1. : 0.);
  y[i] = recovered_value (x[i], sum/number);
 }
}
