//#include "plotprofile.hpp"
#include "processdata.hpp"
#include "cpgplot.h"

#include <math.h>

char prf_xlabel[255] = "Time (\\gm\\fns)"; // x-label
//char prf_ylabel[255] = "Flux density (\\gs\\fn)"; // y-label
char prf_ylabel[255] = "Flux density (mJy)"; // y-label

// get y tics major interval
long ProcessData::gettics (float min, float max) {
 long beg = (long)floor((max - min)/5.);
 long end = (long)floor((max - min)/4.);
 if (beg == 0 && end == 0) end = 1;
 int mult[] = { 1000, 500, 400, 250, 200, 150, 120, 100, 80, 60, 50, 40, 25, 20, 10, 8, 6, 5, 4, 2, 1 };
 for (int j=0; j<21; ++j)
  for (long i=beg; i<=end; ++i) {
   if (((long)((float)i/(float)mult[j]))*mult[j] == i && i!=0) return i;
  }
 return end;
}

// draw to device
void ProcessData::drawing (float *x, float **y, unsigned long destsize, float ymin, float ymax, char *on, char *mjd, char *date, char *time, double period, unsigned long Nperiods, long y_major_tic, long x_major_tic, double tres, unsigned long Nframes) {

 cpgeras ();

 char *cur;
 cur = new char[255];
 if (!cur) { perror("new cur"); exit (1); }

 cpgsch (char_size);
 cpgsvp (0.07, 0.49, 0.65, 0.97);
 cpgswin (0, x[destsize-1], ymin, ymax);
 cpgslw (3);
 cpgbox ("BCNTSI", x_major_tic, 4, "BCNTSI", y_major_tic, 4);
 cpgsitf (0);
 cpgline (destsize, x, y[2]);
 cpgsci (4);
 cpgsch (0.8*char_size);
 memset (cur, 0, 255);
 sprintf (cur, "%s", "RCP 1450");
 cpgptxt (x[destsize-destsize/20], ymax - ymax/5, 0., 1., cur);
 cpgsci (1);

 cpgsch (char_size);
 cpgsvp (0.57, 0.99, 0.65, 0.97);
 cpgswin (0, x[destsize-1], ymin, ymax);
 cpgslw (3);
 cpgbox ("BCNTSI", x_major_tic, 4, "BCNTSI", y_major_tic, 4);
 cpgsitf (0);
 cpgline (destsize, x, y[0]);
 cpgsci (4);
 cpgsch (0.8*char_size);
 memset (cur, 0, 255);
 sprintf (cur, "%s", "RCP 1700");
 cpgptxt (x[destsize-destsize/20], ymax - ymax/5, 0., 1., cur);
 cpgsci (1);

 cpgsch (char_size);
 cpgsvp (0.07, 0.49, 0.25, 0.57);
 cpgswin (0, x[destsize-1], ymin, ymax);
 cpgslw (3);
 cpgbox ("BCNTSI", x_major_tic, 4, "BCNTSI", y_major_tic, 4);
 cpgsitf (0);
 cpgline (destsize, x, y[3]);
 cpgsci (4);
 cpgsch (0.8*char_size);
 memset (cur, 0, 255);
 sprintf (cur, "%s", "LCP 1450");
 cpgptxt (x[destsize-destsize/20], ymax - ymax/5, 0., 1., cur);
 cpgsci (1);

 cpgsch (char_size);
 cpgsvp (0.57, 0.99, 0.25, 0.57);
 cpgswin (0, x[destsize-1], ymin, ymax);
 cpgslw (3);
 cpgbox ("BCNTSI", x_major_tic, 4, "BCNTSI", y_major_tic, 4);
 cpgsitf (0);
 cpgline (destsize, x, y[1]);
 cpgsci (4);
 cpgsch (0.8*char_size);
 memset (cur, 0, 255);
 sprintf (cur, "%s", "LCP 1700");
 cpgptxt (x[destsize-destsize/20], ymax - ymax/5, 0., 1., cur);
 cpgsci(1);

 /* x- y- labels*/
 cpgslw (3);
 cpgsch (char_size);
 cpgsvp (0.07, 0.99, 0.25, 0.97);
 /* x-label */ cpgmtxt ("B", 3.0, 0.5, 0.5, prf_xlabel);
 /* y-label */ cpgmtxt ("L", 2.7, 0.5, 0.5, prf_ylabel);

 /* labels */


 cpgsci (1);
 cpgslw (2);

 memset (cur, 0, 255);

 cpgmtxt ("B", 4.5, 0., 0., "Average profile");
 cpgsch (char_size*0.8);
 sprintf (cur, "GBT   Mark5A/VLBA   B1937+21   Period: %lf \\gm\\fns   Time res: %lg \\gm\\fns   Nperiods: %ld", period/1000., tres/1000., Nperiods);
 cpgmtxt ("B", 7., 0., 0., cur);
 memset (cur, 0, 255);
 sprintf (cur, "MJD: %s   UTC:  %s  %s   Nframes = %ld", mjd, date, time, Nframes);
 cpgmtxt ("B", 8., 0., 0., cur);
 memset (cur, 0, 255);
 sprintf (cur, "File: %s", on);
 cpgmtxt ("B", 9., 0., 0., cur);
 cpgslw (1);
 cpgiden ();

 delete (cur);
}

/* graphics part */
void ProcessData::plot_profile(float *x, float **y, long av, char *basename, char *mjd, char *date, char *time, double period, unsigned long cperiod, unsigned long Nperiods, double tres, unsigned long Nframes, double sefd) {
 unsigned long destsize = cperiod/av;
 double sefd1 = sefd / sqrt (Nperiods * av);
 
 double aver, sigma, ss, ss2, max[4], min[4];
 for (int i=0; i<4; ++i) {
  ss = 0.;
  ss2 = 0.;
  max[i] = y[i][0];
  min[i] = y[i][0];
  for (off_t j=0; j<destsize; ++j) { 
   ss += y[i][j];
   ss2 += (y[i][j] * y[i][j]);
   if (y[i][j] > max[i]) max[i] = y[i][j];
   if (y[i][j] < min[i]) min[i] = y[i][j];
  } // j
  aver = ss / destsize;
  sigma = sqrt ( ss2 / destsize - aver * aver );
  max[i] = ((max[i] - aver) / sigma ) * sefd1;
  min[i] = ((min[i] - aver) / sigma ) * sefd1;
  for (off_t k=0; k<destsize; ++k) y[i][k] = ((y[i][k] - aver) / sigma ) * sefd1;
 } // i
  double ymax = max[0], ymin = min[0];
  for (int i=0; i<4; ++i) {
   if (max[i] > ymax) ymax = max[i];
   if (min[i] < ymin) ymin = min[i];
  }

 for (long i=0; i<destsize; ++i) x[i] = i * tres * av * 0.001;

 long y_major_tic = gettics (ymin, ymax);
 long x_major_tic = gettics (x[0], x[destsize-1]);

 // draw to POSTSCRIPT
 cpgopen (basename);
 drawing (x, y, destsize, ymin, ymax, basename, mjd, date, time, period, Nperiods, y_major_tic, x_major_tic, tres * av, Nframes);
 cpgclos ();
}
