#ifndef __PROCESSDATA_HPP
#define __PROCESSDATA_HPP

#include <fcntl.h>

#include "wisdom.hpp"

extern bool is_wisdom; // true if wisdom file was read successfully
extern int mjd_header;
extern char *outname;
extern char *outputpath;
extern double sec_step;

extern float char_size; // character size

/*++++++++++++++++++ class to work directly with real data ++++++++++++++++++++*/
class ProcessData {

           off_t bitstat_points; // number of points for recovering for bit statistics
   protected:
           size_t samplesize;        // size of every decoded channel
           size_t datasize;          // size of the data (fftsize + bitstat_points)
	   size_t current_size;      // current accumulated size
	   size_t fftsize;           // nearest close size for FFT (calculated from datasize)
	   size_t fftsize2;          // doubled fftsize
	   signed char *dataunit[8]; // data streams 

	   float *databitcor[2];  // data corrected for bit-statistics
	   float *det;            // dedispersed data

           /* dedispersion */ 
	   fftwf_complex *matrix1; // matrix for dedispersion for 1450 MHz
	   fftwf_complex *matrix2; // matrix for dedispersion for 1700 MHz
	   fftwf_complex *part1;  // fftw array to make dedispersion
	   fftwf_complex *dest;
	   fftwf_plan plan1;      // forward plan
	   fftwf_plan backplan;   // backward plan

	   double center_freq[2];
	   double ref_freq[2];
	   double bandwidth;
	   bool band_direct;
	   double timeres;
	   double dm;
	   unsigned long begpoints;
	   unsigned long endpoints;
           /*-----------*/
            
           double period;  // pulsar period
	   unsigned long cperiod; // pulsar period in counts
	   float **profile;  // pulsar profile
	   float **y; // plotting pulsar profile
	   float *x; // time array for plotting
	   long av;  // number of points to average for plotting purposes
	   double sefd; // sefd
	   double threshold;  // threshold to search GPs
	   unsigned long Niter; // number of dedispersion/folding iterations;
	   off_t shift; // shift in points (with double timeres) from the very beginning
	   unsigned long number_of_periods; // total number of periods;
	   unsigned long curNperiods; // current number of periods
	   bool is_extra_sample;  // true if we have to take into account an extra sample due to non-integer ratio between
	                          // points in single channel and combined channel

           char *mjd; // mjd time of ref point of the profile
	   char *mjdGP; // mjd time of GP
	   char *date; // date of the beginning
	   char *time; // time of the beginning of the profile
	   char **profnames;  // output file names with profiles
	   char *basename;
           /*------*/


	   size_t resize (size_t sz); // get power of 2 size

           /* calculate phase due to dispersion */
           double get_disp_phase (double f, double f0, double cf, double rf, double bw, double DM);
	   /* compute dedispersion matrix */
	   fftwf_complex* get_matrix (fftwf_complex *matrix, off_t size, double cf, double rf, double bw, bool is_dir, double tres, double DM);
	   /* making the dedispersion */
	   fftwf_complex* dedispersion (fftwf_complex *ampl_sp, fftwf_complex *matrix, fftwf_plan plan, off_t size, bool do_fraction, bool do_zero);
	   /* get total number of spoiled points */
	   unsigned long get_spoiled_points (double f1, double f2);
	   /* get number of spoiled points in the beginning */
	   unsigned long get_start_spoiled_points ();
	   /* get number of spoiled points in the end */
	   unsigned long get_end_spoiled_points ();
	   /* ship FFT data out in two complex arrays */
	   void ship_out2complex (fftwf_complex *part1, fftwf_complex *part2, off_t size);
	   /* ship FFT data from the GBT to arrange in total band */
	   fftwf_complex* ship_out2gbt_total (fftwf_complex *dest, fftwf_complex *part1, off_t size);
	   // run a number of functions to get a profile
	   void get_profile (signed char *first, signed char *second, fftwf_complex *matrix, float *prof, char *name, bool flag = false);
	   // average the array
	   void avg (float *profile, float *y, unsigned long size, long aver);
	   /* to search for GPs */
	   void searchGP (float *x, unsigned long size, char *name);
	   /* graphics part */
	   void plot_profile(float *x, float **y, long av, char *basename, char *mjd, char *date, char *time, double period, unsigned long cperiod, unsigned long Nperiods, double tres, unsigned long Nframes, double sefd);
	   void drawing (float *x, float **y, unsigned long destsize, float ymin, float ymax, char *on, char *mjd, char *date, char *time, double period, unsigned long Nperiods, long y_major_tic, long x_major_tic, double tres, unsigned long Nframes);
	   long gettics (float min, float max);

   public:
	   ProcessData (size_t ss);  // constructor
	   virtual ~ProcessData();  // destructor
	   void update (signed char **array, size_t current_frame, char *startmjd);  // update sample data
};

#endif // __PROCESSDATA_HPP
