// Print the phase in grad (if calculated in grad by dt) having the data for all channels in both polarizations
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <string.h>
#include <sys/stat.h>

int main (int argc, char *argv[]) {
 int in;
 struct stat info;
 long sz;
 float *lcp, *rcp;

 if (argc == 1) { printf("Usage: %s <lcp.W> <lcp.X> <lcp.Y> <lcp.Z> <rcp.W> <rcp.X> <rcp.Y> <rcp.Z>\n", argv[0]); exit (0); }
 if ( stat(argv[1], &info) != 0) { perror("stat"); exit(1); }
 sz = info.st_size / sizeof (float); 

 lcp = new float[sz];
 rcp = new float[sz];
 if (!lcp || !rcp) { perror("new"); exit(1); }

 for (int i=4; i>=1; i--) {

 memset (lcp, 0, info.st_size);
 memset (rcp, 0, info.st_size);
 
 in = open (argv[i], O_RDONLY | O_LARGEFILE);
 lseek (in , 0, SEEK_SET);
 read (in, lcp, info.st_size);
 close (in);

 in = open (argv[i+4], O_RDONLY | O_LARGEFILE);
 lseek (in , 0, SEEK_SET);
 read (in, rcp, info.st_size);
 close (in);

 float lcp_val, rcp_val;

 // step = 1 MHz
 long step = sz/16;
 if (i==3 || i==1) {  // process direct bands
  for (int k=0; k<sz; k+=step) { 
   lcp_val = lcp[k]; rcp_val = rcp[k];
   printf ("%d %.3f %.3f\n", 2052 + (4-i)*16 + k/step, lcp_val, rcp_val);
  }
 }
 else { // process inverse bands
  for (int k=sz-1, l=0; k>=0; k-=step, l+=step) { 
   lcp_val = lcp[k]; rcp_val = rcp[k];
   printf ("%d %.3f %.3f\n", 2052 + (4-i)*16 + l/step, lcp_val, rcp_val);
  }
 }

 }

 delete (lcp);
 delete (rcp);
 return 0;
}
