#include "plot.hpp"
#include "io.hpp"

/* graphics part */
void plotting (float *y1, float *y2, float *x, char *name, double aver1, double sigma1, double ymin1, double ymax1, double ymaxs1, double aver2, double sigma2, double ymin2, double ymax2, double ymaxs2) {

 char *outfile;
 outfile = new char[MaxString];
 if (!outfile) { perror("new"); exit (1); }
 memset (outfile, 0, MaxString);
 strncpy (outfile, name, strlen(name)-11);
 memset (pgplot_dev, 0, MaxString);
 sprintf (pgplot_dev, "%sps/ps", basename(outfile));
 memset (outfile, 0, MaxString);
 strncpy (outfile, pgplot_dev, strlen(pgplot_dev)-5);

 double ybeg = (ymin2 < ymin1 ? ymin2 : ymin1);
 double yend = (ymax2 > ymax1 ? ymax2 : ymax1);
 dataoffset = (yend - ybeg) * 0.03;
 long major_tic = getytics(ybeg - dataoffset, yend + dataoffset);


 cpgopen (pgplot_dev);
 cpgsch (char_size);
 cpgsvp (0.07, 0.99, 0.22, 0.97);
 cpgswin (x[0], x[window-1], ybeg - dataoffset, yend + dataoffset);
 cpgslw (2);
 cpgbox ("BCNTS", 100.0, 2, "BCNTS", major_tic, 2);
 /* x-label */ cpgmtxt ("B", 2.7, 0.5, 0.5, xlabel);
 /* y-label */ cpgmtxt ("L", 2.7, 0.5, 0.5, ylabel);
 cpgslw (2);
 cpgline (window, x, y1);
 cpgsls (2);
 cpgline (window, x, y2);
 cpgslw (1);
 /* labels */
 cpgsch (char_size*0.8);
 cpgmtxt ("B", getdisp(yend, ybeg, major_tic/4., dataoffset, char_size), 0.95, 1.0, "LCP");
 cpgmtxt ("B", getdisp(yend, ybeg, major_tic/4.-shift, dataoffset, char_size), 0.95, 1.0, "RCP");

 char *cur;
 cur = new char[MaxString];
 if (!cur) { perror("new"); exit (1); }
 memset (cur, 0, MaxString);
 sprintf (cur, "B1937+21  GB NRAO  GBT+Mk5  S-band  13cm  sefd: %lg Jy", sefd);
 cpgmtxt ("B", 5., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "Freq: 2100.0 (2052.0-2116.0) MHz   BW: 64 MHz   tres: %lg ns   DM: 71.0255   %s", tres, header.getType() == 'F' ? "False event": (header.getType() == 'M' ? "Main pulse": "Interpulse"));
 cpgmtxt ("B", 6., 0., 0., cur);
 memset (cur, 0, MaxString);
 int hh, mm; 
 double ss;
 header.getTime (hh, mm, ss);
 sprintf (cur, "Epoch: 2005/06/07  %02d:%02d:%s%.10lf   MJD: %s", hh, mm, (int)floor(ss) < 10 ? "0" : "", ss, header.get_mjd());
 cpgmtxt ("B", 7., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "%02d:%02d:%02d", hh, mm, (int)rint(ss));
 cpgmtxt ("T", -4., 0.95, 1.0, cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "LCP  aver: %lg sigma: %lg max: %lg Jy (%lg %s)", aver1, sigma1, ymax1, ymaxs1, "\\gs\\fn");
 cpgmtxt ("B", 8., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "file: %slcp.cut.det", outfile);
 cpgsch (char_size*0.6);
 cpgmtxt ("B", 10.6, 1., 1., cur);
 cpgsch (char_size*0.8);
 memset (cur, 0, MaxString);
 sprintf (cur, "RCP  aver: %lg sigma: %lg max: %lg Jy (%lg %s)", aver2, sigma2, ymax2+shift, ymaxs2, "\\gs\\fn");
 cpgmtxt ("B", 9., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "file: %srcp.cut.det", outfile);
 cpgsch (char_size*0.6);
 cpgmtxt ("B", 12.0, 1., 1., cur);
 cpgsch (char_size*0.8);

 cpgsch (char_size*0.6);
 memset (cur, 0, MaxString);
 sprintf (cur, "Plot file: %sps", outfile);
 cpgmtxt ("B", 14., 0., 0., cur);

 cpgclos();

 delete (cur);
 delete (outfile);
}

// returns disp in character size
double getdisp (double max, double min, double value, double offset, double chsize) {
 return ( -1. * ((value - min + offset) * 38./chsize) / (max - min + 2*offset));
}

// get y tics major interval
long getytics (double min, double max) {
 long beg = (long)floor((max - min)/6.);
 long end = (long)floor((max - min)/5.);
 int mult[] = { 1000, 500, 250, 200, 150, 100, 50, 25, 10, 5, 4, 2, 1 };
 for (int j=0; j<13; ++j)
  for (long i=beg; i<=end; ++i) {
   if (((long)((float)i/(float)mult[j]))*mult[j] == i) return i;
  }
 return end;
}

