#include "viewgp.hpp"

/*--------------------------------------------------------------------------------------
 *                               M   A   I   N
 * -------------------------------------------------------------------------------------*/

int main (int argc, char *argv[]) {
 if (argc == 1) { 
  printf ("Usage: %s <lcp> <rcp>\n", argv[0]);
  exit (0);
 }

 struct stat info;
 if (stat(argv[1], &info) == -1) { perror("stat"); exit(1); }
 // "1" - because of type of GP
 off_t filesize = (info.st_size - mjd_header - 1) / sizeof(float);

 float *data1, *data2, *x, *y1, *y2;
 data1 = new float[filesize];
 data2 = new float[filesize];
 x    = new float[window];
 y1   = new float[window];
 y2   = new float[window];
 if (!data1 || !data2 || !x || !y1 || !y2) { perror("new"); exit(1); }
 memset (data1, 0, filesize * sizeof(float));
 memset (data2, 0, filesize * sizeof(float));
 memset (x, 0, window * sizeof(float));
 memset (y1, 0, window * sizeof(float));
 memset (y2, 0, window * sizeof(float));

 char *head;
 head = new char[mjd_header];
 if (!head) { perror("new"); exit (1); }
 memset (head, 0, mjd_header);

 // read the input file with LCP
 int in; // file descriptor
 in = open (argv[1], O_RDONLY | O_LARGEFILE);
 if (in == -1) { perror("open"); exit (1); }
// lseek (in, mjd_header, SEEK_SET);
 /* read header */ read (in, head, mjd_header); 
 /* read GP type */ char type; read (in, &type, 1);
 read (in, data1, filesize*sizeof(float));
 close (in);

 // read the input file with RCP
 in = open (argv[2], O_RDONLY | O_LARGEFILE);
 if (in == -1) { perror("open"); exit (1); }
 // mjd_header+1 -- eto razmer mjd + 1byte - type of the GP
 lseek (in, mjd_header+1, SEEK_SET);
 read (in, data2, filesize*sizeof(float));
 close (in);

 double ymax1;
 off_t xmax1 = getmax (data1, filesize, ymax1);
 double ymax2;
 off_t xmax2 = getmax (data2, filesize, ymax2);

 off_t beg, end, xmax;
 if (ymax1 > ymax2) xmax = xmax1; else xmax = xmax2;
 beg = (xmax - ledge >= 0 ? xmax - ledge : 0); 
 end = (xmax - ledge >= 0 ? xmax - ledge + window : window);

 // initialize the header
 header.init (head, type, xmax, tres);

 // setting the x-, y1-, and y2-array
 for (off_t i=beg, j=0; i<end; ++i, ++j) { 
  x[j] = (float)(i-xmax) * tres;
  y1[j] = data1[i];
  y2[j] = data2[i];
 }
 double ymin1;
 off_t xmin1 = getmin (y1, window, ymin1);
 double aver1, sigma1, ymaxs1;
 getavsig (data1, filesize, aver1, sigma1, y1, window, xmax1, ymax1, ymin1, ymaxs1);

 double ymin2;
 off_t xmin2 = getmin (y2, window, ymin2);
 double aver2, sigma2, ymaxs2;
 getavsig (data2, filesize, aver2, sigma2, y2, window, xmax2, ymax2, ymin2, ymaxs2);

 // берем shift как сдвиг на max2 и поднимаем на 1/10 от максимума первого файла
 shift = ymax2 - ymin1 - 0.1 * (ymax1 - ymin1); 
 if (shift < 0) shift = ymax2;
 for (off_t i=0; i<window; ++i) y2[i] -= shift;
 ymax2 -= shift;
 ymin2 -= shift;

 /* graphics part */
 plotting (y1, y2, x, argv[1], aver1, sigma1, ymin1, ymax1, ymaxs1, aver2, sigma2, ymin2, ymax2, ymaxs2);
 
 delete (head);
 delete (data1);
 delete (data2);
 delete (x);
 delete (y1);
 delete (y2);
 return 0;
}
