#include "viewstoks.hpp"

/*--------------------------------------------------------------------------------------
 *                               M   A   I   N
 * -------------------------------------------------------------------------------------*/

int main (int argc, char *argv[]) {
 if (argc == 1) { 
  printf ("Usage: %s <I> <V> <P> <A>\n", argv[0]);
  exit (0);
 }

 struct stat info;
 if (stat(argv[1], &info) == -1) { perror("stat"); exit(1); }
 // "1" - because of type of GP
 off_t filesize = (info.st_size - mjd_header - 1) / sizeof(float);

 float *I, *V, *P, *A, *x, *yI, *yV, *yP, *yA;
 I = new float[filesize];
 V = new float[filesize];
 P = new float[filesize];
 A = new float[filesize];
 x    = new float[window];
 yI   = new float[window];
 yV   = new float[window];
 yP   = new float[window];
 yA   = new float[window];
 if (!I || !V || !P || !A || !x || !yI || !yV || !yP || !yA) { perror("new"); exit(1); }
 memset (I, 0, filesize * sizeof(float));
 memset (V, 0, filesize * sizeof(float));
 memset (P, 0, filesize * sizeof(float));
 memset (A, 0, filesize * sizeof(float));
 memset (x, 0, window * sizeof(float));
 memset (yI, 0, window * sizeof(float));
 memset (yV, 0, window * sizeof(float));
 memset (yP, 0, window * sizeof(float));
 memset (yA, 0, window * sizeof(float));

 char *head;
 head = new char[mjd_header];
 if (!head) { perror("new"); exit (1); }
 memset (head, 0, mjd_header);

 // read the input file with I
 int in; // file descriptor
 in = open (argv[1], O_RDONLY | O_LARGEFILE);
 if (in == -1) { perror("open"); exit (1); }
// lseek (in, mjd_header, SEEK_SET);
 /* read header */ read (in, head, mjd_header); 
 /* read GP type */ char type; read (in, &type, 1);
 read (in, I, filesize*sizeof(float));
 close (in);

 // read the input file with V
 in = open (argv[2], O_RDONLY | O_LARGEFILE);
 if (in == -1) { perror("open"); exit (1); }
 // mjd_header+1 -- eto razmer mjd + 1byte - type of the GP
 lseek (in, mjd_header+1, SEEK_SET);
 read (in, V, filesize*sizeof(float));
 close (in);

 // read the input file with P
 in = open (argv[3], O_RDONLY | O_LARGEFILE);
 if (in == -1) { perror("open"); exit (1); }
 // mjd_header+1 -- eto razmer mjd + 1byte - type of the GP
 lseek (in, mjd_header+1, SEEK_SET);
 read (in, P, filesize*sizeof(float));
 close (in);

 // read the input file with A
 in = open (argv[4], O_RDONLY | O_LARGEFILE);
 if (in == -1) { perror("open"); exit (1); }
 // mjd_header+1 -- eto razmer mjd + 1byte - type of the GP
 lseek (in, mjd_header+1, SEEK_SET);
 read (in, A, filesize*sizeof(float));
 close (in);

 double ymaxI;
 off_t xmax = getmax (I, filesize, ymaxI);

 off_t beg, end;
 beg = (xmax - ledge >= 0 ? xmax - ledge : 0); 
 end = (xmax - ledge >= 0 ? xmax - ledge + window : window);

 // initialize the header
 header.init (head, type, xmax, tres);

 /*
  * we calculate the aver and sigma in order to sort out the values of I which are larger
  * than 3sigma. We will draw the position angle only for these points
  */
// double aver, sigma;
// getavsig (I, filesize, aver, sigma, xmax);
// ymaxI = (ymaxI - aver) / sigma;

 // setting the x-, yI-, yV-, yP-, yA-arrays
 for (off_t i=beg, j=0; i<end; ++i, ++j) { 
  x[j] = (float)(i-xmax) * tres;
  yI[j] = I[i]; // / ymaxI;
  yV[j] = V[i]; // / ymaxI;
  yP[j] = P[i]; // / ymaxI;
//  if ( (P[i] - aver) / sigma < 10.) yA[j] = 1000.; // pgplot doesn't plot values out of range (in our case [-90:90])
//   else yA[j] = A[i] * 0.5;  // because before (using cut2stoks) we got position angle as twice the real value
 }
 double ymin;
 double averI, sigmaI, ymaxsI;
 getavsig (I, filesize, averI, sigmaI, yI, window, xmax, ymaxI, ymin, ymaxsI);
 double averV, sigmaV, ymaxsV, ym = 0.;
 getavsig (V, filesize, averV, sigmaV, yV, window, xmax, ym, ymin, ymaxsV);
 double averP, sigmaP, ymaxsP;
 getavsig (P, filesize, averP, sigmaP, yP, window, xmax, ym, ymin, ymaxsP);

 // setting the x-, yI-, yV-, yP-, yA-arrays
 for (off_t i=beg, j=0; i<end; ++i, ++j) { 
  if ( ((yP[j] / sefd)*sigmaP) / sigmaI < 20.) yA[j] = 1000.; // pgplot doesn't plot values out of range (in our case [-90:90])
   else yA[j] = A[i] * 0.5;  // because before (using cut2stoks) we got position angle as twice the real value
  yI[j] *= sqrt(2.); // /= ymaxI;
  yV[j] *= sqrt(2.); // /= ymaxI;
  yP[j] *= sqrt(2.); // /= ymaxI;
  yP[j] = (yP[j] / sigmaI) * sigmaP;  
  yV[j] = (yV[j] / sigmaI) * sigmaV;  
 }
 ymaxI *= sqrt(2.);

 off_t xmin = getmin (yV, window, ymin);
// ymaxI = 1.;
 // circular polarization can be negative so minimum value we choose using only V
 // but if it's positive so we take minimum value as 0
// if (ymin > 0.) ymin = 0.;

// double ymin2;
// off_t xmin2 = getmin (y2, window, ymin2);
// double aver2, sigma2, ymaxs2;
// getavsig (data2, filesize, aver2, sigma2, y2, window, xmax2, ymax2, ymin2, ymaxs2);

 // берем shift как сдвиг на max2 и поднимаем на 1/10 от максимума первого файла
// shift = ymax2 - ymin1 - 0.1 * (ymax1 - ymin1); 
// if (shift < 0) shift = ymax2;
// for (off_t i=0; i<window; ++i) y2[i] -= shift;
// ymax2 -= shift;
// ymin2 -= shift;

 /* graphics part */
// plotting (y1, y2, x, argv[1], aver1, sigma1, ymin1, ymax1, ymaxs1, aver2, sigma2, ymin2, ymax2, ymaxs2);
 plotting (yI, yV, yP, yA, x, argv[1], ymin, ymaxI);
 
 delete (head);
 delete (I);
 delete (V);
 delete (P);
 delete (A);
 delete (x);
 delete (yI);
 delete (yV);
 delete (yP);
 delete (yA);
 return 0;
}
