#define MAIN_
#include "k5toraw-1ch.hpp"
#include "hdsd.hpp"

/*==========================   P r o m p t i n g   ==================================*/

void Help (char *s) {
  printf("\n%sPROGRAMM%s %s\n"
         "It is used to decode K5/VSSP board data format\n\n"
         "%s[USAGE]:%s %s [options] <file1> [file2 ... fileN]\n\n"
         "%s[OPTIONS]:%s\n"
	 "-d, --path <path>       - path to the output files, default - the same\n"
	 "                          where are the source files\n"
	 "-o, --out <basename>    - basename of files for output data\n"
	 "--swap-bytes            - to reverse bits order in input data,\n"
	 "                          use this option on Apple machines\n"
	 "-b, --begin <frame>     - start processing from this frame\n"
	 "-n, --num <frames>      - number of frames to process\n"
	 "--dummy                 - set dummy mode (no output files)\n"
	 "--check                 - set check mode - the same as dummy but together with\n"
	 "                          checking the integrity of input data, time continuity, ...\n"
	 "--spectra               - calculate average spectra rather than raw data\n"
	 "-q, --quite             - turn on quite output, skip printing the frames number changing\n"
	 "-h, --help              - help\n\n", bold, none, s, bold, none, s, bold, none);
  exit(0);
}

/*=========================  erase path part from filename  =========================*/
  
char * Basename (char *str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
} 

/*========================= Parsing the command line =================================*/

int ParseCmdLine (int argc, char *argv[]) {
 int op;
 struct option long_options[] = { {"help", no_argument, 0, 'h'},
	                          {"out", required_argument, 0, 'o'},
	                          {"path", required_argument, 0, 'd'},
	                          {"begin", required_argument, 0, 'b'},
	                          {"num", required_argument, 0, 'n'},
	                          {"dummy", no_argument, 0, 13},
	                          {"spectra", no_argument, 0, 14},
	                          {"swap-bytes", no_argument, 0, 15},
	                          {"check", no_argument, 0, 16},
				  {"quite", no_argument, 0, 'q'},
	                          {0, 0, 0, 0}
                                };
 while((op = getopt_long(argc, argv, "hd:o:n:b:q", long_options, 0)) != EOF)
  switch(op){
    case 'h': Help (Basename(argv[0]));
	      break;
    case 'd': if (outputpath) { outputpath = 0; delete (outputpath); }
	      outputpath = new char[(int)strlen(optarg)];
	      if (!outputpath) { perror("new outputpath"); exit(1); }
	      memcpy (outputpath, optarg, (int)strlen(optarg));
	      break;
    case 'o': if (outname) { outname = 0; delete (outname); }
	      outname = new char[(int)strlen(optarg)];
	      if (!outname) { perror("new outname"); exit(1); }
	      memcpy (outname, optarg, (int)strlen(optarg));
	      break;
    case 'b': BeginFrame = atol (optarg);
	      if (BeginFrame < 0) BeginFrame = 0;
	      break;
    case 'n': NumberOfFrames = atol (optarg);
	      if (NumberOfFrames < 0) NumberOfFrames = 0;
	      break;
    case 13: is_dummy = true;
	     break;
    case 14: is_spectra = true;
	     break;
    case 15: swap_bytes = true;
	     break;
    case 16: is_check = true;
             is_dummy = true;
	     break;
    case 'q': is_quite = true;
              break;
    case '?': Help (Basename(argv[0]));
              break;
  }
 return(optind);
}

/*=================  level calculation ============================*/
float LevelCalc (signed char *data, size_t sz) {
 float sum = 0.;
 for (size_t i=0; i<sz; ++i) sum += (float)data[i];
 return sum/sz;
}

/*==================== get power of 2 size ================================*/
size_t resize (size_t sz) { 
  long n = 0;
  sz += 1; 
  while ( (1<<(++n)) < sz) ;
  return (1<<(--n));
}

/*==================== calculate power spectrum =============================*/

void getSpectrum (float *sp, signed char *data, size_t sz, float level) {
 for (int i=0; i<sz; ++i) { x[i][0] = (float)data[i]; x[i][1] = 0.; }
 for (int i=0; i<sz; ++i) x[i][0] -= level;
 fftwf_execute (pfor);
 for (int i=0; i<sz/2 + 1; ++i) sp[i] += (x[i][0] * x[i][0] + x[i][1] * x[i][1]);
}

/*++++++++++++++++++++++++++++   m a i n   f u n c t i o n +++++++++++++++++++++++++++*/

int main (int argc, char *argv[]) {

 if (argc == 1) Help (Basename(argv[0]));

 outname = new char[MaxString];
 if (!outname) { perror("new outname"); exit(1); }
 memset (outname, 0, MaxString);
 sprintf (outname, "%s","\0");

 outputpath = new char[MaxString];
 if (!outputpath) { perror("new outputpath"); exit(1); }
 memset (outputpath, 0, MaxString);
 sprintf (outputpath, "%s","\0");

 for (int u=0; u<1; ++u) {
  outnames[u] = new char[MaxString];
  if (!outnames[u]) { perror("new outnames[u]"); exit (1); }
  memset (outnames[u], 0, MaxString);
 }

 // parse command line
 next = ParseCmdLine (argc, argv);
 if (argc < next + 1) { printf ("Not point the file to process!\n"); exit(1); }

 // initialize input data frame array
 raw_framedata = new unsigned long long[framesize/size_of_long_long];
 if (!raw_framedata) { perror("new raw_framedata"); exit(1); }

 SampleData shot (framesize, swap_bytes);  // initialize the class object

 if (is_spectra) {
  spectra = new float*[1];
  if (!spectra) { perror("new spectra"); exit(1); }
  for (int u=0; u<1; ++u) {
   spectra[u] = new float[(resize(shot.getSampleSize()))/2 + 1];
   if (!spectra[u]) { perror("new spectra[u]"); exit(1); }
   memset (spectra[u], 0, ((resize(shot.getSampleSize()))/2 + 1) * sizeof(float));
  }
  x = (fftwf_complex *) fftwf_malloc ((resize(shot.getSampleSize())) * sizeof(fftwf_complex));
  if (!x) { perror("new x"); exit(1); }
  pfor = fftwf_plan_dft_1d (resize(shot.getSampleSize()), x, x, FFTW_FORWARD, FFTW_ESTIMATE);
 }

 printf("\nProcessing...\n");
 for (int f=next; f<argc; f++) {  // loop on input files

 Nframe_processed = 0;
 Nframe_skipped = 0;

 printf(" [%3d]\t%s ... ", f-next, argv[f]); fflush(NULL);

#ifdef apple
 if ((in = open (argv[f], O_RDONLY)) == -1) { perror("open"); exit(1); };
#else
 if ((in = open (argv[f], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
#endif

 syncpos = 0;
 // infinite loop
 while (1) {
  if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
  bytesread = read (in, &syncdata, size_of_long);
  if (bytesread == -1) { perror("read syncdata"); exit(1); }
  if (bytesread != size_of_long) break;
  if (syncdata == syncmask) { syncdetect = true; printf("sync pos: %ld bytes  ", syncpos); fflush (NULL); break; }
   else { syncdetect = false; syncpos += size_of_long; }
 } // end of loop
 
 if (!syncdetect) { printf ("No sync word were found!\n"); exit (1); }

 memset (raw_framedata, 0, framesize);

 if (!is_dummy) {  // on if dummy mode isn't set
 // create output file names
 for (int u=0; u<1; ++u) memset (outnames[u], 0, MaxString);
 if (!strcmp(outputpath, "\0")) for (int u=0; u<1; ++u)
  sprintf (outnames[u], "%s%s%s", (!strcmp(outname, "\0") ? argv[f] : outname), ext[u], (is_spectra ? ".sp2": ""));
 else for (int u=0; u<1; ++u) sprintf (outnames[u], "%s/%s%s%s", outputpath, (!strcmp(outname, "\0") ? Basename(argv[f]) : outname), ext[u], (is_spectra ? ".sp2" : ""));

 // at first remove existing file
 if (!is_spectra) for (int u=0; u<1; ++u) unlink (outnames[u]);
 // open for appending
 if (!is_spectra) { 
   for (int u=0; u<1; ++u) 
#ifdef apple
    if ((out[u] = open (outnames[u], O_APPEND | O_CREAT | O_WRONLY, 0644)) == -1) { perror("open"); exit(1); }
#else
    if ((out[u] = open (outnames[u], O_APPEND | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1) { perror("open"); exit(1); }
#endif
 } else {
   for (int u=0; u<1; ++u)
#ifdef apple
    if ((out[u] = open (outnames[u], O_TRUNC | O_CREAT | O_WRONLY, 0644)) == -1) { perror("open"); exit(1); }
#else
    if ((out[u] = open (outnames[u], O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1) { perror("open"); exit(1); }
#endif
 }
 } // is_dummy

 is_info_shown = false;
 
 { // in this block we check the existence of "preframe" file
   // if it does then we check if its size + sync pos is equal to framesize
   // if it does then we read this preframe file + data before sync pos as extra frame
   struct stat info;
   if (stat ("preframe", &info) != 0) { printf(" | no \"preframe\" file found  \n"); fflush(NULL); }
    else { 
           printf (" | \"preframe\" file found"); fflush (NULL); 
           if (framesize != info.st_size + syncpos) { printf("  [mismatch]\n"); fflush(NULL); }
             else { printf ("  [match]"); fflush(NULL); 
                    if (BeginFrame == Nframe_skipped) {
                    printf ("  - processing extra frame\n");
                    if (!is_dummy) { // process only if dummy mode isn't set
                     int endfr;
                     if ((endfr = open ("preframe", O_RDONLY)) == -1) { perror("open"); exit(1); }
                     read (endfr, raw_framedata, info.st_size);
                     if ((close (endfr)) == -1) { perror("close"); exit(1); };
                     unsigned long long *ptr;
                     ptr = raw_framedata + info.st_size/size_of_long_long;
                     if ((endfr = open (argv[f], O_RDONLY)) == -1) { perror("open"); exit(1); }
                     if ((lseek (in, 0, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
                     read (endfr, ptr, syncpos);
                     if ((close (endfr)) == -1) { perror("close"); exit(1); };
                     shot.update (raw_framedata, Nframe_processed); // fill class object with new frame data
                     shot.getHeader();
		     printf ("preframe seconds: %ld\n", shot.getSeconds());
                     if (!is_spectra) for (int u=0; u<1; ++u) write (out[u], shot.getChannel(u), shot.getSampleSize());
		      else for (int u=0; u<1; ++u) getSpectrum (spectra[u], shot.getChannel(u), resize(shot.getSampleSize()), SpectraLevel[u]);
		     Nframe_processed++;
                     printf ("preframe - done\n");
		     if (NumberOfFrames != 0 && NumberOfFrames == Nframe_processed) {
                       printf ("processed %ld frames (%g secs)\n", Nframe_processed, Nframe_processed * sec_step);
		       break;
		     }
                    } // is_dummy
		    } // BeginFrame
            }  // else !=
         }  // else stat
 }
 
 if (Nframe_skipped < BeginFrame) {
  struct stat information;
  if (stat (argv[f], &information) != 0) { perror("stat information"); exit (1); }
  while (syncpos < information.st_size && Nframe_skipped < BeginFrame) {
   syncpos += framesize;
   Nframe_skipped++;
  }
  if (BeginFrame == Nframe_skipped) printf ("skipped %ld frames (%g secs)\n", Nframe_skipped, Nframe_skipped * sec_step);
  if (syncpos >= information.st_size) continue;
 }
 
 // sync word is found and
 // now we begin to read frame by frame for each track
 printf ("Frames processed: %6ld", Nframe_processed); fflush (NULL);
 
 if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
 // infinite loop
 while (1) {

   memset (raw_framedata, 0, framesize);
   bytesread = read (in, raw_framedata, framesize);
   if (bytesread == -1) { perror("read raw_framedata"); exit(1); }
   if (bytesread != framesize) {
     // write this piece of data
     if (!is_dummy) {
      int endfr;
      if ((endfr = open ("preframe", O_CREAT | O_TRUNC | O_WRONLY, 0644)) == -1) { perror("open"); exit(1); }
      write (endfr, raw_framedata, bytesread);
      if ((close (endfr)) == -1) { perror("close"); exit(1); }
     }
     break;
   }

   shot.update (raw_framedata, Nframe_processed); // fill class object with new frame data
   
   if (!is_info_shown) { // getting info for zero track (default)
    printf ("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b"); 
    shot.getHeader(); 
    printf ("Start seconds: %ld\n", shot.getSeconds());
    is_info_shown = true;
    if (!is_quite) { printf ("Frames processed: %6ld", Nframe_processed); fflush (NULL); }
     else { printf ("Frames processed: ", Nframe_processed); fflush (NULL); }
   } 

   if (!is_dummy) { if (!is_spectra) for (int u=0; u<1; ++u) write (out[u], shot.getChannel(u), shot.getSampleSize());
	             else  { 
		      if (Nframe_processed == 0) for (int u=0; u<1; ++u) SpectraLevel[u] = LevelCalc (shot.getChannel(u), resize(shot.getSampleSize()));
		      for (int u=0; u<1; ++u) getSpectrum (spectra[u], shot.getChannel(u), resize(shot.getSampleSize()), SpectraLevel[u]); 
		     }
   }
   Nframe_processed++;
   if (!is_quite) { printf ("\b\b\b\b\b\b%6ld", Nframe_processed); fflush (NULL); }
   if (NumberOfFrames != 0 && NumberOfFrames == Nframe_processed) break;

 } // end of loop

 if (is_quite) { printf ("%6ld", Nframe_processed); fflush (NULL); }

 if ((close (in)) == -1) { perror("close"); exit(1); };
 if (!is_dummy) { if (!is_spectra) for (int u=0; u<1; ++u) if ((close (out[u])) == -1) { perror("close"); exit(1); }
 }

 if (!is_dummy && is_spectra) {
  for (int u=0; u<1; ++u) {
    for (int i=0; i<resize(shot.getSampleSize())/2+1; ++i) spectra[u][i] /= Nframe_processed;
    write (out[u], spectra[u], ((resize(shot.getSampleSize()))/2 + 1) * sizeof(float));
    if ((close (out[u])) == -1) { perror("close"); exit(1); }
  }
 }
 printf (" - done\n");
 printf ("processed %ld frames (%g secs)\n", Nframe_processed, Nframe_processed * sec_step);

 if (is_check) {
  shot.getCheckInfo();
 }

 } // f

 if (Nframe_skipped < BeginFrame) 
  printf ("Number of frames to skip is more than available number!\n");

 delete (raw_framedata);
 delete (outname);
 delete (outputpath);
 for (int u=0; u<1; ++u) delete (outnames[u]);
 if (is_spectra) { 
  delete [] spectra; 
  fftwf_destroy_plan (pfor);
  fftwf_free (x);
 }

 return 0;
}
