/*==========================================================================
 *  Some general comment
 *  This program was written to decode observations of pulsars 
 *  with the Kalyazin 64-m radio telescope using K5 data acquisition system
 *  4 frequency channels of 4-MHz each (sampling frequency is 8 MHz per channel)
 *  2bit sampling
 *  chan1 - W
 *  chan2 - X
 *  chan3 - Y
 *  chan4 - Z
=============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <math.h>

#include <inttypes.h>
#include <fftw3.h>

#ifdef MAIN_

#include <getopt.h>

#define MaxString 255

// number of sampling bits
#define BITS 2
// // sampling frequnecy (in MHz)
#define FREQ 8 
// // number of channels (1 or 4 in K5)
#define CHAN 4
// // size of the sampling data section (1 sec) in bytes
#define BLOCK_SIZE (((FREQ * CHAN * BITS) / 8) * 1000000)

int in;         // input file descriptor
int out[4];     // output file descriptors for recording all channels
char *ext[4] = { ".W", ".X", ".Y", ".Z"};  // file extensions for different channels
float **spectra; // arrays of average power spectra
char *outname;  // basename for output files, default - the same as input file
char *outnames[4];  // full names of files with output data
char *outputpath; // path to the output data
int bytesread;  // how many bytes we have read from the input file
int size_of_long = sizeof(long);  // size of the long type
int size_of_long_long = sizeof(long long);  // size of the long long type
int next;  // counter to the first file to process
bool is_dummy = false;   // true when dummy mode is set
bool is_check = false;   // true when check mode is set
bool is_info_shown = false;  // true when info for first frame is shown
bool is_quite = false; // true when one wants "quite" output without changing frames number

/*+++ present output info +++*/
char bold[] = "\033[1m"; 
char ul[] = "\033[4m"; 
char none[] = "\033[0m";
char tics[4] = { '-', '\\', '|', '/' };
unsigned char tics_counter = 0;

/*+++++++++ sync section +++++++++*/
unsigned long syncmask = 0xffffffff; // all ones in 4 bytes
unsigned long syncdata;              // sync record (all units);
off_t syncpos = 0;                   // position of sync record in bytes
bool syncdetect;                     // flag for sync record detection

/*+++++++++ frame section ++++++++*/
unsigned long long *raw_framedata;  // raw array of one frame (HD + 1sec of sampling data section)
size_t framesize = BLOCK_SIZE + 8;  // size of frame in bytes
size_t Nframe_processed = 0;   // the number of processed frames
double sec_step = 1.;          // the step in seconds between frames
size_t NumberOfFrames = 0;     // number of frames to process (0 means all frames)
size_t BeginFrame = 0;         // start to process from this frame
size_t Nframe_skipped = 0;     // number of frames skipped before processing

bool is_spectra = false;       // calculate average spectra in each channel rather than raw data
bool swap_bytes = false;       // to swap bytes in input data or not

fftwf_complex *x;  // to calculate spectra
fftwf_plan pfor;   // plan variable for FFTW
float SpectraLevel[4];  // spectrum average level to eliminate

#endif
