#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <getopt.h>

// number of sampling bits
#define BITS 2
// sampling frequnecy (in MHz)
#define FREQ 8 
// number of channels (1 or 4 in K5)
// #define CHAN 4
// size of the sampling data section (1 sec) in bytes
//#define BLOCK_SIZE (((FREQ * CHAN * BITS) / 8) * 1000000)
// should be multiplied by channels number
#define BLOCK_SIZE (((FREQ * BITS) / 8) * 1000000)

int chan = 4;

/* Parse command line */
int ParseCmdLine (int argc, char *argv[]) {
 int op;
 struct option long_options[] = { {"chan", required_argument, 0, 'c'},
                                  {0, 0, 0, 0}
                                };
 while((op = getopt_long(argc, argv, "c:", long_options, 0)) != EOF)
  switch(op){
    case 'c': chan = atoi (optarg);
              if (chan != 1 && chan != 4) chan = 4;
              break;
    case '?': printf ("wrong option!\n");
              exit (1);
              break;
  }
 return(optind);
}

/* m a i n */
int main (int argc, char *argv[]) {

 int next;
 if (argc == 1) { printf("Usage: %s [-c <1|4>] <k5 raw-data file>\n"
                         "  -c <1|4>  -  number of channels (1 or 4), default - 4\n", argv[0]); exit(0); }
 // parse command line
 next = ParseCmdLine (argc, argv);
 if (argc < next + 1) { printf ("Not point the file to process!\n"); exit(1); }

 int counter = 0;
 unsigned long hd1, hd2;
 unsigned long seconds;
 unsigned char Nch, freq, Nbits, sync2;

 struct stat info;
 if ((stat (argv[next], &info)) == -1) { perror("stat"); exit(1); }

 int in;
 in = open (argv[next], O_RDONLY | O_LARGEFILE);
 if (in == -1) { perror("open"); exit(1); }

 printf ("chan = %d\n", chan);
 while (counter*(8 + (BLOCK_SIZE*chan)) < info.st_size) {
  lseek (in, counter*(8 + (BLOCK_SIZE*chan)), SEEK_SET);
  read (in, &hd1, 4);
  read (in, &hd2, 4);
  seconds = hd2 & 0x0001ffffl;
  Nch = (hd2 >> 17) & 0x0001;
  freq = (hd2 >> 18) & 0x000f;
  Nbits = (hd2 >> 22) & 0x0003;
  sync2 = (hd2 >> 24) & 0x00ff;
  printf ("#%d sync: %x  secs: %ld  Nch: %d freq: %d Nbits: %d sync2: %x\n", counter, hd1, seconds, Nch, freq, Nbits, sync2);
  ++counter;
 }

 close (in);
	
 return 0;
}
