#ifndef __FILEHEADER_HPP
#define __FILEHEADER_HPP

#include <stdio.h>

/*++++++++++ abstract header +++++++++++++*/
class JustHeader {
  protected:
	const int header_size = 1000; // header size in bytes
  protected:
	virtual int get_header_size() { return header_size; }
  public:
	JustHeader () {} //constructor
	virtual ~JustHeader () {} //destructor
	virtual void write (int descr) { // write to file
	 write (descr, &header_size, sizeof(int));
	}
};

/*++++++++++ K5 data header ++++++++++++*/
class K5Header : public JustHeader {
  protected:
	string mjd; // julian day (27 bytes)
	string pulsar; //pulsar name (10 bytes)
  public:
	K5Header () {} // constructor
	virtual ~K5Header () {} //destructor
	virtual void write (int descr) { //write to file
	 JustHeader::write (descr);
//	 write (descr, pulsar.c_str(), pulsar);
	 // skip
	}
};

/*++++++++++ Det header ++++++++++++++++*/
class DetHeader : public K5Header {
  protected:
	string mjd; // julian day (27 bytes)
	double reffreq;
	double centerfreq;
	double bandwidth;
	double timeres;
	double dm;
	bool is_direct;
  public:
	DetHeader () {} // constructor
	DetHeader (char *m, double ref, double center, double bw, double tres, double d, bool is_d) { //constructor
	 mjd = m;
	 reffreq = ref;
	 centerfreq = center;
	 bandwidth = bw;
	 timeres = tres;
	 dm = d;
	 is_direct = is_d;
	 pulsar = "1937+21";
	}
	virtual ~DetHeader () {} //destructor
	void write (int descr) { // write to file
	}
};

/*+++++++++ Fold header ++++++++++++*/
class FoldHeader : public DetHeader {
  protected:
	string mjd; // modified taking into account the phase
	double period;
	double phase; //desired phase
	unsigned long Nfold; // number of folded periods
  public:
	FoldHeader () {} //constructor
	FoldHeader (char *m, double per, double ph, unsigned long nf) { // constructor
	 mjd = m;
	 period = per;
	 phase = ph;
	 Nfold = nf;
	}
	virtual ~FoldHeader () {} //destructor
};

#endif // __FILEHEADER_HPP
