#include "bitstat.hpp"

/* get recovered value */
float recovered_value (signed char value, double part) {

 // part - percent of "1"-bits
 double x = inverf(part); // determine "x" (argument of erf function) = t/(sigma * sqrt(2))
 double ts = x*sqrt((double)2.); // t/sigma
 double st = ((double)1.)/ts; // sigma/t
 static double pi2 = sqrt((double)2./(double)M_PI); // sqrt(2/pi)

 double x2 = ((double)-1.)*x*x; // t^2/(2*sigma^2)

 /* Power optimized levels */
 double y = 2. * (fabs(value) == 1. ? ((double)value)*st*sqrt( (double)1. - ts*pi2*(exp(x2)/part) ) : ((double)value/(double)3.)*st*sqrt( (double)1. + ts*pi2*(exp(x2)/((double)1. - part)) ) );

 /* Voltage optimized levels */
// double y = 2. * (fabs(value) == 1. ? ((double)value)*st*pi2*(((double)1.-exp(x2)) / part) : ((double)value/(double)3.)*st*pi2*exp(x2)/((double)1. - part) );

 return (float)y;
}  

/* correct for bit-statistics */
/*
 * usually we set the number equal to number spoiling points due to cycling convolution
 * during correction for dispersion. Spoiling points for direct band (when we setting the values
 * to lowest frequency) are in the beginning of the array. 
 * It's ease to correct values in the beginning of the array with the same statistics -- anyway
 * we have to ignore these values after the dedispersion!!! But if the spoiling points are in the
 * end of the array, we have to change slithly this code !!!
 */
void bits_correction (signed char *x, float *y, off_t size, long number) {
 
 if (number > size) { 
   printf ("warning - number of counts for bits-statistics is larger than file size\n"); 
   printf ("setting number equal to the file size ... %d\n", size);
   number = size;
 }
 double sum = 0.;
 for (off_t i=0; i<number; ++i) sum += (x[i] == 1 || x[i] == -1 ? 1. : 0.);
 y[0] = recovered_value (x[0], sum/number);
 for (off_t i=0, j=number; i<size-number; ++i, ++j) {
  sum -= (x[i] == 1 || x[i] == -1 ? 1. : 0.);
  sum += (x[j] == 1 || x[j] == -1 ? 1. : 0.);
  y[i+1] = recovered_value (x[i+1], sum/number);
 }
 for (off_t i=size-number; i<size; ++i) y[i] = recovered_value (x[i], sum/number);
}
