#include "cmd.hpp"

extern ObsConf obs;
//extern char pulse_type;
extern off_t bitstat_points;
extern off_t size;
extern bool is_calc_prof;

/* help */ 
void help (char *s) {
 printf("\nPROGRAMM %s\n"
	"To reconstruct raw cut-files in total band\n"
        "[USAGE]: %s [options] <W> <X> <Y> <Z>\n\n"
        "[OPTIONS]:\n"
	"-t, --timeres <value>         - time resolution (ns) in total band, default = 7.8125\n"
	"--dm <value>                  - dispersion measure (pc/cm^3), default = 71.0255\n"
	"--bpoints <value>             - bitstat points, usually equal to dispersion spoiled pointes\n"
	"--bw <value>                  - total bandwidth (MHz), default = 64\n"
	"--reffreq <value>             - reference frequency (MHz), default = 2100\n"
	"--centerfreq <value>          - center frequency (MHz) of total band, default = 2084\n"
	"--direct                      - to set the band as direct, band is direct by default\n"
	"--inverse                     - to set the band as inverse, band is direct by default\n"
	"--basename   <name>           - base name for output files: basename + \".det\"\n"
	"--prof                        - calculate average profile, default - no\n"
	"-d, --path   <path>           - path for output files\n"
	"-s, --size <value>            - FFT size, default - 1048576\n"
	"-q, --quite                   - turn on quite output, skip printing the iteration changes\n"
	"-h, --help                    - help\n\n", s, s);
 exit(0);
}

/* parse command line */
int parse_command_line (int argc, char *argv[]) {
  int op, tm;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"timeres", required_argument, 0, 't'},
				   {"dm", required_argument, 0, 10},
				   {"bw", required_argument, 0, 11},
				   {"reffreq", required_argument, 0, 12},
				   {"centerfreq", required_argument, 0, 13},
                                   {"direct", no_argument, 0, 14},
				   {"inverse", no_argument, 0, 15},
				   {"basename", required_argument, 0, 16},
				   {"path", required_argument, 0, 'd'},
				   {"bpoints", required_argument, 0, 19},
				   {"size", required_argument, 0, 's'},
				   {"prof", no_argument, 0, 20},
				   {"quite", no_argument, 0, 'q'},
				   {0, 0, 0, 0}
                                  };

  optind = 0; optarg = 0;
  while((op = getopt_long(argc, argv, "ht:d:s:q", long_options, 0)) != EOF)
    switch(op){
      
      case 'h':
        help (argv[0]);
      break;
      
      case 't':
        obs.set_time_resolution (atof(optarg));
      break;
      
      case 10:
        obs.set_dispersion_measure (atof(optarg));
      break;
      
      case 11:
        obs.set_bandwidth (atof(optarg));
      break;
      
      case 12:
        obs.set_ref_freq (atof(optarg));
      break;
      
      case 13:
        obs.set_center_freq (atof(optarg));
      break;
      
      case 14:
        obs.set_band_direction (true);
      break;

      case 15:
        obs.set_band_direction (false);
      break;

      case 16:
        if (base) { base = 0; free (base); }
	if (!(base = (char *)calloc((int)strlen(optarg), sizeof(char *)))) { perror("new"); exit (1); }
	memcpy (base, optarg, (int)strlen(optarg));
      break;

      case 'd':
        if (output_path) { output_path = 0; free (output_path); }
	if (!(output_path = (char *)calloc((int)strlen(optarg), sizeof(char *)))) { perror("new"); exit (1); }
	memcpy (output_path, optarg, (int)strlen(optarg));
      break;

      case 19:
	bitstat_points = (off_t)atol(optarg);
      break;

      case 's':
	size = (off_t)atol(optarg);
      break;

      case 20:
	is_calc_prof = true;
      break;

      case 'q':
       is_quite = true;
      break;

      case '?':
        help (argv[0]);
      break;
    }
  return(optind);
}
