#include "header.hpp"

#include <string.h>
#include <math.h>
#include <gmp.h> // high-precision math library

// constructor
Header::Header (char *m, int mlen) {
  mjd = m;
  mjd_string_size = mlen;
  first_digits = mjd.find('.');
}

// initialize
void Header::init (char *m, int mlen) {
  mjd = m;
  mjd_string_size = mlen;
  first_digits = mjd.find('.');
}

// update
void Header::update (unsigned long shift, double tres) {
  mpf_t m1, temp, temp2;
  mpf_init2 (m1, 512); mpf_init2 (temp, 512); mpf_init2 (temp2, 512);
  mpf_set_str (m1, get_mjd(), 10);
  mpf_set_ui (temp, (unsigned int)shift);
  mpf_set_d (temp2, tres);
  mpf_mul (temp, temp, temp2);
  mpf_set_str (temp2, "86400000000000", 10);
  mpf_div (temp, temp, temp2);
  mpf_add (temp, temp, m1);
  mpf_clear (m1); mpf_clear (temp2);
  long exponent = first_digits;
  mjd = mpf_get_str (NULL, &exponent, 10, mjd_string_size, temp);
  string s = mjd;
  mjd = s.substr(0, first_digits) + "." + s.substr(first_digits, s.length()-first_digits);
  mpf_clear (temp);
}

// return start mjd in usual format like 53528.4567...
// show only 16 digits after the radix point
// 1 ns -- it's 14'th digit of the fraction of day
char* Header::get_mjd (char *m) {
 memcpy (m, mjd.c_str(), mjd_string_size);
 return m;
}

// return start mjd in usual format like 53528.4567...
// show only 16 digits after the radix point
// 1 ns -- it's 14'th digit of the fraction of day
const char* Header::get_mjd () {
 return mjd.c_str();
}

// get hour, min and sec of GP
void Header::getTime (int& hour, int& min, double& sec) {
 string cur;
 cur = "0." + mjd.substr(first_digits, 15);
 mpf_t temp, temp2;
 mpf_init2 (temp, 512); mpf_init2 (temp2, 512);
 mpf_set_str (temp, cur.c_str(), 10);
 mpf_set_ui (temp2, 24);
 mpf_mul (temp, temp, temp2);
 hour = (int)floor(mpf_get_d (temp));
 mpf_sub_ui (temp, temp, (unsigned long)hour);
 mpf_set_ui (temp2, 60);
 mpf_mul (temp, temp, temp2);
 min = (int)floor(mpf_get_d (temp));
 mpf_sub_ui (temp, temp, (unsigned long)min);
 mpf_mul (temp, temp, temp2);
 sec = mpf_get_d (temp);
 mpf_clear (temp); mpf_clear (temp2);
}
