#include "hlop.hpp"

// observing conf
ObsConf obs;

// cut-size is 548317, so we just ignore the 24029 counts in dedispersion
off_t size = 1048576;

// number of channels combined
int Nmerged_chan = 4;

/*--------------------------------------------------------------------------------------
 *                               M   A   I   N
 * -------------------------------------------------------------------------------------*/

int main (int argc, char *argv[]) {

 if (argc == 1) help (argv[0]);

 /* initialize wisdom */
 is_wisdom = !initialize_wisdom ();
 if (!is_wisdom) { printf ("FFTW Wisdom will not be used!\n"); }

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;
        
 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE, &r);
 /*------------------------------------------*/

 char *outname, *profname;
 outname  = new char[255];
 profname  = new char[255];
 output_path  = new char[255];
 base = new char[255];
 if (!outname || !base || !output_path || !profname) { perror("new"); exit (1); }
 memset (output_path, 0, 255);
 memset (outname, 0, 255);
 memset (profname, 0, 255);
 memset (base, 0, 255);
 sprintf (base, "%s", "dest");
 sprintf (output_path, "%s", ".");

 int next = parse_command_line (argc, argv);
 if (argc < next+1) { printf ("Not point the files to process!\n"); exit (1); }

 sprintf (outname, "%s/%s%s", output_path, base, ".det");
 sprintf (profname, "%s/%s%s", output_path, base, ".prof");

 off_t size4 = Nmerged_chan*size;
 
 struct stat info;
 if (stat(argv[next], &info) != 0) { perror("stat"); exit(1); }
 off_t filesize = info.st_size;
 off_t leftborder = 0;

 unsigned long begpoints = obs.get_start_spoiled_points();
 unsigned long endpoints = obs.get_end_spoiled_points();

 obs.print_info ();
 // number of spoiled points in single merged channel
 off_t chan1_spoiled_points = (off_t)ceil((begpoints + endpoints) / (double)Nmerged_chan);
 if (chan1_spoiled_points * Nmerged_chan > begpoints + endpoints) {
  if (endpoints == 0 && begpoints != 0) begpoints += chan1_spoiled_points * Nmerged_chan - begpoints - endpoints;
  if (begpoints == 0 && endpoints != 0) endpoints += chan1_spoiled_points * Nmerged_chan - begpoints - endpoints;
  if (begpoints != 0 && endpoints != 0) {
   if (begpoints >= endpoints) begpoints += chan1_spoiled_points * Nmerged_chan - begpoints - endpoints;
    else endpoints += chan1_spoiled_points * Nmerged_chan - begpoints - endpoints;
  }
  printf ("spoiled points [UPDATED]: start -> %ld   end -> %ld   total -> %ld\n", begpoints, endpoints, begpoints+endpoints);
 }

 // define bitstat_points
 if (bitstat_points == 0 || bitstat_points > size || bitstat_points < 0) {
//  bitstat_points = (off_t)obs.get_spoiled_points();
  bitstat_points = chan1_spoiled_points;
 }
 off_t cutsize = size + bitstat_points;

 // iterations' number
 off_t Niter = 0;
 while (leftborder+cutsize <= filesize) {
  leftborder += (size-chan1_spoiled_points);
  Niter++;
 }
 leftborder = 0;

 printf ("\n");
 printf ("filesize: %ld  fft size: %ld  cutsize: %ld  bitstat_points: %ld  Niter: %ld\n", (unsigned long)filesize, (unsigned long)size, (unsigned long)cutsize, (unsigned long)bitstat_points, (unsigned long)Niter);

 signed char *src;
 src = new signed char[cutsize];
 if (!src) { perror("new"); exit(1); } 
 memset (src, 0, cutsize);

 float *srcf;
 srcf = new float[size4];
 if (!srcf) { perror("new"); exit(1); } 

 fftwf_complex *matrix;
 matrix = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * (size4/2 + 1));
 if (!matrix) { perror("fftwf_malloc"); exit (1); }
 memset (matrix, 0, (size4/2+1)*sizeof(fftwf_complex));
 matrix = get_matrix (matrix, size4);

 fftwf_complex *part1, *part2, *dest;
 part1 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 part2 = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size);
 dest = (fftwf_complex *) fftwf_malloc (sizeof(fftwf_complex) * size4);
 if (!part1 || !part2 || !dest) { perror("fftwf_malloc"); exit(1); }
 fftwf_plan plan1, plan2, backplan;
 if (!is_wisdom) {
  plan1 = fftwf_plan_dft_1d (size, part1, part1, FFTW_FORWARD, FFTW_ESTIMATE);
  plan2 = fftwf_plan_dft_1d (size, part2, part2, FFTW_FORWARD, FFTW_ESTIMATE);
  backplan = fftwf_plan_dft_1d (size4, dest, dest, FFTW_BACKWARD, FFTW_ESTIMATE);
 } else {
/*
  plan1 = fftwf_plan_dft_1d (size, part1, part1, FFTW_FORWARD, FFTW_EXHAUSTIVE);
  plan2 = fftwf_plan_dft_1d (size, part2, part2, FFTW_FORWARD, FFTW_EXHAUSTIVE);
  backplan = fftwf_plan_dft_1d (size4, dest, dest, FFTW_BACKWARD, FFTW_EXHAUSTIVE);
*/
  plan1 = fftwf_plan_dft_1d (size, part1, part1, FFTW_FORWARD, FFTW_ESTIMATE);
  plan2 = fftwf_plan_dft_1d (size, part2, part2, FFTW_FORWARD, FFTW_ESTIMATE);
  backplan = fftwf_plan_dft_1d (size4, dest, dest, FFTW_BACKWARD, FFTW_ESTIMATE);
 }

 char *head;
 head = new char[MaxString];
 if (!head) { perror("new"); exit (1); } 
 memset (head, 0, MaxString);

 // read MJD in header
 int hdescr;
 if ((hdescr = open (argv[next], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 read (hdescr, head, mjd_header);
 close (hdescr);
 header.init (head, mjd_header);
 printf ("start mjd = %s   ", header.get_mjd()); fflush (NULL);
 header.update (begpoints, obs.get_time_resolution());
 printf ("updated mjd = %s\n", header.get_mjd());
 memset (head, 0, MaxString);

 // initialize class with poluco.dat records
 PolycoFile pol;
 double period = pol.getPeriod (header.get_mjd(), true);
 double phase = pol.getPhase (header.get_mjd());
 double shift; // shift from the beginning to be in phase
 off_t cPeriod = (off_t)rint(period / obs.get_time_resolution()); // period in counts
 off_t profindex = 0; // position within the profile to fold correctle and fully
 unsigned long Nperiods = 0; // number of folded periods
 off_t left = 0; // left edge of the current period

 if (desired_phase >= phase) { shift = (((desired_phase - phase) * period) / obs.get_time_resolution()); }
   else { shift = (((desired_phase + 1. - phase) * period) / obs.get_time_resolution()); }
 left = (off_t)rint(shift + Nperiods * (period / obs.get_time_resolution()));

 printf ("period: %.1lf ns (%ld counts)   phase: %.3lf   shift: %.1lf\n", period, (unsigned long)cPeriod, phase, shift);

 float *prof;
 if (is_calc_prof) {
  prof = new float[cPeriod];
  if (!prof) { perror("new"); exit(1); } 
  memset (prof, 0, cPeriod * sizeof(float));
 }

/* Processing starts */
 printf ("Processing [%ld] ", (unsigned long)Niter); fflush(NULL);
 for (int s=0; s<Niter; ++s) {

  if (!is_quite) { printf(".%5d", s+1); fflush(NULL); }

 memset (part1, 0, size*sizeof(fftwf_complex));
 memset (part2, 0, size*sizeof(fftwf_complex));
 memset (dest, 0, size4*sizeof(fftwf_complex));

 /* read correction functions */
/*
 float *bandlcp, *bandrcp;
 bandlcp = new float[size4/2+1];
 bandrcp = new float[size4/2+1];
 if (!bandlcp || !bandrcp) { perror("new"); exit (1); }
 memset (bandlcp, 0, (size4/2+1)*sizeof(float));
 memset (bandrcp, 0, (size4/2+1)*sizeof(float));
 int bandfile;
 if ((bandfile = open ("lcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 lseek (bandfile, 0, SEEK_SET);
 memset (bandlcp, 0, (size4/2+1)*sizeof(float));
 read (bandfile, bandlcp, (size4/2+1)*sizeof(float));
 close (bandfile);
 if ((bandfile = open ("rcp.corr", O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 lseek (bandfile, 0, SEEK_SET);
 memset (bandrcp, 0, (size4/2+1)*sizeof(float));
 read (bandfile, bandrcp, (size4/2+1)*sizeof(float));
 close (bandfile);
*/
 /* - */

 read_correct_n_fill (src, srcf, part1, mjd_header, size, leftborder, cutsize, size4, bitstat_points, 0, 0, argv[next]);
 read_correct_n_fill (src, srcf, part1, mjd_header, size, leftborder, cutsize, size4, bitstat_points, 0, 1, argv[next+1]);
 read_correct_n_fill (src, srcf, part2, mjd_header, size, leftborder, cutsize, size4, bitstat_points, 0, 0, argv[next+2]);
 read_correct_n_fill (src, srcf, part2, mjd_header, size, leftborder, cutsize, size4, bitstat_points, 0, 1, argv[next+3]);

 fftwf_execute (plan1);
 fftwf_execute (plan2);
 ship_out2gbt_total (dest, part1, part2, size);

 dest = dedispersion (dest, matrix, NULL, backplan, size4, true, false);

 memset (srcf, 0, size4*sizeof(float));
 for (off_t i=begpoints, j=0; i<size4-endpoints; ++i, ++j) { srcf[j] = (dest[i][0] / size4) * (dest[i][0] / size4) + (dest[i][1] / size4) * (dest[i][1] / size4); }
 write2file (srcf, header.get_mjd(head), s, size4-begpoints-endpoints, outname);

 /* fold the profile */
 if (is_calc_prof) {
	 
 if (profindex != 0) { // process the previous part
  for (off_t w=profindex, q=(off_t)begpoints; w<cPeriod; ++w, ++q) { prof[w] = (prof[w] * Nperiods + srcf[q]) / (Nperiods + 1); }
  profindex = 0;
  Nperiods += 1;
  left = (off_t)rint(shift + Nperiods * (period / obs.get_time_resolution()));
  while (left > size4  - (off_t)endpoints - (off_t)begpoints) { left -= (size4  - (off_t)endpoints - (off_t)begpoints); }
  write2file (prof, cPeriod, profname);
  // call plotting here
 }
 while (left < size4  - (off_t)endpoints - (off_t)begpoints) {
  if (left + cPeriod > size4 - (off_t)endpoints - (off_t)begpoints) profindex = size4 - (off_t)endpoints - (off_t)begpoints - left; else profindex = 0;
  if (profindex == 0) { for (off_t w=0, q=(off_t)begpoints+left; w<cPeriod; ++w, ++q) { prof[w] = (prof[w] * Nperiods + srcf[q]) / (Nperiods + 1); } }
   else { if (s < Niter-1) { for (off_t w=0, q=(off_t)begpoints+left; w<profindex; ++w, ++q) { prof[w] = (prof[w] * Nperiods + srcf[q]) / (Nperiods + 1); } } }
  if (profindex == 0) { 
   Nperiods += 1; 
   left = (off_t)rint(shift + Nperiods * (period / obs.get_time_resolution())); 
   while (left > size4 - (off_t)endpoints - (off_t)begpoints) { left -= (size4 - (off_t)endpoints - (off_t)begpoints); }
   write2file (prof, cPeriod, profname);
   // call plotting program
  } else { left = (off_t)rint(shift + (Nperiods + 1) * (period / obs.get_time_resolution())); }
 }
 } // is_calc_prof

 if (!is_quite) { printf("\b\b\b\b\b"); fflush (NULL); }

 // increase leftborder
 leftborder += (size - chan1_spoiled_points);

 } // loop on iterations
 printf (" - done\n");

 if (is_calc_prof) delete (prof);
 delete (head);
 free (output_path);
 delete (outname);
 free (base);
// delete (bandlcp);
 delete (srcf);
 delete (src);
 fftwf_free (matrix);
 fftwf_free (part1);
 fftwf_free (part2);
 fftwf_free (dest);
 fftwf_destroy_plan (plan1);
 fftwf_destroy_plan (plan2);
 fftwf_destroy_plan (backplan);

 return 0;
}
