#ifndef _OBSCONF_HPP
#define _OBSCONF_HPP

class ObsConf {
   double center_freq;
   double ref_freq;
   double bandwidth;
   bool band_direct;
   double timeres;
   double dm;

  public:
   ObsConf ();
   ObsConf (double cen, double ref, double bw, bool bd, double tres, double disp);
   virtual ~ObsConf () {}

   inline void set_center_freq (double c) { center_freq = c; }
   inline void set_ref_freq (double r) { ref_freq = r; }
   inline void set_bandwidth (double b) { bandwidth = b; }
   inline void set_band_direction (bool is) { band_direct = is; }
   inline void set_time_resolution (double t) { timeres = t; }
   inline void set_dispersion_measure (double d) { dm = d; }

   inline double get_center_freq () { return center_freq; }
   inline double get_ref_freq () { return ref_freq; }
   inline double get_bandwidth () { return bandwidth; }
   inline bool is_band_direct () { return band_direct; }
   inline double get_time_resolution () { return timeres; }
   inline double get_dispersion_measure () { return dm; }

   /* print out info */
   void print_info ();

   /* get total number of spoiled points */
   unsigned long get_spoiled_points ();
   inline unsigned long get_spoiled_points (double f1, double f2);

   /* get full value of spoiled time in ns */
   double get_spoiled_time ();
   inline double get_spoiled_time (double f1, double f2);

   /* get number of spoiled points in the beginning */
   unsigned long get_start_spoiled_points ();
   /* get spoiled time in ns in the beginning */
   double get_start_spoiled_time ();

   /* get number of spoiled points in the end */
   unsigned long get_end_spoiled_points ();
   /* get spoiled time in ns in the end */
   double get_end_spoiled_time ();
};

#endif //#ifndef _OBSCONF_HPP
