#include <string>
#include <list>
#include <vector>
#include <gmp.h>  // high-precision math library


using namespace std;

class PolycoRecord {
	string pulsar;
	string date;
	string utc;
	string mjd;
	string dm;
	string doppler;
	string fit;
	string refphase;
	string f0;
	string observatory;
	int span;
	int ncoeff;
	string skyfreq;
	string binphase;
	vector<string> coeffs;

	// get recursive product of value 'iter' times
	// accepts and returns mpf_t data
	void getProduct (mpf_t value, int iter);

   public:
	   PolycoRecord (string p, string d, string u, string m, string disp, string dop, string f, string r, string freq, string o, int s, int n, string sky, string b);
	   virtual ~PolycoRecord () { coeffs.clear(); };

	   string getPulsar () { return pulsar; };
	   string getDate () { return date; };
	   string getUTC () { return utc; };
	   // get time in "hh:mm:ss.ss" format
	   string getTime ();
	   // get hours
	   int getHH ();
	   // get minutes
	   int getMM ();
	   // get seconds
	   double getSS ();
	   // get time in seconds from the beginning of the day
	   double getSeconds ();
	   string getMJD () { return mjd; };
	   string getDM () { return dm; };
	   string getDoppler () { return doppler; };
	   string getFit () { return fit; };
	   string getRefPhase () { return refphase; };
	   string getF0 () { return f0; };
	   string getObs () { return observatory; };
	   int getSpan () { return span; };
	   int getNcoeff () { return ncoeff; };
	   string getSkyFreq () { return skyfreq; };
	   string getBinPhase () { return binphase; };
	   
	   string getCoeff (int index = 0);

	   // calculate the period for the particular MJD
	   // using the coefficients from the current record
	   // if ns = false period is in seconds, otherwise - in nanoseconds
	   double getPeriod (string m, bool ns = false);

	   // calculate the phase for the particular MJD
	   double getPhase (string m);

	   // setting the coefficient of index 'index'
	   void setCoeff (string value, int index);
	   // setting the next coefficient in the vector recs
	   void setCoeff (string value);
};

class PolycoFile {
	string filename;
	list<PolycoRecord> recs;

    public:
	PolycoFile (char* fn = "polyco.dat");
	virtual ~PolycoFile () { recs.clear(); };

	// return pulsar name
	string getPulsar ();

	// get pulsar period for the particular MJD
	// if ns = false period is in seconds, otherwise - in nanoseconds
	double getPeriod (string mjd, bool ns = false);

	// calculate the phase for the particular MJD
	double getPhase (string mjd);

	// check the periods for particular MJD in "polyco.dat"
	// using the coefficients from the previous record
	void checkModel ();
};
