#define MAIN_
#include "k5toraw.hpp"
#include "hdsd.hpp"
#include "stat.hpp"

/*==========================   P r o m p t i n g   ==================================*/

void Help (char *s) {
  printf("\n%sPROGRAMM%s %s\n"
         "It is used to decode K5/VSSP board data format\n\n"
         "%s[USAGE]:%s %s [options] <file1> [file2 ... fileN]\n\n"
         "%s[OPTIONS]:%s\n"
	 "-d, --path <path>       - path to the output files, default - the same\n"
	 "                          where are the source files\n"
	 "-o, --out <basename>    - basename of files for output data\n"
	 "--swap-bytes            - to reverse bits order in input data,\n"
	 "                          use this option on Apple machines\n"
	 "-b, --begin <frame>     - start processing from this frame\n"
	 "-n, --num <frames>      - number of frames to process\n"
	 "--shift <counts>        - don't write counts <counts> from the beginning of the first frame\n"
	 "--dummy                 - set dummy mode (no output files)\n"
	 "--check                 - check the integrity of input data, time continuity, ...\n"
	 "--ignore                - to ignore time-coninuity errors, other errors are ignored by default\n"
	 "--debug                 - to debug the data, i.e. get full info for every frame; with this option\n"
	 "                          --check and --dummy are also set up\n"
	 "--sync                  - to search sync word after every read frame\n"
	 "--spectra[=number]      - calculate average spectra rather than raw data\n"
	 "                          number is optional arg that means number of counts to average in\n"
	 "                          calculated spectra, default = 1024\n"
	 "--show-spectra          - show built spectra during the calculation, use together with --spectra\n"
	 "--skipharm <number>     - number of harmonics to skip from the beginning in calculated spectra, default - 0\n"
	 "--stat                  - to calculate bit statistics\n"
         "--mjd <value>           - MJD of the beginning of the decoded files, default = 0\n"
         "--label <string>        - extra label like the name of the telescope, or experiment, or data taking machine\n"
	 "                          to draw in the very first legend row in spectra or statistics plots\n"
	 "-q, --quite             - turn on quite output, skip printing the frames number changing\n"
	 "-h, --help              - help\n\n", bold, none, s, bold, none, s, bold, none);
  exit(0);
}

/*=========================  erase path part from filename  =========================*/

// get basename
char* basename (char *str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
}

/*=========================  get dirname  =========================*/

char* dirname (char *str, char *dest) {
 int len = (int)strlen(str);
 while (str[--len] != '/' && len >= 0) ;
 if (len > 0) { strncpy (dest, str, len); dest[len] = '\0'; } else { dest[0] = '.'; dest[1] = '\0'; }
 return dest;
}
  
/*========================= Parsing the command line =================================*/

int ParseCmdLine (int argc, char *argv[]) {
 int op;
 struct option long_options[] = { {"help", no_argument, 0, 'h'},
	                          {"out", required_argument, 0, 'o'},
	                          {"path", required_argument, 0, 'd'},
	                          {"begin", required_argument, 0, 'b'},
	                          {"num", required_argument, 0, 'n'},
	                          {"dummy", no_argument, 0, 13},
	                          {"spectra", optional_argument, 0, 14},
	                          {"swap-bytes", no_argument, 0, 15},
	                          {"check", no_argument, 0, 16},
	                          {"show-spectra", no_argument, 0, 17},
	                          {"skipharm", required_argument, 0, 18},
	                          {"stat", no_argument, 0, 19},
	                          {"shift", required_argument, 0, 20},
                                  {"mjd", required_argument, 0, 21},
	                          {"debug", no_argument, 0, 22},
	                          {"ignore", no_argument, 0, 23},
	                          {"sync", no_argument, 0, 24},
				  {"quite", no_argument, 0, 'q'},
                                  {"label", required_argument, 0, 25},
	                          {0, 0, 0, 0}
                                };
 while((op = getopt_long(argc, argv, "hd:o:n:b:q", long_options, 0)) != EOF)
  switch(op){
    case 'h': Help (basename(argv[0]));
	      break;
    case 'd': if (outputpath) { outputpath = 0; free (outputpath); }
	      if (!( outputpath = (char *)calloc((int)strlen(optarg), sizeof(char *)) )) { perror("calloc outputpath"); exit(1); }
	      memcpy(outputpath, optarg, (int)strlen(optarg));
	      break;
    case 'o': if (outname) { outname = 0; free (outname); }
	      if (!( outname = (char *)calloc((int)strlen(optarg), sizeof(char *)) )) { perror("calloc outname"); exit(1); }
	      memcpy(outname, optarg, (int)strlen(optarg));
	      break;
    case 'b': BeginFrame = atol (optarg);
	      if (BeginFrame < 0) BeginFrame = 0;
	      break;
    case 'n': NumberOfFrames = atol (optarg);
	      if (NumberOfFrames < 0) NumberOfFrames = 0;
	      break;
    case 13: is_dummy = true;
	     break;
    case 14: is_spectra = true;
	     if (optarg != NULL) aver = atol (optarg);
	     if (aver < 1) aver = 1;
	     break;
    case 15: swap_bytes = true;
	     break;
    case 16: is_check = true;
	     break;
    case 17: is_spectra_show = true;
	     break;
    case 18: skipharm = atol (optarg);
	     break;
    case 19: is_stat = true;
	     break;
    case 20: shift_counts = atol (optarg);
	     if (shift_counts < 0) shift_counts = 0;
	     break;
    case 21: if (startmjd) { startmjd = 0; free (startmjd); }
             if (!( startmjd = (char *)calloc((int)strlen(optarg), sizeof(char *)) )) { perror("calloc startmjd"); exit(1); }
             memcpy(startmjd, optarg, (int)strlen(optarg)); 
             mpf_set_str (mjd_t, optarg, 10);
             break;
    case 22: is_debug = true;
	     is_check = true;
	     is_dummy = true;
	     break;
    case 23: is_ignore = true;
	     break;
    case 24: is_to_search_sync = true;
	     break; 
    case 'q': is_quite = true;
              break;
    case 25: if (label) { label = 0; free (label); }
	     if (!( label = (char *)calloc((int)strlen(optarg), sizeof(char *)) )) { perror("calloc label"); exit(1); }
	     memcpy(label, optarg, (int)strlen(optarg));
	     break;
    case '?': Help (basename(argv[0]));
              break;
  }
 return(optind);
}

/*=================  level calculation ============================*/
float LevelCalc (signed char *data, size_t sz) {
 float sum = 0.;
 for (size_t i=0; i<sz; ++i) sum += (float)data[i];
 return sum/sz;
}

/*==================== get power of 2 size ================================*/
size_t resize (size_t sz) { 
  long n = 0;
  sz += 1; 
  while ( (1<<(++n)) < sz) ;
  return (1<<(--n));
}

/*==================== calculate power spectrum =============================*/
void getSpectrum (float *sp, signed char *data, size_t sz, float level, int o, size_t frames) {
 for (int i=0; i<sz; ++i) { x[i][0] = (float)data[i]; x[i][1] = 0.; }
 for (int i=0; i<sz; ++i) x[i][0] -= level;
 fftwf_execute (pfor);
 for (int i=0; i<sz/2 + 1; ++i) sp[i] = (sp[i]*frames + (x[i][0] * x[i][0] + x[i][1] * x[i][1])) / (frames + 1);
 lseek (o, 0, SEEK_SET);
 write (o, sp, (sz/2 + 1) * sizeof(float));
}

/*=============== search for the next syncword ===================*/
bool get_next_syncword (int f, off_t seek, off_t& pos) {
 int br; // bytes read
 // infinite loop
 while (1) {
  if ((lseek (f, pos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
  br = read (f, &syncdata, size_of_long);
  if (br == -1) { perror("read syncdata"); exit(1); }
  if (br != size_of_long) break;
  if (syncdata == syncmask) { syncdetect = true; break; }
   else { syncdetect = false; pos += 1; }
 } // end of loop
 if (!syncdetect) return false; else return true;
}

/*++++++++++++++++++++++++++++   m a i n   f u n c t i o n +++++++++++++++++++++++++++*/

int main (int argc, char *argv[]) {

 if (argc == 1) Help (basename(argv[0]));

 /* initialize wisdom */
 if (is_spectra) {
  is_wisdom = !initialize_wisdom ();
  if (!is_wisdom) { printf ("FFTW Wisdom will not be used!\n"); }
 }

 // initialize mjd
 mpf_init2 (mjd_t, 512);

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;

 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE, &r);
 /*------------------------------------------*/

 outname = new char[MaxString];
 if (!outname) { perror("new outname"); exit(1); }
 memset (outname, 0, MaxString);
 sprintf (outname, "%s","\0");

 outputpath = new char[MaxString];
 if (!outputpath) { perror("new outputpath"); exit(1); }
 memset (outputpath, 0, MaxString);
 sprintf (outputpath, "%s","\0");

 startmjd = new char[MaxString];
 if (!startmjd) { perror("new startmjd"); exit (1); }
 memset (startmjd, 0, MaxString);
 sprintf (startmjd, "%d", 0);

 label = new char[MaxString];
 if (!label) { perror("new label"); exit(1); }
 memset (label, 0, MaxString);
 sprintf (label, "%s","\0");

 // parse command line
 next = ParseCmdLine (argc, argv);
 if (argc < next + 1) { printf ("Not point the file to process!\n"); exit(1); }

 if (!is_spectra) is_spectra_show = false;

 // initialize global configuration class
 printf ("\nInitializing... "); fflush (NULL);
 FrameInfo conf;

#ifdef apple
 if ((in = open (argv[next], O_RDONLY)) == -1) { perror("open"); exit(1); };
#else
 if ((in = open (argv[next], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
#endif
 syncpos = 0;
 // infinite loop
 while (1) {
  if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
  bytesread = read (in, &syncdata, size_of_long);
  if (bytesread == -1) { perror("read syncdata"); exit(1); }
  if (bytesread != size_of_long) break;
  if (syncdata == syncmask) { syncdetect = true; break; }
   else { syncdetect = false; syncpos += 1; }
 } // end of loop
 if (!syncdetect) { printf ("Can't initialize the configuration: no sync word found!)\n"); exit (1); }
 if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
 unsigned long long header;
 bytesread = read (in, &header, sizeof(unsigned long long));
 if (bytesread == -1 || bytesread != sizeof(unsigned long long)) { perror("read header"); exit(1); }
 if ((close (in)) == -1) { perror("close"); exit(1); };
 conf.setup (header, 0);

 printf ("done\n");
 if (is_debug) printf (" sync word at %ld bytes\n", (unsigned long)syncpos);
 printf (" sampl. freq = %g  nbits = %d  nchan = %d  start at %ld secs\n", conf.getSamplingFreq(), conf.getNbits(), conf.getNchan(), (unsigned long)conf.getSeconds());
 printf (" bytes in frame = %ld ", (unsigned long)conf.getFrameSize()); fflush (NULL);
 //end of initialization

 // initialize framesize
 framesize = conf.getFrameSize();
 Nchan = conf.getNchan();

 if (!is_dummy) {
  outfilenames = new char*[Nchan];
  if (!outfilenames) { perror("new outfilenames"); exit (1); }
  for (int u=0; u<Nchan; ++u) {
   outfilenames[u] = new char[MaxString];
   if (!outfilenames[u]) { perror("new outfilenames[u]"); exit (1); }
   memset (outfilenames[u], 0, MaxString);
  }

  out = new int[Nchan];
  if (!out) { perror("new out"); exit (1); }
  memset (out, 0, Nchan*sizeof(int));
 } // !is_dummy

 SpectraLevel = new float[Nchan];
 if (!SpectraLevel) { perror("new SpectraLevel"); exit (1); }
 memset (SpectraLevel, 0, Nchan*sizeof(float));
 
 // initialize input data frame array
 raw_framedata = new unsigned long long[framesize/size_of_long_long];
 if (!raw_framedata) { perror("new raw_framedata"); exit(1); }

 SampleData shot (framesize, Nchan, conf.getNbits(), conf.getNbits_id(), swap_bytes);  // initialize the class object
 printf ("  samplesize: %ld counts\n", (unsigned long)shot.getSampleSize());

 if (is_spectra && !is_dummy) {
  spectrumsize = resize(shot.getSampleSize());
  spectra = new float*[Nchan];
  if (!spectra) { perror("new spectra"); exit(1); }
  for (int u=0; u<Nchan; ++u) {
   spectra[u] = new float[spectrumsize/2 + 1];
   if (!spectra[u]) { perror("new spectra[u]"); exit(1); }
   memset (spectra[u], 0, (spectrumsize/2 + 1) * sizeof(float));
  }
  x = (fftwf_complex *) fftwf_malloc (spectrumsize * sizeof(fftwf_complex));
  if (!x) { perror("new x"); exit(1); }
  if (!is_wisdom) pfor = fftwf_plan_dft_1d (spectrumsize, x, x, FFTW_FORWARD, FFTW_ESTIMATE);
   else pfor = fftwf_plan_dft_1d (spectrumsize, x, x, FFTW_FORWARD, FFTW_EXHAUSTIVE);

   if (is_spectra_show) { dev_id = cpgopen (pgplot_dev); cpgask (false); }
 } // is_spectra

 StatInfo bs;
 if (is_stat && !is_dummy) {
  bs.initialize (Nchan, conf.getNbits());
  stat_dev_id = cpgopen (pgplot_dev);
  cpgask (false);
 } // is_stat

 char *mjd;
 mjd = new char[MaxString];
 if (!mjd) { perror("new"); exit (1); }
 memset (mjd, 0, MaxString);

 printf("\nProcessing...\n");
 if (Nframe_skipped < BeginFrame) {
  struct stat information;
  if (stat (argv[next], &information) != 0) { perror("stat information"); exit (1); }
  printf (" file: %s", argv[next]); fflush (NULL);
  while (Nframe_skipped < BeginFrame) {
   Nframe_skipped++;
   syncpos += framesize;
   if (syncpos >= information.st_size) {
     syncpos -= information.st_size;
     next++;
     if (next == argc) { printf (" [skip]\n skipped %ld frames (%g secs)\n syncpos = %ld bytes\n number of available frames (%ld) is less than frames to skip (%ld) !\n done.\n", Nframe_skipped, Nframe_skipped * sec_step, (unsigned long)syncpos, Nframe_skipped, BeginFrame); exit (1); }
     printf (" [skip]\n       %s", argv[next]); fflush (NULL);
     if (stat (argv[next], &information) != 0) { perror("stat information"); exit (1); }
   } // syncpos
  } //while
  printf (" [ok]  syncpos = %ld bytes\n", (unsigned long)syncpos);
  printf (" skipped %ld frames (%g secs)\n", Nframe_skipped, Nframe_skipped * sec_step);
 }

#ifdef apple
 if ((in = open (argv[next], O_RDONLY)) == -1) { perror("open"); exit(1); };
#else
 if ((in = open (argv[next], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
#endif

 if (!is_dummy) {  // on if dummy mode isn't set
 // create output file names
 // if outname is still empty, call its basename as argv[next]
 if (!strcmp(outname, "\0")) {
  memset (outname, 0, MaxString);
  sprintf (outname, "%s", basename(argv[next]));
 }
 if (!strcmp(outputpath, "\0")) {
  memset (outputpath, 0, MaxString);
  outputpath = dirname(argv[next], outputpath);
 }
 for (int u=0; u<Nchan; ++u) { 
  memset (outfilenames[u], 0, MaxString);
  sprintf (outfilenames[u], "%s/%s%s%s", outputpath, outname, ext[u], (is_spectra ? ".sp2" : ""));
  printf (""); fflush (NULL);
 }
 if (Nchan == 1) printf ("\n outname: %s\n", outfilenames[0]);
  else printf ("\n outnames: %s/%s%s%s\n", outputpath, outname, ".[WXYZ]", (is_spectra ? ".sp2" : ""));

 // at first remove existing file
 for (int u=0; u<Nchan; ++u) unlink (outfilenames[u]);
 // open for appending
   for (int u=0; u<Nchan; ++u) {
#ifdef apple
    if ((out[u] = open (outfilenames[u], O_CREAT | (is_spectra ? O_TRUNC : O_APPEND) | O_WRONLY, 0644)) == -1) { perror("open"); exit(1); }
#else
    if ((out[u] = open (outfilenames[u], O_CREAT | (is_spectra ? O_TRUNC : O_APPEND) | O_WRONLY | O_LARGEFILE, 0644)) == -1) { perror("open"); exit(1); }
#endif
   }
 } // !is_dummy

 char *debugmask;
 if (is_debug) {
  debugmask = new char[MaxString];
  if (!debugmask) { perror("new"); exit (1); }
 }

 // true when one wants to see header info from the first frame
 is_info_shown = false;

 // check correctness of shift_counts
 if (shift_counts >= shot.getSampleSize()) shift_counts = 0;

 off_t newpos = syncpos; // new position for frame reading; this var is used when --sync option is set 

 if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
 // infinite loop
 while (1) {

   if (is_to_search_sync) { 
    if (!get_next_syncword(in, newpos-framesize+size_of_long < 0 ? newpos : newpos-framesize+size_of_long, newpos)) { printf ("failed to search for the next sync word!\n"); exit (1); }
    if ((lseek (in, newpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
   }
   memset (raw_framedata, 0, framesize);
   bytesread = read (in, raw_framedata, framesize);
   if (bytesread == -1) { perror("read raw_framedata"); exit(1); }
   if (bytesread != framesize) {
     if (bytesread > framesize) { printf("wow! we can read more than framesize at once! :)\n"); exit (1); }
     if (bytesread < framesize) { syncpos = framesize - bytesread; if (is_to_search_sync) newpos = framesize - bytesread; }
     next++;
     if (next == argc) break;
     else { unsigned long long *ptr;
            ptr = raw_framedata + bytesread/size_of_long_long;
            if ((close (in)) == -1) { perror("close"); exit(1); };
#ifdef apple
            if ((in = open (argv[next], O_RDONLY)) == -1) { perror("open"); exit(1); };
#else
            if ((in = open (argv[next], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
#endif
            if ((lseek (in, 0, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
            bytesread = read (in, ptr, syncpos);
            if (bytesread == -1) { perror("read ptr"); exit(1); }
            if (bytesread > syncpos) { printf("wow! we can read more than we need at once! :)\n"); exit (1); }
            if (bytesread < syncpos) { printf ("size of file \"%s\" is less than framesize!\n", argv[next]); exit (1); }
	    if (is_check && !is_debug) shot.getSimpleCheckInfo();
	    if (!is_quite) { printf ("\n                 %s %6ld", argv[next], Nframe_processed); fflush (NULL); }
     } // else
   } // bytesread != framesize

   shot.update (raw_framedata, Nframe_processed); // fill class object with new frame data
   
   if (!is_info_shown) { // getting info for zero track (default)
    shot.getHeader(); printf ("\n");
    mjd = getMJD (mjd_t, shot.getSeconds(), (int)strlen(startmjd), mjd);
    printf (" start seconds: %ld   mjd: %s\n", shot.getSeconds(), mjd);
    is_info_shown = true;
    if (!is_quite) { printf (" file processed: %s %6ld", argv[next], Nframe_processed); fflush (NULL); }
   } 
   if (is_debug) {
    shot.getHeader(); 
    if (is_to_search_sync) printf ("   offset = %+ld bytes\n", newpos - syncpos); else printf ("\n");
    memset (debugmask, 0, MaxString);
    sprintf (debugmask, "                 %s%ds       ", "%", (int)strlen(argv[next]));
    printf (debugmask, ""); fflush (NULL);
   }

   if (!is_dummy) { if (!is_spectra) { 
	              if (Nframe_processed == 0) for (int u=0; u<Nchan; ++u) write (out[u], mjd, mjd_header);
	              if (shift_counts != 0 && Nframe_processed == 0) {
			signed char *sptr;
			for (int u=0; u<Nchan; ++u) {
			 sptr = shot.getChannel(u) + shift_counts;
			 write (out[u], sptr, shot.getSampleSize() - shift_counts);
			 if (is_stat) { bs.update (shot, Nframe_processed+1); plot_stat (bs, conf, outname, stat_dev_id); }
			} // for
		      } else { for (int u=0; u<Nchan; ++u) write (out[u], shot.getChannel(u), shot.getSampleSize());
			       if (is_stat) { bs.update (shot, Nframe_processed+1); plot_stat (bs, conf, outname, stat_dev_id); }
		              }
                    } else  { 
		      if (Nframe_processed == 0) for (int u=0; u<Nchan; ++u) SpectraLevel[u] = LevelCalc (shot.getChannel(u), spectrumsize);
		      for (int u=0; u<Nchan; ++u) getSpectrum (spectra[u], shot.getChannel(u), spectrumsize, SpectraLevel[u], out[u], Nframe_processed); 
                      if (is_spectra_show) plot_spectrum (spectra, spectrumsize/2+1, aver, skipharm, outname, dev_id, conf, Nframe_processed+1);
                      if (is_stat) { bs.update (shot, Nframe_processed+1); plot_stat (bs, conf, outname, stat_dev_id); }
		     }
   } // !is_dummy
   Nframe_processed++;
   if (!is_quite) { printf ("\b\b\b\b\b\b%6ld", Nframe_processed); fflush (NULL); }
   if (NumberOfFrames != 0 && NumberOfFrames == Nframe_processed) break;
   if (is_to_search_sync) { syncpos = newpos + framesize; newpos += framesize; }
    else syncpos += framesize;

 } // end of loop

 if ((close (in)) == -1) { perror("close"); exit(1); };
 if (!is_dummy) { for (int u=0; u<Nchan; ++u) if ((close (out[u])) == -1) { perror("close"); exit(1); }
 }

 printf (" - done\n");
 printf ("processed %ld frames (%g secs)\n", Nframe_processed, Nframe_processed * sec_step);

 if (is_check) {
  shot.getCheckInfo();
 }

 if (is_spectra_show && is_spectra && !is_dummy) { cpgslct (dev_id); cpgclos(); }
 if (is_stat && !is_dummy) { cpgslct (stat_dev_id); cpgclos (); }

 if (is_debug) delete (debugmask);
 delete (mjd);
 delete (startmjd);
 mpf_clear (mjd_t);
 if (!is_dummy) delete (out);
 if (!is_dummy) { for (unsigned char i=0; i<Nchan; ++i) { delete (outfilenames[i]); } delete (outfilenames); }
 delete (SpectraLevel);
 delete (raw_framedata);
 delete (label);
 delete (outname);
 delete (outputpath);
 if (is_spectra && !is_dummy) { 
  for (unsigned char i=0; i<Nchan; ++i) delete (spectra[i]);
  delete (spectra); 
  fftwf_destroy_plan (pfor);
  fftwf_free (x);
 }

 return 0;
}
