/*==========================================================================
 *  Some general comment
 *  This program was written to decode observations of pulsars 
 *  with the Kalyazin 64-m radio telescope using K5 data acquisition system
 *  4 or 1 frequency channels of 4-MHz (sampling frequency is 8 MHz per channel)
 *  2bit sampling
 *  chan1 - W
   and next three are in case of 4 frequency channels
 *  chan2 - X
 *  chan3 - Y
 *  chan4 - Z
=============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <math.h>

#include <inttypes.h>
//#include <fftw3.h>

#ifdef MAIN_

#include <getopt.h>

#include <sys/resource.h>
#include "wisdom.hpp"
#include "hdsd.hpp"
#include "stat.hpp"
#include "mjd.hpp"

#include "cpgplot.h"

#define MaxString 255

int in;         // input file descriptor
int *out;       // output file descriptors for recording all channels
char *ext[4] = { ".W", ".X", ".Y", ".Z"};  // file extensions for different channels
float **spectra; // arrays of average power spectra
char *outname;  // basename for output files, default - the same as input file
char **outfilenames;  // full names of files with output data
char *outputpath; // path to the output data
int bytesread;  // how many bytes we have read from the input file
int size_of_long = sizeof(long);  // size of the long type
int size_of_long_long = sizeof(long long);  // size of the long long type
int next;  // counter to the first file to process
bool is_dummy = false;   // true when dummy mode is set
bool is_check = false;   // true when check mode is set
bool is_info_shown = false;  // true when info for first frame is shown
bool is_stat = false; // true when bit statistics is calculating
bool is_debug = false; // true if we want to get full info for every frame
                       // if true, is_check=true and is_dummy=true as well
bool is_ignore = false; // true if we want to ignore time-continuity errors, other errors are ignored by default
bool is_to_search_sync = false;  // true if we want to search sync word after every read frame
bool is_quite = false; // true when one wants "quite" output without changing frames number

bool is_wisdom = false; // true if wisdom file was read successfully
long aver = 1024; // number of points to average the spectrum
long skipharm = 0; // number of harmonics to skip in the spectrum

/*+++ present output info +++*/
char bold[] = "\033[1m"; 
char ul[] = "\033[4m"; 
char none[] = "\033[0m";

/*+++++++++ sync section +++++++++*/
unsigned long syncmask = 0xffffffff; // all ones in 4 bytes
unsigned long syncdata;              // sync record (all units);
off_t syncpos = 0;                   // position of sync record in bytes
bool syncdetect;                     // flag for sync record detection

/*+++++++++ frame section ++++++++*/
unsigned long long *raw_framedata;  // raw array of one frame (HD + 1sec of sampling data section)
size_t framesize;              // size of frame in bytes
unsigned char Nchan;           // number of channels
size_t Nframe_processed = 0;   // the number of processed frames
double sec_step = 1.;          // the step in seconds between frames
size_t NumberOfFrames = 0;     // number of frames to process (0 means all frames)
size_t BeginFrame = 0;         // start to process from this frame
size_t Nframe_skipped = 0;     // number of frames skipped before processing
unsigned long shift_counts = 0; // number of counts to shift in the first frame

bool is_spectra = false;       // calculate average spectra in each channel rather than raw data
bool is_spectra_show = false;  // show calculated spectra
bool swap_bytes = false;       // to swap bytes in input data or not
size_t spectrumsize;           // size of the spectrum

fftwf_complex *x;  // to calculate spectra
fftwf_plan pfor;   // plan variable for FFTW
float *SpectraLevel;  // spectrum average level to eliminate

// high precision value for mjd
mpf_t mjd_t;
char *startmjd;
int mjd_header = 27; // length in bytes for MJD string with radix

/* graphics part */
char pgplot_dev[MaxString] = "/xwindow"; // device for pgplot
int dev_id;  // identifier for default xwindow device
int stat_dev_id; // identifier for xwindow device to plot the bit-statistics
char *label;     // extra label in plots

void plot_spectrum (float **spectra, size_t size, long aver, long skipharm, char *basename, int dev_default, FrameInfo& conf, size_t frames);
void plot_stat (StatInfo& bs, FrameInfo& conf, char *basename, int dev_default);

#endif
