#include "plotspectrum.hpp"
#include "op.hpp"

// get y tics major interval
long getytics (float min, float max) {
 long beg = (long)floor((max - min)/6.);
 long end = (long)floor((max - min)/5.);
 if (beg == 0 && end == 0) end = 1;
 int mult[] = { 1000, 500, 250, 200, 150, 100, 50, 25, 10, 5, 4, 2, 1 };
 for (int j=0; j<13; ++j)
  for (long i=beg; i<=end; ++i) {
   if (((long)((float)i/(float)mult[j]))*mult[j] == i && i!=0) return i;
  }
 return end;
}

// draw to device
void drawing (size_t frames, size_t destsize, long aver, long skipharm, float ymin, float ymax, long major_tic, int ostatok, float xres, float *x, float **m, char *outname, FrameInfo& conf) {

 float samp_freq = conf.getSamplingFreq();
 unsigned char nchan = conf.getNchan();
 cpgeras ();
 cpgsch (char_size);
 cpgsvp (0.07, 0.99, 0.22, 0.97);
 cpgswin (0, x[destsize-1-skipharm], ymin, ymax);
 cpgslw (3);
 cpgbox ("BCNTSI", samp_freq/2./4., 4, "BCNTSI", major_tic, 4);
 /* x-label */ cpgmtxt ("B", 2.7, 0.5, 0.5, xlabel);
 /* y-label */ cpgmtxt ("L", 2.7, 0.5, 0.5, ylabel);
 cpgsitf (0);
 for (unsigned char i=0; i<nchan; ++i) {
  cpgsci (i+1);
  cpgline (destsize-ostatok-skipharm, x, m[i]);
 }
 cpgsci (1);

 /* labels */
 cpgslw (4);
 for (unsigned char i=0; i<nchan; ++i) {
  cpgsci (i+1);
  cpgmove (samp_freq/2. - samp_freq/2./8., ymax - (major_tic/5.)*(i+1)); cpgdraw (samp_freq/2. - samp_freq/2./8.+samp_freq/2./16., ymax - (major_tic/5.)*(i+1));
  cpgptxt (samp_freq/2. - samp_freq/2./8.+samp_freq/2./16., ymax - (major_tic/5.)*(i+1), 0.0, 0.0, i == 0 ? " W" : (i == 1 ? " X" : (i == 2 ? " Y" : " Z")));
 }
 cpgsci (1);
 cpgslw (2);

 char *cur;
 cur = new char[MaxString];
 if (!cur) { perror("new"); exit (1); }
 memset (cur, 0, MaxString);

 cpgmtxt ("B", 4.5, 0., 0., "Power spectrum");
 cpgsch (char_size*0.8);
 sprintf (cur, "%sSampl. freq = %g MHz   Nchan = %d   Nbits = %d", label, samp_freq, (int)nchan, (int)conf.getNbits());
 cpgmtxt ("B", 7., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "Size = %d harmonics   Spectral res. = %g MHz (%d counts)   Skip harmonics = %d   Nframes = %d", destsize - ostatok, xres, aver, skipharm, (int)frames);
 cpgmtxt ("B", 8., 0., 0., cur);
 memset (cur, 0, MaxString);
 int hh = (int)(conf.getSeconds()/3600.);
 int mm = (int)((conf.getSeconds() - hh*3600)/60.);
 int ss = conf.getSeconds() - hh*3600 - mm*60;
 sprintf (cur, "Start: %ld secs   UTC: %02d:%02d:%02d      Max: %g dB   Min: %g dB", conf.getSeconds(), hh, mm, ss, ymax, ymin);
 cpgmtxt ("B", 9., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "File: %s", outname);
 cpgmtxt ("B", 10., 0., 0., cur);
 cpgslw (1);
 cpgiden ();

 delete (cur);
}

/* graphics part */
void plot_spectrum (float **spectra, size_t size, long aver, long skipharm, char *basename, int dev_default, FrameInfo& conf, size_t frames) {

 float samp_freq = conf.getSamplingFreq();
 unsigned char nchan = conf.getNchan();
 if (aver >= size) aver = 1;
 int ostatok = (size%aver == 0 ? 0 : 1);
 size_t destsize = size/aver + ostatok;
 if (skipharm >= destsize-1) skipharm = 0;
	
 float **m;
 m = new float*[nchan];
 if (!m) { perror("new m"); exit (1); }
 for (unsigned char i=0; i<nchan; ++i) {
  m[i] = new float[destsize];
  if (!m[i]) { perror("new m[i]"); exit (1); }
  memset (m[i], 0, destsize * sizeof(float));
 }
 float *x;
 x = new float[destsize];
 if (!x) { perror("new x"); exit (1); }
 memset (x, 0, destsize * sizeof(float));

 // average
 for (unsigned char i=0; i<nchan; ++i) m[i] = average (spectra[i], m[i], size, aver);

 for (unsigned char i=0; i<nchan; ++i)
  for (off_t j=skipharm,k,l=0; j<destsize; ++j, ++l) {
   m[i][l] = 10. * log10f (m[i][j]);
   if (isnan(m[i][j]) || isinf(m[i][j]) || isinf(m[i][j]) == -1) 
    if (j-1>=skipharm) m[i][l] = m[i][j-1]; 
     else { k=j; while (isnan(m[i][++k]) || isinf(m[i][++k]) || isinf(m[i][++k]) == -1); m[i][l] = m[i][k]; }
  }
 float xres = ((samp_freq/2.)/(size-1)) * aver;
 for (off_t j=0,k=skipharm; k<destsize; ++j, ++k) x[j] = k * xres;

 float ymax = getmax (m, nchan, destsize-ostatok-skipharm, 0);
 float ymin = getmin (m, nchan, destsize-ostatok-skipharm, 0);
 long major_tic = getytics (ymin, ymax);

 char *outname;
 outname = new char[MaxString];
 if (!outname) { perror("new outname"); exit (1); }
 memset (outname, 0, MaxString);
 sprintf (outname, "%s.WXYZ.sp2.ps/cps", basename);

 // draw to POSTSCRIPT
 cpgopen (outname);
 drawing (frames, destsize, aver, skipharm, ymin, ymax, major_tic, ostatok, xres, x, m, outname, conf);
 cpgclos ();

 // draw in XWINDOW
 cpgslct (dev_default);
 drawing (frames, destsize, aver, skipharm, ymin, ymax, major_tic, ostatok, xres, x, m, outname, conf);

 for (unsigned char i=0; i<nchan; ++i) delete (m[i]);
 delete (m);
 delete (x);
}
