#include "stat.hpp"
#include "hdsd.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//default constructor
StatInfo::StatInfo () {
 stat = NULL;
}

//constructor
StatInfo::StatInfo (unsigned char nch, unsigned char nbits) {
 Nchan = nch;
 Nbits = nbits;
 Nlevels = 1<<nbits;
 samplesize = 0;
 Nframes = 0;

 // initialize stat array
 stat = new unsigned long*[Nchan];
 if (!stat) { perror("new stat"); exit (1); }
 for (unsigned char i=0; i<Nchan; ++i) {
  stat[i] = new unsigned long[Nlevels];
  if (!stat[i]) { perror("new stat[i]"); exit (1); }
  memset (stat[i], 0, Nlevels*sizeof(unsigned long));
 }
}

// destructor
StatInfo::~StatInfo () {
 if (stat != NULL) {
  for (unsigned char i=0; i<Nchan; ++i) if (stat[i] != NULL) delete (stat[i]);
  if (stat != NULL) delete (stat);
 }
}

//initialize array
void StatInfo::initialize (unsigned char nch, unsigned char nbits) {
 Nchan = nch;
 Nbits = nbits;
 Nlevels = 1<<nbits;
 samplesize = 0;
 Nframes = 0;

 // initialize stat array
 stat = new unsigned long*[Nchan];
 if (!stat) { perror("new stat"); exit (1); }
 for (unsigned char i=0; i<Nchan; ++i) {
  stat[i] = new unsigned long[Nlevels];
  if (!stat[i]) { perror("new stat[i]"); exit (1); }
  memset (stat[i], 0, Nlevels*sizeof(unsigned long));
 }
}

// update stat array
void StatInfo::update (SampleData& shot, size_t frames) {
 Nframes = frames;
 samplesize = shot.getSampleSize();
 unsigned char cod = shot.getCoding(Nbits);
 signed char *ptr;
 signed char val;
 for (unsigned char i=0; i<Nchan; ++i) {
  ptr = shot.getChannel (i);  
  for (unsigned int k=0; k<Nlevels; ++k) {
   val = 2*k - cod;
   for (off_t j=0; j<samplesize; ++j) {
    if (ptr[j] == val) stat[i][k]++;
   } // for array
  } // for Nlevels
 } // for chan
}

// get statistics for selected channel
unsigned long* StatInfo::getChanStat (int ch) {
 if (ch < 0 || ch > 3) {
   printf ("Bad channel number!\n");
   exit (1);
 }
 return stat[ch];
}

// get statistics for selected channel in %
float StatInfo::getChanStatPercent (int ch, unsigned int level) {
 if (ch < 0 || ch > 3) {
   printf ("Bad channel number!\n");
   exit (1);
 }
 if (level < 0 || level > Nlevels-1) {
   printf ("Bad level-index number!\n");
   exit (1);
 }
 float res = 100.*((float)stat[ch][level])/(Nframes * (float)samplesize);
 return res;
}
