/*******************************************************************/
/*                                                                 */
/*   COMMON header file for IP-VLBI utilities     chead1           */
/*                                                                 */
/*      Version:  1.00  2003-07-21                                 */
/*                                                                 */
/*******************************************************************/
#ifndef _IP_CHEAD_
#define _IP_CHEAD_

#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#ifdef WIN32
//define gettimeofday to keep source code compatibility with linux/FreeBSD 
#ifndef _WINSOCKAPI_    /* to avoid duplicate include (winsock.h also include */
                        /*    struct timeval definition */
struct timeval {          
        long    tv_sec;         /* seconds */
        long    tv_usec;        /* and microseconds */
};
#endif
struct timezone {
        int    tz_minuteswest; /* minutes W of Greenwich */
        int    tz_dsttime;     /* type of dst correction */
};
int gettimeofday(struct timeval *tv, struct timezone *tz)
{
     struct tm;
     time_t tnow;
     tnow=time(NULL);
     tv->tv_sec=(long)tnow;
     tv->tv_usec=0;
     tz->tz_minuteswest=0;
     tz->tz_dsttime=0;
     return 0;
}
#endif

/* common functions */
int cmnd_get(int argc, char *argv[], char *command, char *param);
char *select_env_dflt(char *env, char *dflt);
void parse_dir_and_name_fm_fpath(char *fpath, char *dir, char *fname); 
int getenv_from_path(char *path, char *env, int nth);
int substitute_env_path_name(char *path, char *fpath);
void get_true_path(char *path, char *dfltenv, char *dfltdir2, char *fpath);

int cmnd_get(int argc, char *argv[], char *command, char *param)
// get argument like -command parameter
// search "-command" in argument, if it is exsted, 0 is returned as
// a function value, and next argument is returned as a parameter (ascii).
// if "-command" is not exsited, -1 is returned as a function value.
// if "-command" is not accompanied by real parameter, NULL character
// is returned as param.
// 
{
    int n;
    char cbuf[80];

    param[0]='\0';

    //search command
    //printf("%s\n",command); // debug
    n=2;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        if(strncmp(cbuf,command,strlen(command))==0){  
            n++;
            if(n > argc) return 0;
            strcpy(param,*(argv+n-1));
            return 0;
        }
        n++;
    }
    return -1;
}

char *select_env_dflt(char *env, char *dflt)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*   Select enviromental variable or dfault                                               */
/*                                                                                        */
/*    IN                                                                                  */
/*        env --  environment variable                                                    */
/*        dflt--  default                                                                 */
/*    RETURN                                                                              */
/*        ret --  real value of env if env has real value, otherwise dflt                 */
/******************************************************************************************/
{
    char *p;
    /* default apriori file directory set */
    if(getenv(env) != NULL){
       p=getenv(env);
    } else {
       p=dflt;
    }
    return p;
}

void parse_dir_and_name_fm_fpath(char *fpath, char *dir, char *fname) 
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*   Parse fpath to directpory portion and file name portion                              */
/*                                                                                        */
/*    IN                                                                                  */
/*        fpath --  given path name                                                       */
/*    RETURN                                                                              */
/*        dir   --  directory portion                                                     */
/*                             -1 for no environment found                                */
/*        fname --  file name                                                             */
/******************************************************************************************/
{
    int len,i,k;
    char cbuf[128];
    len=strlen(fpath);
    // converting '\' to '/' if so
    for(i=0; i<len; i++){
       if(fpath[i]=='\\'){
            cbuf[i]='/';
       } else {
            cbuf[i]=fpath[i];
       }
    }
    cbuf[len]='\0';
    k=-1;
    for (i=len-1; i>=0; i--){
        if(cbuf[i] == '/') {
            k=i;
            break;
        }
    }
    //printf("debug %d %s\n",k,cbuf);
    if(k<0){
        dir[0]='\0';
        strcpy(fname,cbuf);
    } else if (k == len-1){
        strcpy(dir,cbuf);
        fname[0]='\0';
    } else {
        strncpy(dir,cbuf,k+1);
        dir[k+1]='\0';
        strcpy(fname,cbuf+k+1);
    }
    //printf("debug %d %s\n",k,dir);
}

int getenv_from_path(char *path, char *env, int nth)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*   Get enviroment variables in 'path'                                                   */
/*      ex.  $ABCD/cout.txt  or  /ad4/$ABCD/$XYZ/cout.txt                                 */
/*            ABCD and XYZ is extracted then substituted actual enviroment variable       */
/*    IN                                                                                  */
/*        path  --  given path name (may include any environment variables)               */
/*        nth   --  set pick up environment variable by order number                      */
/*                     eg.    $ABCD/$XYZ/fname                                            */
/*                            nth=1 for ABCD  and nth=2 for XYZ                           */
/*    RETURN                                                                              */
/*        getenv_from_path -- number of characters for enviroment variable                */
/*                             -1 for no environment found                                */
/*        env     --  environment variable found                                          */
/******************************************************************************************/
{
    int i,k,j,len,nkai;
    env[0]='\0';
    nkai=0;
    //found $
    len=strlen(path);
    for(i=0; i<len; i++){
        if(path[i]=='$') {
           nkai++;
           if(nkai==nth) break;
        }
    }
    if(i==len) return -1;
    j=0;
    for(k=i+1; k<len; k++){
        if(path[k]=='/' || path[k]=='\\') break;
        env[j]=path[k];
        j++;
    }
    env[j]='\0';
    return j-1;
}


int substitute_env_path_name(char *path, char *fpath)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*   subsitute all enviroment variables in 'path' by real ones                            */
/*      ex.  $ABCD/cout.txt  or  /ad4/$ABCD/$XYZ/cout.txt                                 */
/*            ABCD and XYZ is extracted then substituted actual enviroment variable       */
/*    IN                                                                                  */
/*        path  --  given path name (may include any environment variable)                */
/*                     eg.    $ABCD/fname   or fname                                      */
/*    RETURN                                                                              */
/*        fpath   --  real path                                                           */
/*        substitute_env_path_name -- number of environment variables found               */
/******************************************************************************************/
{
    int i,k,j,len,p1,p2, nenv;
    char cbuf[100], env[50], tdir[80];
    strcpy(fpath,path);
    strcpy(cbuf,path);
    nenv=0;
    while(1){
       env[0]='\0';
       //found $
       len=strlen(cbuf);
       for(i=0; i<len; i++){
           if(cbuf[i]=='$') {
               p1=i;  //  start pointer
               nenv++;
               break;
           }
       }
       if(i==len) goto exit1; // nothing to do
       j=0;
       for(k=i+1; k<len; k++){
          if(cbuf[k]=='/' || cbuf[k]=='\\') {
             p2=k;
             break;
          }
          env[j]=cbuf[k];
          j++;
       }
       env[j]='\0';
       tdir[0]='\0';
       if(j>0){   // get environment
          if(getenv(env) != NULL){
             strcpy(tdir,getenv(env));
             //printf("substitute_env_in_path_name:\n");
             //printf("       \'%s\' is substituted by \'%s\'\n",env,tdir);
          //} else {
             //printf("substitute_env_in_path_name: Warning!!\n");
             //printf("       Env variable \'%s\' has no value\n",env);
          }
       }
       //insert tdir
       strncpy(fpath,cbuf,p1);
       fpath[p1]='\0';
       strcat(fpath,tdir);
       strcat(fpath,cbuf+p2);
       strcpy(cbuf,fpath);
    }
   // 
exit1:;
    return nenv;
}

void get_true_path(char *path, char *dfltenv, char *dfltdir2, char *fpath)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*   subsitute all enviroment variables in 'path' by real ones                            */
/*      ex.  $ABCD/cout.txt  or  /ad4/$ABCD/$XYZ/cout.txt                                 */
/*            ABCD and XYZ is extracted then substituted actual enviroment variable       */
/*    IN                                                                                  */
/*        path  --  given path name (may include any environment variable)                */
/*                     eg.    $ABCD/fname   or fname                                      */
/*        dfltenv --  environment variable for default path when no path (directory       */
/*                     portion) is accompnied by 'path'.                                  */
/*                     Note: this parameter should not start with '$'.                    */
/*        dfltdir2 --  if dfltenv is null, dfltdir2 is applied instead of dfltenv.        */
/*                     This can be enviroment varialble. In this case 1st caharacter      */
/*                     should be '$'                                                      */
/*    RETURN                                                                              */
/*        fpath   --  real path                                                           */
/*                                                                                        */
/*    examples                                                                            */
/*     Case A      path = 'fname',  dfltenv = ABC,  dfltdir2 = 'xyz'                      */
/*                1st environment variable ABC is checked and it has a value (ex.abc),    */
/*                fpath will be 'abc/fname'. If ABC is null, then dfltdir2 (xyz) is       */
/*                applied (fpath will be (xyz/abc)).  Then further environment variable   */
/*                check is carried out.                                                   */
/*     Case B      path = '$TETSU/$YOKO/$POLO/fname',  dfltenv = ABC,  dfltdir2 = 'xyz'   */
/*                1st environment variable TETSU is substituted real one,                 */
/*                next environment variable YOKO is substituted real one,                 */
/*                then environment variable POLO is substituted real one.                 */
/*                                                                                        */
/******************************************************************************************/
{
    int len,nenv;
    char dir[80],fname[80],cbuf[100];

    strcpy(fpath,path);

    /* check file name is acompanied by directory or not */
    parse_dir_and_name_fm_fpath(path, dir, fname);
    if(dir[0]=='\0'){    // then no directory is accompnied
                              //  add directory
        if(getenv(dfltenv) != NULL){
            strcpy(dir,getenv(dfltenv));
        } else {
            strcpy(dir,"./");   //current directory
            strcpy(dir,dfltdir2);   //current directory
        } 
        len=strlen(dir);
        if(dir[len-1] != '/' && dir[len-1] != '\\'){
           dir[len]='/';
           dir[len+1]='\0';
        } 
        strcpy(fpath,dir);
        strcat(fpath,fname);
    } 
    /* check file name include environment variables */
    if((nenv=substitute_env_path_name(fpath, cbuf)) > 0){
         strcpy(fpath,cbuf);
    } 
}
#endif
