/**************************************************************/
/*                                                            */
/*   File name :  libsked.h                                   */
/*                                                            */
/*      Version:   2004-06-17                                 */
/*                                                            */
/*   Copyright (c) 2003-2004 T.Kondo/NICT All Right Reserved  */
/*                                                            */
/**************************************************************/
/* 

2004-06-17
   vex2sked2 was modified a bit

2003-09-17
   'skedlineana' was modified to allow the case when scan duration by 
   station was omitted in $SKED field
   If scan duration by station is omitted, max duration s set for all 
   stations

2003-08-08
   A portion of chedule file read and analysis was extracted from 
   libapri-a.h on 2003-08-08

*/
#ifndef _LIBSKED_
#define _LIBSKED_

#include <math.h>
#include <ctype.h>

#ifndef CONVA
   #define CONVA 4.848136811E-6    // PI*3600/180 arcsec to radian
#endif
#ifndef CONVS
   #define CONVS 7.272205216E-5    // PI/(3600*12) time sec to radian
#endif
#ifndef CONVD
   #define CONVD 1.745329252E-2    // PI/180  degree to radian
#endif
#ifndef CONVH
   #define CONVH 2.617993878E-1    // PI/12 hour to radian
#endif
#ifndef PIRA
   #define PIRA  1.745329252E-2    // PI/180  degree to radian
#endif
#ifndef PIDE
   #define PIDE  57.29577951       // 180/PI radian to degree
#endif
#ifndef TWOPI
   #define TWOPI 6.283185307       // 2PI
#endif
#ifndef PI
   #define PI    3.141592653
#endif
#ifndef VLIGHT
   #define VLIGHT 2.99792458E+8    // speed of light (m/sec)
#endif

#ifndef MAXSTAR
   #define MAXSTAR 1500            // Maximum star numbers in schedule file
#endif

#ifndef MAXOBS
   #define MAXOBS 2000            // Maximum observation (scan) numbers in schedule file
#endif

#ifndef MAXSTAT
   #define MAXSTAT 20          // Maximum stations numbers in schedule file
#endif

#ifdef WIN32
  typedef unsigned int    u_int32_t;
#endif


#ifndef _LIBVEX_
#include "libvex.h"
#endif

extern void ddd2md(int year, int ddd, int* month, int* mday);

    /* data structure for SKED (DRUDG) file data */
typedef struct schedule
{
     char expcode[11];           // experiment code
     char starnam1[MAXSTAR][11]; // star name1
     char starnam2[MAXSTAR][11]; // star name2
     double ra[MAXSTAR];         // star position RA  in degree
     double dec[MAXSTAR];        //       "       DEC in degree
     double epo[MAXSTAR];        //       "       Epoch
     int    nstar;               // number of stars 

     char statnam[MAXSTAT][11];       // station name
     double xyz[MAXSTAT][3];          // station position (x,y,z) in m
     char statid[MAXSTAT][3];         // 1 char or 2 char station ID 
     char statid2[MAXSTAT][3];         // 1 char or 2 char station ID 
     int nstation;               // number of stations

     double freqtab[16];         // RF frequency table (Hz)
     double pcalf[10][16];      // PCAL frequency table (Hz)  modified on 2003-07-18
                                //  up to 10 th PCAL frequency
     int nfreq;                  // number of frequency
     // scan info
     char obsstar[MAXOBS][11];   // Observation sourcer name
     int starid[MAXOBS];         // Star ID  (entry at starpos table)
     int start[MAXOBS][5];       // start time   YY, DDD, HH, MM, SS
     int dura[MAXOBS];           // obs duration (sec)
     char obsid[MAXOBS][21];     // station IDs participated in this obs
     int nobs;                   // Number of observations (scans)
     char skdname[61];           // schedule file name
     // following is added on 2003-03-21 to handle VEX file (not implemented yet)
     char skedtype[10];          // sked type SKED|VEX|OVEX
     //double x_clock_offset;

     /* followings are added  to contain side band info and subnet info */ //2003-07-17
     char sideband[16][4];       // side band  U|L or USB|LSB
     int datastart[MAXOBS][MAXSTAT];   // data start time offset 
                                 //   from obs start time (sec) 
     int datastop[MAXOBS][MAXSTAT];    // data stop  time offset from obs start time (sec)
     int station_obs[MAXOBS][MAXSTAT]; // station participated in scan info (1: obs  0: no obs)
     
} sked_data_t;

typedef struct skedline  // structure for each scan data
{
    char source[10];   // radio source
    int start[5];      // start time  YY,DDD,HH,MM,SS
    int dura;          // max scan length in second
    int nstat;         // participated stations in this scan
    char id_stat[MAXSTAT][3]; // participated station id
    int dura_stat[MAXSTAT];   // scan length by station
} skedline_t;

void SkdMonit(sked_data_t *skd);
void stcode_get(char *stcode_file, char *sid2, char *sid1);
void vex2sked(vex_t *vex, char* stcode_file, sked_data_t *skd);
void vex2sked2(vex_t *vex, sked_data_t *skd);
void skedlineana(char *line, skedline_t *skl);
void readskd(sked_data_t *skd);   
void readskd_for_nozomi(char *skdname, char *baseid, char *apefile, int *iok);



/* Monitor schedule file */
void SkdMonit(sked_data_t *skd)  //version 2003-07-18
{
    int i,n,k,gmax;
    int year,ddd,hh,mm,ss,month,mday;
	printf("SkdMonit: *********** Schedule file monitor **********\n");
	printf("SkdMonit: Schedule file = %s\n",skd->skdname);
	printf("SkdMonit: Expcode       = %s\n",skd->expcode);
	printf("SkdMonit: Total Scan #  = %d\n",skd->nobs);
	printf("SkdMonit: Total Star #  = %d\n",skd->nstar);
	printf("SkdMonit:\n");
    /* 1st and Last Observation Time Print */
	for (i=0; i<2; i++){
		if (i==0){
			  k=0;
			  printf("SkdMonit:  1st   Scan   = ");
		} else {
              k=skd->nobs-1; 
			  printf("SkdMonit:  Last  Scan   = ");
		}
        year=skd->start[k][0]+2000;
        ddd=skd->start[k][1];
        hh=skd->start[k][2];
        mm=skd->start[k][3];
        ss=skd->start[k][4];
        ddd2md(year,ddd,&month,&mday);
        printf("%04d/%02d/%02d %02d:%02d:%02d  %s\n",
			  year,month,mday,hh,mm,ss,skd->obsstar[k]);
    } 
	printf("SkdMonit:\n");
    printf("SkdMonit: ---------- Station ID Table ----------\n");
    for (i=0; i<skd->nstation ; i++){
          //printf("SkdMonit:  %s %s --- %s\n",skd->statid[i],skd->statid2[i],
          printf("SkdMonit:  %s --- %s\n",skd->statid[i],
              skd->statnam[i]);
    }
    if((strncmp(skd->skedtype,"VEX",3)==0)||
        (strncmp(skd->skedtype,"OVEX",4)==0)){
         printf("SkdMonit: (Note that 1 char station ID is not defined in a VEX file)\n");
         //printf("SkdMonit: (so 1 char station ID is generated by stcode_get function)\n");
    }
    printf("SkdMonit: --------------------------------------\n");
/*
    printf("SkdMonit: ---------- Frequency Table -----------\n");
    printf("SkdMonit:   Gr# CH#  RF Freq(MHz)  PCAL Freq(Hz)\n");

    for(n=0; n<skd->nfreq; n++){           
        printf("SkdMonit:  %3d %3d   %10.2f     %8.1f\n",n/4+1,n+1,
                skd->freqtab[n],skd->pcalf[n]);
    }
    printf("SkdMonit: --------------------------------------\n");
*/
    gmax=(skd->nfreq-1)/4+1;
    printf("SkdMonit: ------- Frequency (MHz) Table --------\n");
    for(i=0; i<gmax; i++){
        printf("SkdMonit: Gr# %d ",i+1);
        for(k=0; k<4; k++){
            n=4*i+k;
            if(n >= skd->nfreq) break;
            printf(" %.2f %s",skd->freqtab[n],skd->sideband[n]);
        }
        printf("\n");
    }
    printf("SkdMonit: --------------------------------------\n");
    printf("SkdMonit: ------- PCAL Freq (kHz) Table ---------\n");
    for(i=0; i<gmax; i++){
        printf("SkdMonit: Gr# %d ",i+1);
        for(k=0; k<4; k++){
            n=4*i+k;
            if(n >= skd->nfreq) break;
            printf(" %.1f",skd->pcalf[0][n]/1.0e3);
        }
        printf("\n");
    }
    printf("SkdMonit: --------------------------------------\n");
    printf("SkdMonit: ------- SCAN TABLE ---------\n");

}

void stcode_get(char *stcode_file, char *sid2, char *sid1)  // no more use
{     
/*   2003-07-13 minor update
       obtain 1char station code from 2 chr station code
    [in]
      stcode_file  -- station code file name ex. "stcode.txt"
      sid2         -- Two character station code
    [return]
      sid1         -- 1char station code accompanied by '\0' 
                   -- if No code found "-" returned
*/
    FILE *fin;
    char line[60],s2[10],s1[10];
    int k;

    strcpy(sid1,"-");

    if( (fin  = fopen(stcode_file, "r" )) == NULL ){
        printf("stcode_get: File (%s) not found or Open Error \n",stcode_file);
        printf("stcode_get:   Station code encode from 2 char to 1 char is not made for %s\n",sid2); //2003-07-13
        printf("stcode_get:   So 2 char code will be used for further processing\n"); //2003-07-13
        goto exit2return;
    } 

    while(fgets(line, 60, fin ) != NULL) {
        if(line[0] != '*'){
            k=sscanf(line, "%*s %s %s", s2,s1);
            //printf("stcode_get: %s %s\n",s2,s1);
            if(strncmp(sid2,s2,strlen(s2))==0){
                strcpy(sid1,s1);
                break;
            }
        }
    }
    k=fclose(fin);
exit2return:;
}

void vex2sked(vex_t *vex, char* stcode_file, sked_data_t *skd)
{
    /*
      VEX structure to SKED structure data converter

      PCAL frequency of 10kHz is always supposed
        2003-03-21

     [in]
       vex --  vex items
       stcode_file -- file name contain 2char station code to 1char station code
     [return]
       skd --  sked items

    */
    int i,n,k,m;
    int datastart, datastop, duramax;
    site_t site;
    //track_freq_t trfreq;

    strcpy(skd->skdname,vex->vex_file_name);
    strcpy(skd->expcode,vex->exper[0].exper_name);

    // station info set up

    skd->nstation=vex->nstation;
    for (n=0; n<vex->nstation ; n++){
        // find site info
        if(search_site(vex,vex->station[n].def,&site)==0){
            strcpy(skd->statid2[n], site.id);
            //strncpy(skd->statid[n], site.id,1);  // station ID truncated to 1 char
            stcode_get(stcode_file, site.id, skd->statid[n]); //2003-03-24
            //skd->statid[n][0]='\0';  //  for not to define 1 char station ID
            //skd->statid[n][1]='\0';
            strcpy(skd->statnam[n],site.name);
            for(i=0; i<3; i++){
                skd->xyz[n][i]=site.position[i];
            }
        } else {
            skd->statid2[n][0]='\0';
            skd->statnam[n][0]='\0';
            for(i=0; i<3; i++){
                skd->xyz[n][i]=0.0;
            }
        }
    }

    // scan info set up
    skd->nobs=vex->numobs;
    for(n=0; n<vex->numobs; n++){
        for(i=0; i<5; i++){
            skd->start[n][i]=vex->scan[n].start[i];
        }
        skd->start[n][0]-=2000;   // definition in SKED file is 1 digit year
        strcpy(skd->obsstar[n],vex->scan[n].source);

//printf("OBS# %d  numstat %d  skdnstation  %d\n",n+1, vex->scan[n].numstat, skd->nstation);
        for(k=0; k<skd->nstation; k++){
            datastart=0; datastop=0; duramax=0;

            for(m=0; m< vex->scan[n].numstat; m++){
//printf("m= %d  %s  ",m,vex->scan[n].station[m].id);
                if(strncmp(vex->scan[n].station[m].id,skd->statid2[k],strlen(skd->statid2[k]))==0){
                    datastart=vex->scan[n].station[m].data_start;
                    datastop=vex->scan[n].station[m].data_stop;
//printf("datastart datastop %s %d %d\n",vex->scan[n].station[m].id,datastart,datastop);
                    break;
                }
            }
            skd->datastart[n][k]=datastart;
            skd->datastop[n][k] =datastop;
            if(datastop > duramax) duramax=datastop;
        }
        skd->dura[n]=duramax;  
    }

    //source info
    skd->nstar=vex->numstar;
    for(n=0; n<vex->numstar; n++){
        strcpy(skd->starnam1[n],vex->source[n].name);
        strcpy(skd->starnam2[n],vex->source[n].IAU_name);
        skd->dec[n]=vex->source[n].dec;
        skd->ra[n]=vex->source[n].ra;
        if(strncmp(vex->source[n].epoch,"J2000",5)==0){
           skd->epo[n]=2000.0;
        } else {
           skd->epo[n]=1950.0;
        }
    }

    //Frequency table temporaly set-up by using the 1st freq definition

    for(n=0; n<vex->freq_def[0].numch; n++){
        skd->freqtab[n]=vex->freq_def[0].chan_def[n].rf/1.0e6;  // MHz unit in SKED
        skd->pcalf[0][n]=10000.0;
    }
    skd->nfreq=vex->freq_def[0].numch;

}

void vex2sked2(vex_t *vex, sked_data_t *skd)
// 2004-06-17
{
    /*
      VEX structure to SKED structure data converter


     [in]
       vex --  vex items
     [return]
       skd --  sked items

    */
    int i,n,k,m,iok;
    int datastart, datastop, duramax;
    char mode[50], sid[4];
    char cbuf[50];
    char wmode[20][50];
    int nmode;
    int modemax=20;
    site_t site;
    freq_info_t freq_info;
    //track_freq_t trfreq;

    strcpy(skd->skdname,vex->vex_file_name);
    strcpy(skd->expcode,vex->exper[0].exper_name);

    // station info set up

    skd->nstation=vex->nstation;
    for (n=0; n<vex->nstation ; n++){
        // find site info
        if(search_site(vex,vex->station[n].def,&site)==0){
            strcpy(skd->statid[n], site.id);
            strcpy(skd->statnam[n],site.name);
            for(i=0; i<3; i++){
                skd->xyz[n][i]=site.position[i];
            }
        } else {
            skd->statid[n][0]='\0';
            skd->statnam[n][0]='\0';
            for(i=0; i<3; i++){
                skd->xyz[n][i]=0.0;
            }
        }
    }

    // check mode included in all scan
    nmode=0;
    if(vex->numobs <=0 ){  // in case of no scan
                           // 1st mode in $MODE definition is used
        printf("vex2sked2: Warning! No scan is included in VEX file.\n");
        strcpy(mode,vex->mode[0].def);
    } else {
        strcpy(mode,vex->scan[0].mode);
        strcpy(wmode[nmode],mode);
        nmode++;
        for(n=1; n<vex->numobs; n++){
            strcpy(cbuf,vex->scan[n].mode);
            iok=0;
            for(i=0; i<nmode; i++){
                if(strncmp(wmode[i],cbuf,strlen(cbuf))==0){
                    iok=-1;
                    break;
                }
            }
            if(iok ==0){
                strcpy(wmode[nmode],cbuf);
                nmode++;
            }
        }
        if(nmode > 1){ // 
            printf("vex2sked2: Warning! %d different MODEs are found in scans.\n",nmode);
            printf("vex2sked2:   1st mode (%s) will be used.\n",mode);
        }
    }
    // scan info set up
    skd->nobs=vex->numobs;
    for(n=0; n<vex->numobs; n++){
        for(i=0; i<5; i++){
            skd->start[n][i]=vex->scan[n].start[i];
        }
        skd->start[n][0]-=2000;   // definition in SKED file is 1 digit year
        strcpy(skd->obsstar[n],vex->scan[n].source);
        duramax=0;        
        skd->dura[n]=duramax;  
        for(k=0; k<skd->nstation; k++){
            datastart=0; datastop=0; 
            skd->datastart[n][k]=datastart;
            skd->datastop[n][k] =datastop;
            skd->station_obs[n][k]=0;
            for(m=0; m< vex->scan[n].numstat; m++){
                if(strncmp(vex->scan[n].station[m].id,skd->statid[k],strlen(skd->statid[k]))==0){
                    datastart=vex->scan[n].station[m].data_start;
                    datastop=vex->scan[n].station[m].data_stop;
                    skd->datastart[n][k]=datastart;
                    skd->datastop[n][k] =datastop;
                    if(datastop > duramax) duramax=datastop;
                    if(datastop > 0) skd->station_obs[n][k]=1;
                    break;
                }
            }
        }
        skd->dura[n]=duramax;  
    }

    //source info
    skd->nstar=vex->numstar;
    for(n=0; n<vex->numstar; n++){
        strcpy(skd->starnam1[n],vex->source[n].name);
        strcpy(skd->starnam2[n],vex->source[n].IAU_name);
        skd->dec[n]=vex->source[n].dec;
        skd->ra[n]=vex->source[n].ra;
        if(strncmp(vex->source[n].epoch,"J2000",5)==0){
           skd->epo[n]=2000.0;
        } else {
           skd->epo[n]=1950.0;
        }
    }

    //Frequency table set-up using 1st site id in skd->statid table
    strcpy(sid,skd->statid[0]);

    for(n=0; n<16; n++) {    /* 2004-06-17 add initialize*/
        skd->freqtab[n]=0.0;
        //skd->sideband[n][0]='\0';
        strcpy(skd->sideband[n],"U");  /* default */
        for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
    }

    freq_info=vex_chinfo_get(vex,mode,sid,1);
    skd->nfreq=freq_info.chmax;
//printf("Chmax %d\n",skd->nfreq);
    for(n=0; n< skd->nfreq; n++){
        freq_info=vex_chinfo_get(vex,mode,sid,n+1);
        skd->freqtab[n]=freq_info.rf/1.0e6;  // MHz unit in SKED
        for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
        //printf("freq_info.npcal  %d\n",freq_info.npcal);
        for(i=0; i<freq_info.npcal; i++){
           skd->pcalf[i][n]=freq_info.pcalf[i]; 
           //printf("skd->pcalf  %d\n",skd->pcalf[i][n]);
        }
        strcpy(skd->sideband[n],freq_info.sideband);
    }

}

/*
void skedlineana(char *line, skedline_t *skl)  // 2003-08-08
{
    int len,i,k,j;
    char cbuf[60], s1[21], fmt0[400], fmt[400];
    // get source info
    k=sscanf(line,"%s",skl->source);
    // get start time and max scan length
    k=sscanf(line, "%*s %*s %*s %*s %s %d", cbuf,&skl->dura); 
    strncpy(s1,cbuf,2);    s1[2]='\0';    //YY
    skl->start[0]=atoi(s1);   
    strncpy(s1,cbuf+2,3);  s1[3]='\0';    //DDD
    skl->start[1]=atoi(s1);   
    strncpy(s1,cbuf+5,2);  s1[2]='\0';    //HH
    skl->start[2]=atoi(s1);   
    strncpy(s1,cbuf+7,2);  s1[2]='\0';    //MM
    skl->start[3]=atoi(s1);   
    strncpy(s1,cbuf+9,2);  s1[2]='\0';    //SS
    skl->start[4]=atoi(s1);
    // participated station id get
    k=sscanf(line, "%*s %*s %*s %*s %*s %*s %*s %*s %*s %s", cbuf); 
    len=strlen(cbuf);
    skl->nstat=len/2;
    for(i=0; i<skl->nstat; i++){
       strncpy(skl->id_stat[i],cbuf+2*i,1);      //1character id is supposed
       skl->id_stat[i][1]='\0';
    }
    // scan length by station get
    strcpy(fmt,"%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s");
    for(i=0; i<skl->nstat; i++){  // to skip footage field  
        strcat(fmt," %*s");
    }
    strcat(fmt," %*s");    // to skip prflags field
    strcpy(fmt0,fmt);
    for(i=0; i<skl->nstat; i++){ 
        strcpy(fmt,fmt0);
        for(j=0; j<i; j++) strcat(fmt," %*s");
        strcat(fmt," %s");
        k=sscanf(line, fmt, s1);
        skl->dura_stat[i]=atoi(s1);
    }
}
*/

void skedlineana(char *line, skedline_t *skl)  // debug on 2003-09-17
{
    char abuf[80];
    char *cset=" ";
    char *p=NULL;
    int i,k,j,len;
    char cbuf[60], pid[60], s1[21];

    cbuf[0]='\0';
    pid[0]='\0';
    skl->source[0]='\0';
    skl->dura=0;
    for(i=0; i<5; i++) skl->start[i]=0;
    skl->nstat=0;
    for(i=0; i<MAXSTAT; i++) skl->dura_stat[i]=0;
    for(i=0; i<MAXSTAT; i++) skl->id_stat[i][0]='\0';
   
    k=0;
    sprintf(abuf,"%s\0",strtok(line, cset));
    strcpy(skl->source,abuf);  // source info
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s\0",p);
        if(k==4) strcpy(cbuf,abuf);
        if(k==5) skl->dura=atoi(abuf);
        if(k==9) strcpy(pid,abuf);
        k++;
        if(k==10) break;
    }
    if(k > 4) {
       strncpy(s1,cbuf,2);    s1[2]='\0';    //YY
       skl->start[0]=atoi(s1);   
       strncpy(s1,cbuf+2,3);  s1[3]='\0';    //DDD
       skl->start[1]=atoi(s1);   
       strncpy(s1,cbuf+5,2);  s1[2]='\0';    //HH
       skl->start[2]=atoi(s1);   
       strncpy(s1,cbuf+7,2);  s1[2]='\0';    //MM
       skl->start[3]=atoi(s1);   
       strncpy(s1,cbuf+9,2);  s1[2]='\0';    //SS
       skl->start[4]=atoi(s1);
    }
    if(k > 9) {
       // participated station id get
       len=strlen(pid);
       skl->nstat=len/2;
       for(i=0; i<skl->nstat; i++){
          strncpy(skl->id_stat[i],pid+2*i,1);      //1character id is supposed
          skl->id_stat[i][1]='\0';
       } 
    }
    // scan length by station initialize by maximum svcan length 
    //     just in case for length_by_station parameters are omitted 
    for(i=0; i<skl->nstat; i++){
        skl->dura_stat[i]=skl->dura;
    }

    // scan length by station get according to actual parameters if existed
    j=0;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s\0",p);
        //printf("*%d %s\n",k,abuf);
        if(k > 10+ skl->nstat){
            skl->dura_stat[j]=atoi(abuf);
            j++;
        }
        k++;
    }
    //printf("k j nstat %d %d %d\n",k,j,skl->nstat);
}


int readskd_source(FILE *fin, char *line, sked_data_t *skd)
{
    int k;
    int i1,i2,i3,i4,i5;
    double d1,d2,d3,f3;
    char s1[21],s2[21],s3[21],s4[21];
    char starnam[11],altnam[11];
    skd->nstar=0;
             //printf("readskd: $SOURCES Detected! \n");
 loop:
    if( fgets( line, 500, fin ) == NULL) return -1;
	if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
	if(line[0]=='$') return 0;  // this is command line  
    k=sscanf(line, "%s %s %d %d %s %s %d %s %s",    
                   starnam,altnam,&i1,&i2,s1,s3,&i4,s2,s4);     
    d1=atof(s1);
    d2=atof(s2);
	i3=atoi(s3);
    f3=atof(s4);
    //printf("%d %f %f\n",skd->nstar+1,d1,d2);
    strcpy(skd->starnam1[skd->nstar],starnam);
    strcpy(skd->starnam2[skd->nstar],altnam);
    d3=(double)i1*15.0+(double)i2/4.0 + d1/240.0 ;
    skd->ra[skd->nstar]=d3;
    i5=abs(i3);
    d3=(double)i5+(double)i4/60.0 + d2/3600.0 ;
    if (strncmp(s3,"-",1) == 0) d3 = -d3 ;   
    skd->dec[skd->nstar]=d3;
    skd->epo[skd->nstar]=f3;
    skd->nstar++;
    if(skd->nstar >= MAXSTAR) {
        printf("readskd_source: Number of Stars exceeds limit (%d)\n",MAXSTAR);
        skd->nstar -= 1;    // to avoid array overflow
    }
    goto loop;               
}

int readskd_station(FILE *fin, char *line, sked_data_t *skd)
{
    int i,k;
    int idct;
    int flag;
    double d1,d2,d3;
    char s1[21],s2[21],s3[21],cbuf[20],cbuf2[10];
    char wstatnam[MAXSTAT][11];  // station name work
    char wstatid[MAXSTAT][2];    // 1 char station id
    idct=0;
    skd->nstation=0;
    flag=0;
 loop:
    if( fgets( line, 500, fin ) == NULL) {
        flag=-1;
        goto exitend;
    }
	if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
	if(line[0]=='$') goto exitend;  // this is command line  
    if(line[0]=='A') {    // 1 charcter Station ID set
        k=sscanf(line, "%*s %s %s", s1,s2); 
        //printf("readskd_station: ID NAME length %d %d\n",strlen(s1),strlen(s2));
        strcpy(wstatid[idct],s1);
        strcpy(wstatnam[idct],s2);
        idct++;
    }
    if(line[0]=='P'){    // Station XYZ
        k=sscanf(line, "%*s %s %s %s %s %s", cbuf2,cbuf,s1,s2,s3); 
        d1=atof(s1);
        d2=atof(s2);
        d3=atof(s3);
        strcpy(skd->statid2[skd->nstation],cbuf2);  // 2 char station id
        strcpy(skd->statnam[skd->nstation],cbuf);   // station name
        skd->xyz[skd->nstation][0]=d1;
        skd->xyz[skd->nstation][1]=d2;
        skd->xyz[skd->nstation][2]=d3;
        skd->nstation++;
        if(skd->nobs >= MAXSTAT) {
           printf("readskd_station: Number of stations exceeds limit (%d)\n",MAXSTAT);
           skd->nobs -= 1;    // to avoid array overflow
        }
    }
    goto loop;               
exitend:
    /* Station ID table set up */
    for (i=0; i < skd->nstation; i++){
       strcpy(cbuf,skd->statnam[i]);
       for (k=0; k < idct; k++){
           strcpy(s1,wstatnam[k]);
           //printf("len = %d\n",len);
           if (strncmp(cbuf,s1,strlen(s1))==0){
              //strncpy(skd->statid[i],' ',1);
              strcpy(skd->statid[i],wstatid[k]);
              break; 
           } 
       } 
    }
    return flag;
}

int readskd_freq(FILE *fin, char *line, sked_data_t *skd)
{
    int k;
    int i1;
    double d1;
    double pcal,vbw;
    char s1[21],s2[21],s3[21];
    skd->nfreq=0;
 loop:
    if( fgets( line, 500, fin ) == NULL) return -1;
	if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
	if(line[0]=='$') return 0;  // this is command line  
    if(line[0]=='C') {   // Frequency info
       k=sscanf(line, "%*s %*s %*s %s %s %d %*s %s", s1,s2,&i1,s3); 
       pcal=atof(s2);
       vbw=atof(s3);
       //printf("%d %s %f %d %f %d\n",skd->nfreq,s1,pcal,i1,vbw,strlen(s1));
       if(s1[0] != 'x' && s1[0] != 'X') {    // to avoid  xxxx.xx or XXXX.XX
           d1=atof(s1);
           //printf("d1= %f\n",d1);
           skd->freqtab[i1-1]=d1;
           skd->pcalf[0][i1-1]=pcal;   //added on 2003-02-15
           skd->pcalf[1][i1-1]=0.0;   //
           strcpy(skd->sideband[i1-1],"U"); // added on 2003-07-22 (default)
           skd->nfreq++;
       }
    }
    goto loop;               
}

int readskd_sked(FILE *fin, char *line, sked_data_t *skd)
{
    int i,n;
    skedline_t skl;
    skd->nobs=0;
 loop:
    if( fgets( line, 500, fin ) == NULL) return -1;
	if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
	if(line[0]=='$') return 0;  // this is command line  
    for(i=0; i<MAXSTAT; i++){
        skd->station_obs[skd->nobs][i]=0;
        skd->datastart[skd->nobs][i]=0;
        skd->datastop[skd->nobs][i]=0;
    }
    skedlineana(line, &skl);
    for(i=0; i<5; i++) skd->start[skd->nobs][i]=skl.start[i];
    strcpy(skd->obsstar[skd->nobs],skl.source);
    skd->dura[skd->nobs]=skl.dura;
    if(skd->nstation > 0){ //  then already station info set
        for(n=0; n<skl.nstat; n++){
            for(i=0; i<skd->nstation; i++){
                if(skl.id_stat[n][0]==skd->statid[i][0]){  // found id (1char)
                    skd->station_obs[skd->nobs][i]=1;
                    skd->datastop[skd->nobs][i]=skl.dura_stat[n];
                    break;
                }
            }
        }
    }
    skd->nobs++;
    if(skd->nobs >= MAXOBS) {
       printf("readskd_sked: Number of scans exceeds limit (%d)\n",MAXOBS);
       skd->nobs -= 1;    // to avoid array overflow
    }
    goto loop;               
}

void readskd(sked_data_t *skd)  //2003-08-08   
{
    //sked_data_t skd;
    char line[500];
    char cbuf[21];
	char s1[21];
	//char* fname="D:/IPVLBI/data/tds16a.dat"; 
    int i,k;
    int len;
    int flag_sid; // flag to show already STATION field is read or not
    FILE *stream;

	//printf("FILE : %s\n",skd->skdname);
    if( (stream  = fopen(skd->skdname, "r" )) == NULL ){
       printf("readskd: File (%s) not found or Open Error! \n",skd->skdname);
       exit(1);
	   //goto exitend2;
    }
    //printf("File %s opened\n",skd->skdname);

    skd->nstar=0;
    skd->nstation=0;
    skd->nfreq=0;
    skd->nobs=0;
    flag_sid=0;

    /* first step */
    // station info get first to reflect sub-net in each scan info

loop:
     if( fgets( line, 500, stream ) == NULL) goto nextstep;
     /* Schedule File Analysis */    
     if(strncmp(line,"$STATIONS",5)==0){        // $STATIONS block   
        //printf("readskd: $STATIONS Detected! \n");
        i=readskd_station(stream, line, skd);
        goto nextstep;
     }
     goto loop;

nextstep:
     rewind(stream);

loop1:
     if( fgets( line, 500, stream ) == NULL) goto exitend;
     /* Schedule File Analysis */    
	 if(line[0]=='*') goto loop1;  // this is comment line (nothing to do)  
       
     if(line[0]=='$'){    // this is command line  
loop2:
       len = strlen(line);
       if(strncmp(line,"$EXPER",5)==0){        // $EXPER block   
           k=sscanf(line, "%s %s", cbuf, skd->expcode); 
           //printf("readskd: $EXPER Detected! \n");
           //printf("readskd: EXPCODE = %s\n",skd->expcode);
           goto loop1;
       }
       if(strncmp(line,"$PARAM",5)==0) {       // $PARAM block  
           /* not used in IP-VLBI */     
           //printf("readskd: $PARAM Detected! \n");
           goto loop1;
       }
       if(strncmp(line,"$SOURCES",5)==0) {       // $SOURCES block   
          //printf("readskd: $SOURCES Detected! \n");
          if(readskd_source(stream, line, skd) < 0) goto exitend;
          goto loop2;
       }
//       if(strncmp(line,"$STATIONS",5)==0){        // $STATIONS block   
//          //printf("readskd: $STATIONS Detected! \n");
//          if(readskd_station(stream, line, skd) < 0) goto exitend;
//          goto loop2;
//       }
       if(strncmp(line,"$CODES",5)==0){       // $CODES block (FREQ info)   
          //printf("readskd: $CODES Detected! \n");
          if(readskd_freq(stream, line, skd) < 0) goto exitend;
          goto loop2;
       } 
       if(strncmp(line,"$SKED",5)==0){       // $SKED block 
          //printf("readskd: $SKED Detected! \n");
          if(readskd_sked(stream, line, skd) < 0) goto exitend;
          goto loop2;
       } 
       if(strncmp(line,"$HEAD",5)==0){       // head stack information  
           //printf("$HEAD Detected! \n");
           goto loop1;
       }
       if(strncmp(line,"$FLUX",5)==0){       // FLUX info   
           //printf("$FLUX Detected! \n");
           goto loop1;
       } 
       if(strncmp(line,"$PROC",5)==0){       // station procedure info   
           //printf("$PROC Detected! \n");
           goto loop1;
       }
     } 
     goto loop1;
 exitend:
	   i=fclose( stream ) ;
       //printf("%d bytes read\n",pos); 

       //printf("readskd: Number of Sources = %d\n",skd->nstar);
       //printf("readskd: Number of Stations = %d\n",skd->nstation);
       //printf("readskd: Number of Observations = %d\n",skd->nobs);
 
       /* Obs Star Index Table Setup */
       if (skd->nstar > 0) {
           for (i=0; i < skd->nobs; i++){
               strcpy(cbuf,skd->obsstar[i]);
               for (k=0; k < skd->nstar; k++){
                   strcpy(s1,skd->starnam1[k]);
                   len=strlen(s1);
                   //printf("len = %d\n",len);
                   if (strncmp(cbuf,s1,len)==0){
                      skd->starid[i]=k;
                      goto exithere; 
                   }
                   strcpy(s1,skd->starnam2[k]);
                   len=strlen(s1);
                   if (strncmp(cbuf,s1,len)==0){
                      skd->starid[i]=k;
                      goto exithere; 
                   }
               }
               skd->starid[i]=-1;
   exithere: ;
           }
       }
         
/*
       printf("readskd: Number of Observations = %d\n",skd->nobs);
       
       for (i=0; i < 10; i++)
       {
          printf("%d %s %d %d %d %d %d %d   %d %s\n",i+1,skd->obsstar[i],skd->starid[i],
                 skd->start[i][0],skd->start[i][1],skd->start[i][2],
                 skd->start[i][3],skd->start[i][4],skd->dura[i],skd->obsid[i]);
       }
       for (i=skd->nobs-10; i < skd->nobs; i++)
       {
          printf("%d %s %d %d %d %d %d %d   %d %s\n",i+1,skd->obsstar[i],skd->starid[i],
                 skd->start[i][0],skd->start[i][1],skd->start[i][2],
                 skd->start[i][3],skd->start[i][4],skd->dura[i],skd->obsid[i]);
       }
 */      
 //exitend2:

}

void readskd_for_nozomi(char *skdname, char *baseid, char *apefile, int *iok)  
//2003-04-12   to obtain NOZOMI apriori file name
//    iok =-1  not found in schedule file
{
    //sked_data_t skd;
    char line[500];
	char s1[10];
	char s2[80];
	//char* fname="D:/IPVLBI/data/tds16a.dat"; 
    int i,k;
    int len;
	//char timelab[30];
	//int sfreq;
    FILE *stream;

    *iok=-1;
    apefile[0]='\0';

    if( (stream  = fopen(skdname, "r" )) == NULL ){
       printf("readskd_for_nozomi: File (%s) not found or Open Error! \n",skdname);
       exit(1);
	   //goto exitend2;
    }

     loop1:

     if( fgets( line, 500, stream ) == NULL){
       //printf("File EOF or Read Error!\n");
	    goto exitend;
     }
     /* Schedule File Analysis */    
	 if(line[0]=='*') goto loop1;  // this is comment line (nothing to do)  
       
     if(line[0]=='$') {    // this is command line  
          len = strlen(line);
          if(strncmp(line,"$NOZOMI",7)==0) {       // $NOZOMI block   
              //printf("readskd_for_nozomi: $NOZOMI Detected! \n");
  loop_nozomi:
              if( fgets( line, 500, stream ) == NULL) goto exitend;
	          if(line[0]=='*')  goto loop_nozomi;  // this is comment line (nothing to do)  
	          //if(line[0]=='$')  goto loop2;  // this wil be next comment line  
	          if(line[0]=='$') goto exitend;  // this wil be next comment line  
              k=sscanf(line, "%s %s", s1,s2); 
              if(strncmp(baseid,s1,strlen(baseid))==0) {  // baseline info found
                  strcpy(apefile,s2);
                  *iok=0;
                  goto exitend;
              }
              goto loop_nozomi;               
           }
     }
     goto loop1;
 exitend:
	 i=fclose( stream ) ;
}

/* check schedule file type */
void check_sked_type(char* skdname, char* skedtype)  // 2003-03-21
{
    /*
      check schedule file type by reading 1st line
    */
    int i;
    char cbuf[60];
    char atype[][6]={"SKED","VEX","OVEX","OTHER"};

    FILE *fin;
    if((fin  = fopen(skdname, "r" )) == NULL ){
       printf("check_sked_type: File (%s) not found or Open Error! \n",skdname);
       exit(1);
    }
    // 1st line read
    if( fgets(cbuf, 60, fin) == NULL){
        i=fclose(fin);
	    goto exitend;
    }
    i=fclose(fin);
    // first line analysis */  
    strcpy(skedtype,atype[3]); // other type
    if(strncmp(cbuf,"$EXPER",6)==0) strcpy(skedtype,atype[0]);  // original schedule file (SKED) type
	if(strncmp(cbuf,"VEX",3)==0)  strcpy(skedtype,atype[1]);     // VEX
	if(strncmp(cbuf,"$OVEX",5)==0)  strcpy(skedtype,atype[2]);   // OVEX     
exitend:;
}


#endif
