/*******************************************************************/
/*     library for VEX file analysis                               */
/*      by T.Kondo for Mark5 to K5 conversion and others           */
/*                                                                 */
/*        Ver. 1.23  2004-06-29                                    */
/*                                                                 */
/*      Copyright (c) 2003-2004 T.Kondo/NICT All Right Reserved    */
/*******************************************************************/

/*

2004-09-13
    int headstack was added as a member of track_freq_sub structual valiabl.
    head stack info is added in "search_track_freq" function

2004-06-29
    sub_pass info was added in a structure variable track_freq_sub.
    "search_track_freq" function was modified to include sub_pass info
    in track_freq structure variable.

2004-06-17 
    "search_site" : station name was added in No Clock info message.
    "PCAL_ana" :  debug in no pcal case

2003-10-28 
    Algorithm calculating PCAL frequency was updated in 
    "vex_freq_get" function.
    PCAL_ana was modified not to included tone# 0 in tone number table

2003-10-12 
    # of tracks was increased to support 64 track mode
        in  tracks structure and track_freq structure
    TRACKS_ana -- allow omittion of "sub-pass ID" in fanout_def parameters

2003-07-17
    BBC_ana -- $BBC block analysis was added
    BBC_mon -- $BBC block monitor was added
    IF_ana  -- $IF block analysis was added
    IF_mon  -- $IF block monitor was added
    PCAL_ana  -- $PCAL block analysis was added
    PCAL_mon  -- $PCAL block monitor was added

   in "typedef struct track_freq_sub"
     double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
     int npcal;            //       valid number of data in pcalf
     were added as members

2003-07-13  
    eop_get, eop_get2, sa_clock_get, clock_get are added

2003-07-12  debug in "search_track_freq"
2003-07-04  "typedef struct vex"
              the number of $FREQ block is incresed from 5 to 20
              as follows
              //freq_t freq_def[5];        // $FREQ block info
              freq_t freq_def[20];        // $FREQ block info   2003-07-04
*/
#ifndef _LIBVEX_
#define _LIBVEX_

#ifndef MAXSTAR
#define MAXSTAR  1000
#endif
#ifndef MAXSTAT
#define MAXSTAT  20   // maximum number of stations
#endif
#ifndef MAXOBS
#define MAXOBS  2000
#endif

/* primitive block structure for VEX analysis*/

typedef struct global
{
    char exper[30];     // link id to $EXPER block
    char eop[30];       // link id to $EOP block
} global_t;

typedef struct exper
{
    char def[30];         // def id
    char exper_name[20];  // experiment code (name)
} exper_t;

typedef struct mode_item
{
    char mode[30];   // major link id to other block
    char id[20];     // link id to STATION block
} mode_item_t;

typedef struct fanout_def
{
    char sub_pass[2];      // sub pass character (usually A|B )
    char chan_id[20];      // link id to chan_def in $FREQ block
    char stream_info[5];   // sign | mag
    int headstack;         // head stack number
    int track[4];          // track number from first multiplex to last multiplex
    int fanout;            // fanout
} fanout_def_t;
 

typedef struct chan_def
{
    double rf;         // RF frequency (Hz)
    char sideband[2];  // net side band  U|L
    double vbw;        // band width (Hz)
    char chan_id[20];  // Channel ID link to $TRACK
    char bbc_id[20];   // BBC ID  link to $BBC
    char pcal_id[20];  // Pcal ID  link to $PHASE_CAL
} chan_def_t;

typedef struct tracks
{
    char     def[50];            // def id
    char  format[20];            // track frame format ex. Mark4
    int    multiplex;            // multiplex 1|2|4 
    int    bitssampl;            // bits/sampl
    //fanout_def_t fanout_def[32]; // fanout def info (fanout_def type)
    fanout_def_t fanout_def[64]; // fanout def info (fanout_def type) 2003-10-10
    int nfanout_def;             // number of fanout_def
} tracks_t;

typedef struct freq
{
    char   def[50];              // def id
    double samplerate;           // sampling rate (Hz/s)
    int numch;                   // number of channels
    //chan_def_t chan_def[16];     // channel def info (chan_def type)
    chan_def_t chan_def[64];     // channel def info (chan_def type)
} freq_t;

typedef struct if_def            // added on 2003-07-17
{
    char if_id[20];              // IF ID link to $BBC
    char pol[2];                 // Polarization R|L
    double lo;                   // total local frequency (Hz)
    char sideband[2];            // net side band  U|L
    double pcal_space;           // PCAL freq spacing (Hz)
    double pcal_base;            // PCAL freq base (Hz)
} if_def_t;

typedef struct if_block          // added on 2003-07-17
{
    char   def[50];              // def id
    int nif;                   // number of IF definition
    if_def_t if_def[10];         // IF def info (if_def type)
} if_block_t;

typedef struct pcal_detect
{
    char pcal_id[20];     // Pcal id link to $FREQ block
    int ntone;            // number of tones
    int tone[10];  // tone number info for pcal detection
} pcal_detect_t;

typedef struct pcal_block             // added on 2003-07-17
{
    char def[50];                   // def id
    int npcal;                         // number of PCAL definition
    pcal_detect_t pcal[20];         // PCAL def info
} pcal_block_t;

typedef struct bbc          // added on 2003-07-17
{
    char bbc_id[20];     // BBC id link to $FREQ block
    char if_id[20];     // IF id link to $IF block
    int bbcch;        // Physical BBC number
} bbc_t;

typedef struct bbc_block          // added on 2003-07-17
{
    char   def[50];       // def id
    char nbbc;     // number of BBC block
    bbc_t bbc_def[20];    // BBC def 
} bbc_block_t;

typedef struct siteclock
{
    char   def[50];        // def id
    int validity_epoch[5]; // yyyy,ddd,hh,mm,ss
    double clock_offset;   // clock offset wrt UTC  >0 for formatter tick early
    int clock_epoch[5];    // yyyy,ddd,hh,mm,ss
    double clock_rate;     // s/s
} siteclock_t;

typedef struct site
{
    char   def[50];        // def id
    char   type[20];       // site type 'fixed' or others
    char   name[20];       // site name
    char   id[3];          // 2 chars site id
    double position[3];    // site position X,Y,Z (m)
    siteclock_t sclock;    // site clock info [siteclock_t type]
} site_t;


typedef struct source
{
    char    def[50];         // def id
    char    type[20];        // star|satellite
    char    name[20];        // source name
    char    IAU_name[20];    // IAU name
    double  ra;              // RA in degree
    double  dec;             // DEC in degree
    char    epoch[10];       // epoch  J2000|B1950 etc
} source_t;

typedef struct eop
{
    char  def[50];     // def id
    int  epoch[5];     // EOP epoch yyyy, ddd, hh, mm, ss
    int    points;     // number of eop data points
    int interval_hr;   // eop data interval (unit is hour)
    double ut1[10];    // UT1-UTCs (sec)
    double xwob[10];   // Wobbling X parameters (asec)
    double ywob[10];   // Wobbling Y parameters (asec)
} eop_t;

typedef struct station_def
{
    char def[20];      // def id
    char das[20];      // link id to DAS block
    char clock[20];    // link id to CLOCK block
    char site[20];     // link id to SITE block
    char antenna[20];  // link id to ANTENNA block
} station_def_t;

typedef struct station_in_scan
{
    char id[3];       // 2 chars station id
    int data_start;   // sec  valid data offset from start
    int data_stop;    // sec  stop time from start
} station_in_scan_t;

typedef struct scan
{
    char scanid[20];  // scan id (used as a Mark 5 data file name)
    char mode[30];    // link id to MODE block (describing track, freq link)
    int number;       // scan (obs) number in an experiment
    int start[5];     // scan start time   yyyy,ddd,hh,mm,ss
    char source[20];  // source name
    station_in_scan_t station[MAXSTAT];  // scan period by station
    int numstat;      // acutual stations in station[MAXSTAT}
} scan_t;

typedef struct roll
{
    char    def[50];      // def ID
    char    rolldef[4];   // on|off
} roll_t;

typedef struct vexmode    // structure for MODE block data
{
    char def[30];                    // def id
    mode_item_t mode_freq[MAXSTAT];  // link to $FREQ block info by station (link id to $STATION)
    mode_item_t mode_track[MAXSTAT]; // link to $TRACK block info by station ( " )
    mode_item_t mode_roll[MAXSTAT];  // link to $ROLL block info by station  ( " )
    mode_item_t mode_pcal_detect[MAXSTAT]; // link to PCAL block info by station ( " )
    mode_item_t mode_if[MAXSTAT]; // link to $IF block info by station ( " )
    mode_item_t mode_bbc[MAXSTAT]; // link to $BBC block info by station ( " )
    int nmode;                // used only for count mode_freq and mode_track count
} vexmode_t;
   

typedef struct vex
{
    char vex_file_name[60];    // VEX file name
    char block[50][20];        // found $block name table  50x20chars
    int nblock;                // found $block number
    char flag[10];             // vex type identifier VEX|OVEX
    global_t global;           // $GLOBAL block info
    exper_t exper[10];         // $EXPER block info up to 10
    int nexper;                // number of def in $EXPER block
    vexmode_t mode[10];        // $MODE block info
    int nmode;                 // number of def in $MODE block
    site_t site[MAXSTAT];      // $SITE block info
    int  numstat;              // number of def in $SITE block
    source_t source[MAXSTAR];  // $SOURCE block info
    int  numstar;              // number of def in $SOURCE block
    //freq_t freq_def[5];        // $FREQ block info
    freq_t freq_def[20];        // $FREQ block info   2003-07-04
    int  nfreq_def;            // number of def in $FREQ block
    tracks_t track_def[10];    // $TRACK block info
    int ntrack_def;            // number of def in $TRACK block
    scan_t  scan[MAXOBS];      // $SCHED info
    int  numobs;               // number of observations in $SCHED block
    station_def_t station[MAXSTAT]; // $STATION block info
    int nstation;                   // number of def in $STATION block
    siteclock_t siteclock[MAXSTAT]; // $CLOCK block info
    int nclock;                     // number of def in $CLOCK block
    eop_t   eop[10];           // $EOP block info
    int neop;                  // number of def in $EOP block
    roll_t  roll[10];          // $ROLL block info
    int nroll;                 // number of def in $ROLL block
    pcal_block_t pcal[20];    // $PCAL block info
    int npcal;                 // number of def in $PCAL block
    bbc_block_t  bbc[20];    // $BBC block info
    int nbbc;                 // number of def in $BBC block
    if_block_t  ifb[20];    // $IF block info
    int nifb;                 // number of def in $IF block
} vex_t;


typedef struct sidtable
{
    char sid[10];
} sidtable_t;

typedef struct freq_info
{
    char chanid[10];       // channel ID
    char sideband[10];    // net sideband  usualy U|L or USB|LSB
    double rf;            // RF frequency (Hz)
    double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
    int npcal;            //       valid number of data in pcalf
    double vbw;           // Video frequency band width (Hz)
    int nch;              // channel # for this info
    int chmax;            // maximum channels in original table
} freq_info_t;


typedef struct track_freq_sub
{
    int fan_out_pos;      // 1 for 1st   2 for secondary
    int headstack;    // head stack position  1,2,3,4  added 2004-09-13
    int track;            // track number
    char chan_id[20];     // chan id
    char sideband[10];    // net sideband  usualy U|L or USB|LSB
    char bitstream[10];       //  sign|mag
    double rf;            // RF frequency (Hz)
    double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
    int npcal;            //       valid number of data in pcalf
    double vbw;           // Video frequency band width (Hz)
    char sub_pass[2];    // pass info (A|B) 2004-06-29
} track_freq_sub_t;

typedef struct track_freq
{
    //track_freq_sub_t trk[32];
    track_freq_sub_t trk[64];     // 2003-10-10
    char trackdef[30];
    char freqdef[30];
    int ntrack;
    int nfreq;
    int adbit;
    double samplrate;     // Sampling rate sampls/sec
    int fanout;        // fanout 1|2|4
    char roll[30];
} track_freq_t;

typedef struct vparam
{
    int  n;          //number of param
    char param[60][128];   // 60x128 characters
} vparam_t;

double clock_get(siteclock_t clock, int *prt)  //2003-07-13
{
    // get clock parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT
    double coffset,t1,t2;
    coffset=0.0;
    t1=(double)clock.clock_epoch[1]*86400.0   // DDD
        +(double)clock.clock_epoch[2]*3600.0  // HH
        +(double)clock.clock_epoch[3]*60.0    // MM
        +(double)clock.clock_epoch[4]*1.0;     // SS
    t2=(double)prt[1]*86400.0   // DDD
        +(double)prt[2]*3600.0  // HH
        +(double)prt[3]*60.0    // MM
        +(double)prt[4]*1.0;     // SS
    coffset=(t2-t1)*clock.clock_rate + clock.clock_offset;
    return coffset;
}

void sa_clock_get(vex_t *vex, char *stat_id1, char *stat_id2, int *prt, 
                  double *coffset, double *crate)  //2003-07-13
{
    // get clock parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT
    int i,k1,k2;
    char cw1[10],cw2[10];
    double o12, r12,o1,o2,r1,r2;
    strcpy(cw1,"--");
    strcpy(cw2,"--");
    k1=-1; k2=-1;
    o1=0.0; o2=0.0;
    r1=0.0; r2=0.0;
    o12=0.0; r12=0.0;
    //find station key for stat_id1
    for(i=0; i<vex->nstation; i++){
       if(strncmp(stat_id1,vex->station[i].def,strlen(stat_id1))==0){
            strcpy(cw1,vex->station[i].clock);
            break;
       }
    }
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw1,vex->siteclock[i].def,strlen(cw1))==0){
           k1=i;
           break;
       }
    }
    if(k1 >= 0) {
        o1=clock_get(vex->siteclock[k1], prt);
        r1=vex->siteclock[k1].clock_rate;
    }
    //find station key for stat_id2
    for(i=0; i<vex->nstation; i++){
       if(strncmp(stat_id2,vex->station[i].def,strlen(stat_id2))==0){
            strcpy(cw2,vex->station[i].clock);
            break;
       }
    }
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw2,vex->siteclock[i].def,strlen(cw2))==0){
           k2=i;
           break;
       }
    }
    if(k1 >= 0) {
        o2=clock_get(vex->siteclock[k2], prt);
        r2=vex->siteclock[k2].clock_rate;
    }
    o12=o2-o1;
    r12=r2-r1;
    //o12=o1-o2;   // debug
    //r12=r1-r2;   // debug
    *coffset=o12;
    *crate=r12;
}


void eop_get2(eop_t eop, int *prt, double *ut1, double *xwob, double *ywob)  //2003-07-13
{
    // get EOP parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT
    int i;
    double t0,tspan,t1;
    double tdat[20];
    t0=(double)eop.epoch[1]*86400.0   // DDD
        +(double)eop.epoch[2]*3600.0  // HH
        +(double)eop.epoch[3]*60.0    // MM
        +(double)eop.epoch[4]*1.0;     // SS
    tspan=(double)eop.interval_hr*3600.0;
    tdat[0]=t0;
    for(i=1; i<eop.points; i++){
        tdat[i]=tdat[i-1]+tspan;
    }
    // found prt location
    t1=(double)prt[1]*86400.0   // DDD
        +(double)prt[2]*3600.0  // HH
        +(double)prt[3]*60.0    // MM
        +(double)prt[4]*1.0;     // SS
    if(t1 < tdat[0]){
        *ut1=eop.ut1[0];
        *xwob=eop.xwob[0];
        *ywob=eop.ywob[0];
        goto exit2return;
    }
    if(t1 >= tdat[eop.points-1]){
        *ut1=eop.ut1[eop.points-1];
        *xwob=eop.xwob[eop.points-1];
        *ywob=eop.ywob[eop.points-1];
        goto exit2return;
    }
    for(i=0; i<eop.points-1; i++){
        if((t1 >= tdat[i]) && (t1 < tdat[i+1])){
            *ut1=eop.ut1[i]+(eop.ut1[i+1]-eop.ut1[i])*(t1-tdat[i])/tspan;
            *xwob=eop.xwob[i]+(eop.xwob[i+1]-eop.xwob[i])*(t1-tdat[i])/tspan;
            *ywob=eop.ywob[i]+(eop.ywob[i+1]-eop.ywob[i])*(t1-tdat[i])/tspan;
            break;
        }
    }
exit2return:;
}

void eop_get(vex_t *vex, int *prt, 
                  double *ut1, double *xwob, double *ywob)  //2003-07-13
{
    // get EOP parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT
    int i,k;
    k=-1;
    //find EOP data set
    for(i=0; i<vex->neop; i++){
       if(strncmp(vex->global.eop,vex->eop[i].def,strlen(vex->global.eop))==0){
           k=i;
           break;
       }
    }
    if(k<0) goto exit2return;

    eop_get2(vex->eop[k], prt, &(*ut1), &(*xwob), &(*ywob));
exit2return:;
}


void parsetime1(char *ctime, int *itime)  //2003-03-21
{
//   convert YYYYyDDDdHHhMMmSSs to  yyyy,ddd,hh,mm,ss
    int i,len;
    char abuf[10];
    for(i=0; i<5; i++) itime[i]=0;

    len=strlen(ctime);
    if(len <4 ) goto exitend;
    strncpy(abuf,ctime,4); abuf[4]='\0';
    itime[0]=atoi(abuf);
    if(len <8 ) goto exitend;
    strncpy(abuf,ctime+5,3); abuf[3]='\0';
    //printf("debug abuf %s\n",abuf);
    itime[1]=atoi(abuf);
    if(len <11 ) goto exitend;
    strncpy(abuf,ctime+9,2); abuf[2]='\0';
    itime[2]=atoi(abuf);
    if(len <14 ) goto exitend;
    strncpy(abuf,ctime+12,2);abuf[2]='\0';
    itime[3]=atoi(abuf);
    if(len <17 ) goto exitend;
    strncpy(abuf,ctime+15,2);abuf[2]='\0';
    itime[4]=atoi(abuf);
exitend:;
}

void asc_ra2deg(char *ra, double *radeg) //2003-03-21
{
//   convert HHyMMmSS.SSSSSs to degree
    char abuf[30];
    int i,i1,i2,len;
    int iw[2];
    double dsec,dmin;

    len=strlen(ra);
    *radeg=0.0;
    // found 'h'
    i1=0;
    i2=0;
    for(i=0; i<len; i++){
        if(ra[i]=='h'){
            i2=i;
            break;
        }
    }
    if(i2==0) goto exitend;
    strncpy(abuf,ra,i2); abuf[i2]='\0';
    iw[0]=atoi(abuf);
    // found 'm'
    i1=i2+1;
    for(i=i1; i<len; i++){
        if(ra[i]=='m'){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    strncpy(abuf,ra+i1,i2); abuf[i2]='\0';
    iw[1]=atoi(abuf);
    // found 's'
    i1=i1+i2+1;
    for(i=i1; i<len; i++){
        if(ra[i]=='s'){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    strncpy(abuf,ra+i1,i2); abuf[i2]='\0';
    dsec=atof(abuf);

    dmin=(double)iw[1]+dsec/60.0;
    *radeg=15.0*(double)iw[0]+dmin/4.0;
exitend:;
}


void asc_dec2deg(char *dec, double *decdeg) //2003-03-21
{
//   convert +/-DDHHdM'SS.SSSSS" to degree
    char abuf[30];
    int i,i1,i2,len;
    int iw[2];
    double dsec,dmin,ddeg;

    //printf("%s\n",dec);
    len=strlen(dec);
    *decdeg=0.0;
    // found 'd'
    i1=0;
    i2=0;
    for(i=0; i<len; i++){
        if(dec[i]=='d'){
            i2=i;
            break;
        }
    }
    if(i2==0) goto exitend;
    strncpy(abuf,dec,i2); abuf[i2]='\0';
    iw[0]=atoi(abuf);
    // found '''
    i1=i2+1;
    for(i=i1; i<len; i++){
        if(dec[i]=='\''){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    strncpy(abuf,dec+i1,i2); abuf[i2]='\0';
    //printf("debug %d %d %s\n",i1,i2,abuf);
    iw[1]=atoi(abuf);
    // found '"'
    i1=i1+i2+1;
    for(i=i1; i<len; i++){
        if(dec[i]=='\"'){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    strncpy(abuf,dec+i1,i2); abuf[i2]='\0';
    dsec=atof(abuf);

    dmin=(double)iw[1]+dsec/60.0;

    if(iw[0]<0) {
        ddeg=-(double)iw[0];
    } else {
        ddeg=(double)iw[0];
    }
    //printf("DEBUG %d %d %f\n",iw[0],iw[1],dsec);
    *decdeg=ddeg+dmin/60.0;
    if(iw[0]<0) *decdeg=-*decdeg;
exitend:;
}


int vex_parse(char *cbuf, vparam_t *vparam)  //2003-03-21
{
/*
    Parse VEX parameters by '=',':',';', and ' '.
    '=',':',';' are returned as a parsed item for further analysis
    Total number of items including '=',':',';' is defined in
    type define statesment of vparam as

    typedef struct vparam
    {
        int  n;          //number of param
        char param[60][128];   // 60x128 characters
    } vparam_t;

    In this case 60 is a maximum number of items and
    128 characters are maximum char in an item.

    If runtime error occurrs, please try to increase the number of
    items in the vparam definition.


*/

    // vex_parse =0;  single statement in a cbuf
    //           =1;  more statement left in a cbuf, start point in cbuf 
    //                is updated 
    char abuf[500];
    char *cset=" ";
    char *p=NULL;
    int i,ipos,k,kpos,flag,mflag,len;

    len=strlen(cbuf);
    //printf("len = %d\n",len);

    //remove extra space in front

    if(cbuf[0]==' '){
        kpos=0;
        for(k=0; k<len; k++){
            if(cbuf[k]!=' '){
                kpos=k;
                break;
            }
        }
        i=0;
        for(k=kpos; k<len; k++){
            cbuf[i]=cbuf[k];
            i++;
        }
        cbuf[i]='\0';
    }
    // len again

    len=strlen(cbuf);


    // parse start

    if(cbuf[0] == '*') { // comment
        vparam->n=0; 
        return 0;
    }
    i=0;
    ipos=0;
    flag=0;
    for(k=0; k<len; k++){
        if(cbuf[k] == ' ' || cbuf[k] == '=' || cbuf[k] == ':'){
            if(flag==1) {  // one parameter end
                vparam->param[ipos][i]='\0';
                flag=0;
                i=0;
                ipos++;
            }
            if(cbuf[k] != ' '){
                vparam->param[ipos][i]=cbuf[k];
                vparam->param[ipos][i+1]='\0';
                flag=0;
                i=0;
                ipos++;
            }
            goto nextk;
        }
        if(cbuf[k] == ';'){
            kpos=k;
            goto exithere;
        }
        if(cbuf[k] != ' '){
            flag=1;
            vparam->param[ipos][i]=cbuf[k];
            i++;
        }
nextk:;
    }
    kpos=k;
exithere:
    vparam->param[ipos][i]='\0';
    ipos++;
    vparam->n=ipos;
    i=0;
    mflag=0;
    if(kpos < len-1){
        for(k=kpos+1; k<len; k++) {
            if(cbuf[k] != '\0'){
                abuf[i]=cbuf[k];
                i++;
            }
            if(cbuf[k] == ';') mflag=1;
        }
    }
    if(mflag==1){
        abuf[i]='\0';
        strcpy(cbuf,abuf);
        return 1;
    }
    return 0;
}


/* GLOBAL block analysis */
int GLOBAL_ana(FILE *fin, char *line, vex_t *vex) //2003-03-21
{
    int k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
loop:    
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop00:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[1],"$EXPER",6)==0){
         strcpy(vex->global.exper,vp.param[3]); // save def param
         if(k > 0) goto loop00;
         goto loop;
    }
    if(vp.n > 0 && strncmp(vp.param[1],"$EOP",4)==0){
         strcpy(vex->global.eop,vp.param[3]); // save def param
         if(k > 0) goto loop00;
         goto loop;
    }
    if(k > 0) goto loop00;
    goto loop;
}

/* EXPER block analysis */
int EXPER_ana(FILE *fin, char *line, vex_t *vex)  //2003-03-21
{
    int k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->nexper=0;
loop:
    
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
         strcpy(vex->exper[vex->nexper].def,vp.param[1]); // save def param
         if(k > 0) goto loop01;
loop00:
         if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
         k=vex_parse(line, &vp);
         if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->nexper++;
             goto loop;
         }
         if(vp.n>0 && strncmp(vp.param[0],"exper_name",10)==0){ // 
             strcpy(vex->exper[vex->nexper].exper_name,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
         }
         goto loop00;
    } 
    if(k > 0) goto loop1;
    goto loop;
}

/* SCHEDULE block analysis */
int SKED_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int i,k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->numobs=0;
loopscan:
    
    i=0;
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"scan",4)==0){
         strcpy(vex->scan[vex->numobs].scanid,vp.param[1]); // save scan param
        if(k > 0) goto loopscan01;
loopscan00:
         if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loopscan01:
         k=vex_parse(line, &vp);
         if(vp.n>0 && strncmp(vp.param[0],"endscan",7)==0){
             vex->scan[vex->numobs].number=vex->numobs+1;
             vex->scan[vex->numobs].numstat=i;
             vex->numobs++;
             goto loopscan;
         }
         if(vp.n>0 && strncmp(vp.param[0],"start",5)==0){ // 
             parsetime1(vp.param[2],vex->scan[vex->numobs].start);
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         if(vp.n>0 && strncmp(vp.param[0],"mode",4)==0){ // 
             strcpy(vex->scan[vex->numobs].mode,vp.param[2]);
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         if(vp.n>0 && strncmp(vp.param[0],"source",6)==0){ // 
             strcpy(vex->scan[vex->numobs].source,vp.param[2]);
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         if(vp.n>0 && strncmp(vp.param[0],"station",7)==0){ // 
             strcpy(vex->scan[vex->numobs].station[i].id,vp.param[2]);
            //printf("DDD %s %s\n",vex->scan[vex->numobs].station[i].id,vp.param[4]);
             vex->scan[vex->numobs].station[i].data_start=atoi(vp.param[4]);
             vex->scan[vex->numobs].station[i].data_stop=atoi(vp.param[7]);
             i++;
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         goto loopscan00;
    } 
    if(k > 0) goto loop1;
    goto loopscan;
}

/* TRACKS block analysis */
int TRACKS_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int i,k,m,ioffset;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->ntrack_def=0;
looptrack:
    i=0;
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->track_def[vex->ntrack_def].def,vp.param[1]); // save def param
        vex->track_def[vex->ntrack_def].format[0]='\0';
        if(k > 0) goto looptrack01;
looptrack00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
looptrack01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->ntrack_def++;
             goto looptrack;
        }
        if(vp.n>0 && strncmp(vp.param[0],"track_frame_format",17)==0){ // 
             strcpy(vex->track_def[vex->ntrack_def].format,vp.param[2]);
             if(k > 0) goto looptrack01;
             goto looptrack00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"fanout_def",10)==0){ // 
            //printf("TRACK: %s\n",line);
            m=0;

            ioffset=0;
            if(strncmp(vp.param[2],":",1)==0) {  // no sub_pass character
                strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].sub_pass,"\0");
                ioffset=-1;
            } else {
                strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].sub_pass,vp.param[2]);
            }
            strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].chan_id,vp.param[4+ioffset]);
            strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].stream_info,vp.param[6+ioffset]);
            vex->track_def[vex->ntrack_def].fanout_def[i].headstack=atoi(vp.param[8+ioffset]);
            while(2*m+10+ioffset < vp.n){
               vex->track_def[vex->ntrack_def].fanout_def[i].track[m]=atoi(vp.param[10+ioffset+2*m]);
               m++;
            } 
            vex->track_def[vex->ntrack_def].fanout_def[i].fanout=m;
            i++;
            vex->track_def[vex->ntrack_def].nfanout_def=i;

            if(k > 0) goto looptrack01;
            goto looptrack00;
        }
        goto looptrack00;
    }
    if(k > 0) goto loop1;
    goto looptrack;
}

/* SITE block analysis */
int SITE_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->numstat=0;
loop:
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->site[vex->numstat].def,vp.param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->numstat++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"site_name",9)==0){ // 
             strcpy(vex->site[vex->numstat].name,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"site_ID",7)==0){ // 
             strcpy(vex->site[vex->numstat].id,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"site_position",13)==0){ // 
             vex->site[vex->numstat].position[0]=atof(vp.param[2]);
             vex->site[vex->numstat].position[1]=atof(vp.param[5]);
             vex->site[vex->numstat].position[2]=atof(vp.param[8]);

             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* STATION block analysis */
int STATION_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->nstation=0;

loop:
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    //printf("STATION_ana: %s\n",line);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->station[vex->nstation].def,vp.param[1]); // save def param
        vex->station[vex->nstation].antenna[0]='\0';
        vex->station[vex->nstation].clock[0]='\0';
        vex->station[vex->nstation].das[0]='\0';
        vex->station[vex->nstation].site[0]='\0';
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->nstation++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$DAS",4)==0){ // 
             strcpy(vex->station[vex->nstation].das,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$CLOCK",6)==0){ // 
             strcpy(vex->station[vex->nstation].clock,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$SITE",5)==0){ // 
             strcpy(vex->station[vex->nstation].site,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$ANTENNA",8)==0){ // 
             strcpy(vex->station[vex->nstation].antenna,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* SOURCE block analysis */
int SOURCE_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->numstar=0;
loop:
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->source[vex->numstar].def,vp.param[1]); // save def param
        vex->source[vex->numstar].name[0]='\0';
        vex->source[vex->numstar].IAU_name[0]='\0';
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->numstar++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"source_name",11)==0){ // 
             strcpy(vex->source[vex->numstar].name,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"IAU_name",8)==0){ // 
             strcpy(vex->source[vex->numstar].IAU_name,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"ref_coord_frame",15)==0){ //
             strcpy(vex->source[vex->numstar].epoch,vp.param[2]);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"ra",2)==0){ // 
            asc_ra2deg(vp.param[2],&vex->source[vex->numstar].ra);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"dec",3)==0){ // 
            asc_dec2deg(vp.param[2],&vex->source[vex->numstar].dec);
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* CLOCK block analysis */
int CLOCK_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k,i;
    vparam_t vp;
    char cbuf[50];
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vex->nclock=0;
loop:
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    //printf("CLOCK: %s\n",line);
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        //printf("CLOCK2:%s\n",vp.param[0]);
        strcpy(vex->siteclock[vex->nclock].def,vp.param[1]); // save def param
        // default set
        for(i=0; i<5; i++){
           vex->siteclock[vex->nclock].validity_epoch[i]=0;
           vex->siteclock[vex->nclock].clock_epoch[i]=0;
        }
        vex->siteclock[vex->nclock].clock_offset=0.0;
        vex->siteclock[vex->nclock].clock_rate=0.0;
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->nclock++;
             //printf("CLOCK3: %d\n",vex->nclock);
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"clock_early",11)==0){ // 
             strcpy(cbuf,vp.param[2]);
             parsetime1(cbuf,vex->siteclock[vex->nclock].validity_epoch);
             strcpy(cbuf,vp.param[4]);
             vex->siteclock[vex->nclock].clock_offset=atof(cbuf);
             if(strncmp(vp.param[5],"msec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-3;
             if(strncmp(vp.param[5],"usec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-6;
             if(strncmp(vp.param[5],"nsec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-9;
             if(strncmp(vp.param[5],"psec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-12;
             if(vp.n>7){
                strcpy(cbuf,vp.param[7]);
                parsetime1(cbuf,vex->siteclock[vex->nclock].clock_epoch);
                strcpy(cbuf,vp.param[9]);
                vex->siteclock[vex->nclock].clock_rate=atof(cbuf);
                vex->siteclock[vex->nclock].clock_rate*=1.0e-6; //2003-07-13
             }
             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* EOP block analysis */
int EOP_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k,m;
    vparam_t vp;
    char cbuf[50];
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vex->neop=0;
loop:
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->eop[vex->neop].def,vp.param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->neop++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"TAI-UTC",7)==0){ // 
            // nothing to do 
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"eop_ref_epoch",13)==0){ // 
            strcpy(cbuf,vp.param[2]);
            parsetime1(cbuf,vex->eop[vex->neop].epoch);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"num_eop_points",14)==0){ // 
            strcpy(cbuf,vp.param[2]);
            vex->eop[vex->neop].points=atoi(cbuf);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"eop_interval",12)==0){ // 
            strcpy(cbuf,vp.param[2]);
            vex->eop[vex->neop].interval_hr=atoi(cbuf);
            // if u want to check unit use following method
            //if(strncmp(vp.param[3],"hr",2)==0) 
            //    vex->eop[vex->neop].interval_hr*=1;
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"ut1-utc",7)==0){ // 
            m=0;
            strcpy(cbuf,vp.param[2]);
            while(1){
               strcpy(cbuf,vp.param[m*3+2]);
               vex->eop[vex->neop].ut1[m]=atof(cbuf);
               if(strncmp(vp.param[m*3+3],"msec",4)==0) 
                    vex->eop[vex->neop].ut1[m]*=1.0e-3;
               m++;
               //if(m>=vp.n) break;
               if(m*3+2 >=vp.n) break;
            }
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"x_wobble",8)==0){ // 
            m=0;
            strcpy(cbuf,vp.param[2]);
            while(1){
               strcpy(cbuf,vp.param[m*3+2]);
               vex->eop[vex->neop].xwob[m]=atof(cbuf);
               //if(strncmp(vp.param[m*3+3],"asec",4)==0) // asec unit 
               //     vex->eop[vex->neop].xwob[m]*=1.0;
               m++;
               //if(m>=vp.n) break;
               if(m*3+2 >=vp.n) break;
            }
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"y_wobble",8)==0){ // 
            m=0;
            strcpy(cbuf,vp.param[2]);
            while(1){
               strcpy(cbuf,vp.param[m*3+2]);
               vex->eop[vex->neop].ywob[m]=atof(cbuf);
               //if(strncmp(vp.param[m*3+3],"asec",4)==0) // asec unit 
               //     vex->eop[vex->neop].ywob[m]*=1.0;
               m++;
               //if(m>=vp.n) break;
               if(m*3+2 >=vp.n) break;
            }
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* ROLL block analysis */
int ROLL_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vex->nroll=0;
loop:
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->roll[vex->nroll].def,vp.param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->nroll++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"roll",4)==0){ // 
            //printf("ROLL_ana: %s\n",vp.param[2]);
            strcpy(vex->roll[vex->nroll].rolldef,vp.param[2]);
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* FREQ block analysis */
int FREQ_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int i,k,m,m1,mpos,ioffset;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->nfreq_def=0;

    ioffset=1;
    if(strncmp(vex->flag,"OVEX",4)==0) ioffset=2;
loop:
    i=0;
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->freq_def[vex->nfreq_def].def,vp.param[1]); // save def param
        vex->freq_def[vex->nfreq_def].samplerate=0.0;
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
            vex->freq_def[vex->nfreq_def].numch=i;
             vex->nfreq_def++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"sample_rate",11)==0){ // 
             vex->freq_def[vex->nfreq_def].samplerate=atof(vp.param[2]);
             if(strncmp(vp.param[3],"Ms/sec",6)==0) {
                  vex->freq_def[vex->nfreq_def].samplerate*=1.0e6;
             }
             if(strncmp(vp.param[3],"ks/sec",6)==0) {
                  vex->freq_def[vex->nfreq_def].samplerate*=1.0e3;
             }
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"chan_def",8)==0){ // 
             vex->freq_def[vex->nfreq_def].chan_def[i].rf=atof(vp.param[3]);
             m1=0;
             for(m=0; m<vp.n; m++){
                 if(strncmp(vp.param[m],":",1)==0) {
                     m1++;
                     if(m1==ioffset){
                         mpos=m+1;
                         break;
                     }
                 }
             }
             vex->freq_def[vex->nfreq_def].chan_def[i].rf=atof(vp.param[mpos]);
             if(strncmp(vp.param[mpos+1],"GHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].rf*=1.0e9;
             }
             if(strncmp(vp.param[mpos+1],"MHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].rf*=1.0e6;
             }
             if(strncmp(vp.param[mpos+1],"kHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].rf*=1.0e3;
             }
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].sideband,vp.param[mpos+3]);
             vex->freq_def[vex->nfreq_def].chan_def[i].vbw=atof(vp.param[mpos+5]);
             if(strncmp(vp.param[mpos+6],"GHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].vbw*=1.0e9;
             }
             if(strncmp(vp.param[mpos+6],"MHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].vbw*=1.0e6;
             }
             if(strncmp(vp.param[mpos+6],"kHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].vbw*=1.0e3;
             }
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].chan_id,vp.param[mpos+8]);
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].bbc_id,vp.param[mpos+10]);
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].pcal_id,vp.param[mpos+12]);
             i++;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* MODE block analysis */
int MODE_ana(FILE* fin, char *line, vex_t *vex)  //2003-07-17
{
    int i,k,nfreq,nroll,ntrack,npcal,m;
    int nif,nbbc;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->nmode=0;

loop:
    nfreq=0; nroll=0; ntrack=0; npcal=0; nif=0; nbbc=0;
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->mode[vex->nmode].def,vp.param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
            if(nfreq != ntrack) printf("MODE_ana: Warning!! \n");
             vex->mode[vex->nmode].nmode=nfreq;
             vex->nmode++;
             goto loop;
        }
        if(vp.n > 0 && strncmp(vp.param[1],"$FREQ",5)==0){
           i=5;
           while(1){
              // first check already existed id or not
              if(nfreq >1){
                   for(m=0; m<nfreq; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_freq[m].id,vp.param[i],strlen(vp.param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_freq[m].mode,vp.param[3]);
                           goto next_fid;
                       }
                   }
              }
               // for new id
              strcpy(vex->mode[vex->nmode].mode_freq[nfreq].mode,vp.param[3]); // 
              strcpy(vex->mode[vex->nmode].mode_freq[nfreq].id,vp.param[i]); 
              nfreq++;
next_fid:;
              i+=2;
              if(i >= vp.n) break;
           } 
           if(k > 0) goto loop01;
           goto loop00;
        }

        if(vp.n > 0 && strncmp(vp.param[1],"$TRACKS",7)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(ntrack >1){
                   for(m=0; m<ntrack; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_track[m].id,vp.param[i],strlen(vp.param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_tid;
                       }
                   }
               }
               // for new id
               strcpy(vex->mode[vex->nmode].mode_track[ntrack].mode,vp.param[3]); // 
               strcpy(vex->mode[vex->nmode].mode_track[ntrack].id,vp.param[i]); 
               ntrack++;
next_tid:;
               i+=2;
               if(i >= vp.n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }
        if(vp.n > 0 && strncmp(vp.param[1],"$ROLL",5)==0){
           i=5;
           while(1){
              strcpy(vex->mode[vex->nmode].mode_roll[nroll].mode,vp.param[3]); // 
              strcpy(vex->mode[vex->nmode].mode_roll[nroll].id,vp.param[i]); 
              nroll++;
              i+=2;
              if(i >= vp.n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }

        if(vp.n > 0 && strncmp(vp.param[1],"$PHASE_CAL_DETECT",17)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(npcal >1){
                   for(m=0; m<npcal; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_pcal_detect[m].id,vp.param[i],strlen(vp.param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_pcid;
                       }
                   }
               }
              strcpy(vex->mode[vex->nmode].mode_pcal_detect[npcal].mode,vp.param[3]); // 
              strcpy(vex->mode[vex->nmode].mode_pcal_detect[npcal].id,vp.param[i]); 
              npcal++;
next_pcid:;
              i+=2;
              if(i >= vp.n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        } 
        // $IF mode
        if(vp.n > 0 && strncmp(vp.param[1],"$IF",3)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(nif >1){
                   for(m=0; m<nif; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_if[m].id,vp.param[i],strlen(vp.param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_ifid;
                       }
                   }
               }
               // for new id
               strcpy(vex->mode[vex->nmode].mode_if[nif].mode,vp.param[3]); // 
               strcpy(vex->mode[vex->nmode].mode_if[nif].id,vp.param[i]); 
               nif++;
next_ifid:;
               i+=2;
               if(i >= vp.n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }

        // $BBC mode
        if(vp.n > 0 && strncmp(vp.param[1],"$BBC",3)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(nbbc >1){
                   for(m=0; m<nbbc; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_bbc[m].id,vp.param[i],strlen(vp.param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_bbcid;
                       }
                   }
               }
               // for new id
               strcpy(vex->mode[vex->nmode].mode_bbc[nbbc].mode,vp.param[3]); // 
               strcpy(vex->mode[vex->nmode].mode_bbc[nbbc].id,vp.param[i]); 
               nbbc++;
next_bbcid:;
               i+=2;
               if(i >= vp.n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}


/* PCAL block analysis */
int PCAL_ana(FILE* fin, char *line, vex_t *vex) /* 2004-06-17 */
{
    int i,k,m,mm;
    char cbuf[50];
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->npcal=0;
    //printf("PCAL ANA\n");
loop:
    i=0;
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->pcal[vex->npcal].def,vp.param[1]); // save def param
//printf("DEBUG %s\n",vex->pcal[vex->npcal].def);
//printf("k= %d\n",k);
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
//printf("debug\n");
        k=vex_parse(line, &vp);
//printf("debug2 k=%d %s\n",k,line);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
            vex->pcal[vex->npcal].npcal=i;
             vex->npcal++;
//printf("PCAL enddef\n");
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"phase_cal_detect",16)==0){ // 
             strcpy(vex->pcal[vex->npcal].pcal[i].pcal_id,vp.param[2]);
//printf("debug3 %s\n",vp.param[2]);
//printf("debug4 %s\n",vex->pcal[vex->npcal].pcal[i].pcal_id);
            m=0;
            mm=0;
            strcpy(cbuf,vp.param[2]);
            if(vp.n > 4){     /* 2004-06-17 */
//printf("debug40 %d\n",vp.n);
            while(1){
//printf("debug41\n");

               strcpy(cbuf,vp.param[m*2+4]);
//printf("debug42\n");
               if(atoi(cbuf) != 0) {    // 2003-10-28
                  vex->pcal[vex->npcal].pcal[i].tone[mm]=atoi(cbuf);
                  mm++;
               } // 2003-10-28
               //vex->pcal[vex->npcal].pcal[i].tone[m]=atoi(cbuf);
//printf("debug %d\n",vex->pcal[vex->npcal].pcal[i].tone[m]);
               m++;
//printf("m,vp.n %d %d\n",m,vp.n);
               if(m*2+4 >=vp.n) {
                   //vex->pcal[vex->npcal].pcal[i].ntone=m;
                   vex->pcal[vex->npcal].pcal[i].ntone=mm;  // 2003-10-28
                   break;
               }
            }
            } else {
                vex->pcal[vex->npcal].pcal[i].ntone=0;  /* 2004-06-17 */
            }
            i++;
//printf("i,k %d %d\n",i,k);
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}


/* BBC block analysis */
int BBC_ana(FILE* fin, char *line, vex_t *vex) //2003-07-17
{
    int i,k;
    char cbuf[50];
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->nbbc=0;
//printf("BBC ANA\n");
loop:
    i=0;
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->bbc[vex->nbbc].def,vp.param[1]); // save def param
//printf("DEBUG %s\n",vex->bbc[vex->nbbc].def);

        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
            vex->bbc[vex->nbbc].nbbc=i;
             vex->nbbc++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"BBC_assign",10)==0){ // 
//printf("BBC assign field\n");
             strcpy(vex->bbc[vex->nbbc].bbc_def[i].bbc_id,vp.param[2]);
             strcpy(cbuf,vp.param[4]);
             vex->bbc[vex->nbbc].bbc_def[i].bbcch=atoi(cbuf);
             strcpy(vex->bbc[vex->nbbc].bbc_def[i].if_id,vp.param[6]);
             i++;
        }
//printf("i,k %d %d\n",i,k);
        if(k > 0) goto loop01;
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* IF block analysis */
int IF_ana(FILE* fin, char *line, vex_t *vex) //2003-07-17
{
    int i,k;
    vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->nifb=0;
//printf("IF ANA\n");
loop:
    i=0;
    if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->ifb[vex->nifb].def,vp.param[1]); // save def param
//printf("DEBUG %s\n",vex->ifb[vex->nifb].def);

        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
            vex->ifb[vex->nifb].nif=i;
             vex->nifb++;
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"if_def",6)==0){ // 
//printf("IF assign field\n");
             strcpy(vex->ifb[vex->nifb].if_def[i].if_id,vp.param[2]);
             strcpy(vex->ifb[vex->nifb].if_def[i].pol,vp.param[6]);

             vex->ifb[vex->nifb].if_def[i].lo=atof(vp.param[8]);
             if(strncmp(vp.param[9],"GHz",3)==0) {
                  vex->ifb[vex->nifb].if_def[i].lo*=1.0e9;
             }
             if(strncmp(vp.param[9],"MHz",3)==0) {
                  vex->ifb[vex->nifb].if_def[i].lo*=1.0e6;
             }
             if(strncmp(vp.param[9],"kHz",3)==0) {
                  vex->ifb[vex->nifb].if_def[i].lo*=1.0e3;
             }
             strcpy(vex->ifb[vex->nifb].if_def[i].sideband,vp.param[11]);
             vex->ifb[vex->nifb].if_def[i].pcal_space=0.0;
             if(vp.n >= 13){
                 vex->ifb[vex->nifb].if_def[i].pcal_space=atof(vp.param[13]);
                 if(strncmp(vp.param[14],"GHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_space*=1.0e9;
                 }
                 if(strncmp(vp.param[14],"MHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_space*=1.0e6;
                 }
                 if(strncmp(vp.param[14],"kHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_space*=1.0e3;
                 }
             }
             vex->ifb[vex->nifb].if_def[i].pcal_base=0.0;
             if(vp.n >= 16){
                 vex->ifb[vex->nifb].if_def[i].pcal_base=atof(vp.param[16]);
                 if(strncmp(vp.param[17],"GHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_base*=1.0e9;
                 }
                 if(strncmp(vp.param[17],"MHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_base*=1.0e6;
                 }
                 if(strncmp(vp.param[17],"kHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_base*=1.0e3;
                 }
             }
             i++;
        }
//printf("i,k %d %d\n",i,k);
        if(k > 0) goto loop01;
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

void MODE_mon(vex_t *vex)  //2003-03-21
{
/*      $MODE block monitor  */
    int i,k;
    printf("MODE %d\n",vex->nmode);
    for (k=0; k<vex->nmode; k++){  
        printf(" %s\n",vex->mode[k].def);
        for (i=0;i<vex->mode[k].nmode;i++){
          printf("%d %s %s\n",i+1,vex->mode[k].mode_freq[i].mode, vex->mode[k].mode_freq[i].id);
          printf("%d %s %s\n",i+1,vex->mode[k].mode_track[i].mode, vex->mode[k].mode_track[i].id);
          printf("%d %s %s\n",i+1,vex->mode[k].mode_roll[i].mode, vex->mode[k].mode_roll[i].id);
          printf("%d %s %s\n",i+1,vex->mode[k].mode_pcal_detect[i].mode, vex->mode[k].mode_pcal_detect[i].id);
        } 
    } 
}

void SKED_mon(vex_t *vex)  //2003-03-21
{
/*      $SCHED block monitor  */
    int i,k;
    printf("NUMOBS  %d\n",vex->numobs);
    for (i=0; i<vex->numobs; i++){
        
        printf("%d %s %s %s %d %d %d %d %d",
            vex->scan[i].number,vex->scan[i].scanid, vex->scan[i].mode,
            vex->scan[i].source,
            vex->scan[i].start[0],
            vex->scan[i].start[1],
            vex->scan[i].start[2],
            vex->scan[i].start[3],
            vex->scan[i].start[4]);
        //printf("numstat = %d\n", vex->scan[i].numstat);
        for(k=0; k< vex->scan[i].numstat; k++){
            printf("  %s ",vex->scan[i].station[k].id);
            printf("  %d %d  ",
                vex->scan[i].station[k].data_start,vex->scan[i].station[k].data_stop);
        }
        printf("\n");
    }
}

void TRACKS_mon(vex_t *vex)  //2003-03-21
{
/*      $TRACKS block monitor  */
    int i,k,m;
    printf("NUMTRACK  %d\n",vex->ntrack_def);
    for (i=0; i<vex->ntrack_def; i++){
        printf("%d %s %s\n",i+1,vex->track_def[i].def,vex->track_def[i].format);
        //printf("%d\n",vex->track_def[i].nfanout_def);
        for(k=0; k<vex->track_def[i].nfanout_def; k++){
            printf("   %s ",vex->track_def[i].fanout_def[k].sub_pass);
            printf("   %d",vex->track_def[i].fanout_def[k].fanout);
            printf(" %s ",vex->track_def[i].fanout_def[k].chan_id);
            printf(" %s ",vex->track_def[i].fanout_def[k].stream_info);
            for(m=0; m< vex->track_def[i].fanout_def[k].fanout; m++){
                printf(" %d",vex->track_def[i].fanout_def[k].track[m]);
            }
            printf("\n");
        }
    }
}

void SITE_mon(vex_t *vex)  //2003-03-21
{
/*      $SITE block monitor  */
    int i,k;
    printf("NUMSITE  %d\n",vex->numstat);
    for (i=0; i<vex->numstat; i++){
        printf("%d %s %s %s\n",i+1,vex->site[i].def,vex->site[i].name,vex->site[i].id);
        for(k=0; k<3; k++){
            printf(" %f\n",vex->site[i].position[k]);
        }
    }
}

void STATION_mon(vex_t *vex)  //2003-03-21
{
/*      $STATION block monitor  */
    int i;
    printf("NUMSTATION  %d\n",vex->nstation);
    for (i=0; i<vex->nstation; i++){
        printf("%d %s %s %s %s %s\n",i+1,vex->station[i].def,vex->station[i].site,
            vex->station[i].antenna,vex->station[i].clock,vex->station[i].das);
    }
}

void SOURCE_mon(vex_t *vex)  //2003-03-21
{
/*      $SOURCE block monitor  */
    int i;
    printf("NUMSTAR  %d\n",vex->numstar);
    for (i=0; i<vex->numstar; i++){
        printf("%d %s %s %s ",i+1,vex->source[i].def,vex->source[i].name,vex->source[i].IAU_name);
        printf(" %f %f %s \n",vex->source[i].ra,vex->source[i].dec,vex->source[i].epoch);
    }
}

void FREQ_mon(vex_t *vex)  //2003-03-21
{
/*      $FREQ block monitor  */
    int i,k;
    printf("NUMFREQ def %d\n",vex->nfreq_def);
    for (i=0; i<vex->nfreq_def; i++){
        printf("%d %s %f\n",i+1,vex->freq_def[i].def,vex->freq_def[i].samplerate);
        for(k=0; k<vex->freq_def[i].numch; k++){
           printf(" %f %s %f %s %s %s\n",vex->freq_def[i].chan_def[k].rf,
               vex->freq_def[i].chan_def[k].sideband,
               vex->freq_def[i].chan_def[k].vbw,
               vex->freq_def[i].chan_def[k].chan_id,
               vex->freq_def[i].chan_def[k].bbc_id,
               vex->freq_def[i].chan_def[k].pcal_id);
        }
    }
}

void CLOCK_mon(vex_t *vex)  //2003-03-21
{
/*      $CLOCK block monitor  */
    int i,k;
    printf("NUMCLOCK def %d\n",vex->nclock);
    for (i=0; i<vex->nclock; i++){
        printf("%d %s\n",i+1,vex->siteclock[i].def);
        printf(" validity epoch: ");
        for(k=0; k<5; k++) printf(" %d",vex->siteclock[i].validity_epoch[k]);
        printf("\n");
        printf(" clock early : %e\n",vex->siteclock[i].clock_offset);
        printf(" clock epoch : ");
        for(k=0; k<5; k++) printf(" %d",vex->siteclock[i].clock_epoch[k]);
        printf("\n");
        printf(" clock rate  : %e\n",vex->siteclock[i].clock_rate);
    }
}


void CLOCK_mon2(vex_t *vex, char *stat_id)  //2003-07-13
{
/*      $CLOCK block monitor only for stat_id */
    int i,k;
    char cw1[20];
    strcpy(cw1,"--");
    k=-1;
    //find station key for stat_id
    for(i=0; i<vex->nstation; i++){
       if(strncmp(stat_id,vex->station[i].def,strlen(stat_id))==0){
            strcpy(cw1,vex->station[i].clock);
            break;
       }
    }
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw1,vex->siteclock[i].def,strlen(cw1))==0){
           k=i;
           break;
       }
    }
    if(k < 0){
        printf("Clock parameters not found for %s\n",stat_id);
        goto exit2return;
    }
    //printf("NUMCLOCK def %d\n",vex->nclock);
    printf("  %s %s\n",stat_id,vex->siteclock[k].def);
    printf("   validity epoch: ");
    for(i=0; i<5; i++) printf("   %d",vex->siteclock[k].validity_epoch[i]);
    printf("\n");
    printf("   clock early : %e\n",vex->siteclock[k].clock_offset);
    printf("   clock epoch : ");
    for(i=0; i<5; i++) printf("   %d",vex->siteclock[k].clock_epoch[i]);
    printf("\n");
    printf("   clock rate  : %e\n",vex->siteclock[k].clock_rate);
exit2return:;
}

void CLOCK_get(vex_t *vex, char *stat_id, siteclock_t *clock)  //2003-07-19
{
/*      Get clock parameter */
    int i,k;
    char cw1[20];
    strcpy(cw1,"--");
    strcpy(clock->def,"--");
    for(i=0; i<5; i++){
       clock->clock_epoch[i]=0;
       clock->validity_epoch[i]=0;
    }
    clock->clock_offset=0.0;
    clock->clock_rate=0.0;
    k=-1;
    //find station key for stat_id
    for(i=0; i<vex->nstation; i++){
       if(strncmp(stat_id,vex->station[i].def,strlen(stat_id))==0){
            strcpy(cw1,vex->station[i].clock);
            break;
       }
    }
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw1,vex->siteclock[i].def,strlen(cw1))==0){
           k=i;
           break;
       }
    }
    if(k < 0){
        printf("Clock parameters not found for %s\n",stat_id);
        goto exit2return;
    }
    //printf("NUMCLOCK def %d\n",vex->nclock);
    strcpy(clock->def,vex->siteclock[k].def);
    for(i=0; i<5; i++) {
        clock->validity_epoch[i]=vex->siteclock[k].validity_epoch[i];
        clock->clock_epoch[i]=vex->siteclock[k].clock_epoch[i];
    }
    clock->clock_offset=vex->siteclock[k].clock_offset;
    clock->clock_rate=vex->siteclock[k].clock_rate;
exit2return:;
}

void EOP_mon(vex_t *vex)  //2003-03-21
{
/*      $EOP block monitor  */
    int i,k,kk;
    printf("NUMEOP def %d\n",vex->neop);
    for (i=0; i<vex->neop; i++){
        printf("%d %s\n",i+1,vex->eop[i].def);
        printf(" eop ref epoch   : ");
        for(k=0; k<5; k++) printf(" %d",vex->eop[i].epoch[k]);
        printf("\n");
        printf(" eop_interval(hr): %d\n",vex->eop[i].interval_hr);
        printf(" eop_ponts : %d\n",vex->eop[i].points);
        kk=vex->eop[i].points;
        printf(" ut1-utc  : ");
        for(k=0; k<kk; k++) printf(" %f",vex->eop[i].ut1[k]);
        printf("\n");
        printf(" x_wobble : ");
        for(k=0; k<kk; k++) printf(" %f",vex->eop[i].xwob[k]);
        printf("\n");
        printf(" y_wobble : ");
        for(k=0; k<kk; k++) printf(" %f",vex->eop[i].ywob[k]);
        printf("\n");
    }
}

void EOP_mon2(vex_t *vex)  //2003-03-21
{
/*      $EOP block monitor  */
    int i,k,kk;
    k=-1;
    //find EOP data set
    for(i=0; i<vex->neop; i++){
       if(strncmp(vex->global.eop,vex->eop[i].def,strlen(vex->global.eop))==0){
           k=i;
           break;
       }
    }
    if(k<0) {
        printf("No EOP data for this experiment\n");
        goto exit2return;
    }

    //printf("NUMEOP def %d\n",vex->neop);
    printf("   %s\n",vex->eop[k].def);
    printf("   eop ref epoch   : ");
    printf("  %d/%03d %02d:%02d:%02d\n",
                vex->eop[k].epoch[0],
                vex->eop[k].epoch[1],
                vex->eop[k].epoch[2],
                vex->eop[k].epoch[3],
                vex->eop[k].epoch[4]);
    printf("   eop_interval(hr): %d\n",vex->eop[k].interval_hr);
    printf("   eop_points      : %d\n",vex->eop[k].points);
    kk=vex->eop[k].points;
    printf("   ut1-utc (s)     : ");
    for(i=0; i<kk; i++) printf(" %f",vex->eop[k].ut1[i]);
    printf("\n");
    printf("   x_wobble (asec) : ");
    for(i=0; i<kk; i++) printf(" %f",vex->eop[k].xwob[i]);
    printf("\n");
    printf("   y_wobble (asec) : ");
    for(i=0; i<kk; i++) printf(" %f",vex->eop[k].ywob[i]);
    printf("\n");
exit2return:;
}

void PCAL_mon(vex_t *vex)  //2003-07-16
{
/*      $PCAL block monitor  */
    int i,kk,m;
    printf("vex.npcal %d\n",vex->npcal);
    for(i=0; i<vex->npcal; i++){

        printf("PCAL def %s\n",vex->pcal[i].def);
        for(m=0; m< vex->pcal[i].npcal; m++){
            printf("  %s\n",vex->pcal[i].pcal[m].pcal_id);
            for(kk=0; kk< vex->pcal[i].pcal[m].ntone; kk++){
                printf(" %d\n",vex->pcal[i].pcal[m].tone[kk]);
            }
        }
    }
}

void BBC_mon(vex_t *vex)  //2003-07-16
{
/*      $BBC block monitor  */
    int i,m;
    printf("vex.nbbc %d\n",vex->nbbc);
    for(i=0; i<vex->nbbc; i++){

        printf("BBC def %s\n",vex->bbc[i].def);
        for(m=0; m< vex->bbc[i].nbbc; m++){
            printf("  %s ",vex->bbc[i].bbc_def[m].bbc_id);
            printf("  %d ",vex->bbc[i].bbc_def[m].bbcch);
            printf("  %s\n",vex->bbc[i].bbc_def[m].if_id);
        }
    }
}

void IF_mon(vex_t *vex)  //2003-07-16
{
/*      $IF block monitor  */
    int i,m;
    printf("vex.nifb %d\n",vex->nifb);
    for(i=0; i<vex->nifb; i++){

        printf("IF def %s\n",vex->ifb[i].def);
        for(m=0; m< vex->ifb[i].nif; m++){
            printf("  %s ",vex->ifb[i].if_def[m].if_id);
            printf("  %s ",vex->ifb[i].if_def[m].pol);
            printf("  %f ",vex->ifb[i].if_def[m].lo);
            printf("  %s ",vex->ifb[i].if_def[m].sideband);
            printf("  %f ",vex->ifb[i].if_def[m].pcal_space);
            printf("  %f\n",vex->ifb[i].if_def[m].pcal_base);
        }
    }
}

void  vex_read(vex_t *vex)  
{
    /*
       VEX file read and analyze by $BLOCK
    */
     int k,loop;
     char line[500];
     FILE *fin;
     vparam_t vp;

     if( (fin  = fopen(vex->vex_file_name, "r" )) == NULL ){
        printf("vex_read: File (%s) not found or Open Error! \n",vex->vex_file_name);
        exit(1);
     } 

     /* initialize counter */
     vex->nblock=0;
     loop=0;
     // first line analysis to identify VEX or OVEX
     if( fgets(line, 500, fin ) == NULL) goto exitend;
     k=vex_parse(line, &vp);
     if(strncmp(vp.param[0],"$OVEX",5)==0){  // This is OVEX format
         strcpy(vex->block[vex->nblock],vp.param[0]); vex->nblock++;
         strcpy(vex->flag,"OVEX");
     } else {
         strcpy(vex->flag,"VEX");
     }


mainloop:
     if( fgets( line, 500, fin ) == NULL) goto exitend;
     //printf("%s",line);
loop1:
     k=vex_parse(line, &vp);
     //if(line[0]=='$') printf("%s\n",line);

     //block analysis
     if(vp.n <= 0) goto mainloop;
     if(strncmp(vp.param[0],"$",1)==0){  // found block name
         strcpy(vex->block[vex->nblock],vp.param[0]); 
         vex->nblock++;
     }

     /* $GLOBAL analysis */
     if(strncmp(vp.param[0],"$GLOBAL",7)==0){  // GLOBAL
        if(GLOBAL_ana(fin, line, vex)==-1) goto exitend;
         //printf("GLOBAL analysis end\n");
         //printf("%s %s\n",vex->global.exper, vex->global.eop);
         goto loop1;
     } 

     /* $EXPER analysis */
     if(strncmp(vp.param[0],"$EXPER",7)==0){  // EXPER
        if(EXPER_ana(fin, line, vex)==-1) goto exitend;
        //printf("EXPER analysis end\n");
        //printf("exper = %s\n",vex->exper[0].exper_name);
        goto loop1;
     }

     /* $MODE analysis */
     if(strncmp(vp.param[0],"$MODE",5)==0){  // MODE
        if(MODE_ana(fin, line, vex)==-1) goto exitend;
        //printf("MODE analysis end\n");
        goto loop1;
     }

     /* $SCHED analysis */
     if(strncmp(vp.param[0],"$SCHED",6)==0){  // SCHED
         if(SKED_ana(fin, line, vex)==-1) goto exitend;
         //printf("SCHED analysis end\n");
         goto loop1;
     }

     /* $TRACKS analysis */
     if(strncmp(vp.param[0],"$TRACKS",7)==0){  // TRACKS
         if(TRACKS_ana(fin, line, vex)==-1) goto exitend;
         //printf("TRACK analysis end\n");
         goto loop1;
     }

     /* $SITE analysis */
     if(strncmp(vp.param[0],"$SITE",5)==0){  // SITE
         if(SITE_ana(fin, line, vex)==-1) goto exitend;
         //printf("SITE analysis end\n");
         goto loop1;
     }
     
     /* $STATION analysis */
     if(strncmp(vp.param[0],"$STATION",8)==0){  // SITE
         if(STATION_ana(fin, line, vex)==-1) goto exitend;
         //printf("STATION analysis end\n");
         goto loop1;
     }

     /* $SOURCE analysis */
     if(strncmp(vp.param[0],"$SOURCE",7)==0){  // SOURCE
         if(SOURCE_ana(fin, line, vex)==-1) goto exitend;
         //printf("SOURCE analysis end\n");
         goto loop1;
     }

     /* $FREQ analysis */
     if(strncmp(vp.param[0],"$FREQ",5)==0){  // FREQ
         if(FREQ_ana(fin, line, vex)==-1) goto exitend;
         //printf("FREQ analysis end\n");
         goto loop1;
     }

     /* $CLOCK analysis */
     if(strncmp(vp.param[0],"$CLOCK",6)==0){  // CLOCK
         if(CLOCK_ana(fin, line, vex)==-1) goto exitend;
         //printf("CLOCK analysis end\n");
         goto loop1;
     }

     /* $EOP analysis */
     if(strncmp(vp.param[0],"$EOP",4)==0){  // EOP
         if(EOP_ana(fin, line, vex)==-1) goto exitend;
         //printf("EOP analysis end\n");
         goto loop1;
     }


     /* $ROLL analysis */
     if(strncmp(vp.param[0],"$ROLL",5)==0){  // ROLL
         if(ROLL_ana(fin, line, vex)==-1) goto exitend;
         //printf("ROLL analysis end\n");
         //printf("ROLL %s %s\n",vex->roll[0].def,vex->roll[0].rolldef);
         goto loop1;
     }


     /* $PHASE_CAL_DETECT analysis */   //added on 2003-07-17

     if(strncmp(vp.param[0],"$PHASE_CAL_DETECT",17)==0){  // PHASE_CAL
         if(PCAL_ana(fin, line, vex)==-1) goto exitend;
         //printf("PHASE_CAL analysis end\n");
         //PCAL_mon(vex);
         goto loop1;
     }

     /* $BBC analysis */   //added on 2003-07-17

     if(strncmp(vp.param[0],"$BBC",4)==0){  // IF
         if(BBC_ana(fin, line, vex)==-1) goto exitend;
         //printf("BBC analysis end\n");
         //printf(" BBC debug %d\n",vex->nbbc);
         //BBC_mon(vex);
         goto loop1;
     }

     if(strncmp(vp.param[0],"$IF",4)==0){  // IF
         if(IF_ana(fin, line, vex)==-1) goto exitend;
         //printf("IF analysis end\n");
         //printf(" IF debug %d\n",vex->nifb);
         //IF_mon(vex);
         goto loop1;
     }
             
     if(k==1) goto loop1;
     loop++;
     //if(loop==50) goto exitend; // for check
     goto mainloop;
exitend:;

     /*      $SCHED block monitor  */
  //   SKED_mon(vex);

     /* $TRACKS monitor */
  //   TRACKS_mon(vex);

    /* $STATION monitor */
  //   STATION_mon(vex);

    /* $SITE monitor */
  //   SITE_mon(vex);

    /* $SOURCE monitor */
  //   SOURCE_mon(vex);

    /* $FREQ monitor */
  //   FREQ_mon(vex);

    /* $MODE monitor */
  //   MODE_mon(vex);

    /* $CLOCK monitor */
  //   CLOCK_mon(vex);

    /* $EOP monitor */
  //   EOP_mon(vex);
}


int search_site(vex_t *vex, char *key, site_t *site)  /* 2004-06-17 */
{
/* 
   Get site info correspond to 'key' in $MODE (station id)
       return=-1; not found   =0; found
*/

    int i,id,id2,iok;
    id=-1;
    for(i=0; i<vex->nstation; i++){
       if(strncmp(vex->station[i].def,key,strlen(key))==0){  // STATION found
              //printf("%s %s %s %s\n",vex.station[i].def,vex.station[i].site,
              //         vex.station[i].antenna,vex.station[i].clock);
          id=i;
          break;
       }
    }
    if(id < 0) return -1;

    // site position get
    id2=-1;
    for(i=0; i<vex->numstat; i++){
       if(strncmp(vex->station[id].site,vex->site[i].def,strlen(vex->site[i].def))==0){  // STATION found
            //printf("%s %s %s\n",vex.site[i].def,vex.site[i].name,
            //           vex.site[i].id);
            //printf("%f %f %f\n",vex.site[i].position[0],
            //            vex.site[i].position[1],vex.site[i].position[2]);
            *site=vex->site[i];
            id2=i;
            break;
       }
    }
    if(id2<0) return -1;

    // site clock info get
    iok=-1;
    for(i=0; i<vex->nclock; i++){
       if(strncmp(vex->station[id].clock,vex->siteclock[i].def,strlen(vex->siteclock[i].def))==0){  // STATION found
            //printf("%s %s %s\n",vex.site[i].def,vex.site[i].name,
            //           vex.site[i].id);
            //printf("%f %f %f\n",vex.site[i].position[0],
            //            vex.site[i].position[1],vex.site[i].position[2]);
            site->sclock=vex->siteclock[i];
            iok=0;
            break;
       }
    }
    if(iok<0) {
        printf("search_site: No CLOCK info for %s included in the VEX FILE.\n",
            site->name);
        printf("search_site: So all 0 for clock information was set.\n");
        for(i=0; i<5; i++) site->sclock.clock_epoch[i]=0;
        for(i=0; i<5; i++) site->sclock.validity_epoch[i]=0;
        site->sclock.clock_offset=0.0;
        site->sclock.clock_rate=0.0;
        site->sclock.def[0]='\0';
    }
    return 0;
}


freq_info_t vex_freq_get(vex_t *vex, char *mode, char *sid, char *chanid)
//2003-10-28
{
    int i,ipos,k,kf,iok, kp,j,kbbc,kif;
    char freqid[30];//, chanid[30];
    char pcalid[30], pcalmode[50], bbcid[30],bbcmode[50],ifid[30],ifmode[50];
    int tones[10],ntone;
    int nji;
    double pcalspace,pcalbase, pcalf;
    freq_info_t freq;

    // initialize
    freq.npcal=0;
    freq.pcalf[0]=-1.0;
    freq.npcal=0;
    freq.rf=0.0;
    freq.sideband[0]='\0';
    freq.vbw=0.0;
    strcpy(freq.chanid,chanid);
    freq.nch=0;
    freq.chmax=0;
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return freq;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas1 %d %s\n",i,vex->mode[ipos].mode_freq[i].id);
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    // bbc mode finding for sid

    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas2 %d %s\n",i,vex->mode[ipos].mode_bbc[i].id);
        if(strncmp(vex->mode[ipos].mode_bbc[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(bbcmode,vex->mode[ipos].mode_bbc[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // ifmode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas3 %d %s\n",i,vex->mode[ipos].mode_if[i].id);
        if(strncmp(vex->mode[ipos].mode_if[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(ifmode,vex->mode[ipos].mode_if[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // pcal_detection mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas4 %d %s\n",i,vex->mode[ipos].mode_pcal_detect[i].id);
        if(strncmp(vex->mode[ipos].mode_pcal_detect[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(pcalmode,vex->mode[ipos].mode_pcal_detect[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    /*freq data set up*/
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    // get frequency info from chanid
    //printf("debug chan id %s\n",chanid);//debug
    iok=-1;
    freq.chmax=vex->freq_def[kf].numch;
    for(k=0; k<vex->freq_def[kf].numch; k++){
        if(strncmp(vex->freq_def[kf].chan_def[k].chan_id,chanid,strlen(chanid))==0) {  // MODE found
            freq.rf=vex->freq_def[kf].chan_def[k].rf;
            strcpy(freq.sideband,vex->freq_def[kf].chan_def[k].sideband);
            freq.vbw=vex->freq_def[kf].chan_def[k].vbw;
 //printf("debug chan id  k freq.rf %s %d %f\n",chanid,k,freq.rf);//debug
            // pcal id get
            strcpy(pcalid,vex->freq_def[kf].chan_def[k].pcal_id);
            // bbc id get
            strcpy(bbcid,vex->freq_def[kf].chan_def[k].bbc_id);
            freq.nch=k+1;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;

    //pcal tone info get
    for(i=0; i<vex->npcal; i++){
        if(strncmp(vex->pcal[i].def,pcalmode,strlen(pcalmode))==0) {  // MODE found
            kp=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;
    for(k=0; k< vex->pcal[kp].npcal; k++){
        if(strncmp(vex->pcal[kp].pcal[k].pcal_id,pcalid,strlen(pcalid))==0){
            ntone=vex->pcal[kp].pcal[k].ntone;
            for(j=0; j<ntone; j++){
                tones[j]=vex->pcal[kp].pcal[k].tone[j];
            }
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    //pcal freq info get through BBC info
    iok=-1;
    for(i=0; i<vex->nbbc; i++){
        if(strncmp(vex->bbc[i].def,bbcmode,strlen(bbcmode))==0) {  // MODE found
            kbbc=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;
    for(k=0; k< vex->bbc[kbbc].nbbc; k++){
        if(strncmp(vex->bbc[kbbc].bbc_def[k].bbc_id,bbcid,strlen(bbcid))==0){
            strcpy(ifid,vex->bbc[kbbc].bbc_def[k].if_id);  //if_id get
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //if_id analyze
    iok=-1;
    for(i=0; i<vex->nifb; i++){
        if(strncmp(vex->ifb[i].def,ifmode,strlen(ifmode))==0) {  // MODE found
            kif=i;
            iok=0;
            break;
        }
    }
    for(k=0; k< vex->ifb[kif].nif; k++){
        if(strncmp(vex->ifb[kif].if_def[k].if_id,ifid,strlen(ifid))==0){
            pcalspace=vex->ifb[kif].if_def[k].pcal_space;
            pcalbase =vex->ifb[kif].if_def[k].pcal_base;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //pcal frequency calc
    iok=-1;

    if(fabs(pcalspace) < 0.1 ) return freq;
    // following is valid for USB  
    if(freq.sideband[0]=='U' || freq.sideband[0]=='u'){
       /* new algorithm was applied on 2003-10-28 */
       if(ntone > 0){
           for(i=0; i<ntone; i++){
               k=tones[i];
               if(k > 0) {
                   nji=(int)(freq.rf/pcalspace);
                   pcalf=pcalspace*(double)(nji+k);
                   freq.pcalf[i]=pcalf-freq.rf;
               }
               if(k < 0) {
                   nji=(int)((freq.rf+freq.vbw)/pcalspace);
                   pcalf=pcalspace*(double)(nji+k+1);
                   freq.pcalf[i]=pcalf-freq.rf;
               }
               if(k == 0) {  // nothing to do
                   freq.pcalf[i]=0.0;   
               }
           }
       }
       /* old algorithm before 2003-10-28 version */       
       //nji=(int)(freq.rf/pcalspace)-1;
       //k=0;
       //i=0;
       //if(ntone > 0) {
       //   while(1){
       //      pcalf=pcalspace*(double)nji;
       //      if(pcalf > freq.rf) k++;
       //      if(k==tones[i]){
       //         freq.pcalf[i]=pcalf-freq.rf;
       //         i++;
       //         if(i >= ntone) break;
       //      } 
       //     nji++;
       //   } 
       //} 
    }
    // following is valid for LSB  
    if(freq.sideband[0]=='L' || freq.sideband[0]=='l'){
       /* new algorithm was applied on 2003-10-28 */
       if(ntone > 0){
           for(i=0; i<ntone; i++){
               k=tones[i];
               if(k > 0) {
                   nji=(int)(freq.rf/pcalspace);
                   pcalf=pcalspace*(double)(nji-k+1);
                   freq.pcalf[i]=freq.rf-pcalf;
               }
               if(k < 0) {
                   nji=(int)((freq.rf-freq.vbw)/pcalspace);
                   pcalf=pcalspace*(double)(nji-k);
                   freq.pcalf[i]=freq.rf-pcalf;
               }
               if(k == 0) {  // nothing to do
                   freq.pcalf[i]=0.0;   
               }
           }
       }
       /* old algorithm before 2003-10-28 version */       
       //nji=(int)(freq.rf/pcalspace)+2;
       //k=0;
       //i=0;
       //if(ntone > 0) {
       //   while(1){
       //      pcalf=pcalspace*(double)nji;
       //      if(pcalf < freq.rf) k++;
       //      if(k==tones[i]){
       //         freq.pcalf[i]=freq.rf-pcalf;
       //         i++;
       //         if(i >= ntone) break;
       //      } 
       //     nji--;
       //   } 
       //} 
    }
    freq.npcal=ntone;
    return freq;
}


freq_info_t vex_chinfo_get(vex_t *vex, char *mode, char *sid, int nch)
//2003-07-17
//   nch ranging 1 - 16 
{
    int i,ipos,k,kf,iok, kp,j,kbbc,kif;
    char freqid[30];//, chanid[30];
    char pcalid[30], pcalmode[50], bbcid[30],bbcmode[50],ifid[30],ifmode[50];
    int tones[10],ntone;
    int nji;
    double pcalspace,pcalbase, pcalf;
    freq_info_t freq;

    // initialize
    // initialize
    freq.npcal=0;
    freq.pcalf[0]=-1.0;
    freq.npcal=0;
    freq.rf=0.0;
    freq.sideband[0]='\0';
    freq.vbw=0.0;
    freq.chanid[0]='\0';
    freq.nch=nch;
    freq.chmax=0;
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return freq;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas1 %d %s\n",i,vex->mode[ipos].mode_freq[i].id);
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // bbc mode finding for sid

    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas2 %d %s\n",i,vex->mode[ipos].mode_bbc[i].id);
        if(strncmp(vex->mode[ipos].mode_bbc[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(bbcmode,vex->mode[ipos].mode_bbc[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // ifmode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_if[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(ifmode,vex->mode[ipos].mode_if[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // pcal_detection mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_pcal_detect[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(pcalmode,vex->mode[ipos].mode_pcal_detect[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    /*freq data set up*/
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    freq.chmax=vex->freq_def[kf].numch;
    //chanid get for nch
    if(nch <=0 || nch > vex->freq_def[kf].numch) return freq;
    strcpy(freq.chanid,vex->freq_def[kf].chan_def[nch-1].chan_id);
    // get frequency info 
    freq.rf=vex->freq_def[kf].chan_def[nch-1].rf;
    strcpy(freq.sideband,vex->freq_def[kf].chan_def[nch-1].sideband);
    freq.vbw=vex->freq_def[kf].chan_def[nch-1].vbw;
    // pcal id get
    strcpy(pcalid,vex->freq_def[kf].chan_def[nch-1].pcal_id);
    // bbc id get
    strcpy(bbcid,vex->freq_def[kf].chan_def[nch-1].bbc_id);

    iok=-1;

    //pcal tone info get
    for(i=0; i<vex->npcal; i++){
        if(strncmp(vex->pcal[i].def,pcalmode,strlen(pcalmode))==0) {  // MODE found
            kp=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;
    for(k=0; k< vex->pcal[kp].npcal; k++){
        if(strncmp(vex->pcal[kp].pcal[k].pcal_id,pcalid,strlen(pcalid))==0){
            ntone=vex->pcal[kp].pcal[k].ntone;
            for(j=0; j<ntone; j++){
                tones[j]=vex->pcal[kp].pcal[k].tone[j];
            }
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    //pcal freq info get through BBC info
    iok=-1;
    for(i=0; i<vex->nbbc; i++){
        if(strncmp(vex->bbc[i].def,bbcmode,strlen(bbcmode))==0) {  // MODE found
            kbbc=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;
    for(k=0; k< vex->bbc[kbbc].nbbc; k++){
        if(strncmp(vex->bbc[kbbc].bbc_def[k].bbc_id,bbcid,strlen(bbcid))==0){
            strcpy(ifid,vex->bbc[kbbc].bbc_def[k].if_id);  //if_id get
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //if_id analyze
    iok=-1;
    for(i=0; i<vex->nifb; i++){
        if(strncmp(vex->ifb[i].def,ifmode,strlen(ifmode))==0) {  // MODE found
            kif=i;
            iok=0;
            break;
        }
    }
    for(k=0; k< vex->ifb[kif].nif; k++){
        if(strncmp(vex->ifb[kif].if_def[k].if_id,ifid,strlen(ifid))==0){
            pcalspace=vex->ifb[kif].if_def[k].pcal_space;
            pcalbase =vex->ifb[kif].if_def[k].pcal_base;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //pcal frequency calc
    iok=-1;

    if(fabs(pcalspace) < 0.1 ) return freq;
    // following is valid for USB now 
    if(freq.sideband[0]=='U' || freq.sideband[0]=='u'){
       nji=(int)(freq.rf/pcalspace)-1;
       k=0;
       i=0;
       if(ntone > 0) {
          while(1){
             pcalf=pcalspace*(double)nji;
             if(pcalf > freq.rf) k++;
             if(k==tones[i]){
                freq.pcalf[i]=pcalf-freq.rf;
                i++;
                if(i >= ntone) break;
             } 
            nji++;
          } 
       } 
    }
    // following is valid for LSB now 
    if(freq.sideband[0]=='L' || freq.sideband[0]=='l'){
       nji=(int)(freq.rf/pcalspace)+2;
       k=0;
       i=0;
       if(ntone > 0) {
          while(1){
             pcalf=pcalspace*(double)nji;
             if(pcalf < freq.rf) k++;
             if(k==tones[i]){
                freq.pcalf[i]=freq.rf-pcalf;
                i++;
                if(i >= ntone) break;
             } 
            nji--;
          } 
       } 
    }
    freq.npcal=ntone;
    return freq;
}



int search_track_freq(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq)
//2004-06-29   
{
/*
       Get track and frequency info correspond to 'def' mode in $MODE for 
       station id sid
       return=-1; not found   =0; found
       pcal frequency info was added to search_track_freq2
       
       2004-06-29 sub-pass was added in track_freq_sub

*/

    int i,ipos,k,kk,kt,kf,iok,m;
    char freqid[30], trackid[30], rollid[30], chanid[30];
    freq_info_t freq_info;

    trfreq->ntrack=0;
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return -1;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->freqdef,freqid);
 
    // track mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_track[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(trackid,vex->mode[ipos].mode_track[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->trackdef,trackid);

    // roll mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_roll[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(rollid,vex->mode[ipos].mode_roll[i].mode);
            //printf("DEBUG!!: %s\n",rollid);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;


    //track data set up
    iok=-1;
    for(i=0; i<vex->ntrack_def; i++){
        if(strncmp(vex->track_def[i].def,trackid,strlen(trackid))==0) {  // MODE found
            kt=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;


    //freq data set up
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;

    //found roll mode and value
    for(i=0; i<vex->nroll; i++){
        if(strncmp(vex->roll[i].def,rollid,strlen(rollid))==0) {  // MODE found
           strcpy(trfreq->roll,vex->roll[i].rolldef);
           break;
        }
    }

    trfreq->samplrate=vex->freq_def[kf].samplerate;
    trfreq->adbit=1;
    trfreq->nfreq=vex->freq_def[kf].numch;

    m=0;
    trfreq->fanout=vex->track_def[kt].fanout_def[0].fanout;
    //printf("Debug 1 %d \n",trfreq->fanout);
    for(i=0; i<vex->track_def[kt].nfanout_def; i++){
        for(k=0; k<vex->track_def[kt].fanout_def[i].fanout; k++){
            strcpy(trfreq->trk[m].chan_id,vex->track_def[kt].fanout_def[i].chan_id);
            strcpy(chanid,vex->track_def[kt].fanout_def[i].chan_id);
            //printf("%s\n",vex->track_def[kt].fanout_def[m].stream_info);
            strcpy(trfreq->trk[m].bitstream,vex->track_def[kt].fanout_def[i].stream_info);
            if(strncmp(trfreq->trk[m].bitstream,"mag",3)==0) trfreq->adbit=2;

            trfreq->trk[m].fan_out_pos=k+1;
            trfreq->trk[m].track=vex->track_def[kt].fanout_def[i].track[k];
            //printf("%d %d %d\n",i,k,vex->track_def[kt].fanout_def[m].track[k]);
            strcpy(trfreq->trk[m].sub_pass,vex->track_def[kt].fanout_def[i].sub_pass); //2004-06-29
            trfreq->trk[m].headstack=vex->track_def[kt].fanout_def[i].headstack; // added 2004-09-13
            
            // get frequency info from chan id
            //printf("debug mode,sid,chan id %s %s %s\n",mode,sid,chanid);//debug
            freq_info=vex_freq_get(vex,mode,sid,chanid);
            //printf("i,k %d %d\n",i,k);  //debug
            trfreq->trk[m].rf=freq_info.rf;
            strcpy(trfreq->trk[m].sideband,freq_info.sideband);
            trfreq->trk[m].vbw=freq_info.vbw;
            trfreq->trk[m].npcal=freq_info.npcal;
            for(kk=0; kk< freq_info.npcal; kk++){
                trfreq->trk[m].pcalf[kk]=freq_info.pcalf[kk];
            }
            m++;
        }
    }
    trfreq->ntrack=m;
    return 0;
}

int search_track_freq2(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq)
//2003-07-12 debug
{
/*
       Get track and frequency info correspond to 'def' mode in $MODE for 
       station id sid
       return=-1; not found   =0; found
       pcal frequency info was addwd to search_track_freq2
*/

    int i,ipos,k,kk,kt,kf,iok,m;
    char freqid[30], trackid[30], rollid[30], chanid[30];

    trfreq->ntrack=0;
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return -1;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->freqdef,freqid);
 
    // track mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_track[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(trackid,vex->mode[ipos].mode_track[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->trackdef,trackid);

    // roll mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_roll[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(rollid,vex->mode[ipos].mode_roll[i].mode);
            //printf("DEBUG!!: %s\n",rollid);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;


    //track data set up
    iok=-1;
    for(i=0; i<vex->ntrack_def; i++){
        if(strncmp(vex->track_def[i].def,trackid,strlen(trackid))==0) {  // MODE found
            kt=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;

    //freq data set up
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;

    //found roll mode and value
    for(i=0; i<vex->nroll; i++){
        if(strncmp(vex->roll[i].def,rollid,strlen(rollid))==0) {  // MODE found
           strcpy(trfreq->roll,vex->roll[i].rolldef);
           break;
        }
    }

    trfreq->samplrate=vex->freq_def[kf].samplerate;
    trfreq->adbit=1;
    trfreq->nfreq=vex->freq_def[kf].numch;

    m=0;
    trfreq->fanout=vex->track_def[kt].fanout_def[0].fanout;
    for(i=0; i<vex->track_def[kt].nfanout_def; i++){
        for(k=0; k<vex->track_def[kt].fanout_def[i].fanout; k++){
            strcpy(trfreq->trk[m].chan_id,vex->track_def[kt].fanout_def[i].chan_id);
            strcpy(chanid,vex->track_def[kt].fanout_def[i].chan_id);
            //printf("%s\n",vex->track_def[kt].fanout_def[m].stream_info);
            strcpy(trfreq->trk[m].bitstream,vex->track_def[kt].fanout_def[i].stream_info);
            if(strncmp(trfreq->trk[m].bitstream,"mag",3)==0) trfreq->adbit=2;

            trfreq->trk[m].fan_out_pos=k+1;
            trfreq->trk[m].track=vex->track_def[kt].fanout_def[i].track[k];
            //printf("%d %d %d\n",i,k,vex->track_def[kt].fanout_def[m].track[k]);
            
            // get frequency info from chan id
            //printf("debug chan id %s\n",chanid);//debug
            iok=-1;
            for(kk=0; kk<vex->freq_def[kf].numch; kk++){
               if(strncmp(vex->freq_def[kf].chan_def[kk].chan_id,chanid,strlen(chanid))==0) {  // MODE found
                   //printf("debug chan id  kk %s %d %f\n",chanid,kk,);//debug
                   iok=0;
                   //trfreq->trk[m].rf=vex->freq_def[kf].chan_def[k].rf;
                   //strcpy(trfreq->trk[m].sideband,vex->freq_def[kf].chan_def[k].sideband);
                   //trfreq->trk[m].vbw=vex->freq_def[kf].chan_def[k].vbw;
                   trfreq->trk[m].rf=vex->freq_def[kf].chan_def[kk].rf;                     //2003-07-12
                   strcpy(trfreq->trk[m].sideband,vex->freq_def[kf].chan_def[kk].sideband); //2003-07-12
                   trfreq->trk[m].vbw=vex->freq_def[kf].chan_def[kk].vbw;                   //2003-07-12
               } 
            } 
            if(iok<0) return -1;
            m++;
        }
    }
    trfreq->ntrack=m;
    return 0;
}


void create_siteid_table(vex_t *vex, int *nstation, sidtable_t *siteid)  //2003-03-21
{
/*
    Create site id (corresponds to 'def' keyword in $STAION) table
*/

    int i;
    *nstation=vex->nstation;
    for(i=0; i<vex->nstation; i++){
       strcpy(siteid[i].sid,vex->station[i].def);
    }
}



int check_block(vex_t *vex)  //2003-03-21
{
/* 
     Check $BLOCK names that contains all blocks necessary for
     correlation processing or not

     return 0: OK,  -1 for no good

*/

    int i,k,iok,iok2;
    char kijun1[][20]={"$GLOBAL","$MODE","$STATION","$SITE","$TRACKS",
        "$FREQ","$ROLL","$SCHED","$SOURCE","$CLOCK","$EOP",
        "$BBC","$IF","$PHASE_CAL_DETECT"};
    int nkijun1=14;
    char *kijun2="$EOP";
    iok2=0;
    for(k=0; k<nkijun1; k++){
       iok=-1;
       for(i=0; i<vex->nblock; i++){
           if(strncmp(vex->block[i],kijun1[k],strlen(kijun1[k]))==0) {
               iok=0;
               break;
           }
       }
       if(iok<0){
           printf("%s is not found in VEX file\n",kijun1[k]);
           iok2=-1;
       }
    }
    if(iok2<0) return -1;

    return 0;
}

#endif
