/*************************************************************************/
/*                                                                       */
/*   Program Name :  psrobs                                              */
/*   Updated by Vlad Kondratiev - maximum file size was changed          */
/*                                file names are changed from XXXX.datN  */
/*                                 to XXXX.dat000N                       */
/*                                                                       */
/*     Autoobs carries out IP-VLBI observations automatically            */
/*          according to a schedule file                                 */
/*                                                                       */
/*     Enviroment variables                                              */
/*           K5SKED     ---   default directory for schedule file        */
/*           K5OUTINFO   --   default output directory info file name    */
/*           K5FREQG     --   default frequency group of PC              */
/*           K5NAMING    --   default out file naming rule               */
/*           K5SUBNET    --   default subnet mode                        */
/*           K5SATKEY    --   default satellite keyword for 1ch obs mode */
/*                                                                       */
/*      Version:  0.0   2001-08-17                                       */
/*                2.21  2002-07-22                                       */
/*                3.01  2003-06-20                                       */
/*                3.02  2003-07-20  add env variable monitor             */
/*                3.10  2003-07-31  add naming rule selection            */
/*                3.11  2003-08-02  add K5SKED env variable              */
/*                3.20  2003-08-09  add subnet obs mode                  */
/*                3.21  2003-11-01  add K5SATKEY                         */
/*                3.22  2003-11-20  add file size non-limit mode         */
/*                3.23  2003-11-22  bug fix in Ver 3.22 (had a bug for   */
/*                                  4ch mode)                            */
/*                3.30  2003-11-27  Allow multi satellite keywords.      */
/*                                  Allow satellite keywords set in an   */
/*                                  run information file.                */
/*                                  Modify information displayed on CRT  */
/*                                  so as to give information about      */
/*                                  sampling mode of the next scan       */
/*                3.31  2004-05-21  Debug in K5SUBNET environment        */
/*                                  variable analysis.                   */
/*                                  Allow max 8 character satellite key  */
/*                3.40  2004-05-25  Reflect the updated "sampleget"      */
/*                                                                       */
/*                                                                       */
/*      Copyright (c) 2002-2004 T.Kondo/NICT All Right Reserved          */
/*                                                                       */
/*************************************************************************/
 
/*
ho|ukah@ItCϑvO(2000ϑсû݁vϑT|[gj
        iZbg͕ʂ̃\tgōsj
         by T.Kondo 2003.11.1

        uNOZOMIvϑ͂Pch[hAϑ͂Sch[hɎIɐ؂ւAfBXNeʂ
               ߖsB
               PF1chTv[{[hɂ4chTv[{[hch1ƓrfIM
               ͂邱ƁI
               QFû݁vϑ͓dg̍ŏ̂R"NOZ"ǂŔB
               ϐK5SATKEYł̃L[[hZbgiύXj\B
               CvO K5SATKEYDFLT ύX邱ƂŃftHg̉qɕύX

    点  autoobs sked_file station_id [span [sfreq [adbit [numch [outdir [y_or_n][]]]]]
           sked_file ---- XPW[t@C
                          擪'-'tƁAp[^t@Cirun information file)A
                          ȉ̃p[^͕svijB
                          
                staion_id ----  ID (XPW[t@CŋLqĂǂhc)
                span ---- ϑ (sec)
                          iXPW[t@CŋK肵Ă
                           ϑԂZړIj
                           ftHg͂POb
                           XPW[t@Cʂ̊ϑԂƂƂ
                           OƂ
                sfreq  TvOg
                       40,100,200,500  (kHz)
                       1,2,4,8,16      (MHz)
                          ftHg16
                adbit  `c\irbg)
                       1,2,4,8
                          ftHg1
                numch  `l
                       1@܂ 4
                          ftHg1
                outdir  f[^t@Co̓fBNg
                          擪'-'tƁAo̓fBNg؂ւt@CwB
                          ftHg̓JgfBNg
                y_or_n  ϑ͈͂𐧌ꍇ@Y,@ׂĊϑ̏ꍇN
                        ȗ͑Θbŉ񓚂
                naming_type o̓t@C@@
                       1:]^CviftHgj
                           XDDDNNNN.dat  
                             
                              X    -- ǂhc(P) 
                              DDD  -- Ԗڂ̃XL̒ʓ
                              NNNN -- ϑԍ  (S)
                        2:V^Cv
                            sidDDDHHMMSSG.dat
                             
                               sid -- ID (PQ)
                               DDD -- XLJnʓiRj
                               HH  -- XLJniQj
                               MM  -- XLJniQj
                               SS  -- XLJnbiQj
                               G   -- gO[vID (a|b|c|d) ܂
                                       null
                freqg   V^Cvt@CŎgpgO[vhc
                        (1|2|3|4)܂(a|b|c|d)iftHgnullj
                subnet   --  0: subnet mode off  1:subnet on (default)
                f_size_mode -- 0: no limitation  1: divided into each 2GB size(default) -- !!! updated (200MB)


                        
      ϐ
          K5SKED   --- XPW[t@CTfBNg
          K5LOGDIR --- Ot@Co͂fBNg
                       ftHg̓JgfBNg
          K5OUTINFO -- o̓fBNgt@C
          K5NAMING --- ϑt@C@ols | new
          K5FREQG  --- ob̎󂯎gO[v@1|2|3|4 ܂́@a|b|c|d
          K5SUBNET --- ϑ̃Tulbg[h on | off
          K5SATKEY --- 1chϑsqR[h  2003-11-01 ǉ

*/
#define AUTOOBS_VERSION "Ver 3.40    2004-05-25"
#include "chead.h"      // common header for IP-VLBI utilities
#ifndef WIN32
#include <sys/tdsio.h>  // IP-VLBI board header
#endif
#include "libsked.h"     // schedule file analizing functions
#include "libipvlbi.h"  // IP-VLBIp֐Q(by T.Kondo)

#ifdef WIN32
#define SIMULATION       // IP-BOARD simulation mode ON
#endif

#ifdef SIMULATION
#include "libipvlbi_board_dummy.h"  // IP-VLBITv[{[hi_~[j֐Q(by T.Kondo)
#else
#include "libipvlbi_board.h"  // IP-VLBITv[{[hp֐Q(by T.Kondo)
#endif

#define SLEEP_USEC 10000    // watch dog timer sep (10usec)
#define SLEEP_MSEC 10       // watch dog timer sep (10msec)

#define GET_TIME_FROM_BOARD 1  //  ^p̎{[h瓾邩ob̎瓾邩̔
	                           //  1: {[h    0:ob
#define LOGFILE_TYPE 0   // log file name type
                         //  0: exocode+station_id+"."+hostname+".log"
                         //  1: YYYY+hostname+".log"
                         //       where  YYYY is 4 digit year
                         //  -1: no log
/* environment variables definition */
#define K5SKED    "K5SKED"     // dfault directory for schedule file
#define K5OUTINFO "K5OUTINFO"  // dfault for output directory info file name
#define K5LOGDIR  "K5LOGDIR"   // default for log out directry
#define K5FREQG   "K5FREQG"    // default frequency group of PC  (1|2|3|4 or a|b|c|d)
#define K5NAMING  "K5NAMING"   // default out file naming rule  "old" | "new"  
#define K5SUBNET  "K5SUBNET"   // default subnet mode  "on" | "off"  
#define K5SATKEY  "K5SATKEY"   // default satellite code to switch to 1ch mode  

/* default used in program set */
#define K5SKEDDFLT    "./"             // dfault directory for schedule file
#define K5OUTINFODFLT "k5outinfo.txt"  // default out directories 2nd(1st), 3rd(2nd),....
#define K5LOGDIRDFLT  "./"             // default log file directory
#define K5FREQGDFLT   "null"           // default frequency group (NULL)
#define K5NAMINGDFLT  "old"            // default file naming rule
#define K5SUBNETDFLT  "on"             // default subnet mode
//#define K5SATKEYDFLT  "NOZ"            // default satellite code for 1ch mode
#define K5SATKEYDFLT  "HYB"            // default satellite code for 1ch mode

// global variables
//char logdir[60];               // logfile directory
//char hostname[20];             // host name
//char logfile[80];              //log file name
//int logfiletype=LOGFILE_TYPE;  // logfile name type
//  now structure variable 
logval_t lg;


typedef struct listoutdir    
{            
    int  dirnum;
    char dir[10][80];
} listoutdir_t;

typedef struct satkeys    
{            
    int  num;
    char keys[10][9];  // support 10 max 8-char satellite keys
} satkeys_t;

/* set interrupt handler */
void Handler(int n);

void SetHandler(void)
{
    if(signal(SIGINT, Handler) == SIG_ERR){
        printf("Signal handler set up failure!\n");
        abort();
    }
}

void Handler(int n)
{
     //static int x=0;
     char cbuf[80];
     printf("\n\n Interrupt occurred !!\n");
     sprintf(cbuf,"Interrupt occurred !!  AUTOOBS_S OBSERVATION TERMINATED by OPERATOR");
     //LogWrite3(logdir,hostname,logfile,logfiletype,0,0.0,3,cbuf);
     LogWrite4(&lg,0,0.0,3,cbuf);
     exit(0);
}


void p_parse(char *cbuf, char *para1, char *para2)  //2003-06-18
{
/*
    parse   a = c   to  a (para1)  and c (para2)
*/
    // vex_parse =0;  single statement in a cbuf
    //           =1;  more statement left in a cbuf, start point in cbuf 
    //                is updated 
    char wbuf[10][60];
    int i,ipos,k,kpos,flag,len;

    len=strlen(cbuf);

    //remove extra space in front

    if(cbuf[0]==' '){
        kpos=0;
        for(k=0; k<len; k++){
            if(cbuf[k]!=' '){
                kpos=k;
                break;
            }
        }
        i=0;
        for(k=kpos; k<len; k++){
            cbuf[i]=cbuf[k];
            i++;
        }
        cbuf[i]='\0';
    }
    // len again

    len=strlen(cbuf);


    // parse start

    i=0;
    ipos=0;
    flag=0;
    for(k=0; k<len; k++){
        if(cbuf[k] == ' ' || cbuf[k] == '=' || cbuf[k] == ':'){
            if(flag==1) {  // one parameter end
                wbuf[ipos][i]='\0';
                flag=0;
                i=0;
                ipos++;
                if(ipos > 3) goto exithere;
            }
            if(cbuf[k] != ' '){
                wbuf[ipos][i]=cbuf[k];
                wbuf[ipos][i+1]='\0';
                flag=0;
                i=0;
                ipos++;
                if(ipos > 3) goto exithere;
            }
            goto nextk;
        }
        if(cbuf[k] == ';'){
            kpos=k;
            goto exithere;
        }
        if(cbuf[k] != ' '){
            flag=1;
            wbuf[ipos][i]=cbuf[k];
            i++;
        }
nextk:;
    }
    kpos=k;
exithere:
    strcpy(para1,wbuf[0]);
    strcpy(para2,wbuf[2]);

}

void dir_name_check(char *cbuf)
{
    /* check out dir name accompanied by "/" or not */
    int i;
    i=strlen(cbuf);
    if(cbuf[i-1] != '/') {
       cbuf[i]='/';
       cbuf[i+1]='\0';
    }
}

void get_run_info(char *run_info_file, char *skdname, int *fixspan, int *sfreq, 
                  int *adbit, int *numch, char *stat_id, char *logdir, 
                  listoutdir_t *outdirtab, int *naming_type, char *freqg, 
                  int *subnet, int *f_size_mode, char *keyword)
{
       int i,k,j,len;
       char ibuf[100];
       char cbuf[80],para1[20],para2[20];
       char freqgtab[][3]={"a","b","c","d"};
       FILE *stream;
              

       if( (stream  = fopen( run_info_file, "r" )) == NULL ){
          printf("get_run_info: File (%s) not found or Open Error! \n",run_info_file);
          goto exitend2;
       }

  loop1:

       if( fgets( ibuf, 100, stream ) == NULL){
          //printf("File EOF or Read Error!\n");
          goto exitend;
       }
       if(ibuf[0]=='*')  // this is comment line
          goto exit1;
       if(ibuf[0]=='$'){ // this is command line
  cmd_entry:
          if(strncmp(ibuf,"$SKED",5)==0){    // schedule file name get
          sloop1:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 2) goto sloop1;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop1;  // carrige return code check
              if(ibuf[0]=='*') goto sloop1;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", skdname);
              goto loop1;
          }
          
          if(strncmp(ibuf,"$OUT",4)==0){    // out directory info get
             outdirtab->dirnum=0;
          sloop2:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 2) goto sloop2;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop2;  // carrige return code check
              if(ibuf[0]=='*') goto sloop2;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              dir_name_check(cbuf);
              strcpy(outdirtab->dir[outdirtab->dirnum],cbuf);
              outdirtab->dirnum++;
              goto sloop2;
          }
          
          
          if(strncmp(ibuf,"$SAMPL",6)==0){    // sampling parameters get
          sloop3:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 2) goto sloop3;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop3;  // carrige return code check
              if(ibuf[0]=='*') goto sloop3;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              p_parse(ibuf, para1, para2);
              if(strncmp(para1,"span",4)==0 || strncmp(para1,"fixs",4)==0 ){    // span
                  *fixspan=atoi(para2);
              }
              if(strncmp(para1,"sfreq",5)==0){    // sampling frequency
                  *sfreq=atoi(para2);
              }
              if(strncmp(para1,"adbit",5)==0){    // adbit
                  *adbit=atoi(para2);
              }
              if(strncmp(para1,"numch",5)==0){    // numch
                  *numch=atoi(para2);
              }
              goto sloop3;
          }
          
          if(strncmp(ibuf,"$STAT",5)==0){    // station ID  get
          sloop4:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop4;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop4;  // carrige return code check
              if(ibuf[0]=='*') goto sloop4;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              strcpy(stat_id,cbuf);
              goto loop1;
          }
          
          
          if(strncmp(ibuf,"$LOG",4)==0){    // Log directory  get
          sloop5:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop5;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop5;  // carrige return code check
              if(ibuf[0]=='*') goto sloop5;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              dir_name_check(cbuf);
              strcpy(logdir,cbuf);
              goto loop1;
          }
          
          if(strncmp(ibuf,"$NAMING",7)==0){    // Naming type get
          sloop6:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop6;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop6;  // carrige return code check
              if(ibuf[0]=='*') goto sloop6;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              *naming_type=atoi(cbuf);
              goto loop1;
          }
          
          if(strncmp(ibuf,"$FREQ_G",7)==0){    // Frequency Group get
          sloop7:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop7;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' ')) goto sloop7;  // carrige return code check
              if(ibuf[0]=='*') goto sloop7;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              j=0;
              if(cbuf[0]=='1') j=1;
              if(cbuf[0]=='2') j=2;
              if(cbuf[0]=='3') j=3;
              if(cbuf[0]=='4') j=4;
              if(j!=0){
                  strcpy(freqg,freqgtab[j-1]);
              } else {
                  strcpy(freqg,cbuf);
              }
              goto loop1;
          }
          
          if(strncmp(ibuf,"$SUBNET",7)==0){    // subnet mode get get
          sloop8:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop8;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop8;  // carrige return code check
              if(ibuf[0]=='*') goto sloop8;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              if(strncmp(cbuf,"off",3)==0 || strncmp(cbuf,"OFF",3)==0) *subnet=0;
              if(strncmp(cbuf,"on",2)==0 || strncmp(cbuf,"ON",2)==0) *subnet=1;
              goto loop1;
          }
          
          
          if(strncmp(ibuf,"$FILE_SIZE",10)==0){    // file size limit mode get get
          sloop9:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop9;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop9;  // carrige return code check
              if(ibuf[0]=='*') goto sloop9;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              k=sscanf(ibuf, "%s", cbuf);
              if(strncmp(cbuf,"off",3)==0 || strncmp(cbuf,"OFF",3)==0) *f_size_mode=0;
              if(strncmp(cbuf,"on",2)==0 || strncmp(cbuf,"ON",2)==0) *f_size_mode=1;
              goto loop1;
          }
          
          
          if(strncmp(ibuf,"$SATKEYS",8)==0){    // satellite key words for 1ch mode
          sloop10:
              if( fgets( ibuf, 100, stream ) == NULL) goto exitend;
              if(strlen(ibuf) < 1) goto sloop10;
              if(strlen(ibuf) == 1 && (ibuf[0]=='\n' || ibuf[0]==' '))goto sloop10;  // carrige return code check
              if(ibuf[0]=='*') goto sloop10;  // this is comment line
              if(ibuf[0]=='$') goto cmd_entry;  // this is command line
              len=strlen(ibuf);
              /* remove first extra spaces if existed */ 
              for(i=0; i<len; i++){
                  if(ibuf[i] == ' ') continue;
                  if(ibuf[i] != ' ') {
                      k=i;
                      break;
                  }
              }
              j=0;
              for(i=k; i<len; i++){
                  if(ibuf[i] == '*') break;
                  cbuf[j]=ibuf[i];
                  j++;
              }
              cbuf[j]='\0';
              len=strlen(cbuf);
              /* remove last extra spaces if existed */ 
              for(i=len-1; i >= 0; i--){
                  if(cbuf[i] != ' ') {
                      k=i;
                      break;
                  }
                  if(cbuf[i] == ' ') continue;
              }
              for(i=0; i <= k; i++){
                  keyword[i]=cbuf[i];
              }
              keyword[k+1]='\0';

              //printf("Debug %s\n",keyword);
              goto loop1;
          }
          
          
        }
    exit1:
        if (feof(stream) == 0) 
           goto loop1;         // next line read
  exitend:
	    i=fclose( stream ) ;
  exitend2:;
}


void get_outdir(char* k5outinfo, listoutdir_t *outdirtab)
//void get_outdir(char* k5outinfo, int *dirnum)
//void get_outdir(char* k5outinfo, char** outdirtab, int *dirnum)
{
       int i;
       char line[100];
       FILE *stream;

       if( (stream  = fopen( k5outinfo, "r" )) == NULL ){
          printf("get_outdir: File (%s) not found or Open Error! \n",k5outinfo);
          goto exitend2;
       }

  loop1:

       if( fgets( line, 100, stream ) == NULL){
          //printf("File EOF or Read Error!\n");
          goto exitend;
       }
       //printf("%s",line);
       if(line[0]=='*') {  
          goto loop1;  // this is comment line (nothing to do)  
       } else {
          /* remove CR code */
          i=strlen(line);
          line[i-1]='\0';
          /* check last character */
          dir_name_check(line);
          //printf("dirnum = %d\n",outdirtab->dirnum);
          strcpy(outdirtab->dir[outdirtab->dirnum],line);
          outdirtab->dirnum++;
       }
       goto loop1;
  exitend:
       i=fclose(stream);
  exitend2:;
}

void parse_sat_keys(char* keywords, satkeys_t *sat)
/*  2003-11-27 */
{
    // keywords should be separated by space and each keyword should be 8 or
    // less characters
    // Max number of key words should be 10
    char abuf[80], cbuf[80];
    char *cset=" ";
    char *p=NULL;
    int k;
   
    strcpy(cbuf,keywords);
    sat->num=0;
    sat->keys[0][0]='\0';
    k=0;
    if (strlen(keywords) <= 0) goto exithere;
    sprintf(abuf,"%s\0",strtok(cbuf, cset));
    strcpy(sat->keys[k], abuf);  // 1st key words
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s\0",p);
        strcpy(sat->keys[k],abuf);
        k++;
        if(k==10) break;
    }
    sat->num=k;
exithere:;
}


int main(int argc, char *argv[])
{
       char dev[] = "/dev/tds0";
       char outfile [80];
       char fname [50];
       char outdir [80];
       char* dflt_dir="./";           // default out file directory (1st)
       char skdname[80];
       char run_info_file[80];        // run info file
       char mystation[11];
       char stat_id[4];
       char cbuf[120];
       char afreq[20];
       char y_or_n[2];
       char freqg[3];   // frequency group info (allw max 2char)
       char freqgtab[][3]={"a","b","c","d"};
       char k5outinfo[80];            // file contains out directories
       //char* keyword="NOZ";         // key word for 1ch mode (source name)
       //char keyword[10];            // key word for 1ch mode (source name)
       char keyword[80];              // key words for 1ch mode (source name)
       //char logdir[60];             // logfile directory
       char hostname[20];           // host name
       //char logfile[80];            // log file name
       int sindex;                    // station index #
       int ndir;                      //current directory number 
       //int fd_in,rv;
       int rv;
       int i, span, sfreq, adbit, numch;
       int outdirmode, runinfomode, len;
       int nobs, obsspan, fixspan;  
       int year,ddd,hh,mm,ss ;
       int yearf,dddf,hhf,mmf,ssf ;   // to keep 1st obs info
       int yearl,dddl,hhl,mml,ssl ;   // to keep last obs info
       int year1,ddd1,hh1,mm1,ss1 ;   // 
       int year2,ddd2,hh2,mm2,ss2 ;   // 
       int yearn,dddn,hhn,mmn,ssn ;
       int yearPC;  // to keep yaer get from PC
       int month,mday;
       int monthf,mdayf;
       int monthl,mdayl;
       int month1,mday1;
       int month2,mday2;
       int monthn,mdayn;
       int iofset;
       int chmode;   // 1 or 4  (1ch mode or 4 ch mode indicator)
       int n;
       unsigned int day; // foCXǂݍ񂾌݂̎i[
       unsigned int secold, sec1st, seclast, secnow, secnext;
       int day1st, daylast, daynext;
       int time_mode;  // ^p̎{[h瓾邩ob̎瓾邩̔
                       //  0: {[h@@@1:ob
       int naming_type;   // out file naming rule  1: XDDDNNNN.dat  2: sidDDDHHMMSSG.dat
       int subnet;        // 0: subnet mode OFF   1: subnet mode ON (default)
       int f_size_mode;   // 0: no limitation on file size  1: divided into 2GB size (default) !!! updated - 200MB
       float tofset, fsec;
       double dtime1st,dtimelast,dtimenow, dtimenext, dtimenext2;
       double dtime1, dtime2;  // manually set obs time range
       double d1sec;    // 1sec in day unit
       double dwork;
       double freeb, freeb2, reqb, reqmb, dsfreq;
       listoutdir_t outdirtab;
       satkeys_t sat;
       //logval_t lg;
       spget_t spg;
       static sked_data_t skd;
       
       
       if(argc == 1){
          printf("\nautoobs %s\n",AUTOOBS_VERSION);
          printf("Usage: autoobs -runparam_file\n");
          printf("         where  runparam_file ---- run parameters file name\n");
          printf("or\n");
          printf("Usage: autoobs sked_file station_id [span [sfreq [adbit [numch [outdir\n");
          printf("            [y_or_no [naming_type [freqg [subnet [f_size_mode]]]]]]]]]]\n");
          //printf("\n");
          printf("     where  sked_file ---- schedule file name\n");
          printf("            staion_id ---- station ID (A1)\n");
          printf("            span ---- duration (sec)\n");
          printf("                      compulsory shorten obs length (dflt 10)\n");
          printf("                      =0 means no effect (observed as scheduled)\n");
          printf("            sfreq ---- sampling frequency\n");
          printf("                       40,100,200,500 (for kHz)\n");
          printf("                       1,2,4,8,16 (for MHz)  (dflt 16)\n");
          printf("            adbit ---- A/D bit  1|2|4|8 (dflt 1)\n");
          printf("            numch ---- number of channel 1|4 (dflt 1)\n");
          printf("            outdir ---- output file directory\n");
          printf("                         -outdir means directory info file\n");
          printf("                      default is current directory\n");
          printf("            y_or_n ---- Obs range limitation reply N|Y\n");
          printf("                      if omitted, u will be asked later\n");
          printf("            naming_type ---- out file naming type selectio\n");
          printf("                       1 : XDDDNNNN.dat (default) 2: sidDDDHHMMSSG.dat\n");
          printf("            freqg ---- frequency group id used in TypeII file naming\n");
          printf("                      1|2|3|4 or a|b|c|d  (default null)\n");
          printf("            subnet --- Subnet mode control 0: no-subnet, 1: subnet on(dflt)\n");
          printf("            f_size_mode -- 0: no limitation  1: divide into each 200MB size(dflt)\n");
          printf("\n");
          printf("   How to monitor environment variablesand defaults:   autoobs env\n");
          exit(0);
       }
       if (argc>=2){
         if(strncmp(*(argv+1),"env",3)==0){
           printf("   Enviroment variables\n");
           //   K5SKED
           printf("        %s ---   default directory for schedule file\n",K5SKED);
           printf("                      ( %s )",getenv(K5SKED));
           if(getenv(K5SKED)==NULL) {
              printf(",  program deflt is ( %s )\n",K5SKEDDFLT);
           } else {
              printf("\n");
           }
           //   K5OUTINFO
           printf("        %s ---   default out directories information file\n",K5OUTINFO);
           printf("                      ( %s )",getenv(K5OUTINFO));
           if(getenv(K5OUTINFO)==NULL) {
              printf(",  program deflt is ( %s )\n",K5OUTINFODFLT);
           } else {
              printf("\n");
           }
           //   K5LOGDIR
           printf("        %s ---   default log file directory\n",K5LOGDIR);
           printf("                      ( %s )",getenv(K5LOGDIR));
           if(getenv(K5LOGDIR)==NULL) {
              printf(",  program deflt is ( %s )\n",K5LOGDIRDFLT);
           } else {
              printf("\n");
           }
           //   K5NAMING
           printf("        %s ---   default out file naming rule\n",K5NAMING);
           printf("                    old : old type   new : newtype\n");
           printf("                      ( %s )",getenv(K5NAMING));
           if(getenv(K5NAMING)==NULL) {
              printf(",  program deflt is ( %s )\n",K5NAMINGDFLT);
           } else {
              printf("\n");
           }
           //   K5FREQG
           printf("        %s ---   default frequency group of this PC\n",K5FREQG);
           printf("                    1|2|3|4 or a|b|c|d\n");
           printf("                      ( %s )",getenv(K5FREQG));
           if(getenv(K5FREQG)==NULL) {
              printf(",  program deflt is ( %s )\n",K5FREQGDFLT);
           } else {
              printf("\n");
           }
           //   K5SUBNET
           printf("        %s ---   default subnet mode of this PC\n",K5SUBNET);
           printf("                    on: subnet on, off: subnet off\n");
           printf("                      ( %s )",getenv(K5SUBNET));
           if(getenv(K5SUBNET)==NULL) {
              printf(",  program deflt is ( %s )\n",K5SUBNETDFLT);
           } else {
              printf("\n");
           }
            //   K5SATKEY
           printf("        %s ---   default satellite keyword of 1ch obs of this PC\n",K5SATKEY);
           printf("                      ( %s )",getenv(K5SATKEY));
           if(getenv(K5SATKEY)==NULL) {
              printf(",  program deflt is ( %s )\n",K5SATKEYDFLT);
           } else {
              printf("\n");
           }
          printf("\n");
           exit(0);
         } 
       } 
      
       /* initialize variables */

       d1sec=1.0/86400.0;
       stat_id[0]='\0';
       //logdir[0]='\0';
       skdname[0]='\0';
       fixspan=10;   // default fix span (sec)
       sfreq=16;
       adbit=1;
       numch=1;
       for(i=0; i<10; i++){
          strcpy(outdirtab.dir[i],"NONE");
       }
       strcpy(outdirtab.dir[0],"./");
       outdirtab.dirnum=1;
       strcpy(y_or_n," ");
       f_size_mode=1; // file size limit mode default 

       sat.num=0;
       sat.keys[0][0]='\0';
       k5outinfo[0]='\0';

       lg.hostname[0]='\0';
       lg.logdir[0]='\0';
       lg.logfile[0]='\0';
       lg.logfiletype=LOGFILE_TYPE;  // logfile name type
       
       /* naming type set from environment variable */

       if(getenv(K5NAMING) != NULL){
          strcpy(cbuf,getenv(K5NAMING));
       } else {
          strcpy(cbuf,K5NAMINGDFLT);
       }
       naming_type=1; // naming type default (old type)
       if(strncmp(cbuf,"old",3)==0 || strncmp(cbuf,"OLD",3)==0) naming_type=1;
       if(strncmp(cbuf,"new",3)==0 || strncmp(cbuf,"NEW",3)==0) naming_type=2;
       
       /* frequency group set from environment variable */

       if(getenv(K5FREQG) != NULL){
          strcpy(cbuf,getenv(K5FREQG));
       } else {
          strcpy(cbuf,K5FREQGDFLT);
       }

       freqg[0]='\0';  //frequency group default
       i=0;
       if(strncmp(cbuf,"1",1)==0 || strncmp(cbuf,"a",1)==0) i=1;
       if(strncmp(cbuf,"2",1)==0 || strncmp(cbuf,"b",1)==0) i=2;
       if(strncmp(cbuf,"3",1)==0 || strncmp(cbuf,"c",1)==0) i=3;
       if(strncmp(cbuf,"4",1)==0 || strncmp(cbuf,"d",1)==0) i=4;
       if(i != 0) strcpy(freqg,freqgtab[i-1]);

       /* subnet mode set from environment variable */

       if(getenv(K5SUBNET) != NULL){
          strcpy(cbuf,getenv(K5SUBNET));
       } else {
          strcpy(cbuf,K5SUBNETDFLT);
       }
       subnet=1; // default (subnet ON)
       if(strncmp(cbuf,"on",2)==0 || strncmp(cbuf,"ON",2)==0) subnet=1;
       if(strncmp(cbuf,"off",3)==0 || strncmp(cbuf,"OFF",3)==0) subnet=0;

       /* get satellite keyword for 1ch obs */

       if(getenv(K5SATKEY) != NULL){
          strcpy(keyword,getenv(K5SATKEY));
       } else {
          strcpy(keyword,K5SATKEYDFLT);
       }

       /* get run parameters */
       
       runinfomode=0;
       if (argc>=2){
          strcpy(skdname,argv[1]);  
          if(skdname[0]=='-'){   // this is k5 run info file name mode
             runinfomode=1;
             len=strlen(skdname);
             // remove first '-' from skdname
             for(i=0; i<len-1; i++){
                 run_info_file[i]=skdname[i+1];
             }
             run_info_file[len-1]='\0';
          }
       }

       if(runinfomode==1) {
          get_run_info(run_info_file, skdname, &fixspan, &sfreq, &adbit, 
                  //&numch, stat_id, logdir, &outdirtab, &naming_type, freqg, 
                  &numch, stat_id, lg.logdir, &outdirtab, &naming_type, freqg, 
                  &subnet, &f_size_mode, keyword);
          strcpy(y_or_n,"N");
          strcpy(outdir,outdirtab.dir[0]);
          goto bypass1;
       }
       
      
       if (argc>=3) strcpy(stat_id,argv[2]);  
       if (argc>=4) fixspan = atoi(*(argv + 3 ));
       if (argc>=5) sfreq = atoi(*(argv + 4 ));
       if (argc>=6) adbit = atoi(*(argv + 5 ));
       if (argc>=7) numch = atoi(*(argv + 6 ));
       
       outdirmode=0;
       if (argc>=8){
          strcpy(outdir,*(argv+7));
          if(outdir[0]=='-'){   // this is k5out info file name mode
             outdirmode=1;
             len=strlen(outdir);
             // remove first '-' from outdir
             for(i=0; i<len-1; i++){
                 k5outinfo[i]=outdir[i+1];
             }
             k5outinfo[len-1]='\0';
          }
       } else {
          strcpy(outdir,dflt_dir);
       }

       if (argc>=9) strcpy(y_or_n,*(argv+8));
       if (argc>=10) naming_type = atoi(*(argv + 9 ));

       if (argc>=11){
          strcpy(freqg,*(argv + 10 ));
          i=0;
          if(freqg[0]=='1') i=1;
          if(freqg[0]=='2') i=2;
          if(freqg[0]=='3') i=3;
          if(freqg[0]=='4') i=4;
          if(i!=0){
             strcpy(freqg,freqgtab[i-1]);
          }
       }
       
       if (argc>=12) subnet = atoi(*(argv + 11 ));
       if (argc>=13) f_size_mode = atoi(*(argv + 12 ));

       
       /* check out dir name accompanied by "/" or not */

       if(outdirmode==0){
          dir_name_check(outdir);
       }
       
       
       /* set up out dir table */

       if(outdirmode==0){
           strcpy(outdirtab.dir[0],outdir);
           outdirtab.dirnum=1;
       } else {
           outdirtab.dirnum=0;
       }


       /* get file name that contains 2nd, 3rd, 4th, ... directory candidate for output */

       if(outdirmode==0){
          if(getenv(K5OUTINFO) != NULL){
             strcpy(k5outinfo,getenv(K5OUTINFO));
          } else {
             strcpy(k5outinfo,K5OUTINFODFLT);
          }
       }
       
       get_outdir(k5outinfo, &outdirtab);
       //get_outdir(k5outinfo, &dirnum);
       for(i=0; i<outdirtab.dirnum; i++){
          printf("%d  %s\n",i,outdirtab.dir[i]);
       }
       
 bypass1:

       /* log directory name get from enviroment variable K5LOGDIR if it is set */

       if(lg.logdir[0]=='\0'){
          if(getenv(K5LOGDIR) != NULL){
             strcpy(lg.logdir,getenv(K5LOGDIR));
          } else {
             strcpy(lg.logdir,K5LOGDIRDFLT);
          }
       }
       //strcpy(lg.logdir,logdir);

       /* check schedule file name is acompanied by directory or not */

       get_true_path(skdname, K5SKED, K5SKEDDFLT,cbuf);
       strcpy(skdname,cbuf);


       /* get host name */

       hostget(hostname);
       strcpy(lg.hostname,hostname);



       time_mode=GET_TIME_FROM_BOARD;  //  current TIME get from IP-VLBI board or not
       //time_mode=0;  //  current TIME get from PC board

       /* parse sat keys for 1 ch observation */

       parse_sat_keys(keyword, &sat);

       if(sat.num==0) strcpy(keyword,"N.A.");

       printf(" skedname = %s\n",skdname);    // debug
       printf(" station_id = %s\n",stat_id);    // debug
       printf(" span = %d\n",fixspan);    // debug
       printf(" sfreq= %d\n",sfreq);   // debug
       printf(" adbit= %d\n",adbit);   // debug
       printf(" numch= %d\n",numch);   // debug
       printf(" outdir= %s\n",outdir);   // debug
       printf(" naming_type = %d\n",naming_type);   // debug
       printf(" freq_g = %s\n",freqg);   // debug
       printf(" satellite keys : %s\n",keyword); // debug
       for(i=0; i< outdirtab.dirnum; i++){
            printf(" outdir candidate :  %s\n",outdirtab.dir[i]); 
       }

       spg.adbit=adbit;
       spg.sfreq=sfreq;

       /* sampling frequency convert into text for print */

       strcpy(afreq," ");
       if(sfreq < 40){
          sprintf(afreq,"%dMHz\0",sfreq);
       }  else {
          sprintf(afreq,"%dkHz\0",sfreq);
       } 
       //if(sfreq==40){ strcpy(afreq,"40kHz");}
       //else if(sfreq==100){ strcpy(afreq,"100kHz");}
       //else if(sfreq==200){ strcpy(afreq,"200kHz");}
       //else if(sfreq==500){ strcpy(afreq,"500kHz");}
       //else if(sfreq==1){ strcpy(afreq,"1MHz");}
       //else if(sfreq==2){ strcpy(afreq,"2MHz");}
       //else if(sfreq==4){ strcpy(afreq,"4MHz");}
       //else if(sfreq==8){ strcpy(afreq,"8MHz");}
       //else if(sfreq==16){ strcpy(afreq,"16MHz");}
       //else { strcpy(afreq,"INVALID");}
	
       /* convert sampling frequency index to real frequency (Hz) */

	   if(sfreq < 40){
	      dsfreq=(double)sfreq*1.0e6;
	   } else {
	      dsfreq=(double)sfreq*1.0e3;
	   }

       /* schedule file read and analysis */

       strcpy(skd.skdname,skdname);
       //readskd(skdname, &skd);
       readskd(&skd);
       if (skd.expcode[0]=='\0'){
           printf(" %s may not be a schedule file\n",skdname);
           printf(" Check schedule file name or run parameters!!\n");
           exit(0);
       }
       if (skd.nobs <= 0) { 
           printf(" No observation scan found in the schedule file %s\n",skdname);
           printf(" Check schedule file!!\n");
           exit(0);
       }

       /* Observation Time Range Check */

       yearf=skd.start[0][0]+2000;
       dddf=skd.start[0][1];
       hhf=skd.start[0][2];
       mmf=skd.start[0][3];
       ssf=skd.start[0][4];
       sec1st=hhf*3600+mmf*60+ssf;
       day1st=dddf;
       printf("day1st  %d\n",day1st);
       dtime1st=day1st+sec1st/86400.0;
       ddd2md(yearf,dddf,&monthf,&mdayf);
       printf("1st  Obseravtion : %04d/%02d/%02d %02d:%02d:%02d  %s\n",yearf,monthf,mdayf,hhf,mmf,ssf,
                      skd.obsstar[0]);
       yearl=skd.start[skd.nobs-1][0]+2000;
       dddl=skd.start[skd.nobs-1][1];
       hhl=skd.start[skd.nobs-1][2];
       mml=skd.start[skd.nobs-1][3];
       ssl=skd.start[skd.nobs-1][4];
       seclast=hhl*3600+mml*60+ssl;
       daylast=dddl;
       dtimelast=daylast+seclast/86400.0;
       ddd2md(yearl,dddl,&monthl,&mdayl);
       printf("Last Obseravtion : %04d/%02d/%02d %02d:%02d:%02d  %s\n",yearl,monthl,mdayl,hhl,mml,ssl,
                      skd.obsstar[skd.nobs-1]);
       // printf("Date range %f -- %f\n",dtime1st,dtimelast);

       dtime1=dtime1st;     dtime2=dtimelast;
       year1=yearf;       year2=yearl;
       ddd1=dddf;       ddd2=dddl;
       month1=monthf;   month2=monthl;
       mday1=mdayf;     mday2=mdayl;
       hh1=hhf;        hh2=hhl;
       mm1=mmf;        mm2=mml;
       ss1=ssf;        ss2=ssl;

       /* Observation range limit if selected so */

       if (strncmp(y_or_n," ",1)==0) { // interactive mode
          strcpy(cbuf,"\0");
          //printf(" Observation Time range Change? (Y/[N]) ----> ");
          printf(" Observation Time range Change? (Y/N) ----> ");
          scanf("%s",&cbuf);
          if(strncmp(cbuf,"Y",1)==0 || strncmp(cbuf,"y",1)==0 ){
             year1=yearf;
             printf("Enter Start Time (month mday hour min sec)\n");
             scanf("%d %d %d %d %d", &month1,&mday1,&hh1,&mm1,&ss1);
             md2ddd(year1,&ddd1,month1,mday1);
             secnow=hh1*3600+mm1*60+ss1;
             dtime1=(double)ddd1+secnow/86400.0;
             printf("Enter Stop  Time (month mday hour min sec)\n");
             scanf("%d %d %d %d %d", &month2,&mday2,&hh2,&mm2,&ss2);
             md2ddd(year1,&ddd2,month2,mday2);
             secnow=hh2*3600+mm2*60+ss2;
             dtime2=(double)ddd2+secnow/86400.0;
             printf("dtime1 dtime2 %f %f\n)",dtime1,dtime2);
          } 
       } else if(strncmp(y_or_n,"Y",1)==0){
            year1=yearf;
            printf("Enter Start Time (month mday hour min sec)\n");
            scanf("%d %d %d %d %d", &month1,&mday1,&hh1,&mm1,&ss1);
            md2ddd(year1,&ddd1,month1,mday1);
            secnow=hh1*3600+mm1*60+ss1;
            dtime1=(double)ddd1+secnow/86400.0;
            printf("Enter Stop  Time (month mday hour min sec)\n");
            scanf("%d %d %d %d %d", &month2,&mday2,&hh2,&mm2,&ss2);
            md2ddd(year1,&ddd2,month2,mday2);
            secnow=hh2*3600+mm2*60+ss2;
            dtime2=(double)ddd2+secnow/86400.0;
            printf("dtime1 dtime2 %f %f\n)",dtime1,dtime2);
       }
       	    


       /* slect station if Station is not yet selected */

       if (stat_id[0]=='\0') 
       //if (argc<=2)
       {
          printf(" ****** STATION ID TABLE *********\n");
          for (i=0; i<skd.nstation ; i++)
          {
             printf("  %s --- %s\n",skd.statid[i],skd.statnam[i]);
          }
          printf(" *********************************\n");
          printf(" Enter Station ID ----> ");
          scanf("%s",&stat_id);
          printf(" Selected station_id is %s\n",stat_id);
       }

       /* Set Up my station Name */

       strcpy(mystation,"\0");
       for (i=0; i<skd.nstation ; i++)
       {
           if (strncmp(skd.statid[i],stat_id,strlen(stat_id))==0)
           {
                //strncpy(mystation,skd.statnam[i],strlen(skd.statnam[i]));
                strcpy(mystation,skd.statnam[i]);
                printf("  %s --- %s\n",stat_id,mystation);
                sindex=i;
                break;
           }
       }


       /* log file name create */
       
       dir_name_check(lg.logdir);
       sprintf(lg.logfile,"%s%s%s.%s.log",lg.logdir,skd.expcode,stat_id,hostname);
       //strcpy(lg.logdir,logdir);
       //strcpy(lg.logfile,logfile);

       /* copy log information to spg */
       spg.lg=lg;       // where lg is gloabal variable, while spg is local variable 

       /* file size mode set tp spgettype variable */
       spg.f_size_mode=f_size_mode;

       /* log start */

       sprintf(cbuf,"*********** AUTOOBS %s START **************",AUTOOBS_VERSION);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," skedname = %s",skdname); 
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," station_id = %s",stat_id); 
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," span = %d",fixspan);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," sfreq= %s",afreq);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," adbit= %d",adbit);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," numch= %d",numch);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf," outinfo= %s",k5outinfo); 
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(subnet == 1){
          sprintf(cbuf," subnetmode =  ON"); 
       } else {
          sprintf(cbuf," subnetmode =  OFF"); 
       }
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(naming_type == 1){
          sprintf(cbuf," naming_type= 1    (sidDDDNNNN.dat)"); 
       } else if (naming_type == 2){
          sprintf(cbuf," naming_type= 2    (sidDDDHHMMSSG.dat)"); 
       } else {
          sprintf(cbuf," naming_type= %d",naming_type); 
       }
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(freqg[0]=='\0'){
           sprintf(cbuf," freq_group = NULL"); 
       } else {
           sprintf(cbuf," freq_group = %s",freqg); 
       }
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       //sprintf(cbuf," 1st outdir= %s",outdir); 
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       for(i=0; i< outdirtab.dirnum; i++){
            sprintf(cbuf," outdir candidate :  %s",outdirtab.dir[i]); 
            //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
            LogWrite4(&lg,time_mode,0.0,2,cbuf);
       }
       sprintf(cbuf," satellite key words :  %s",keyword); 
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"***************************************************************");
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
	
	   SetHandler();   // Interrapt handler



       /* check Board Time and System Time, and get offset */

       iofset=0;
       tofset=0.0;

       if(toffset_get(&yearPC, &tofset)<0) exit(0);
        
       ndir=0;
       strcpy(outdir,outdirtab.dir[ndir]);
       ndir++;
        
       /* log obs range info */

       freeb=GetFreeInfo(outdir);
       if(strncmp(outdirtab.dir[ndir],"NONE",4)==0){
          freeb2=0.0;
       } else {
          freeb2=GetFreeInfo(outdirtab.dir[ndir]);
       }
       sprintf(cbuf,"*****************************************************************");
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*   AUTOOBS (Automatic observation program for IP-VLBI board)   *");
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*        (%s by Kashima VLBI Group)         *",AUTOOBS_VERSION);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*                                                               *");
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       if(time_mode==1){
       sprintf(cbuf,"*         Schedule Time Get from IP-BOARD : Naming Type %d       *",
           naming_type);
       } else {
       sprintf(cbuf,"*         Time for scheduling Get from PC : Naming Type %d       *",
           naming_type);
       }
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"*****************************************************************");
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Schedule File     :  %s",skdname);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Experiment Code   :  %s        Freq_G (%s)",skd.expcode,freqg);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Log File          :  %s",lg.logfile);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Sampling Frequency:  %s,  AD bits: %d,   # of CH: %d",afreq, adbit, numch);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Stations Included :  %s",skd.statnam[0]);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       for (i=1; i<skd.nstation; i++){ 
       sprintf(cbuf,"                    :  %s",skd.statnam[i]); 
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       }
       if(subnet==1){
       sprintf(cbuf,"  My Station Name   :  %s (My Station ID : %s)   Subnet Mode : ON",mystation,stat_id);
       } else {
       sprintf(cbuf,"  My Station Name   :  %s (My Station ID : %s)   Subnet Mode : OFF",mystation,stat_id);
       }
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Current Out Dir   :  %s    %.1fMB left",outdir,freeb/1.0e6);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Next    Out Dir   :  %s    %.1fMB left",outdirtab.dir[ndir],freeb2/1.0e6);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Total Scan Number :  %d",skd.nobs);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Satellite Keywords:  %s",keyword);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);

       sprintf(cbuf,"  1st  Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec)  %s",
                      yearf,monthf,mdayf,hhf,mmf,ssf,skd.dura[0],skd.obsstar[0]);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Last Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec)  %s",
                      yearl,monthl,mdayl,hhl,mml,ssl,skd.dura[skd.nobs-1],skd.obsstar[skd.nobs-1]);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);

       sprintf(cbuf,"  Obs Range (start) :  %04d/%02d/%02d %02d:%02d:%02d",
                      year1,month1,mday1,hh1,mm1,ss1);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       sprintf(cbuf,"  Obs Range (end)   :  %04d/%02d/%02d %02d:%02d:%02d",
                      year2,month2,mday2,hh2,mm2,ss2);
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);


        /* Schedule Time Check */
        /* wait till next obs */
       
        nobs=0;

 obsloop:
       
        if(nobs > skd.nobs) goto exitend;

        /* satellite key word check for setup 1ch mode */

        chmode=numch;
        n=0;
        for(i=0; i<sat.num; i++){
            if(strncmp(skd.obsstar[nobs],sat.keys[i],strlen(sat.keys[i]))==0) n=1; // 1ch obs     addded on 2003-03-13
        }
        if(n==1){ // compulsory set 1ch mode
            chmode=1;
        }


       secold=90000;

       /* check disk free space */

       freeb=GetFreeInfo(outdir);
       if(strncmp(outdirtab.dir[ndir],"NONE",4)==0){
          freeb2=0.0;
       } else {
          freeb2=GetFreeInfo(outdirtab.dir[ndir]);
       }

       /* display observation informations */

       printf("\033[2J");   //  Erase the screen ( move the cursor to home ) for VT100
       printf("\033[H");   //  move the cursor to home
       printf("*****************************************************************\n");
       //printf("*                                                               *\n");
       printf("*    AUTOOBS (%s  by Kashima VLBI Group)    *\n",AUTOOBS_VERSION);
       //printf("*                                                               *\n");
       if(time_mode==1){
       printf("*              Time Get from IP-BOARD : Naming Type %d           *\n",
           naming_type);
       } else {
       printf("*              Time Get from PC       : Naming Type %d           *\n",
           naming_type);
       }
       printf("*****************************************************************\n");
       printf("\n");

       printf("  Schedule File     :  %s\n",skdname);
       printf("  Experiment Code   :  %s          Freq_G (%s)\n",skd.expcode,freqg);
       printf("  Log File          :  %s\n",lg.logfile);
       //printf("  Sampling Frequency:  %s,  AD bits: %d,   # of CH: %d\n",afreq, adbit, numch);
       //printf("  Sampling Frequency:  %s,  AD bits: %d,   # of CH: %d\n",afreq, adbit, chmode);
       printf("  Stations Included :  %s",skd.statnam[0]);
       for (i=1; i<skd.nstation; i++){ printf("  %s",skd.statnam[i]); }
       printf("\n");
       if(subnet==1){
       printf("  My Station Name   :  %s (My Station ID : %s)   Subnet Mode : ON\n",mystation,stat_id);
       } else {
       printf("  My Station Name   :  %s (My Station ID : %s)   Subnet Mode : OFF\n",mystation,stat_id);
       }
       printf("  Current Out Dir   :  %s    %.1fMB left\n",outdir,freeb/1.0e6);
       printf("  Next    Out Dir   :  %s    %.1fMB left\n",outdirtab.dir[ndir],freeb2/1.0e6);
       printf("  Total Scan Number :  %d       Satellite Mode Keys: %s\n",skd.nobs,keyword);

       printf("  1st  Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec)  %s\n",
                      yearf,monthf,mdayf,hhf,mmf,ssf,skd.dura[0],skd.obsstar[0]);
       printf("  Last Scan         :  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec)  %s\n",
                      yearl,monthl,mdayl,hhl,mml,ssl,skd.dura[skd.nobs-1],skd.obsstar[skd.nobs-1]);

       printf("  Obs Range (start) :  %04d/%02d/%02d %02d:%02d:%02d\n",
                      year1,month1,mday1,hh1,mm1,ss1);
       printf("  Obs Range (end)   :  %04d/%02d/%02d %02d:%02d:%02d\n",
                      year2,month2,mday2,hh2,mm2,ss2);
       printf("  Sampling Condition for Next Scan :  %d-ch %s %d-bit sampling\n",
                      chmode, afreq, adbit);
       printf("\n");

       printf("\033[2B");  // J[\ɂQs

       if(time_mode == 1) {
           if(ip_board_time(&day, &secnow, &hh, &mm, &ss)==-1) return -1 ;
           ddd=day;
           year=yearPC;
           ddd2md(year, ddd, &month, &mday);
       } else {
           getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
           secnow=hh*3600+mm*60+ss;
           md2ddd(year, &ddd, month, mday);
       }
       dtimenow=ddd+secnow/86400.0;
       printf("  Time Now (UTC)    :  %04d/%02d/%02d %02d:%02d:%02d\n\033[A",
                   year,month,mday,hh,mm,ss);
       printf("\033[2A");  // J[\ɂQs


       //fixspan=10;   // compulsory fix span length (sec)
       //nobs=0;
       //flagobs=0;
       while(1)
      {                                 // observation loop

           /* next observation start time get */

           yearn=skd.start[nobs][0]+2000;
           dddn=skd.start[nobs][1];
           hhn=skd.start[nobs][2];
           mmn=skd.start[nobs][3];
           ssn=skd.start[nobs][4];
           secnext=hhn*3600+mmn*60+ssn;
           daynext=dddn;
           //printf("daynext  %d\n",daynext);
           dtimenext=daynext+secnext/86400.0;
           ddd2md(yearn,dddn,&monthn,&mdayn);
           obsspan=skd.dura[nobs];
           
           /* subnet consideration */

           if(subnet==1){ // sub-net mode
              // get scan duration of this station
              obsspan=skd.datastop[nobs][sindex];
           }
           if(obsspan <= 0) {
              nobs++;
              goto obsloop;  // skip this scan
           }           
           
           dtimenext2=dtimenext+(double)obsspan/86400.0;
           
           /* disk capacity check */

           if (fixspan > obsspan || fixspan==0){
               span=obsspan; 
           } else {
               span=fixspan; 
           }

           //if(strncmp(skd.obsstar[nobs],keyword,strlen(keyword))==0){   // 1ch obs addded on 2003-03-13
           //     reqb=dsfreq*(double)adbit*(double)span;
           //} else {
           //     reqb=dsfreq*(double)adbit*(double)span*(double)numch;
           //}
           reqb=dsfreq*(double)adbit*(double)span*(double)chmode;  // 2003-11-27

           reqb/=8.0;   // in byte unit
           reqmb=reqb/1.0e6;
           freeb=GetFreeInfo(outdir);
           //if(nobs == 1 && ndir==1) freeb=10.0e6;  // for check
           //if(nobs == 3 && ndir==2) freeb=10.0e6;  // for check
           if(1.5*reqb > freeb){   //  out directory change
              if(ndir < outdirtab.dirnum){
                  strcpy(outdir,outdirtab.dir[ndir]);
                  ndir++;
                  sprintf(cbuf,"OUT directory changed  :  %s",outdir);
                  //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,0,cbuf);
                  LogWrite4(&lg,time_mode,0.0,0,cbuf);
                  goto obsloop;
              }
           }

           
           if(fixspan > obsspan || fixspan==0){
              printf("\033[2K  Next Scan(No.%04d):  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec)  %s [%.1fMB]\n",
                      nobs+1,yearn,monthn,mdayn,hhn,mmn,ssn,obsspan,skd.obsstar[nobs],reqmb);
                       // \033[2K  means erases the entire current line
              sprintf(cbuf,"OBS# %04d :  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec)  %s [%.1fMB]",
                      nobs+1,yearn,monthn,mdayn,hhn,mmn,ssn,obsspan,skd.obsstar[nobs],reqmb);
           } else {
              printf("\033[2K  Next Scan(No.%04d):  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec ==>%3dsec)  %s [%.1fMB]\n",
                      nobs+1,yearn,monthn,mdayn,hhn,mmn,ssn,obsspan,fixspan,skd.obsstar[nobs], reqmb);
              sprintf(cbuf,"OBS# %04d :  %04d/%02d/%02d %02d:%02d:%02d  (%3dsec ==>%3dsec)  %s [%.1fMB]",
                      nobs+1,yearn,monthn,mdayn,hhn,mmn,ssn,obsspan,fixspan,skd.obsstar[nobs],reqmb);
           }
           printf("\n");
           //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,0,cbuf);
           LogWrite4(&lg,time_mode,0.0,0,cbuf);
           
           
           
           
           /*
           if (dtimenow > dtimenext2 ) // next observation set
           {
              nobs++;
              printf("\033[2A");  // J[\ɂQs
              goto pass2;
           }
           */

       loop1:

           /* compare system time */

#ifndef WIN32
           usleep(SLEEP_USEC);
#else
           tk_sleep(SLEEP_MSEC);
#endif
           //getsystime(&year, &month, &mday, &hh, &mm, &ss, &iofset);
           if(time_mode == 1) {
              if(ip_board_time(&day, &secnow, &hh, &mm, &ss)==-1) return -1 ;
              ddd=day;
              year=yearPC;
              ddd2md(year, ddd, &month, &mday);
           } else {
              getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
              secnow=hh*3600+mm*60+ss;
              md2ddd(year, &ddd, month, mday);
           }
           dtimenow=ddd+secnow/86400.0;
           //printf("Total Day  %d  %f\n", ddd, dtimenow); 
           if (dtime1 > dtimenext2 ) // next observation set
           {
              nobs++;
              if(nobs > skd.nobs) { goto exitend; }   // add on 2002/7/22
              printf("\033[2A");  // J[\ɂQs
              goto pass2;
           }
      	/*
           if (dtimenow < dtime1) { goto pass1; }
           if (dtimenow < dtime1st) { goto pass1; }
           if (dtimenow > dtime2) { goto exitend; }
           if (dtimenow > dtimelast) { goto exitend; }
           if (dtimenow >= dtimenext && dtimenow < dtimenext2 ) 
      	*/

           /* check sampling start conditions */

           if ((dtimenow+d1sec) < dtime1) goto pass1; 
           if ((dtimenow+d1sec) < dtime1st) goto pass1; 
           if (dtimenow > dtime2) goto exitend; 
           if (dtimenow > dtimelast) goto exitend; 

           if ((dtimenow+d1sec) >= dtimenext && dtimenow < dtimenext2 ) { /* sampling start */

              /* out file name creation */

              /*  Definition of out file name
              
                Type I

                  XDDDNNNN.dat   
                   where X    -- satation id (1 char) 
                         DDD  -- total day at 1st scan (3 digits)
                         NNNN -- obs number  (4digits)
                Type II

                  sidDDDHHMMSSG.dat
                    where sid   -- station id  (1 char or 2 char)
                          DDD   -- total day at current scan (3digits)
                          HH    -- hour at the start of scan (2digits)
                          MM    -- minute at the start of scan (2digits)
                          SS    -- second at the start of scan (2 digits)
                          G     -- frequency group id (a|b|c|d)
              */

               if(naming_type == 1){  // naming type I  
                   sprintf(fname,"%1s%03d%04d.dat\0",stat_id,dddf,nobs+1);  
               } else {               // naming type II
                   sprintf(fname,"%s%03d%02d%02d%02d%s.dat\0",
                       stat_id,dddn,hhn,mmn,ssn,freqg);
               }

              strcpy(outfile,outdir);
              strcat(outfile,fname);
              //printf("%s\n",outfile);
              /* obsspan re-calc */
              //printf("obsspan   %d\n",obsspan);
              //dwork=(dtimenext2 - dtimenow)*86400.0;  // 2002-08-15
              dwork=(dtimenext2 - dtimenow-d1sec)*86400.0;  // 2002-08-15
              //obsspan=(int)dwork ;                      // 2002-08-15
              obsspan=nint(dwork) ;                      // 2002-08-15
              //printf("obsspan(after)   %d\n",obsspan);
              if (fixspan > obsspan || fixspan==0)
              {  span=obsspan; }
              else
              {  span=fixspan; }
              printf("\033[B");  // J[\ɂPs
              printf("\n  OUT file name ==> %s\n",outfile);


              if(numch == 4 && chmode ==1){
                 printf("NOZOMI (satellite) mode!!  1ch sampling now\n");   
              }
              spg.span=span;
              spg.numch=chmode;
              strcpy(spg.outfile,outfile);
              //if(f_size_mode==1) {                             
              //    rv=sampleget_old(outfile, span, sfreq, adbit, chmode);    
              //} else {
              //    rv=sampleget_big(outfile, span, sfreq, adbit, chmode);    
              //}
              rv=sampleget(&spg);    
              if(rv > 0) {
                 sprintf(cbuf,"smapler read error occured at least once (%d)",rv);
                 LogWrite4(&lg, time_mode, 0.0, 1, cbuf);
              } 
              if(numch == 4 && chmode ==1){
                 sprintf(cbuf,"NOZOMI (satellite) mode!!  1ch sampled"); 
                 //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
                 LogWrite4(&lg,time_mode,0.0,2,cbuf);
              }
              sprintf(cbuf,"OBS# %04d : %s %d %d %d %d",nobs+1,outfile,span,sfreq,adbit,chmode);           
              //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,1,cbuf);
              LogWrite4(&lg,time_mode,0.0,1,cbuf);
             
           	
              nobs++;
              //printf("\033[A\033[A");  // J[\ɂQs
              //goto pass2;
              goto obsloop;
           
           }
           if (dtimenow > dtimenext2 ) // next observation set
           {
              nobs++;
              printf("\033[2A");  // J[\ɂQs
              goto pass2;
           }

       pass1:
           if (secnow != secold)
           {
             printf("  Time Now (UTC)    :  %04d/%02d/%02d %02d:%02d:%02d\n\033[A",
                      year,month,mday,hh,mm,ss);
           }
           secold=secnow; 
           goto loop1;
        pass2:;

       }

  exitend:

       printf("\n\n\n"); 
       
       sprintf(cbuf,"*********** AUTOOBS OBSERVATION END **************");
       //LogWrite3(logdir,hostname,logfile,logfiletype,time_mode,0.0,2,cbuf);
       LogWrite4(&lg,time_mode,0.0,2,cbuf);
       
       return 0;
}

