#define MAIN_
#include "readme.hpp"
#include "k5search.hpp"   
#include "mjd.hpp"

unsigned char Link = 0, NoEnd = 0;
off_t shift = 0, bl = 0, window = -1, left, step, Counts = -1, sz;
off_t Begin = 0, Number = -1;
off_t szc, szc2;
off_t Size;
double razmer, lc;
double Period = -1., TimeResolution = -1.;
int Header = 0;
char *OutFile, *Mask, *OutputPath, *ProfFile, *GPLogFile;
char *startmjd;          // MJD data as a char*
char *gpmjd;             // MJD of GP
TYPE *data;
float *prof;
float *record;
float search_threshold = 22.; // threshold to search for GPs
float write_threshold;        // threshold to write single periods
off_t schetchik = 0;
bool is_search = true;
bool is_quite = false;
bool is_caption = false; // if false to print the caption of the GP table
bool is_startmjd = false; // if true then GP's time in MJD will be written to log-file
bool is_single_write = false; // if true then single periods are to be written (all or larger 
                              // than some threshold
bool is_write_threshold = false; // if true than default threshold to write single periods exists
                                 // and is equal to search threshold
double level = 0., max_level;
unsigned long width = 1;

/* change the size to the nearest power of 2 (upwards) */
long resize (unsigned long sz) {
 unsigned long n = 0;
 if (sz == 1) return sz;
 while ( (1<<(++n)) < sz ) ;
 return (1<<n);
}

/* to rebin input time-series */
/* consider input array as not that large */
template <class tFormat>
off_t rebinning (tFormat x, off_t size, unsigned long w) {
 float sum;
 off_t newsize = (off_t)(size / w);

 for (off_t i=0, j, k=0; k<newsize; i+=w, k++) {
  sum = 0.;
  for (j=i; j<(i+w >= size ? size : i+w); j++) sum += x[j];
  x[k] = sum / w;
 }

 return newsize;
}

/* merge adjacent samples */
template <class tFormat>
off_t merge_adjacent (tFormat x, off_t size) {
 off_t newsize = (off_t)(size / 2);

 for (off_t i=0, k=0; k<newsize; i+=2, k++)  x[k] = (((float)(x[i] + x[i+1])) / 2.);

 return newsize;
}

/* to rebin input time-series */
/* using other array for output rebinned samples */
template <class tFormat>
off_t rebinning2 (float *res, tFormat x, off_t size, unsigned long w) {
 float sum;
 off_t newsize = (off_t)(size / w);

 memset (res, 0, newsize*sizeof(float));

 for (unsigned long i=0, j, k=0; k<newsize; i+=w, k++) {
  sum = 0.;
  for (j=i; j<(i+w >= size ? size : i+w); j++) sum += x[j];
  res[k] = sum / w;
 } //for

 return newsize;
}


/* searching for GPs */
template <class tFormat>
bool is_gp (tFormat m, off_t size, off_t number, char *name, unsigned long w) {
 double max;
 double aver, aver2, sigma;
 double sum = 0., sum2 = 0.;
 off_t xmax;
 off_t otstup = (off_t)(size * 0.05); // it is calculated as 5% shift from the maximum

 xmax = 0;
 max = m[0];

 for (off_t i=0; i<size; ++i) {
  xmax = ((double)m[i] > max ? i : xmax);
  max  = ((double)m[i] > max ? (double)m[i] : max);
 }

 if (xmax > size - xmax) {
  for (off_t i=0; i<xmax - otstup; ++i) {
   sum  += ((double)m[i]);
   sum2 += ((double)m[i])*((double)m[i]);
  }
  aver  = (double)(sum/(xmax - otstup));
  aver2 = (double)(sum2/(xmax - otstup));
 } else {
  for (off_t i=xmax + otstup; i<size; ++i) {
   sum  += ((double)m[i]);
   sum2 += ((double)m[i])*((double)m[i]);
  }
  aver  = (double)(sum/(size - xmax - otstup));
  aver2 = (double)(sum2/(size - xmax - otstup));
 }

  sigma = sqrt (aver2 - aver * aver);
  level = (max - aver) / sigma;

  if (level >= search_threshold) {
   if (!is_caption) printf ("\n SNR         Phase          Seconds              #per     Naver\n---------------------------------------------------------------------------\n");
   printf (" %6lg     %5.3lg  %22.15lg   %7ld     %5ld\n", level, ((double)xmax)/size, ((shift + xmax*w)*TimeResolution + number*Period)/1.e9, (unsigned long)number, w);
   FILE *gpinfo = fopen(GPLogFile, "at");
                  fseek(gpinfo, 0, SEEK_SET);;
                  if (!is_caption) {
                   fprintf (gpinfo, "#\n");
                   fprintf (gpinfo, "# FILE: %s   Threshold: %.1f   Period: %.15lg   Shift: %lld\n", name, search_threshold, Period, (unsigned long long)shift);
                   fprintf (gpinfo, "# Noise prob: %lg  Sample size: %lld  Nevents: %.1f\n", exp(-1. * search_threshold), (unsigned long long)(Size - shift), exp(-1. * search_threshold) * (Size - shift));
                   fprintf (gpinfo, "#\n");
                   if (!is_startmjd) { 
		     fprintf (gpinfo, "# SNR         Phase              Seconds              Nper     xmax      max        aver       sigma       Naver\n");
                     fprintf (gpinfo, "#-----------------------------------------------------------------------------------------------------------------\n");
		   } else {
		     fprintf (gpinfo, "# SNR         Phase              Seconds              MJD                    Nper     xmax      max        aver       sigma       Naver\n");
                     fprintf (gpinfo, "#----------------------------------------------------------------------------------------------------------------------------------------\n");
		   }
                  }
                  if (!is_startmjd) fprintf (gpinfo, " %lg      %14.12lg     %.15lg     %ld      %lld    %lg    %lg    %lg    %ld\n", level, ((double)xmax)/size, ((shift+xmax*w)*TimeResolution + number*Period)/1.e9, (unsigned long)number, (unsigned long long)xmax, max, aver, sigma, w);
		   else { 
		    // calculate MJD moment of GP
		    memset (gpmjd, 0, 23);
		    gpmjd = getMJD (startmjd, gpmjd, (unsigned long)shift, (unsigned long)xmax, TimeResolution, (unsigned long)number, Period, w); 
		    fprintf (gpinfo, " %lg      %14.12lg     %.15lg     %21s  %ld      %lld    %lg    %lg    %lg    %ld\n", level, ((double)xmax)/size, ((shift+xmax*w)*TimeResolution + number*Period)/1.e9, gpmjd, (unsigned long)number, (unsigned long long)xmax, max, aver, sigma, w);
		  }
                  fclose (gpinfo);
   
   if (!is_caption) is_caption = true;
   return true;
  } 
  return false;
}

/* To convert from Host Byte Order to Network Byte Order and vice versa */
TYPE nh2hn ( TYPE data, int razm1 )
{
  char tmp;
  const int razm = sizeof(TYPE);
  union { TYPE d;
          unsigned char uc[razm];
         } b;
  b.d = data;
  for (int i=0;i<razm/2;i++) { tmp = b.uc[i];
                               b.uc[i] = b.uc[razm-1-i];
			       b.uc[razm-1-i] = tmp;
                              }
  return b.d;           
 } 

   
/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to fold the average pulsar profile with the pointed period\n"
	 "and keep the periods with level larger than set\n\n"
	 "[USAGE]: %s [options] <file(s) to process>\n\n"
	 "[OPTIONS]:\n"
	 "-c, --counts  <value>       - period in counts\n"
	 "-p, --period  <value>       - period of pulsar in ns. If option -c is given then\n"
	 "                              period in counts will be used\n"
	 "-t, --timeres <value>       - time resolution in ns. Work only together with\n"
	 "                              -p option. In case of absence -p or -t AND -c\n"
	 "                              error will be registrated\n"
	 "--yessingle[=threshold]     - write files with single pulses, default - not write.\n"
	 "                              'threshold' is optional argument. If it is specified\n"
	 "                              only single periods with maximum value larger than this\n"
	 "                              threshold (in sigmas) will be written. By default,\n"
	 "                              this threshold is the same as threshold for searching.\n"
	 "                              If option --nosearch is used, then all periods will be written\n"
	 "                              even if you are using 'threshold' in this option\n"
	 "--noend                     - don't write file with not full period (at the end of\n"
	 "                              file), default - write (with name \"remainder\")\n"
	 "--number      <value>       - number of periods to process, default - all\n"
	 "-b, --begin   <value>       - number of first period to process, default - 0\n"
	 "-s, --shift   <shift>       - shift from the file beginning in counts, default - 0\n"
	 "-l, --left    <left>        - left boundary of chosen window in counts relative to\n"
	 "                              the shift, default - 0\n"
	 "-w, --window  <window>      - size of chosen window in counts, default - from left\n"
	 "                              boundary to the end of period\n"
	 "--link                      - to link periods of next input file(s) with previous,\n"
	 "                              default - no\n"
	 "-d, --path    <path>        - path to the output file(s), default - the same\n"
	 "                              where are the source files\n"
	 "--prof        <profname>    - name of the average profile with destination directory\n"
	 "                              default - where are the source files with name = input file + .prof\n"
	 "--mask        <mask>        - mask for the extension of output file(s),\n"
	 "                              default - .r%%05lld  If it will be other '%%' symbols\n"
	 "                              except that for the output numbers in type of int,\n"
	 "                              long etc. (%%d for example) then the output file\n"
	 "                              extension will be non-predicted\n"
	 "--out         <filename>    - the base name of file for output data. The output\n"
	 "                              filename is base name + mask\n"
	 "                              default - input file + .r%%05ld (see --mask)\n"
	 "--header      <size>        - header in bytes of input file(s), default - 0\n"
	 "--nosearch                  - don't search for individual pulses with the set threshold\n"
         "--threshold   <sigmas>      - value of threshold level in sigma, default - 22.\n"
	 "--gplog <log-file>          - name of GPs log-file, default - gplog\n"
	 "--width <value>             - the maximum width in samples to which average every period during the search\n"
	 "                              The value will be changed by the nearest power of 2 upwards.\n"
	 "-m, --mjd <MJD>             - MJD date of the beginning of the file. If specified,\n"
	 "                              GP's time in MJD will be written to log-gile determined by --gplog option\n"
	 "-q, --quite                 - turn on quite output, skip printing the iteration changes\n"
	 "-h, --help                  - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"shift", required_argument, 0, 's'},
				   {"counts", required_argument, 0, 'c'},
				   {"period", required_argument, 0, 'p'},
				   {"timeres", required_argument, 0, 't'},
				   {"left", required_argument, 0, 'l'},
				   {"window", required_argument, 0, 'w'},
				   {"begin", required_argument, 0, 'b'},
				   {"path", required_argument, 0, 'd'},
				   {"header", required_argument, 0, 21},
				   {"out", required_argument, 0, 22},
				   {"mask", required_argument, 0, 23},
				   {"number", required_argument, 0, 24},
                                   {"link", no_argument, 0, 25},
                                   {"yessingle", optional_argument, 0, 26},
                                   {"noend", no_argument, 0, 27},
				   {"prof", required_argument, 0, 28},
				   {"threshold", required_argument, 0, 29},
                                   {"nosearch", no_argument, 0, 30},
                                   {"gplog", required_argument, 0, 31},
				   {"quite", no_argument, 0, 'q'},
                                   {"mjd", required_argument, 0, 'm'},
                                   {"width", required_argument, 0, 32},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "hvp:t:c:s:l:w:b:d:qm:", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 's':
        shift = (off_t)atof(optarg);
	if (shift < 0) shift = 0;
      break;
            
      case 'c':
        Counts = (off_t)atof(optarg);
	if (Counts < 1) Counts = -1;
      break;
      
      case 'p':
        Period = atof(optarg);
	if (Period <= 0.) Period = -1.;
      break;
      
      case 't':
        TimeResolution = atof(optarg);
	if (TimeResolution <= 0.) TimeResolution = -1.;
      break;
      
      case 'l':
        bl = (off_t)atof(optarg);
	if (bl < 0) bl = 0;
      break;
      
      case 'w':
        window = (off_t)atof(optarg);
	if (window < 1) window = -1;
      break;
      
      case 'b':
        Begin = (off_t)atof(optarg);
	if (Begin < 0) Begin = 0;
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 21:
       Header = atoi(optarg);
       if (Header < 0) Header = 0;
      break;
      
      case 22:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
      
      case 23:
	  if( Mask ) {
	    Mask = 0;
	    free(Mask);
	                 }
	  if( !( Mask = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(Mask, optarg, (int)strlen(optarg));
      break;
            
      case 24:
        Number = (off_t)atof(optarg);
	if (Number < 1) Number = -1;
      break;
      
      case 25:
        Link = 1;
      break;
      
      case 26:
        is_single_write = true;
	if (optarg != NULL) { is_write_threshold = true; write_threshold = atof (optarg); }
	 else { is_write_threshold = false; }
      break;
      
      case 27:
        NoEnd = 1;
      break;

      case 28:
	  if( ProfFile ) {
	    ProfFile = 0;
	    free(ProfFile);
	                 }
	  if( !( ProfFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(ProfFile, optarg, (int)strlen(optarg));
      break;

      case 29:
        search_threshold = atof(optarg);
	if (search_threshold < 1.) search_threshold = 22.;
      break;

      case 30:
        is_search = false;
      break;

      case 31:
	  if( GPLogFile ) {
	    GPLogFile = 0;
	    free(GPLogFile);
	                 }
	  if( !( GPLogFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(GPLogFile, optarg, (int)strlen(optarg));
      break;

      case 'q':
       is_quite = true;
      break;

      case 'm':
        if (startmjd) { startmjd = 0; free (startmjd); }
	if (!(startmjd = (char *)calloc((int)strlen(optarg), sizeof(char *)))) { perror("new startmjd"); exit (1); }
        memcpy (startmjd, optarg, (int)strlen(optarg));
        is_startmjd = true;
      break;

      case 32:
       width = atol (optarg);
       if (width < 1) width = 1;
       width = resize (width);
      break;
            
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
 }

/* Split the name of the file from its path */
char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/* checks string to be a floating-point number */
bool is_float (char *src) {
 int len = (int)strlen (src);
 for (int i=0; i<len; ++i) {
  if (isdigit(src[i]) != 0 || src[i] == '.') ;
   else return false;
 }
 return true;
}

/* To write to file */
template <class tFormat>
void Write (tFormat m, off_t size, int razm, char *name, off_t number, int flag)
{
   int out;
   char *outfilename, *base;
   outfilename = new char[MaxString+1];
          base = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);
   memset(base, 0, MaxString+1);

   if (!strcmp(OutputPath, "\0")) {
     sprintf(base, "%s%s", "%s", Mask);
     sprintf(outfilename, base, (!strcmp(OutFile, "\0") ? name : OutFile), number);
    } else {
             sprintf(base, "%s%s%s", "%s/", "%s", Mask);
             sprintf(outfilename, base, OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), number);
            }
	    
    if (flag == 3) {
     struct stat is_exist;
     if (stat (outfilename, &is_exist) == 0) {
      char *newname = new char[MaxString + 1];
      memset (newname, 0, MaxString +1);
      if (!strcmp(OutputPath, "\0")) sprintf (newname, "%s", "remainder");
       else sprintf (newname, "%s/%s", OutputPath, "remainder");
      
      rename (outfilename, newname);
      delete (newname);
      delete (outfilename);
      delete (base);
      return;
     }
    }

    if (flag == 0) { 
     struct stat is_exist;
     if (stat (outfilename, &is_exist) == 0) remove(outfilename);
    }
     else
  {
      
   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
//   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
	   write(out, m, size*razm);	                                           
           close(out);
        }
  }
      
   memset(m, 0, razm*size);
   	
   delete(outfilename);
   delete(base);   
 }

/* To read from file */
void ReadSrc (off_t l, off_t size, char *name) {
  int file;

     memset(data, 0, size*sizeof(TYPE));
       
       if ( (file = open(name, O_RDONLY | O_LARGEFILE)) == -1 )
//       if ( (file = open(name, O_RDONLY)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
       if (lseek(file, (off_t)Header + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, data, size*sizeof(TYPE));
        close(file);

  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (sizeof(TYPE) != 1) { for (off_t q=0;q<size;q++) data[q] = nh2hn(data[q], sizeof(TYPE)); }
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (sizeof(TYPE) != 1) { for (off_t q=0;q<size;q++) data[q] = nh2hn(data[q], sizeof(TYPE)); }
   #endif
  #endif

}
     
/*====================== Main Function ===================================*/
  
int main ( int argc, char *argv[] )
 {
   struct stat info;
   int next;
   off_t countup;

   if (argc == 1) Help(argv[0]);

   /* Making this process to be able to create files larger than 2GB */
   struct rlimit r;

   r.rlim_cur = r.rlim_max = RLIM_INFINITY;
   setrlimit(RLIMIT_FSIZE,&r);
   /*------------------------------------------*/
   
   OutFile     = new char[MaxString+1];
   Mask        = new char[MaxString+1];
   OutputPath  = new char[MaxString+1];
   ProfFile    = new char[MaxString+1];
   GPLogFile   = new char[MaxString+1];
   if (!OutFile || !Mask || !OutputPath || !ProfFile || !GPLogFile) { printf("Not enough memory!\n"); exit(1); }
   memset(OutFile, 0, MaxString+1);
   memset(Mask, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   memset(ProfFile, 0, MaxString+1);
   memset(GPLogFile, 0, MaxString+1);
   sprintf(OutFile, "%s", "\0");
   sprintf(Mask, "%s", ".r%05lld");
   sprintf(OutputPath, "%s", "\0");
   sprintf(ProfFile, "%s", "\0");
   sprintf(GPLogFile, "%s", "gplog");

   startmjd  = new char[255];
   if (!startmjd) { perror("new startmjd"); exit (1); }
   memset (startmjd, 0, 255);
   
   next = ParseCmdLine(argc, argv);   
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }

   if (is_single_write) {
    if (!is_search) { is_write_threshold = false; write_threshold = -1.; }
     else { if (!is_write_threshold) write_threshold = search_threshold; }
   }

   if (is_startmjd) if (!is_float(startmjd)) {
    printf ("Specified MJD value is bad. Check it!\n");
    exit (1);
   }
   if (is_startmjd) {
    gpmjd = new char[23];
    if (!gpmjd) { perror("new gpmjd"); exit (1); }
   }

   if (!strcmp(ProfFile, "\0")) {
    memset (ProfFile, 0, MaxString+1);
    sprintf (ProfFile, "%s.prof", argv[next]);
   }

   if ( stat(argv[next], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[next]); exit(1); }

   Size = (info.st_size - Header)/sizeof(TYPE);
   if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }
   if (shift >= Size) shift = 0;
   if (bl >= Size || shift + bl >= Size) bl = 0;
   
   step = ( Counts == -1 ? ( Period == -1. || TimeResolution == -1. ? (off_t)-10 : (off_t)rint(Period/TimeResolution) ) : Counts );
   if ( step == (off_t)-10 ) { printf("\nError:\nYou need to point period in counts (--counts) or\npoint both period and time resolution (--period , --timeres)\n"); exit(1); }

//   if (step > Size) { printf("\nError:\nPulsar period is more then size of file !\n"); exit(1); }
   
   left = shift + bl;
   if (window > step) { window = step; printf("\n- warning ! - chosen window is more then pulsar period!\nWindow will be equaled to period\n"); }
   if (window == -1) sz = step; else sz = window;   
   if (left + sz > Size && Link == 0) printf("\n- warning ! - processed window is more then size between end of file\nand left position! Save only available counts.\n");

    data = new TYPE[sz];
    if (!data) { printf("Not enough memory!\n"); exit(1); }

    prof = new float[sz];
    if (!prof) { printf("Not enough memory!\n"); exit(1); }
    memset (prof, 0, sz*sizeof(float));

    if (width > 1) {
     while (step/width < 100) width /= 2;
     record = new float[sz/2];
     if (!record) { printf("Not enough memory!\n"); exit(1); }
    }


printf("\nProcessing...\n");

 for (int i=next;i<argc;i++) {

 is_caption = false; 

 printf(" [%3d]\t%s ... ", i-next, argv[i]);
 fflush(NULL);

 countup = (Link == 0 ? 0 : (i == next ? 0 : countup) );
 razmer = (Link == 0 ? 0 : (i == next ? 0. : razmer + Size) );
 lc = (Link == 0 ? left : (i == next ? left : lc - razmer) );
 szc = (Link == 0 ? sz : (i == next ? sz : sz - szc) );

 if ( stat(argv[i], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[i]); exit(1); }
     
 Size = (info.st_size - Header)/sizeof(TYPE);
 if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }

   if (szc < sz ) {
     if (szc > Size) szc = Size;
     if (countup-1 >= Begin && (Number == -1 ? 1 : (countup-1 < Begin + Number)) ) {
     printf(" + "); fflush(NULL);
     ReadSrc(0, szc, argv[i]);   
     if (NoEnd == 1 && szc == Size) ; else {
      schetchik++;
      for (off_t m=0; m<szc; ++m) prof[m] += (float)data[m];
}
     
     // searching ...
     if (is_search) {
        is_gp(data, szc, countup-1, argv[i-1], 1);
	max_level = level;
	// if we want to search in a range of pulse widths
	// now I suppose that we gonna average as a power of 2, if not than in rebinning2 we have to average for the next width (rather than 2)
	if (width > 1) {
	  szc2 = rebinning2 (record, data, szc, 2);
          is_gp(record, szc2, countup-1, argv[i-1], 2);
	  if (level > max_level) max_level = level;
	  for (unsigned long z=4; z<=width; z*=2) {
	   szc2 = merge_adjacent (record, szc2);
           is_gp(record, szc2, countup-1, argv[i-1], z);
	   if (level > max_level) max_level = level;
	  }
	}
        if (is_single_write && max_level >= write_threshold) {
          if (NoEnd == 1 && szc == Size) Write (data, szc, sizeof(TYPE), argv[i-1], countup-1, 0);
            else { Write (data, szc, sizeof(TYPE), argv[i-1], countup-1, 2); }
        } // is_single_write
     // no search 
     } else {
       if (is_single_write) {
          if (NoEnd == 1 && szc == Size) Write (data, szc, sizeof(TYPE), argv[i-1], countup-1, 0);
            else { Write (data, szc, sizeof(TYPE), argv[i-1], countup-1, 2); }
       } // is_single_write
     }

     }
    szc = sz;
   }

  if (i != argc-1 && (lc >= Size/* || step > Size*/) ) { printf("\nError:\nPulsar period is more then size of current file!\n"); exit(1); }
   
 while (lc < Size) {
   
   if (Number != -1) if (countup >= Begin + Number) { countup++; break; }
   
   if (lc + szc > Size) szc -= ((off_t)rint(lc) - Size) + szc;  

  if (countup >= Begin) {
    if (!is_quite) {
     printf(".%5d", countup+1); 
     fflush(NULL); 
     if (szc < sz) printf(" + ");
    } 
    ReadSrc( (off_t)rint(lc) ,  szc , argv[i]);
    if (NoEnd == 1 && Link == 0 && szc < sz) ; else {
     schetchik++;
     for (off_t m=0; m<szc; ++m) prof[m] += (float)data[m];
    }

   // searching ...
   if (is_search) {
      is_gp(data, szc, countup, argv[i], 1);
      max_level = level;
      // if we want to search in a range of pulse widths
      // now I suppose that we gonna average as a power of 2, if not than in rebinning2 we have to average for the next width (rather than 2)
      if (width > 1) {
        szc2 = rebinning2 (record, data, szc, 2);
        is_gp(record, szc2, countup, argv[i], 2);
        if (level > max_level) max_level = level;
        for (unsigned long z=4; z<=width; z*=2) {
          szc2 = merge_adjacent (record, szc2);
          is_gp(record, szc2, countup, argv[i], z);
          if (level > max_level) max_level = level;
         }
      }
      if (is_single_write && max_level >= write_threshold) {
       if (NoEnd == 1 && Link == 0 && szc < sz) ; else Write (data, szc, sizeof(TYPE), argv[i], countup, 1);
      }
   // no search
   } else {
       if (is_single_write) {
        if (NoEnd == 1 && Link == 0 && szc < sz) ; else Write (data, szc, sizeof(TYPE), argv[i], countup, 1);
       } // is_single_write
   }

  } else { if (!is_quite) { printf(".     "); fflush(NULL); } }      
    ++countup;
    lc = ( Counts == -1 ? (double)left + rint((countup*Period)/TimeResolution) : ((double)left) + (double)(countup*step));
    if (lc < razmer + Size) { lc -= razmer; if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); } }
     else if (!is_quite) fflush(NULL);
                       }

  if (countup-1 >= Begin && (Number == -1 ? 1 : (countup-1 < Begin + Number))) printf("  - done -\n");
    else printf("  - skip -\n");

  } //for

  if (NoEnd == 1 && Link == 1 && szc < sz && is_single_write) Write (data, szc, sizeof(TYPE), argv[argc-1], countup-1, 0);

  // rename file for "remainder"
  if (NoEnd == 0 && szc < sz && is_single_write) Write (data, szc, sizeof(TYPE), argv[argc-1], countup-1, 3);



  for (off_t m=0; m<sz; ++m) prof[m] /= schetchik;

  /* writing the profile */
  int outdescr;
  if ( (outdescr = open(ProfFile, O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", ProfFile);
        exit(1);
       } 
     else
       {
           lseek(outdescr, 0, SEEK_SET);
           write(outdescr, prof, sz*sizeof(float));
           close(outdescr);
        }

  if (is_startmjd) delete (gpmjd);
  delete(startmjd);
  delete(GPLogFile);
  delete(OutFile);
  delete(Mask);
  delete(OutputPath);
  delete(ProfFile);
  delete(data);
  delete(prof);
  if (width > 1) delete (record);

  return 0;
}
