#define MAIN_
#include "mk5decode.hpp"
#include "framedata.hpp"

/*==========================   P r o m p t i n g   ==================================*/

void Help (char *s) {
  printf("\n%sPROGRAMM%s %s\n"
         "It is used to decode Mark5 VLBA special format\n\n"
         "%s[USAGE]:%s %s [options] <file1> [file2 ... fileN]\n\n"
         "%s[OPTIONS]:%s\n"
	 "-d, --path <path>       - path to the output files, default - the same\n"
	 "                          where are the source files\n"
	 "-o, --out <basename>    - basename of files for output data\n"
	 "-b, --begin <frame>     - start processing from this frame\n"
	 "-n, --num <frames>      - number of frames to process\n"
	 "--dummy                 - set dummy mode (no output files)\n"
	 "-h, --help              - help\n\n", bold, none, s, bold, none, s, bold, none);
  exit(0);
}

/*=========================  erase path part from filename  =========================*/
  
char * Basename (char *str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
} 

/*========================= Parsing the command line =================================*/

int ParseCmdLine (int argc, char *argv[]) {
 int op;
 struct option long_options[] = { {"help", no_argument, 0, 'h'},
	                          {"out", required_argument, 0, 'o'},
	                          {"path", required_argument, 0, 'd'},
	                          {"begin", required_argument, 0, 'b'},
	                          {"num", required_argument, 0, 'n'},
	                          {"dummy", no_argument, 0, 13},
	                          {0, 0, 0, 0}
                                };
 while((op = getopt_long(argc, argv, "hd:o:n:b:", long_options, 0)) != EOF)
  switch(op){
    case 'h': Help (Basename(argv[0]));
	      break;
    case 'd': if (outputpath) { outputpath = 0; delete (outputpath); }
	      outputpath = new char[(int)strlen(optarg)];
	      if (!outputpath) { perror("new outputpath"); exit(1); }
	      memcpy (outputpath, optarg, (int)strlen(optarg));
	      break;
    case 'o': if (outname) { outname = 0; delete (outname); }
	      outname = new char[(int)strlen(optarg)];
	      if (!outname) { perror("new outname"); exit(1); }
	      memcpy (outname, optarg, (int)strlen(optarg));
	      break;
    case 'b': BeginFrame = atol (optarg);
	      if (BeginFrame < 0) BeginFrame = 0;
	      break;
    case 'n': NumberOfFrames = atol (optarg);
	      if (NumberOfFrames < 0) NumberOfFrames = 0;
	      break;
    case 13: is_dummy = true;
	     break;
    case '?': Help (Basename(argv[0]));
              break;
  }
 return(optind);
}

/*++++++++++++++++++++++++++++   m a i n   f u n c t i o n +++++++++++++++++++++++++++*/

int main (int argc, char *argv[]) {

 if (argc == 1) Help (Basename(argv[0]));

 syncdata = new unsigned long[syncsize/size_of_long];
 if (!syncdata) { perror("new syncdata"); exit(1); }

 outname = new char[MaxString];
 if (!outname) { perror("new outname"); exit(1); }
 memset (outname, 0, MaxString);
 sprintf (outname, "%s","\0");

 outputpath = new char[MaxString];
 if (!outputpath) { perror("new outputpath"); exit(1); }
 memset (outputpath, 0, MaxString);
 sprintf (outputpath, "%s","\0");

 for (int u=0; u<8; ++u) {
  outnames[u] = new char[MaxString];
  if (!outnames[u]) { perror("new outnames[u]"); exit (1); }
  memset (outnames[u], 0, MaxString);
 }

 // parse command line
 next = ParseCmdLine (argc, argv);
 if (argc < next + 1) { printf ("Not point the file to process!\n"); exit(1); }

 // initialize input data frame array
 raw_framedata = new unsigned long[tracksize/size_of_long];
 if (!raw_framedata) { perror("new raw_framedata"); exit(1); }

 SampleData shot (Ntracks, framesize, tracksize);  // initialize the class object

 printf("\nProcessing...\n");
 for (int f=next; f<argc; f++) {  // loop on input files

 printf(" [%3d]\t%s ... ", f-next, argv[f]); fflush(NULL);

 if ((in = open (argv[f], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };

 syncpos = 0;
 // infinite loop
 while (1) {
  memset (syncdata, 0, syncsize);
  if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
  bytesread = read (in, syncdata, syncsize);
  if (bytesread == -1) { perror("read syncdata"); exit(1); }
  if (bytesread != syncsize) break;

  syncdetect = true;
  syncpos += syncsize;
  for (int i=(syncsize/size_of_long) - 1; i>=0; --i, syncpos -= size_of_long) {
   if (syncdata[i] != syncmask) { syncdetect = false; break; }
  }
  if (syncdetect) { printf("sync pos: %ld bytes  ", syncpos); fflush (NULL); break; }
 } // end of loop
 
 if (!syncdetect) { printf ("No sync word were found!\n"); exit (1); }

 memset (raw_framedata, 0, tracksize);

 if (!is_dummy) {  // on if dummy mode isn't set
 // create output file names
 for (int u=0; u<8; ++u) memset (outnames[u], 0, MaxString);
 if (!strcmp(outputpath, "\0")) for (int u=0; u<8; ++u)
  sprintf (outnames[u], "%s%s", (!strcmp(outname, "\0") ? argv[f] : outname), ext[u]);
 else for (int u=0; u<8; ++u) sprintf (outnames[u], "%s/%s%s", outputpath, (!strcmp(outname, "\0") ? Basename(argv[f]) : outname), ext[u]);

 // at first remove existing file
 for (int u=0; u<8; ++u) unlink (outnames[u]);
 // open for appending
 for (int u=0; u<8; ++u) 
  if ((out[u] = open (outnames[u], O_APPEND | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1) { perror("open"); exit(1); }
 } // is_dummy

 is_info_shown = false;
 
 { // in this block we check the existence of "preframe" file
   // if it does then we check if its size + sync pos is equal to tracksize
   // if it does then we read this preframe file + data before sync pos as extra frame
   struct stat info;
   if (stat ("preframe", &info) != 0) { printf(" | no \"preframe\" file found  \n"); fflush(NULL); }
    else { 
           printf (" | \"preframe\" file found"); fflush (NULL); 
           if (tracksize != info.st_size + syncpos) { printf("  [mismatch]\n"); fflush(NULL); }
             else { printf ("  [match]"); fflush(NULL); 
                    if (BeginFrame == Nframe_skipped) {
                    printf ("  - processing extra frame\n");
                    if (!is_dummy) { // process only if dummy mode isn't set
                     int endfr;
                     if ((endfr = open ("preframe", O_RDONLY)) == -1) { perror("open"); exit(1); }
                     read (endfr, raw_framedata, info.st_size);
                     if ((close (endfr)) == -1) { perror("close"); exit(1); };
                     unsigned long *ptr;
                     ptr = raw_framedata + info.st_size/size_of_long;
                     if ((endfr = open (argv[f], O_RDONLY)) == -1) { perror("open"); exit(1); }
                     if ((lseek (in, 0, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
                     read (endfr, ptr, syncpos);
                     if ((close (endfr)) == -1) { perror("close"); exit(1); };
                     shot.update (raw_framedata); // fill class object with new frame data
                     shot.getAllInfo();
		     printf ("preframe seconds: %lf\n", shot.getSeconds());
                     for (int u=0; u<8; ++u) write (out[u], shot.getChannel(u), shot.getSampleSize());
		     Nframe_processed++;
                     printf ("preframe - done\n");
		     if (NumberOfFrames != 0 && NumberOfFrames == Nframe_processed) {
                       printf ("processed %ld frames (%g secs)\n", Nframe_processed, Nframe_processed * sec_step);
		       break;
		     }
                    } // is_dummy
		    } // BeginFrame
            }  // else !=
         }  // else stat
 }
 
 if (Nframe_skipped < BeginFrame) {
  struct stat information;
  if (stat (argv[f], &information) != 0) { perror("stat information"); exit (1); }
  while (syncpos < information.st_size && Nframe_skipped < BeginFrame) {
   syncpos += tracksize;
   Nframe_skipped++;
  }
  if (BeginFrame == Nframe_skipped) printf ("skipped %ld frames (%g secs)\n", Nframe_skipped, Nframe_skipped * sec_step);
  if (syncpos >= information.st_size) continue;
 }
 
 // sync word is found and
 // now we begin to read frame by frame for each track
 
 if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
 // infinite loop
 while (1) {

// I commented the next line to avoid a terrible log file
//   printf ("%c", tics[(tics_counter++)%4]); fflush (NULL);

   memset (raw_framedata, 0, tracksize);
   bytesread = read (in, raw_framedata, tracksize);
   if (bytesread == -1) { perror("read raw_framedata"); exit(1); }
   if (bytesread != tracksize) {
     // write this piece of data
     if (!is_dummy) {
      int endfr;
      if ((endfr = open ("preframe", O_CREAT | O_TRUNC | O_WRONLY, 0644)) == -1) { perror("open"); exit(1); }
      write (endfr, raw_framedata, bytesread);
      if ((close (endfr)) == -1) { perror("close"); exit(1); }
     }
     break;
   }
   
   shot.update (raw_framedata); // fill class object with new frame data
   
   if (!is_info_shown) { // getting info for zero track (default)
    printf ("\b"); 
    shot.getAllInfo(); 
    printf ("Start seconds: %lf\n", shot.getSeconds());
    is_info_shown = true; 
   } 

   if (!is_dummy) for (int u=0; u<8; ++u) write (out[u], shot.getChannel(u), shot.getSampleSize());
   Nframe_processed++;
   if (NumberOfFrames != 0 && NumberOfFrames == Nframe_processed) break;

// I commented this line to make my log-file more easible to read 
//   printf ("\b"); fflush (NULL);
 } // end of loop

 if ((close (in)) == -1) { perror("close"); exit(1); };
 if (!is_dummy) for (int u=0; u<8; ++u) if ((close (out[u])) == -1) { perror("close"); exit(1); }

 printf (" - done\n");
 printf ("processed %ld frames (%g secs)\n", Nframe_processed, Nframe_processed * sec_step);

 } // f

 if (Nframe_skipped < BeginFrame) 
  printf ("Number of frames to skip is more than available number!\n");

 delete (syncdata);
 delete (raw_framedata);
 delete (outname);
 delete (outputpath);
 for (int u=0; u<8; ++u) delete (outnames[u]);

 return 0;
}
