#include "framedata.hpp"
#include "crc16.hpp"

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include <algorithm>
#include <string>
#include <strstream>

#define OUTPUT_INFO_STRING_LENGTH 65536

extern bool is_check;   // true when check mode is set
Error er;  // error global object

// initialize TrackTable
// 99 - mean bad track number in this program

// this table keep correspondences between group (5) and track (8) numbers
// to the real track number
//                       Group (Digit A10)  Track (Digit A9)
//                                       |  |
const unsigned int FrameData::TrackTable[5][8] = { {  3,  5,  7,  9, 11, 13, 15, 17 },
                                                   { 19, 21, 23, 25, 27, 29, 31, 33 },
	                                           {  2,  4,  6,  8, 10, 12, 14, 16 },
			                           { 18, 20, 22, 24, 26, 28, 30, 32 },
			                           {  1, 35,  0, 34, 99, 99, 99, 99 } };

// sign & magn bit correspondence
const signed char SampleData::vlba[4] = { -3, 1, -1, 3 };
// Codings:
// --------------------------------------------
// VLBA |  sign | magn       AT |  sign | magn
// --------------------------------------------
// +3       1      1         +3     0      1
// +1       1      0         +1     0      0
// -1       0      1         -1     1      0
// -3       0      0         -3     1      1


// constructor
FrameCheckInfo::FrameCheckInfo() {
 crc_mismatch_number = 0;
 spare_mismatch_number = 0;
 cont_mismatch_number = 0;
 fields_mismatch_number = 0;
 labels_mismatch_number = 0;
}

// destructor
FrameCheckInfo::~FrameCheckInfo() {
 crc_mismatch_tracks.clear();
 spare_mismatch_tracks.clear();
 cont_mismatch_tracks.clear();
 fields_mismatch_tracks.clear();
 labels_mismatch_tracks.clear();
}

// add spare digit mismatch
void FrameCheckInfo::addSpare(size_t frame, int tr) {
 spare_mismatch_number++;
 spare_mismatch_tracks[frame].push_back(tr);
}

// get number of spare mismatches
long FrameCheckInfo::getSpareN () {
 return spare_mismatch_number;
}

// get the list of all frame and trackes with spare mismatches
char * FrameCheckInfo::getSpareList (char * str) {
 std::ostrstream output (str, OUTPUT_INFO_STRING_LENGTH);
 std::list<int>::iterator jt, kt;
 bool new_seq;
 
 std::map<size_t, std::list<int> >::iterator it = spare_mismatch_tracks.begin();
 for (; it != spare_mismatch_tracks.end(); ++it) {
  output << "[" << it->first << "(";
  new_seq = true;
  jt = it->second.begin();
  for (; jt != it->second.end(); ++jt) {
   if (new_seq) {
    if (jt != it->second.begin()) output << ", ";
    output << *jt;
    new_seq = false;
   }
   if (jt != it->second.begin()) {
    kt = jt, ++kt;
    if ((*jt) + 1 == *kt) ;
    else { output << ".." << *jt; new_seq = true; } 
   }
  }
  output << ")]";
 }
 return str;
}

// add crc mismatch
void FrameCheckInfo::addCRC(size_t frame, int tr) {
 crc_mismatch_number++;
 crc_mismatch_tracks[frame].push_back(tr);
}

// get number of crc mismatches
long FrameCheckInfo::getCRCN () {
 return crc_mismatch_number;
}

// get the list of all frame and trackes with crc mismatches
char * FrameCheckInfo::getCRCList (char * str) {
 std::ostrstream output (str, OUTPUT_INFO_STRING_LENGTH);
 std::list<int>::iterator jt, kt;
 bool new_seq;
 
 std::map<size_t, std::list<int> >::iterator it = crc_mismatch_tracks.begin();
 for (; it != crc_mismatch_tracks.end(); ++it) {
  output << "[" << it->first << "(";
  new_seq = true;
  jt = it->second.begin();
  for (; jt != it->second.end(); ++jt) {
   if (new_seq) {
    if (jt != it->second.begin()) output << ", ";
    output << *jt;
    new_seq = false;
   }
   if (jt != it->second.begin()) {
    kt = jt, ++kt;
    if ((*jt) + 1 == *kt) ;
    else { output << ".." << *jt; new_seq = true; } 
   }
  }
  output << ")]";
 }
 return str;
}

// add cont mismatch
void FrameCheckInfo::addCont(size_t frame, int tr) {
 cont_mismatch_number++;
 cont_mismatch_tracks[frame].push_back(tr);
}

// get number of cont mismatches
long FrameCheckInfo::getContN () {
 return cont_mismatch_number;
}

// get the list of all frame and trackes with cont mismatches
char * FrameCheckInfo::getContList (char * str) {
 std::ostrstream output (str, OUTPUT_INFO_STRING_LENGTH);
 std::list<int>::iterator jt, kt;
 bool new_seq;
 
 std::map<size_t, std::list<int> >::iterator it = cont_mismatch_tracks.begin();
 for (; it != cont_mismatch_tracks.end(); ++it) {
  output << "[" << it->first << "(";
  new_seq = true;
  jt = it->second.begin();
  for (; jt != it->second.end(); ++jt) {
   if (new_seq) {
    if (jt != it->second.begin()) output << ", ";
    output << *jt;
    new_seq = false;
   }
   if (jt != it->second.begin()) {
    kt = jt, ++kt;
    if ((*jt) + 1 == *kt) ;
    else { output << ".." << *jt; new_seq = true; } 
   }
  }
  output << ")]";
 }
 return str;
}

// add fields mismatch
void FrameCheckInfo::addFields(size_t frame, int tr) {
 fields_mismatch_number++;
 fields_mismatch_tracks[frame].push_back(tr);
}

// get number of fields mismatches
long FrameCheckInfo::getFieldsN () {
 return fields_mismatch_number;
}

// get the list of all frame and trackes with fields mismatches
char * FrameCheckInfo::getFieldsList (char * str) {
 std::ostrstream output (str, OUTPUT_INFO_STRING_LENGTH);
 std::list<int>::iterator jt, kt;
 bool new_seq;
 
 std::map<size_t, std::list<int> >::iterator it = fields_mismatch_tracks.begin();
 for (; it != fields_mismatch_tracks.end(); ++it) {
  output << "[" << it->first << "(";
  new_seq = true;
  jt = it->second.begin();
  for (; jt != it->second.end(); ++jt) {
   if (new_seq) {
    if (jt != it->second.begin()) output << ", ";
    output << *jt;
    new_seq = false;
   }
   if (jt != it->second.begin()) {
    kt = jt, ++kt;
    if ((*jt) + 1 == *kt) ;
    else { output << ".." << *jt; new_seq = true; } 
   }
  }
  output << ")]";
 }
 return str;
}

// add labels mismatch
void FrameCheckInfo::addLabels(size_t frame, int tr) {
 labels_mismatch_number++;
 labels_mismatch_tracks[frame].push_back(tr);
}

// get number of labels mismatches
long FrameCheckInfo::getLabelsN () {
 return labels_mismatch_number;
}

// get the list of all frame and trackes with labels mismatches
char * FrameCheckInfo::getLabelsList (char * str) {
 std::ostrstream output (str, OUTPUT_INFO_STRING_LENGTH);
 std::list<int>::iterator jt, kt;
 bool new_seq;
 
 std::map<size_t, std::list<int> >::iterator it = labels_mismatch_tracks.begin();
 for (; it != labels_mismatch_tracks.end(); ++it) {
  output << "[" << it->first << "(";
  new_seq = true;
  jt = it->second.begin();
  for (; jt != it->second.end(); ++jt) {
   if (new_seq) {
    if (jt != it->second.begin()) output << ", ";
    output << *jt;
    new_seq = false;
   }
   if (jt != it->second.begin()) {
    kt = jt, ++kt;
    if ((*jt) + 1 == *kt) ;
    else { output << ".." << *jt; new_seq = true; } 
   }
  }
  output << ")]";
 }
 return str;
}


// clear all the fields
void FrameCheckInfo::clear () {
 spare_mismatch_number = 0;
 spare_mismatch_tracks.clear();
 crc_mismatch_number = 0;
 crc_mismatch_tracks.clear();
 cont_mismatch_number = 0;
 cont_mismatch_tracks.clear();
 fields_mismatch_number = 0;
 fields_mismatch_tracks.clear();
 labels_mismatch_number = 0;
 labels_mismatch_tracks.clear();
}



// constructor
FrameData::FrameData(int Ntr, size_t fs, size_t ts, bool swap) {
  size_of_long = sizeof (long);
  size_of_long_long = sizeof (long long);
  frame_number = 0;
  Ntracks = Ntr;
  framesize = fs;
  tracksize = ts;
  streamsize = framesize - headersize - auxsize;
  to_reverse = swap;

  output_info = new char[OUTPUT_INFO_STRING_LENGTH];
  if (!output_info) { printf ("Not enough memory!\n"); exit (1); }
  memset (output_info, 0, OUTPUT_INFO_STRING_LENGTH);

  raw = new unsigned long long[tracksize/size_of_long_long];
  if (!raw) { perror("new raw"); exit(1); }
  memset (raw, 0, tracksize);

  tracks = new unsigned long*[Ntracks];
  if (!tracks) { perror("new tracks"); exit(1); }
  for (int i=0; i<Ntracks; ++i) {
   tracks[i] = new unsigned long[framesize/size_of_long];
   if (!tracks[i]) { perror("new tracks[i]"); exit(1); }
   memset (tracks[i], 0, framesize);
  }

  info = (FrameInfo *)calloc (Ntracks, sizeof (FrameInfo));
  if (!info) { perror("calloc info"); exit(1); }

  modpattern = new unsigned long long[(tracksize - Ntracks * (headersize + auxsize))/size_of_long_long];
  if (!modpattern) { perror("new modpattern"); exit(1); } 
  memset (modpattern, 0, tracksize - Ntracks * (headersize + auxsize));

  // initialize modulation pattern
  init_mod_pattern ();
}

// destructor
FrameData::~FrameData() {
 delete (output_info);
 delete (raw);
 delete [] tracks;
 free (info);
 delete (modpattern);
}

// this function generates modulation pattern for bits coding (see p.4 memo #230.2)
void FrameData::init_mod_pattern () {
  static unsigned ff[16] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
  unsigned output;
  for (int j=0; j<(tracksize - Ntracks * (headersize + auxsize))/size_of_long_long; j++) {
   output = ff[10] ^ ff[12] ^ ff[13] ^ ff[15];
   for (int i=15; i>0; i--) ff[i] = ff[i-1];
   ff[0] = output;
   modpattern[j] = output * 0xffffffffffffffffll;
   if (j % 8 == 7) {  // this check is necessary because VLBA formatter advances by one step
	   // including parity bits. Now we haven't these parity bits so we need
	   // to make extra empty step
    output = ff[10] ^ ff[12] ^ ff[13] ^ ff[15];
    for (int i=15; i>0; i--) ff[i] = ff[i-1];
    ff[0] = output;
   }
  }
}

// extract each track from raw data array
void FrameData::fillTracks () {
  for (int t=0; t<Ntracks; ++t) 
   for (int d=0; d<tracksize/size_of_long_long; ++d) 
     tracks[t][d / 32] |=  ( (raw[d] >> t) & 1 ) << (d % 32);  
}

// update frame with new data
void FrameData::update (unsigned long long * src, size_t current_frame) {
  frame_number = current_frame;
  if (frame_number == 0) fci.clear();
  memset (output_info, 0, OUTPUT_INFO_STRING_LENGTH);
  
  memset (raw, 0, tracksize);
  raw = (unsigned long long *)memcpy (raw, src, tracksize); 
  // reverse bits
  if (to_reverse) for (int i=0; i<tracksize/size_of_long_long; ++i) raw[i] = swapBytes (raw[i]);
  for (int i=0; i<Ntracks; ++i) memset (tracks[i], 0, framesize);
  // demodulate our data
  for (int k=(headersize * Ntracks)/size_of_long_long, n=0; k<(tracksize - auxsize * Ntracks)/size_of_long_long; ++k, ++n)
   raw[k] ^= modpattern[n];
  fillTracks ();
  
  for (int i=0; i<Ntracks; ++i) {
   try {
    info[i].setup(tracks[i], framesize/size_of_long, getFrameNumber());
   } // try
   catch (Error er) {
    if (!is_check) {
     printf ("%s\b (track #%d)!\nCheck your data with --check option!\n", er.getMsg(), i);
     exit (1);
    }
    else {
     switch (er.getType()) {
      case Error::SPARE: fci.addSpare(getFrameNumber(), i);
                         break;
      case Error::CRC: fci.addCRC(getFrameNumber(), i);
                       break;
      case Error::CONTINOUS: fci.addCont(getFrameNumber(), i);
                             break;
      case Error::FIELDS: fci.addFields(getFrameNumber(), i);
                          break;
      default: break;
     } // switch
    }
   }
   catch (...) {
    printf ("Unknown exception!\n");
    exit (1);
   }
  }

  // check if the time labels are the same in all the tracks
   double prev_secs, cur_secs;
   for (int i=0; i<Ntracks; ++i) {
    try {
     cur_secs = info[i].getSeconds();
     if (i == 0) { prev_secs = cur_secs; }
      else { if ( (long)round((prev_secs - info[i].getSeconds()) * 10000) != 0) { er.setType(Error::LABELS); er.setMsg("\nTime labels are different in the tracks for the same frame!"); throw er; }
      }
    } // try
    catch (Error er) {
     if (!is_check) {
      printf ("%s\nCheck your data with --check option!\n", er.getMsg());
      exit (1);
     }
     else { if (er.getType() == Error::LABELS) fci.addLabels(getFrameNumber(), i);
     }
    }
    catch (...) {
     printf ("Unknown exception!\n");
     exit (1);
    }
   }
}

// print header info for all tracks
void FrameData::getHeaders () {
 for (int i=0; i<Ntracks; ++i) { printf ("track #%2d:  ", i); info[i].getHeader(); }
}

// print aux info for all tracks
void FrameData::getAuxInfos () {
 for (int i=0; i<Ntracks; ++i) { printf ("track #%2d:  ", i); info[i].getAuxInfo(); }
}

// print all info for all tracks
void FrameData::getAllInfos () {
 for (int i=0; i<Ntracks; ++i) { printf ("track #%2d:  ", i); info[i].getAllInfo(); }
}

// return seconds of the track tr (default tr = 0)
double FrameData::getSeconds (unsigned int tr) {
 return info[tr].getSeconds();
}

// return real track number
unsigned int FrameData::getTrackNumber (unsigned int tr, unsigned int gr) {
 if (tr > 7 || gr > 4) return 99; // return bad value
 return TrackTable[gr][tr];
}

// print header info for pointed track (default - 0)
void FrameData::getHeader (int track) {
 if (track < 0 || track > Ntracks - 1) track = 0;
 printf ("track #%2d:  ", track); 
 info[track].getHeader();
}

// print aux info for pointed track (default - 0)
void FrameData::getAuxInfo (int track) {
 if (track < 0 || track > Ntracks - 1) track = 0;
 printf ("track #%2d:  ", track); 
 info[track].getAuxInfo();
}

// print all info for pointed track (default - 0)
void FrameData::getAllInfo (int track) {
 if (track < 0 || track > Ntracks - 1) track = 0;
 printf ("track #%2d:  ", track); 
 info[track].getAllInfo();
}

// this function swap bytes in 64 bits word
unsigned long long FrameData::swapBytes (unsigned long long word) {
 int i;
 unsigned long long nVal = 0;
 for (i=7; i>=0; i--) { nVal |= (word & 255) << 8*i; word >>= 8; }
 return nVal;
}

// print all check info
void FrameData::getCheckInfo () {
 printf ("\n");
 memset(output_info, 0, OUTPUT_INFO_STRING_LENGTH);
 printf ("CRC mismatches: %d %s\n", fci.getCRCN(), fci.getCRCList(output_info));
 memset(output_info, 0, OUTPUT_INFO_STRING_LENGTH);
 printf ("Spare digits mismatches: %d %s\n", fci.getSpareN(), fci.getSpareList(output_info));
 memset(output_info, 0, OUTPUT_INFO_STRING_LENGTH);
 printf ("Time continuity mismatches: %d %s\n", fci.getContN(), fci.getContList(output_info));
 memset(output_info, 0, OUTPUT_INFO_STRING_LENGTH);
 printf ("Fields differencies events: %d %s\n", fci.getFieldsN(), fci.getFieldsList(output_info));
 memset(output_info, 0, OUTPUT_INFO_STRING_LENGTH);
 printf ("Time labels differencies events: %d %s\n", fci.getLabelsN(), fci.getLabelsList(output_info));
}

// get the frame number
size_t FrameData::getFrameNumber () {
 return frame_number;
}



// constructor
SaveInfo::SaveInfo () {
}

// destructor
SaveInfo::~SaveInfo () {
}

// return seconds record
double SaveInfo::getSeconds () {
 return seconds;
}

// return mjd record
unsigned int SaveInfo::getMJD () {
 return mjd;
}

// return aux info field [0] when index = false (default) or [1] when index = true
unsigned long SaveInfo::getAuxInfo (bool index) {
 return aux_info[index];
}

// return frame number
size_t SaveInfo::getFrameNumber () {
 return frame_number;
}

// set aux info
void SaveInfo::setAuxInfo (unsigned long first, unsigned long second) {
 aux_info[0] = first;
 aux_info[1] = second;
}

// set MJD
void SaveInfo::setMJD (unsigned int src) {
 mjd = src;
}

// set seconds
void SaveInfo::setSeconds (double src) {
 seconds = src;
}

// set frame number
void SaveInfo::setFrameNumber (size_t frame) {
 frame_number = frame;
}






// constructor
FrameInfo::FrameInfo () {
 initialized = false;
}

// destructor
FrameInfo::~FrameInfo () {
}

// this function reverse bits order in 16-bit word
unsigned short FrameInfo::BCD16_reverse (unsigned short word) {
 int i;
 unsigned short nVal = 0;
 for (i=15; i>=0; i--) { nVal |= (word & 1) << i; word >>= 1; }
 return nVal;
}

// this function reverse bits order in 8-bit word
unsigned char FrameInfo::BCD8_reverse (unsigned char word) {
 int i;
 unsigned char nVal = 0;
 for (i=7; i>=0; i--) { nVal |= (word & 1) << i; word >>= 1; }
 return nVal;
}

// calculate CRC-16 on time record
unsigned short FrameInfo::calcCRC (unsigned char *ti, int len) {
 CCrc16 crcode;
 crcode.m_Polynomial = 0x8005;      // Most commonly 0x8005 or 0x1021
 crcode.m_Reflect = false;          // use TRUE to reverse bits of input bytes
 crcode.m_ReflectCrc = false;       // use TRUE to reverse bits of output crc
 crcode.m_CrcXorOutput = 0;         // use 0xFFFF to invert crc output
 crcode.m_ResetValue = 0;           // Most commonly 0 or 0xFFFF

 crcode.CreateTable();
 return crcode.ComputeCrcUsingTable (ti, len);
}

// initialize class data from track
void FrameInfo::setup (unsigned long *tr, int len, size_t frame) {
  frame_number = frame;
  if (frame_number == 0) initialized = false;
  syncword = tr[0];
  for (int i=0; i<4; ++i) time[i] = (tr[1] >> (8*i)) & 255;
  for (int i=0; i<2; ++i) time[i+4] = (tr[2] >> (8*i)) & 255;
  for (int i=0; i<6; ++i) time[i] = BCD8_reverse(time[i]);  // reverse bits
  crc = (tr[2] >> 16) & 65535;
  crc = BCD16_reverse(crc);  // reverse bits
  calc_crc = calcCRC (time, 6);  // calculate CRC-16
  if (calc_crc == crc) is_crc_ok = true; else is_crc_ok = false;  // check if CRC is OK
  mjd = (time[0]>>4) * 100 + (time[0] & 15) * 10 + (time[1]>>4);
  seconds = (time[1] & 15) * 10000 + (time[2]>>4) * 1000 + (time[2] & 15) * 100 + (time[3]>>4) * 10 + (time[3] & 15) + (time[4]>>4) / 10. + (time[4] & 15) / 100. + (time[5]>>4) / 1000. + (time[5] & 15) / 10000.;
  station = tr[len - 2] & 255;
  station = BCD8_reverse (station);  // reverse bits
  station_id = (station>>4)*10 + (station & 15);
  for (int i=1; i<4; ++i) rec[i-1] = (tr[len - 2] >> (8*i)) & 255;
  for (int i=0; i<3; ++i) rec[i] = BCD8_reverse (rec[i]);  // reverse bits 
  recorder = rec[0] >> 4;
  headstack = rec[0] & 15;
  pos_sign = rec[1] >> 4;
  pos_sign_char = (pos_sign == 0 ? '+' : (pos_sign == 8 ? '-' : '?'));
  position = (rec[1] & 15) * 100 + (rec[2]>>4) * 10 + (rec[2] & 15);
  for (int i=0; i<3; ++i) dar[i] = (tr[len - 1] >> (8*i)) & 255;
  track = dar[0] & 15;
  group = dar[0] >> 4;
  trackN = FrameData::getTrackNumber (track, group);
  bit = dar[1] & 15;
  bit_char = (bit == 0 ? 'm' : (bit == 1 ? 's' : '?'));
  sideband = dar[1] >> 4;
  sideband_char = (sideband == 0 ? 'l' : (sideband == 1 ? 'u' : '?'));
  converter = dar[2] & 15;
  bbc = converter + 1;
  darN = dar[2] >> 4;
  spare = (tr[len - 1] >> 24) & 255;
  if (spare == 0) is_spare_zero = true; else is_spare_zero = false;

  if (!initialized) { save.setAuxInfo (tr[len-2], tr[len-1]); save.setMJD (mjd); save.setSeconds (seconds); initialized = true; save.setFrameNumber(frame_number); }
   else {
    are_fields_same = (save.getAuxInfo(0) == tr[len-2] ? true : false);
    are_fields_same = (save.getAuxInfo(1) == tr[len-1] ? true : false);
    are_fields_same = (save.getMJD() == mjd ? true : false);
    // check continuous of time
    if ( (long)round((seconds - save.getSeconds())*10000) != 25*(frame_number-save.getFrameNumber()) ) is_time_continous = false;
     else { is_time_continous = true; save.setFrameNumber(frame_number); }
    save.setAuxInfo (tr[len-2], tr[len-1]); save.setMJD (mjd); save.setSeconds (seconds);

    // check the info is ok, if not exit
    if (!is_crc_ok) { er.setType(Error::CRC); er.setMsg("\nCRC mismatch %d [should be %d]!", crc, calc_crc); throw er; }
    if (!is_spare_zero) { er.setType(Error::SPARE); er.setMsg("\nSpare digits are not zero!"); throw er; }
    if (!is_time_continous) { er.setType(Error::CONTINOUS); er.setMsg("\nTime is not continuous!"); throw er; }
    if (!are_fields_same) { er.setType(Error::FIELDS); er.setMsg("\nSome header fields are different for different frames (not time :))!"); throw er; }
   }
}

// print header info
void FrameInfo::getHeader () {
  printf ("MJD = [%d]%d  secs = %.4lf  CRC = 0x%0x [%s]\n", mjd_prefix, mjd, seconds, crc, is_crc_ok ? "OK" : "Mismatch");
}

// print aux info
void FrameInfo::getAuxInfo () {
  printf ("station: %3d rec: %d head: %2d pos: %c%d TRACK# %2d bit: \"%c\" band: \"%c\" bbc: %d dar# %d [spare: %s]\n", station_id, recorder, headstack, pos_sign_char, position, trackN, bit_char, sideband_char, bbc, darN, is_spare_zero ? "OK" : "Mismatch");
}

// print all info
void FrameInfo::getAllInfo () {
  printf ("MJD = [%d]%d  secs = %.4lf  CRC = 0x%0x [%s] station: %3d rec: %d head: %2d pos: %c%d TRACK# %2d bit: \"%c\" band: \"%c\" bbc: %d dar# %d [spare: %s]\n", mjd_prefix, mjd, seconds, crc, is_crc_ok ? "OK" : "Mismatch", station_id, recorder, headstack, pos_sign_char, position, trackN, bit_char, sideband_char, bbc, darN, is_spare_zero ? "OK" : "Mismatch");
}

// return seconds record
double FrameInfo::getSeconds () {
 return seconds; 
}





// constructor
SampleData::SampleData(int Ntr, size_t fs, size_t ts, bool swap) : FrameData (Ntr, fs, ts, swap) {
  samplesize = 32 * streamsize;
  for (int i=0; i<2; ++i) {
   for (int j=0; j<4; ++j) cp[i].WXYZ[j] = new signed char[samplesize];
  }
}

// destructor
SampleData::~SampleData() {
  for (int i=0; i<2; ++i) {
   for (int j=0; j<4; ++j) delete (cp[i].WXYZ[j]); 
  }
}

// update sample data
void SampleData::update (unsigned long long *src, size_t current_frame) {

  FrameData::update (src, current_frame);
  int samplecounter;

  for (int p=0; p<2; ++p) { // cycle on polarizations
   for (int b=0; b<4; ++b) { // cycle on sidebands
    samplecounter = 0;
    for (int i=3; i<framesize/size_of_long - 2; ++i) { // cycle on each long-word of data of track 
     for (int j=0; j<32; ++j) {  // cycle on bits of long-word
      // just for fan-out = 4
      // cycle on number decimated tracks belonged to the same stream
      for (int k=0; k<8; k+=2) {
       // p*16 means that for RCP tracks are shifted by 16
       // (b/2)*32 means that for each polarizations sidebands on other central freq  are shifted by 32
       // 1-(b%2) it's just to correctly give values to W and X
       //   because for the first W we have shift by 1
       //   and for the second X -- no shift at all
       cp[p].WXYZ[b][samplecounter++] = vlba[((tracks[k+(1-(b%2))+p*16+(b/2)*32][i] >> j) & 1) + (((tracks[k+(1-(b%2))+8+p*16+(b/2)*32][i] >> j) & 1) << 1)];
      } // k
     } // j
    } // i
   } // b
  } //p
}

// return the size of the sample
size_t SampleData::getSampleSize () {
  return samplesize;
}

// return pointer to the LCP most upper sideband (0)
signed char* SampleData::getLCPW () {
  return cp[0].WXYZ[0];
}

// return pointer to the LCP lower sideband in a upper frequency (1)
signed char* SampleData::getLCPX () {
  return cp[0].WXYZ[1];
}

// return pointer to the LCP upper sideband in a lower frequency (2)
signed char* SampleData::getLCPY () {
  return cp[0].WXYZ[2];
}

// return pointer to the LCP lower sideband in a lower frequency (3)
signed char* SampleData::getLCPZ () {
  return cp[0].WXYZ[3];
}

// return pointer to the RCP most upper sideband (0)
signed char* SampleData::getRCPW () {
  return cp[1].WXYZ[0];
}

// return pointer to the RCP lower sideband in a upper frequency (1)
signed char* SampleData::getRCPX () {
  return cp[1].WXYZ[1];
}

// return pointer to the RCP upper sideband in a lower frequency (2)
signed char* SampleData::getRCPY () {
  return cp[1].WXYZ[2];
}

// return pointer to the RCP lower sideband in a lower frequency (3)
signed char* SampleData::getRCPZ () {
  return cp[1].WXYZ[3];
}

// return pointer to the channel by its number from LCP W -> Z and then RCP W -> Z
signed char* SampleData::getChannel (int ch) {
  if (ch < 0 || ch > 7) {
   printf ("Bad channel number!\n");
   exit (1);
  }
  return cp[ch / 4].WXYZ[ch % 4];	
}

// constructor
Error::Error () {
 msg = new char[255];
 if (!msg) { printf("Not enough memory!\n"); exit (1); }
 memset (msg, 0, 255);
};

// another constructor
Error::Error (char * src) {
 msg = new char[255];
 if (!msg) { printf("Not enough memory!\n"); exit (1); }
 memset (msg, 0, 255);
 strcpy (msg, src);
};

// constructor with template and 2 arguments
Error::Error (char * mask, int arg1, int arg2) {
 msg = new char[255];
 if (!msg) { printf("Not enough memory!\n"); exit (1); }
 memset (msg, 0, 255);
 sprintf (msg, mask, arg1, arg2);
}

// destructor
Error::~Error () {
 msg = 0;
}

// get message
char * Error::getMsg () {
 return msg;
}

// set message
void Error::setMsg (char * src) {
 memset (msg, 0, 255);
 strcpy (msg, src);
}

// set message by template and two arguments
void Error::setMsg (char * mask, int arg1, int arg2) {
 memset (msg, 0, 255);
 sprintf (msg, mask, arg1, arg2);
}

// get type of the error
Error::ErrorType Error::getType () {
 return type;
}

// set type of the error
void Error::setType (Error::ErrorType src) {
 type = src;
}

