/*==========================================================================
 *  Some general comment
 *  This programm was written in order to decode our test observations of
 *  Giant Pulses from the Millisecond pulsar B1937+21 using 100-n GBT radio
 *  telescope at Green Bank with VLBA Mark5A back-end
 *  Data of observations:  Feb 5, 2005
 *  2bit sampling, 512 Mbit/s -- pure data rate
 *  2 circular polarizations: LCP & RCP
 *  4 16-MHz bands for each pol., i.e.:
 *    central freq.  2068 MHz ->  2052 - 2068  (lower, inverse, Z)
 *                                2068 - 2084  (upper, direct, Y)
 *    central freq.  2100 MHz ->  2084 - 2100  (lower, inverse, X)
 *                                2100 - 2116  (upper, direct, W)
 *  
 *  This corresponds to BBC's as:
 *  LCP-W  ->  BBC1 - L       RCP-W  ->  BBC2 - L
 *  LCP-X  ->  BBC1 - U       RCP-X  ->  BBC2 - U
 *  LCP-Y  ->  BBC3 - L       RCP-Y  ->  BBC4 - L
 *  LCP-Z  ->  BBC3 - U       RCP-Z  ->  BBC4 - U
=============================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>

#include <inttypes.h>
#include <fftw3.h>

#ifdef MAIN_

#include <getopt.h>

#define MaxString 255

int in;         // input file descriptor
int out[8];     // output file descriptors for recording all channels
char *ext[8] = { ".lcp.W", ".lcp.X", ".lcp.Y", ".lcp.Z", 
	         ".rcp.W", ".rcp.X", ".rcp.Y", ".rcp.Z"};  // file extensions for different channels
float **spectra; // arrays of average power spectra
char *outname;  // basename for output files, default - the same as input file
char *outnames[8];  // full names of files with output data
char *outputpath; // path to the output data
int bytesread;  // how many bytes we have read from the input file
int size_of_long = sizeof(long);  // size of the long type
int size_of_long_long = sizeof(long long);  // size of the long long type
int next;  // counter to the first file to process
bool is_dummy = false;   // true when dummy mode is set
bool is_check = false;   // true when check mode is set
bool is_info_shown = false;  // true when info for first frame is shown

/*+++ present output info +++*/
char bold[] = "\033[1m"; 
char ul[] = "\033[4m"; 
char none[] = "\033[0m";
char tics[4] = { '-', '\\', '|', '/' };
unsigned char tics_counter = 0;

/*+++++++++ sync section +++++++++*/
unsigned long syncmask = 0xffffffff; // all ones in 4 bytes
int syncsize = 256;                  // number of sync record for all tracks in bytes
unsigned long *syncdata;             // array for searching sync record;
off_t syncpos = 0;                   // position of sync record in bytes
bool syncdetect;                     // flag for sync record detection

/*+++++++++ frame section ++++++++*/
unsigned long long *raw_framedata;  // raw array of one frame for all 64 tracks
int Ntracks = 64;              // number of tracks
size_t framesize = 2520;       // size of frame in bytes
size_t tracksize = 161280;     // size of frame for all tracks in bytes = 64 tracks * 2520 bytes
size_t Nframe_processed = 0;   // the number of processed frames
double sec_step = 0.0025;      // the step in seconds between frames
size_t NumberOfFrames = 0;     // number of frames to process (0 means all frames)
size_t BeginFrame = 0;         // start to process from this frame
size_t Nframe_skipped = 0;     // number of frames skipped before processing

bool is_spectra = false;       // calculate average spectra in each channel rather than raw data
bool swap_bytes = false;       // to swap bytes in input data or not

fftwf_complex *x;  // to calculate spectra
fftwf_plan pfor;   // plan variable for FFTW
float SpectraLevel[8];  // spectrum average level to eliminate

#endif
