#include "cmd.hpp"

#include <getopt.h>
#include <stdio.h>
#include <string.h>

/* help */
void help (char *s) {
  printf("\n%sPROGRAMM%s %s\n"
         "To decode Mark5/VLBA data. Works only for data recorded in 512-8-2 and 512-16-2 modes\n\n"
         "%s[USAGE]:%s %s [options] <file1> [file2 ... fileN]\n\n"
         "%s[OPTIONS]:%s\n"
	 "-d, --path <path>       - path to the output files, default - the same where are the source files\n"
	 "-o, --out <basename>    - basename of files for output data, if basename is \"-\" then\n"
	 "                          decoded data (not spectra) will be written to the standard output\n"
	 "--swap-bytes            - to reverse bits order in input data, use this option on Apple machines\n"
	 "-b, --begin <frame>     - start processing from this frame\n"
	 "-n, --num <frames>      - number of frames to process\n"
	 "--dummy                 - set dummy mode (no output files)\n"
	 "--check                 - set check mode - the same as dummy but together with\n"
	 "                          checking the integrity of input data, checksums,\n"
	 "                          time continuity, identity of some data in all the tracks, ...\n"
	 "--debug[=track]         - to debug the data, i.e. get full info for every frame; with this option\n"
	 "                          --check and --dummy are also set up\n"
	 "                          track is optional, by default info from track #0 will be written, and\n"
	 "                          one can change the output to another track\n"
	 "--trackmap              - to get correspondance map between channels and tracks, BBCs, sidebands, bits\n"
	 "                          --dummy set up also, number of frames set up to 1\n"
	 "--spectra[=number]      - calculate average spectra rather than raw data\n"
	 "                          number is optional arg that means number of counts to average in\n"
	 "                          calculated spectra, default = 16\n"
	 "--show-spectra          - show built spectra during the calculation, use together with --spectra\n"
	 "--skipharm <number>     - number of harmonics to skip from the beginning in calculated spectra, default - 0\n"
	 "--stat                  - to calculate bit statistics\n"
	 "--mjd_prefix <value>    - two first digits of MJD date of the beginning of the decoded files, default = 53\n"
	 "-q, --quite             - turn on quite output, skip printing frames number changes\n"
	 "--chan <num>            - number of channels. Only 8-channel and 16-channel modes are implemented, default - 8\n"
	 "-e, --extract <chan>    - number of channel to decode. It should be >= 0 and less than the number of channels\n"
	 "                          defined by --chan option. This option works only to decode the data rather than\n"
	 "                          to make a spectra. By default, all channels will be decoded\n"
	 "--fanout <num>          - fan-out, Only 1:4 and 1:2 fan-outs are implemented. <num> is 4 for fan-out 1:4, and\n"
	 "                          2 for fan-out 1:2. Default - 4\n"
	 "--label <string>        - extra label like the name of the telescope, or experiment, or data taking machine\n"
	 "                          to draw in the very first legend row in spectra or statistics plots\n"
	 "-h, --help              - help\n\n", bold, none, s, bold, none, s, bold, none);
  exit(0);
}

/* parse command line */
int parse_command_line (int argc, char *argv[]) {
 int op;
 struct option long_options[] = { {"help", no_argument, 0, 'h'},
	                          {"out", required_argument, 0, 'o'},
	                          {"path", required_argument, 0, 'd'},
	                          {"begin", required_argument, 0, 'b'},
	                          {"num", required_argument, 0, 'n'},
	                          {"dummy", no_argument, 0, 13},
	                          {"spectra", optional_argument, 0, 14},
	                          {"swap-bytes", no_argument, 0, 15},
	                          {"check", no_argument, 0, 16},
				  {"mjd_prefix", required_argument, 0, 17},
				  {"show-spectra", no_argument, 0, 18},
				  {"skipharm", required_argument, 0, 19},
				  {"stat", no_argument, 0, 20},
				  {"debug", optional_argument, 0, 21},
				  {"trackmap", no_argument, 0, 22},
				  {"quite", no_argument, 0, 'q'},
				  {"chan", required_argument, 0, 23},
				  {"fanout", required_argument, 0, 24},
				  {"extract", required_argument, 0, 'e'},
				  {"label", required_argument, 0, 25},
	                          {0, 0, 0, 0}
                                };
 while((op = getopt_long(argc, argv, "hd:o:n:b:qe:", long_options, 0)) != EOF)
  switch(op){
    case 'h': help (basename(argv[0]));
	      break;
    case 'd': if (outputpath) { outputpath = 0; delete (outputpath); }
	      outputpath = new char[(int)strlen(optarg)];
	      if (!outputpath) { perror("new outputpath"); exit(1); }
	      memcpy (outputpath, optarg, (int)strlen(optarg));
	      break;
    case 'o': if (outname) { outname = 0; delete (outname); }
	      outname = new char[(int)strlen(optarg)];
	      if (!outname) { perror("new outname"); exit(1); }
	      memcpy (outname, optarg, (int)strlen(optarg));
	      break;
    case 'b': BeginFrame = atol (optarg);
	      if (BeginFrame < 0) BeginFrame = 0;
	      break;
    case 'n': NumberOfFrames = atol (optarg);
	      if (NumberOfFrames < 0) NumberOfFrames = 0;
	      break;
    case 13: is_dummy = true;
	     break;
    case 14: is_spectra = true;
             if (optarg != NULL) aver = atoi (optarg);
	     if (aver < 1) aver = 1;
	     break;
    case 15: swap_bytes = true;
	     break;
    case 16: is_check = true;
             is_dummy = true;
	     break;
    case 17: if (atoi(optarg) >= 0) mjd_prefix = atoi(optarg);
	     break;
    case 18: is_spectra_show = true;
             break;
    case 19: skipharm = atoi (optarg);
             break;
    case 20: is_stat = true;
             break;
    case 21: is_debug = true;
             is_check = true;
	     is_dummy = true;
	     if (optarg != NULL) debug_track = atoi (optarg);
	     if (debug_track < 0 || debug_track > 63) debug_track = 0;
	     break;
    case 22: is_trackmap = true;
             is_dummy = true;
	     break;
    case 'q': is_quite = true;
              break;
    case 23: Nchan = (unsigned char)atoi (optarg);
             if (Nchan != 8 && Nchan != 16) Nchan = 8;
	     break;
    case 24: fanout = atoi (optarg);
             if (fanout != 4 && fanout != 2) fanout = 4;
	     break;
    case 'e': is_chan_extract = true; 
             chan_extract = atoi (optarg);
             break;
    case 25: if (label) { label = 0; delete (label); }
	      label = new char[(int)strlen(optarg)];
	      if (!label) { perror("new label"); exit(1); }
	      memcpy (label, optarg, (int)strlen(optarg));
	      break;
    case '?': help (basename(argv[0]));
              break;
  }
 return(optind);
}
