#include "mjd.hpp"
#include <string>
#include <math.h>

extern int mjd_header;

// get mjd value of the beginning of the output decoded files
// this MJD will be written as first 27 bytes of every output files (with decimal point)
char* getMJD (unsigned int days, double seconds, int mjd_len, char *mjd) {
 mpf_t temp1, temp2;
 mpf_init2 (temp1, 512); mpf_init2 (temp2, 512);
 
 mpf_set_d (temp1, seconds);
 mpf_set_str (temp2, "86400.0", 10);
 mpf_div (temp1, temp1, temp2);
 mpf_set_ui (temp2, days);
 mpf_add (temp1, temp2, temp1);
 mpf_clear (temp2);
 long exponent = mjd_len;
 char *cur;
 cur = new char[mjd_header-1];
 if (!cur) { perror("new cur"); exit (1); }
 memset (cur, 0, mjd_header-1);
 cur = mpf_get_str (cur, &exponent, 10, mjd_header-1, temp1); // "-1" because of no radix at this point
 std::string s = cur, s1;
 s1 = s.substr(0, mjd_len) + "." + s.substr(mjd_len, (int)strlen(cur)-mjd_len);
 memcpy (mjd, s1.c_str(), mjd_header);
 delete (cur);
 mpf_clear (temp1);
 return mjd;
}

// convert seconds to string of hh:mm:ss.ssss
char* getTime (double seconds, char *starttime) {
 int hour = (int)(seconds/3600.);
 int min = (int)((seconds - hour * 3600.)/60.);
 double secs = seconds - hour * 3600. - min * 60.;
 sprintf (starttime, "%02d:%02d:%07.4lf", hour, min, secs);
 return starttime;
}

// to calculate the date from the given MJD
// this code is taken from slalib.c from sigproc package
void slaDjcal (int ndp, double djm, int iymdf[4], int *j) {
/*
**  - - - - - - - - -
**   s l a D j c a l
**  - - - - - - - - -
**
**  Modified Julian Date to Gregorian calendar, expressed
**  in a form convenient for formatting messages (namely
**  rounded to a specified precision, and with the fields
**  stored in a single array).
**
**  Given:
**     ndp      int       number of decimal places of days in fraction
**     djm      double    Modified Julian Date (JD-2400000.5)
**
**  Returned:
**     iymdf    int[4]    year, month, day, fraction in Gregorian calendar
**     *j       int       status:  nonzero = out of range
**
**  Any date after 4701BC March 1 is accepted.
**
**  Large ndp values risk internal overflows.  It is typically safe
**  to use up to ndp=4.
**
**  The algorithm is derived from that of Hatcher 1984 (QJRAS 25, 53-55).
**
**  Defined in slamac.h:  dmod
**
**  Last revision:   17 August 1999
**
**  Copyright P.T.Wallace.  All rights reserved.
*/
   double fd, df, f, d;
   int jd, n4, nd10;

/* Validate */
   if ( ( djm <= -2395520.0 ) || ( djm >= 1.0e9 ) ) {
      *j = - 1;
      return;
   } else {

   /* Denominator of fraction */
      fd = pow ( 10.0, (double) gmax ( ndp, 0 ) );
      fd = dnint ( fd );

   /* Round date and express in units of fraction */
      df = djm * fd;
      df = dnint ( df );

   /* Separate day and fraction */
      f = dmod ( df, fd );
      if ( f < 0.0 ) f += fd;
      d = ( df - f ) / fd;

   /* Express day in Gregorian calendar */
      jd = (int) dnint ( d ) + 2400001L;
      n4 = 4L * ( jd + ( ( 2L * ( ( 4L * jd - 17918L ) / 146097L)
                                       * 3L ) / 4L + 1L ) / 2L - 37L );
      nd10 = 10L * ( ( ( n4 - 237L ) % 1461L ) / 4L ) + 5L;
      iymdf[0] = (int) ( ( n4 / 1461L ) - 4712L );
      iymdf[1] = (int) ( ( ( nd10 / 306L + 2L ) % 12L ) + 1L );
      iymdf[2] = (int) ( ( nd10 % 306L ) / 10L + 1L );
      iymdf[3] = (int) dnint ( f );
      *j = 0;
   }
}

// convert mjd to YYYY/MM/DD
char* getDate (unsigned int mjd, char *startdate) {
 int iymdf[4], j;
 slaDjcal (1, (double)mjd, iymdf, &j);
 if (j == 0) sprintf (startdate, "%04d/%02d/%02d", iymdf[0], iymdf[1], iymdf[2]);
  else sprintf (startdate, "%s/%s/%s", "????", "??", "??");
 return startdate;
}
