/*====================================================================================
 *  This programm decodes Mk5/VLBA data recorded in 512-8-2 and 512-16-2 modes only
 *
 *  written
 *  (c) Vlad Kondratiev, York University, 2005
 *  updated
 *  (c) Vlad Kondratiev, West Virginia University, 2007
 *
 *===================================================================================*/
#ifndef _MK5DECODE_HPP
#define _MK5DECODE_HPP

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>

#include <inttypes.h>

#include <sys/resource.h>

#include "cmd.hpp"
#include "wisdom.hpp"
#include "framedata.hpp"
#include "mjd.hpp"
#include "stat.hpp"
#include "op.hpp"
#include "plotspectrum.hpp"
#include "plotstat.hpp"

#define MaxString 255

#define NUMCHAN 8
#define NUMBITS 2
#define FANOUT 4

int in;       // input file descriptor
int *out;     // output file descriptors for recording all channels
char ext[] = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '0', '1', '2', '3', '4', '5' }; 
float **spectra; // arrays of average power spectra
char *outname;  // basename for output files, default - the same as input file
char **outnames;  // full names of files with output data
char *outputpath; // path to the output data
int bytesread;  // how many bytes we have read from the input file
int size_of_int = sizeof(int);  // size of the int type
int size_of_long_long = sizeof(long long);  // size of the long long type
int next;  // counter to the first file to process
bool is_dummy = false;   // true when dummy mode is set
bool is_check = false;   // true when check mode is set
bool is_info_shown = false;  // true when info for first frame is shown
bool is_trackmap = false;  // true when one wants to get track map
bool is_quite = false; // true when one wants "quite" output without changing frames number
bool is_stdout = false; // true when files are written to the standard output

bool is_debug = false; // true if we want to get full info for every frame
                       // if true, is_check=true and is_dummy=true as well
int debug_track = 0;   // track to debug if is_debug = true
bool is_stat = false; // true when bit statistics is calculating
bool is_wisdom = false; // true if wisdom file was read successfully
int aver = 16; // number of points to average the spectrum
int skipharm = 0; // number of harmonics to skip in the spectrum

bool is_chan_extract = false; // true when one wants to extract only one channel
int chan_extract;             // number of channel to extract

/*+++ present output info +++*/
char bold[] = "\033[1m";
char ul[] = "\033[4m";
char none[] = "\033[0m";
char tics[4] = { '-', '\\', '|', '/' };
unsigned char tics_counter = 0;

/*+++++++++ sync section +++++++++*/
unsigned int syncmask = 0xffffffff;  // all ones in 4 bytes
int syncsize = 256;                  // number of sync record for all tracks in bytes
unsigned int *syncdata;              // array for searching sync record;
off_t syncpos = 0;                   // position of sync record in bytes
bool syncdetect;                     // flag for sync record detection

/*+++++++++ frame section ++++++++*/
unsigned long long *raw_framedata;  // raw array of one frame for all 64 tracks
int Ntracks = 64;              // number of tracks
int fanout  = FANOUT;          // fan-out, "4" means fan-out of 1:4
size_t framesize = 2520;       // size of frame in bytes (2500 bytes of data, header = 20 bytes: 12 bytes in the beginning of the frame + 8 bytes of auxiliary info in the end of the frame)
size_t tracksize;              // size of frame for all tracks in bytes = 64 tracks * 2520 bytes
size_t Nframe_processed = 0;   // the number of processed frames
size_t NumberOfFrames = 0;     // number of frames to process (0 means all frames)
size_t BeginFrame = 0;         // start to process from this frame
size_t Nframe_skipped = 0;     // number of frames skipped before processing
unsigned char Nchan = NUMCHAN; // number of channels
unsigned char Nbits = NUMBITS; // number of bits
float samp_freq;               // sampling frequency (in MHz)
double sec_step;               // the step in seconds between frames (0.0025 sec)

bool is_spectra = false;       // calculate average spectra in each channel rather than raw data
bool is_spectra_show = false;  // show calculated spectra
bool swap_bytes = false;       // to swap bytes in input data or not
size_t spectrumsize;           // size of the spectrum

fftwf_complex *x;  // to calculate spectra
fftwf_plan pfor;   // plan variable for FFTW
float *SpectraLevel;  // spectrum average level to eliminate

char *startdate; // start date in YYYY/MM/DD
char *starttime; // start time in hh:mm:ss.ssss
int mjd_header = 27; // length in bytes for MJD string with radix
int mjd_prefix = 53; // first 2 digits of MJD date

/* graphics part */
char pgplot_dev[MaxString] = "/xwindow"; // device for pgplot
float char_size = 1.;                    // character size
int dev_id;      // identifier for default xwindow device
int stat_dev_id; // identifier for xwindow device to plot the bit-statistics
char *label;     // Label like "GBT Mark5" to draw in the plots

#endif // _MK5DECODE_HPP
