#include "plotstat.hpp"

char stat_xlabel[MaxString] = "ADC Value"; // x-label
char stat_ylabel[MaxString] = "Bit Fraction (%)"; // y-label

// draw to device
void drawing (float *x, float *y, StatInfo& bs, char *outname, char *mjd, char *startdate, char *starttime, double dur) {

 char *cur, *xaxisfmt, *yaxisfmt;
 cur = new char[MaxString];
 xaxisfmt = new char[MaxString];
 yaxisfmt = new char[MaxString];
 if (!cur || !xaxisfmt || !yaxisfmt) { perror("new"); exit (1); }
 memset (cur, 0, MaxString);

 unsigned int nlev = bs.getNlevels();
 unsigned char nchan = bs.getNchan();
 cpgeras ();
 cpgslw (2);
 int channel;
 float hstep = 0.24;
 float vstep = 0.405 / (nchan / 8);
 float vbeg = 1.01 - vstep;

 for (unsigned int row=0; row<(nchan/4); row++) {
  for (unsigned int col=0; col<4; col++) {

  channel = row * 4 + col;

  memset (xaxisfmt, 0, MaxString);
  memset (yaxisfmt, 0, MaxString);
  sprintf (xaxisfmt, "BCTS%sI", row == (nchan/4)-1 ? "N" : "");
  sprintf (yaxisfmt, "BCTS%sI", col == 0 ? "N" : "");

  cpgsch (0.9*char_size);
  cpgsvp (0.06 + hstep * col, 0.27 + hstep * col, vbeg - vstep * row, 0.97 - vstep * row);
  cpgswin (x[0]-0.99999, x[nlev-1]+0.99999, -1., 100.);
  cpgslw (3);
  cpgbox (xaxisfmt, nlev <= 4 ? 1 : nlev/4, nlev <= 4 ? 1 : 4, yaxisfmt, 25, 5);
  cpgsitf (0);
  cpgsci (2);
  cpgsfs (1);
  // cpgbin (nlev, x, y, true);
  for (unsigned int i=0; i<nlev; ++i) {
   cpgrect (x[i]-0.5, x[i]+0.5, 0, bs.getChanStatPercent(channel, i));
   if (nlev <= 4) { 
    memset (cur, 0, MaxString);
    sprintf (cur, "%.1f", bs.getChanStatPercent(channel, i));
    cpgsch (0.8*char_size);
    cpgsci (1);
    cpgptxt (x[i], bs.getChanStatPercent(channel, i)+2, 0., 0.5, cur);
    cpgsci (2);
    cpgsch (char_size);
   }
  }
  cpgsci (1);
  cpgsch ((1.3 - 0.1 * (nchan/8))*char_size);
  sprintf (cur, "%c", ext[channel]);
  cpgptxt (x[nlev-1]+0.2, 70. + 10.*(16/nchan), 0., 0., cur);
  memset (cur, 0, MaxString);
 }
}

  cpgsch (char_size);
  cpgsvp (0.07, 0.99, 0.20, 0.97);
  cpgsch (char_size);
  /* x-label */ cpgmtxt ("B", 2.7, 0.5, 0.5, stat_xlabel);
  /* y-label */ cpgmtxt ("L", 2.9, 0.5, 0.5, stat_ylabel);

 /* labels */

 cpgsci (1);
 cpgslw (2);

 cpgmtxt ("B", 4.5, 0., 0., "Bit Statistics");
 cpgsch (char_size*0.8);
 sprintf (cur, "%sSampl. freq = %g MHz   Nchan = %d   Nbits = %d   Sample size = %d", label, samp_freq, (int)bs.getNchan(), (int)bs.getNbits(), bs.getSampleSize());
 cpgmtxt ("B", 7., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "MJD: %.20s   UTC:  %s  %s   Dur: %g secs   Nframes = %d", mjd, startdate, starttime, dur * bs.getNframes(), bs.getNframes());
 cpgmtxt ("B", 8., 0., 0., cur);
 memset (cur, 0, MaxString);
 sprintf (cur, "File: %s", outname);
 cpgmtxt ("B", 9., 0., 0., cur);
 cpgslw (1);
 cpgiden ();

 delete (cur);
 delete (xaxisfmt);
 delete (yaxisfmt);
}

/* graphics part */
void plot_stat (StatInfo& bs, char *basename, int dev_default, char *mjd, char *startdate, char *starttime, double dur) {

 unsigned int nlev = bs.getNlevels();
 unsigned int *data = bs.getChanStat(0);
 float *x, *y;
 x = new float[nlev];
 y = new float[nlev];
 if (!x || !y) { perror("new"); exit (1); }
 memset (x, 0, nlev);
 memset (y, 0, nlev);

 if (Nbits == 2) {
  x[0] = -3.; x[1] = -1.; x[2] = 1.; x[3] = 3.;
 } else {
  printf ("Number of bits other than 2 is not implemented!\n");
  exit (1);
 }

 for (unsigned int i=0; i<nlev; ++i) {
  y[i] = 100.*((float)data[i])/(bs.getSampleSize() * (float)bs.getNframes());
 }
	
 char *outname;
 outname = new char[MaxString];
 if (!outname) { perror("new outname"); exit (1); }
 memset (outname, 0, MaxString);
 sprintf (outname, "%s.stat.ps/cps", basename);

 // draw to POSTSCRIPT
 cpgopen (outname);
 drawing (x, y, bs, outname, mjd, startdate, starttime, dur);
 cpgclos ();

 // draw in XWINDOW
 cpgslct (dev_default);
 drawing (x, y, bs, outname, mjd, startdate, starttime, dur);

 delete (y);
 delete (x);
}
