#include "stat.hpp"
#include "framedata.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// coding
const unsigned char StatInfo::coding[4] = { 1, 3, 15, 255 };

//default constructor
StatInfo::StatInfo () {
 stat = NULL;
}

//constructor
StatInfo::StatInfo (unsigned char nch, unsigned char nbits) {
 Nchan = nch;
 Nbits = nbits;
 Nlevels = 1<<nbits;
 samplesize = 0;
 Nframes = 0;
 cod = getCoding(Nbits);

 // initialize stat array
 stat = new unsigned int*[Nchan];
 if (!stat) { perror("new stat"); exit (1); }
 for (unsigned char i=0; i<Nchan; ++i) {
  stat[i] = new unsigned int[Nlevels];
  if (!stat[i]) { perror("new stat[i]"); exit (1); }
  memset (stat[i], 0, Nlevels*sizeof(unsigned int));
 }
}

// destructor
StatInfo::~StatInfo () {
 if (stat != NULL) {
  for (unsigned char i=0; i<Nchan; ++i) if (stat[i] != NULL) delete (stat[i]);
  if (stat != NULL) delete (stat);
 }
}

//initialize array
void StatInfo::initialize (unsigned char nch, unsigned char nbits) {
 Nchan = nch;
 Nbits = nbits;
 Nlevels = 1<<nbits;
 samplesize = 0;
 Nframes = 0;
 cod = getCoding(Nbits);

 // initialize stat array
 stat = new unsigned int*[Nchan];
 if (!stat) { perror("new stat"); exit (1); }
 for (unsigned char i=0; i<Nchan; ++i) {
  stat[i] = new unsigned int[Nlevels];
  if (!stat[i]) { perror("new stat[i]"); exit (1); }
  memset (stat[i], 0, Nlevels*sizeof(unsigned int));
 }
}

// update stat array
void StatInfo::update (SampleData& shot, size_t frames) {
 Nframes = frames;
 samplesize = shot.getSampleSize();
 signed char *ptr;
 signed char val;
 for (unsigned char i=0; i<Nchan; ++i) {
  ptr = shot.getChannel (i);  
  for (unsigned int k=0; k<Nlevels; ++k) {
   val = 2*k - cod;
   for (off_t j=0; j<samplesize; ++j) {
    if (ptr[j] == val) stat[i][k]++;
   } // for array
  } // for Nlevels
 } // for chan
}

// get statistics for selected channel
unsigned int* StatInfo::getChanStat (int ch) {
 if (ch < 0 || ch > Nchan-1) {
   fprintf (stderr, "Bad channel number!\n");
   exit (1);
 }
 return stat[ch];
}

// get statistics for selected channel in %
float StatInfo::getChanStatPercent (int ch, unsigned int level) {
 if (ch < 0 || ch > Nchan-1) {
   fprintf (stderr, "Bad channel number!\n");
   exit (1);
 }
 if (level < 0 || level > Nlevels-1) {
   fprintf (stderr, "Bad level-index number!\n");
   exit (1);
 }
 float res = 100.*((float)stat[ch][level])/(Nframes * (float)samplesize);
 return res;
}

// get coding
unsigned char StatInfo::getCoding (unsigned char nbits) {
 int index = 0;
 if (nbits < 1 || nbits > 8) { perror("get coding"); exit (1); }
 while ((nbits>>=1) !=0) index++;
 if (index < 0 || index > 3) { perror("get coding"); exit (1); }
 return coding[index];
}
