#define MAIN_
#include "mk5decode.hpp"

/*==========================   P r o m p t i n g   ==================================*/

void Help (char *s) {
  printf("\n%sPROGRAMM%s %s\n"
         "It is used to decode Mark5/VLBA data recorded in 512-8-2 mode\n\n"
         "%s[USAGE]:%s %s [options] <file1> [file2 ... fileN]\n\n"
         "%s[OPTIONS]:%s\n"
	 "-d, --path <path>       - path to the output files, default - the same where are the source files\n"
	 "-o, --out <basename>    - basename of files for output data, if basename is \"-\" then\n"
	 "                          decoded data (not spectra) will be written to the standard output\n"
	 "--swap-bytes            - to reverse bits order in input data, use this option on Apple machines\n"
	 "-b, --begin <frame>     - start processing from this frame\n"
	 "-n, --num <frames>      - number of frames to process\n"
	 "--dummy                 - set dummy mode (no output files)\n"
	 "--check                 - set check mode - the same as dummy but together with\n"
	 "                          checking the integrity of input data, checksums,\n"
	 "                          time continuity, identity of some data in all the tracks, ...\n"
	 "--debug[=track]         - to debug the data, i.e. get full info for every frame; with this option\n"
	 "                          --check and --dummy are also set up\n"
	 "                          track is optional, by default info from track #0 will be written, and\n"
	 "                          one can change the output to another track\n"
	 "--trackmap              - to get correspondance map between channels and tracks, BBCs, sidebands, bits\n"
	 "                          --dummy set up also, number of frames set up to 1\n"
	 "--spectra[=number]      - calculate average spectra rather than raw data\n"
	 "                          number is optional arg that means number of counts to average in\n"
	 "                          calculated spectra, default = 16\n"
	 "--show-spectra          - show built spectra during the calculation, use together with --spectra\n"
	 "--skipharm <number>     - number of harmonics to skip from the beginning in calculated spectra, default - 0\n"
	 "--stat                  - to calculate bit statistics\n"
	 "--mjd_prefix <value>    - two first digits of MJD date of the beginning of the decoded files, default = 53\n"
	 "-q, --quite             - turn on quite output, skip printing frames number changes\n"
	 "-h, --help              - help\n\n", bold, none, s, bold, none, s, bold, none);
  exit(0);
}

/*=========================  erase path part from filename  =========================*/
  
// get basename
char* basename (char *str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
} 

/*=========================  get dirname  =========================*/

char* dirname (char *str, char *dest) {
 int len = (int)strlen(str);
 while (str[--len] != '/' && len >= 0) ;
 if (len > 0) { strncpy (dest, str, len); dest[len] = '\0'; } else { dest[0] = '.'; dest[1] = '\0'; }
 return dest;
}

/*========================= Parsing the command line =================================*/

int ParseCmdLine (int argc, char *argv[]) {
 int op;
 struct option long_options[] = { {"help", no_argument, 0, 'h'},
	                          {"out", required_argument, 0, 'o'},
	                          {"path", required_argument, 0, 'd'},
	                          {"begin", required_argument, 0, 'b'},
	                          {"num", required_argument, 0, 'n'},
	                          {"dummy", no_argument, 0, 13},
	                          {"spectra", optional_argument, 0, 14},
	                          {"swap-bytes", no_argument, 0, 15},
	                          {"check", no_argument, 0, 16},
				  {"mjd_prefix", required_argument, 0, 17},
				  {"show-spectra", no_argument, 0, 18},
				  {"skipharm", required_argument, 0, 19},
				  {"stat", no_argument, 0, 20},
				  {"debug", optional_argument, 0, 21},
				  {"trackmap", no_argument, 0, 22},
				  {"quite", no_argument, 0, 'q'},
	                          {0, 0, 0, 0}
                                };
 while((op = getopt_long(argc, argv, "hd:o:n:b:q", long_options, 0)) != EOF)
  switch(op){
    case 'h': Help (basename(argv[0]));
	      break;
    case 'd': if (outputpath) { outputpath = 0; delete (outputpath); }
	      outputpath = new char[(int)strlen(optarg)];
	      if (!outputpath) { perror("new outputpath"); exit(1); }
	      memcpy (outputpath, optarg, (int)strlen(optarg));
	      break;
    case 'o': if (outname) { outname = 0; delete (outname); }
	      outname = new char[(int)strlen(optarg)];
	      if (!outname) { perror("new outname"); exit(1); }
	      memcpy (outname, optarg, (int)strlen(optarg));
	      break;
    case 'b': BeginFrame = atol (optarg);
	      if (BeginFrame < 0) BeginFrame = 0;
	      break;
    case 'n': NumberOfFrames = atol (optarg);
	      if (NumberOfFrames < 0) NumberOfFrames = 0;
	      break;
    case 13: is_dummy = true;
	     break;
    case 14: is_spectra = true;
             if (optarg != NULL) aver = atol (optarg);
	     if (aver < 1) aver = 1;
	     break;
    case 15: swap_bytes = true;
	     break;
    case 16: is_check = true;
             is_dummy = true;
	     break;
    case 17: if (atoi(optarg) >= 0) mjd_prefix = atoi(optarg);
	     break;
    case 18: is_spectra_show = true;
             break;
    case 19: skipharm = atol (optarg);
             break;
    case 20: is_stat = true;
             break;
    case 21: is_debug = true;
             is_check = true;
	     is_dummy = true;
	     if (optarg != NULL) debug_track = atoi (optarg);
	     if (debug_track < 0 || debug_track > 63) debug_track = 0;
	     break;
    case 22: is_trackmap = true;
             is_dummy = true;
	     break;
    case 'q': is_quite = true;
              break;
    case '?': Help (basename(argv[0]));
              break;
  }
 return(optind);
}

/*=================  level calculation ============================*/
float LevelCalc (signed char *data, size_t sz) {
 float sum = 0.;
 for (size_t i=0; i<sz; ++i) sum += (float)data[i];
 return sum/sz;
}

/*==================== get power of 2 size ================================*/
size_t resize (size_t sz) { 
  long n = 0;
  sz += 1; 
  while ( (1<<(++n)) < sz) ;
  return (1<<(--n));
}

/*==================== calculate power spectrum =============================*/

void getSpectrum (float *sp, signed char *data, size_t sz, float level, int o, size_t frames) {
 for (int i=0; i<sz; ++i) { x[i][0] = (float)data[i]; x[i][1] = 0.; }
 for (int i=0; i<sz; ++i) x[i][0] -= level;
 fftwf_execute (pfor);
 for (int i=0; i<sz/2 + 1; ++i) sp[i] = (sp[i]*frames + (x[i][0] * x[i][0] + x[i][1] * x[i][1])) / (frames + 1);
 lseek (o, 0, SEEK_SET);
 write (o, sp, (sz/2 + 1) * sizeof(float));
}

/*++++++++++++++++++++++++++++   m a i n   f u n c t i o n +++++++++++++++++++++++++++*/

int main (int argc, char *argv[]) {

 if (argc == 1) Help (basename(argv[0]));

 outname = new char[MaxString];
 if (!outname) { perror("new outname"); exit(1); }
 memset (outname, 0, MaxString);
 sprintf (outname, "%s","\0");

 outputpath = new char[MaxString];
 if (!outputpath) { perror("new outputpath"); exit(1); }
 memset (outputpath, 0, MaxString);
 sprintf (outputpath, "%s","\0");

 startdate = new char[MaxString];
 if (!startdate) { perror("new startdate"); exit (1); }
 memset (startdate, 0, MaxString);

 starttime = new char[MaxString];
 if (!starttime) { perror("new starttime"); exit (1); }
 memset (starttime, 0, MaxString);

 // parse command line
 next = ParseCmdLine (argc, argv);
 if (argc < next + 1) { fprintf (stderr, "Not point the file to process!\n"); exit(1); }

 /* initialize wisdom */
 if (is_spectra) {
  is_wisdom = !initialize_wisdom (is_quite);
  if (!is_wisdom) { fprintf (stderr, "FFTW Wisdom will not be used!\n"); }
 }

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;

 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE, &r);
 /*------------------------------------------*/

 if (!is_spectra) is_spectra_show = false;
 if (is_trackmap) { NumberOfFrames = 1; is_debug = false; is_check = false; }

 // initializing
 if (!is_trackmap) { fprintf (stderr, "\nInitializing... "); fflush (NULL); }
 syncdata = new unsigned long[syncsize/size_of_long];
 if (!syncdata) { perror("new syncdata"); exit(1); }

#ifdef apple
 if ((in = open (argv[next], O_RDONLY)) == -1) { perror("open"); exit(1); };
#else
 if ((in = open (argv[next], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
#endif
 syncpos = 0;
 // infinite loop
 while (1) {
  memset (syncdata, 0, syncsize);
  if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
  bytesread = read (in, syncdata, syncsize);
  if (bytesread == -1) { perror("read syncdata"); exit(1); }
  if (bytesread != syncsize) break;

  syncdetect = true;
  syncpos += syncsize;
  for (int i=(syncsize/size_of_long) - 1; i>=0; --i, syncpos -= size_of_long) {
   if (syncdata[i] != syncmask) { syncdetect = false; break; }
  }
  if (syncdetect) { if (!is_trackmap) { fprintf(stderr, "done\n"); }  break; }
 } // end of loop
 
 if (!syncdetect) { fprintf (stderr, "No sync word were found!\n"); exit (1); }
  else if (!is_trackmap) fprintf(stderr, " sync word at %lld bytes\n", (unsigned long long)syncpos);

 if (!is_dummy) {
  outnames = new char*[Nchan];
  if (!outnames) { perror("new outnames"); exit (1); }
  for (int u=0; u<Nchan; ++u) {
   outnames[u] = new char[MaxString];
   if (!outnames[u]) { perror("new outnames[u]"); exit (1); }
   memset (outnames[u], 0, MaxString);
  }

  out = new int[Nchan];
  if (!out) { perror("new out"); exit (1); }
  memset (out, 0, Nchan*sizeof(int));
 } // !is_dummy

 SpectraLevel = new float[Nchan];
 if (!SpectraLevel) { perror("new SpectraLevel"); exit (1); }
 memset (SpectraLevel, 0, Nchan*sizeof(float));

 // initialize input data frame array
 raw_framedata = new unsigned long long[tracksize/size_of_long_long];
 if (!raw_framedata) { perror("new raw_framedata"); exit(1); }

 SampleData shot (Ntracks, framesize, tracksize, swap_bytes);  // initialize the class object

 if (!is_trackmap) {
  fprintf (stderr, " Ntracks = %d   framesize = %d x %d bytes = %d bytes   samplesize = %d counts\n", Ntracks, Ntracks, framesize, tracksize, shot.getSampleSize());
  fprintf (stderr, " output header = %d bytes  with MJD string\n", mjd_header);
 }

 // end of initialization

 if (is_spectra && !is_dummy) {
  spectrumsize = resize(shot.getSampleSize());
  spectra = new float*[Nchan];
  if (!spectra) { perror("new spectra"); exit(1); }
  for (int u=0; u<Nchan; ++u) {
   spectra[u] = new float[spectrumsize/2 + 1];
   if (!spectra[u]) { perror("new spectra[u]"); exit(1); }
   memset (spectra[u], 0, (spectrumsize/2 + 1) * sizeof(float));
  }
  x = (fftwf_complex *) fftwf_malloc (spectrumsize * sizeof(fftwf_complex));
  if (!x) { perror("new x"); exit(1); }
  if (!is_wisdom) pfor = fftwf_plan_dft_1d (spectrumsize, x, x, FFTW_FORWARD, FFTW_ESTIMATE);
   else pfor = fftwf_plan_dft_1d (spectrumsize, x, x, FFTW_FORWARD, FFTW_EXHAUSTIVE);

  if (is_spectra_show) { dev_id = cpgopen (pgplot_dev); cpgask (false); }
 } // is_spectra

 StatInfo bs;
 if (is_stat && !is_dummy) {
  bs.initialize (Nchan, Nbits);
  stat_dev_id = cpgopen (pgplot_dev);
  cpgask (false);
 } // is_stat

 char *mjd;
 mjd = new char[MaxString];
 if (!mjd) { perror("new"); exit (1); }
 memset (mjd, 0, MaxString);

 if (!is_trackmap) fprintf(stderr, "\nProcessing...\n");
 if (Nframe_skipped < BeginFrame) {
  struct stat information;
  if (stat (argv[next], &information) != 0) { perror("stat information"); exit (1); }
  if (!is_trackmap) { fprintf (stderr, " file: %s", argv[next]); fflush (NULL); }
  while (Nframe_skipped < BeginFrame) {
   Nframe_skipped++;
   syncpos += tracksize;
   if (syncpos >= information.st_size) {
     syncpos -= information.st_size;
     next++;
     if (next == argc) { fprintf (stderr, " [skip]\n skipped %ld frames (%g secs)\n syncpos = %lld bytes\n number of available frames (%ld) is less than frames to skip (%ld) !\n done.\n", Nframe_skipped, Nframe_skipped * sec_step, (unsigned long long)syncpos, Nframe_skipped, BeginFrame); exit (1); }
     if (!is_trackmap) { fprintf (stderr, " [skip]\n       %s", argv[next]); fflush (NULL); }
     if (stat (argv[next], &information) != 0) { perror("stat information"); exit (1); }
   } //syncpos
  } //while
  if (!is_trackmap) {
   fprintf (stderr, " [ok]  syncpos = %lld bytes\n", (unsigned long long)syncpos);
   fprintf (stderr, " skipped %ld frames (%g secs)\n", Nframe_skipped, Nframe_skipped * sec_step);
  } // !is_trackmap
 }

#ifdef apple
 if ((in = open (argv[next], O_RDONLY)) == -1) { perror("open"); exit(1); };
#else
 if ((in = open (argv[next], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
 #endif

 if (!is_dummy) {  // on if dummy mode isn't set
  if (!strcmp(outname, "-") && !is_spectra) is_stdout = true;
 // create output file names
 // if outname is still empty or if "-", call its basename as argv[f]
  if (!strcmp(outname, "\0") || !strcmp(outname, "-")) {
   memset (outname, 0, MaxString);
   sprintf (outname, "%s", basename(argv[next]));
  }
  if (!strcmp(outputpath, "\0")) {
   memset (outputpath, 0, MaxString);
   outputpath = dirname(argv[next], outputpath);
  }
  for (int u=0; u<Nchan; ++u) {
   memset (outnames[u], 0, MaxString);
   sprintf (outnames[u], "%s/%s.%c%s", outputpath, outname, ext[u], (is_spectra ? ".sp2" : ""));
  }
  if (is_stdout) fprintf (stderr, "\n outname: standard output\n");
   else fprintf (stderr, "\n outnames: %s/%s.%s%s\n", outputpath, outname, "[ABCDEFGH]", (is_spectra ? ".sp2" : ""));

  // at first remove existing file
  if (!is_stdout) for (int u=0; u<Nchan; ++u) unlink (outnames[u]);
  // open for appending
    if (!is_stdout) {
     for (int u=0; u<Nchan; ++u) {
#ifdef apple
      if ((out[u] = open (outnames[u], O_CREAT | (is_spectra ? O_TRUNC : O_APPEND) | O_WRONLY, 0644)) == -1) { perror("open"); exit(1); }
#else
      if ((out[u] = open (outnames[u], O_CREAT | (is_spectra ? O_TRUNC : O_APPEND) | O_WRONLY | O_LARGEFILE, 0644)) == -1) { perror("open"); exit(1); }
#endif
    }
   } else { // is_stdout = true
     for (int u=0; u<Nchan; ++u) out[u] = 1; // stdout
   }
 } // is_dummy

 char *debugmask;
 if (is_debug) {
  debugmask = new char[MaxString];
  if (!debugmask) { perror("new debugmask"); exit (1); }
 }

 // true when one wants to see header info from the first frame
 is_info_shown = false;

 if ((lseek (in, syncpos, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
 // infinite loop
 while (1) {
   memset (raw_framedata, 0, tracksize);
   bytesread = read (in, raw_framedata, tracksize);
   if (bytesread == -1) { perror("read raw_framedata"); exit(1); }
   if (bytesread != tracksize) {
     if (bytesread > tracksize) { fprintf(stderr, "wow! we can read more than tracksize at once! :)\n"); exit (1); }
     if (bytesread < tracksize) syncpos = tracksize - bytesread;
     next++;
     if (next == argc) break;
      else { unsigned long long *ptr;
             ptr = raw_framedata + bytesread/size_of_long_long;
             if ((close (in)) == -1) { perror("close"); exit(1); };
#ifdef apple
             if ((in = open (argv[next], O_RDONLY)) == -1) { perror("open"); exit(1); };
#else
             if ((in = open (argv[next], O_RDONLY | O_LARGEFILE)) == -1) { perror("open"); exit(1); };
#endif
             if ((lseek (in, 0, SEEK_SET)) == -1) { perror("lseek"); exit(1); };
	     bytesread = read (in, ptr, syncpos);
	     if (bytesread == -1) { perror("read ptr"); exit(1); }
	     if (bytesread > syncpos) { fprintf(stderr, "wow! we can read more than we need at once! :)\n"); exit (1); }
	     if (bytesread < syncpos) { fprintf (stderr, "size of file \"%s\" is less than tracksize!\n", argv[next]); exit (1); }
	     if (is_check && !is_debug) shot.getSimpleCheckInfo();
	     if (!is_trackmap) { fprintf (stderr, "\n                 %s %6ld", argv[next], Nframe_processed); fflush (NULL); }
	  } // else
   } // bytesread != tracksize

   shot.update (raw_framedata, Nframe_processed); // fill class object with new frame data
   
   if (!is_trackmap) {
   if (!is_info_shown) { // getting info for zero track (default)
    shot.getAllInfo(); 
    mjd = getMJD (shot.getMJD(), shot.getSeconds(), 5, mjd);
    starttime = getTime (shot.getSeconds(), starttime);
    startdate = getDate (shot.getMJD(), startdate);
    fprintf (stderr, " UTC: %s  %s\n", startdate, starttime);
    fprintf (stderr, " start seconds: %lg   MJD: %s\n", shot.getSeconds(), mjd);
    is_info_shown = true;
    if (!is_quite) {
     if (!is_debug) fprintf (stderr, " file processed: %s %6ld", argv[next], Nframe_processed);
      else fprintf (stderr, " file processed: %s %s%6ld%s", argv[next], bold, Nframe_processed, none);
     fflush (NULL);
    } else { /* is_quite */ fprintf (stderr, " file processed: %s\n", argv[next]); }
   } 
   } // !is_trackmap
   if (is_debug) {
    fprintf (stderr, "\n");
    shot.getAllInfo(debug_track);
    memset (debugmask, 0, MaxString);
    sprintf (debugmask, "                 %s%ds       ", "%", (int)strlen(argv[next]));
    fprintf (stderr, debugmask, ""); fflush (NULL);
   }
   if (is_trackmap) {
    printf ("\n");
    shot.getTracksMap();
   }

   if (!is_dummy) { if (!is_spectra) { 
                     if (Nframe_processed == 0) { 
		      if (!is_stdout) { for (int u=0; u<Nchan; ++u) write (out[u], mjd, mjd_header); }
		       else { 
		        int ss = shot.getSampleSize();
		        write (1, &ss, sizeof(int)); // in stdout we are writing first the size of every channel
		        write (1, mjd, mjd_header);  // then we are writing MJD string
		       }
		     }
                     for (int u=0; u<Nchan; ++u) write (out[u], shot.getChannel(u), shot.getSampleSize());
		     if (is_stat) { bs.update (shot, Nframe_processed+1); plot_stat (bs, outname, stat_dev_id, mjd, startdate, starttime, sec_step); }
		    } else  { 
		      if (Nframe_processed == 0) for (int u=0; u<Nchan; ++u) SpectraLevel[u] = LevelCalc (shot.getChannel(u), spectrumsize);
		      for (int u=0; u<Nchan; ++u) getSpectrum (spectra[u], shot.getChannel(u), spectrumsize, SpectraLevel[u], out[u], Nframe_processed); 
		      if (is_spectra_show) plot_spectrum (spectra, spectrumsize/2+1, aver, skipharm, outname, dev_id, Nframe_processed+1, mjd, startdate, starttime, sec_step);
		      if (is_stat) { bs.update (shot, Nframe_processed+1); plot_stat (bs, outname, stat_dev_id, mjd, startdate, starttime, sec_step); }
		     }
   } // !is_dummy
   Nframe_processed++;
   if (!is_trackmap) {
    if (!is_quite) {
     if (!is_debug) fprintf (stderr, "\b\b\b\b\b\b%6ld", Nframe_processed);
      else fprintf (stderr, "\b\b\b\b\b\b%s%6ld%s", bold, Nframe_processed, none);
     fflush (NULL);
    }
   }
   if (NumberOfFrames != 0 && NumberOfFrames == Nframe_processed) break;
   syncpos += tracksize;

 } // end of loop

 if ((close (in)) == -1) { perror("close"); exit(1); };
 if (!is_dummy) { for (int u=0; u<Nchan; ++u) if ((close (out[u])) == -1) { perror("close"); exit(1); }
 }

 if (!is_trackmap) {
  fprintf (stderr, " - done\n");
  fprintf (stderr, "processed %ld frames (%g secs)\n", Nframe_processed, Nframe_processed * sec_step);
 }

 if (is_check) {
  shot.getCheckInfo();
 }

 if (is_spectra_show && is_spectra && !is_dummy) { cpgslct (dev_id); cpgclos(); }
 if (is_stat && !is_dummy) { cpgslct (stat_dev_id); cpgclos (); }

 if (is_debug) delete (debugmask);
 delete (mjd);
 delete (startdate);
 delete (starttime);
 delete (syncdata);
 delete (SpectraLevel);
 delete (raw_framedata);
 delete (outname);
 delete (outputpath);
 if (!is_dummy) delete (out);
 if (!is_dummy) { for (unsigned char i=0; i<Nchan; ++i) { delete (outnames[i]); } delete (outnames); }
 if (is_spectra && !is_dummy) { 
  for (unsigned char i=0; i<Nchan; ++i) delete (spectra[i]);
  delete (spectra); 
  fftwf_destroy_plan (pfor);
  fftwf_free (x);
 }

 return 0;
}
