#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>

#include <complex.h>
#include <fftw3.h>



/*        */
#define BETA1 8.987554e16
#define BETA2 (BETA1 / (2 * M_PI))


long size = 2097152;
float *matrix;

double ind  = 1.0e6,
       sign = 1.,           // direct band 
       freq = 1387.,        // lower freq [MHz] (upper freq in case of indirect band: sign = -1)
       timeres = 38.461538, // time resolution [ns]
       RM = 1000.,          // rotation measure [rad/m^2]
       T = (timeres * size)/1000.;

double Calc ( long j ) {
  return ( ( ( ( ( BETA1 * RM ) / ind ) / ind )  / (freq + (sign)*(((double)j)/T)) ) / (freq + (sign)*(((double)j)/T)) );
}

void ComputeMatrix () {
 double z;
 
 for (long i=0, j=size/2-1; i < size+2; i+=2, ++j) {
   z = Calc (j);
   matrix[i]   = (float)cos(z);
   matrix[i+1] = -1. * (float)sin(z); // +1. for inverse band
 }
 for (long i=size+2, j=0; i < 2*size; i+=2, ++j) {
   z = Calc (j);
   matrix[i]   = (float)cos(z);
   matrix[i+1] = -1. * (float)sin(z); // +1. for inverse band
 }
}



int main (int argc, char* argv[]) {

 if (argc == 1) {
  printf ("Usage: %s <outfile>\n", argv[0]);
  exit (0);
 }
 if (argc < 2) {
  printf ("Not output file!\n");
  exit (1);
 }

 matrix = new float[2*size];
 if (!matrix) { printf("Not enough memory!\n"); exit(1); }
 memset(matrix, 0, 2*size*sizeof(float));

 ComputeMatrix();

 int out;

 if ( (out = open(argv[1], O_TRUNC | O_CREAT | O_WRONLY, 0644)) == -1 )
  { printf("\nError: Couldn't open file %s for writing!\n", argv[1]);
    exit(1);
   } else { lseek(out, 0, SEEK_SET);
            write(out, matrix, 2*size*sizeof(float));
            close(out);
           }

 delete (matrix);
 return 0;
}
