#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>

#include <inttypes.h>

#include <complex.h>
#include <fftw3.h>




/*        */
#define BETA1 8.987554e16
#define BETA2 (BETA1 / (2 * M_PI))


size_t size = 2097152, badpoints, zeroshift = 8192;
fftwf_complex *matrix, *x, *y, *z;
fftwf_plan pforwardx, pforwardy, pbackwardz;
float Level1, Level2;

// arrays for real, imagine, absolute value and phase
float *re, *im, *av, *ph;

double freq = 1387.,        // lower freq
       bandwidth = 26.,     // band
       timeres = 38.461538, // time resolution
       RM;                  // rotation measure
bool inverse_band = false;  // если полоса - прямая -> false
                            // а если обратная, то - true

// возвращаем количество плохих точек
// при обратной полосе плохие точки оказываются в конце массива,
// а если полоса - прямая, то - в начале
void BadPoints () {
 double f = freq + bandwidth,
        tbad = ( RM * BETA2 * ( 1./(freq * freq * freq) - 1./(f * f * f) ) * 1.e-9 ) / timeres;

 badpoints = (size_t)ceil(fabs(tbad));
}

// this function calculates sigma
double Sigma (fftwf_complex * x) {
 double sum = 0., sum2 = 0.;
 for (size_t i = zeroshift; i < size; ++i) { sum += x[i][0]; sum += x[i][1]; sum2 += x[i][0]*x[i][0]; sum2 += x[i][1]*x[i][1]; }
 return  (sum2 / size) - (sum / size) * (sum / size) ;
}

// дедисперсия
void DeDispersion () {
 
 double facf = Sigma (x);
 double sacf = Sigma (y);
 double norm = sqrt (facf * sacf) * size * size;

 fftwf_execute (pforwardx);
 fftwf_execute (pforwardy);

 // zanulyaem v nachale, i poluchim v CCF +/- zeroshift horoshih sdvigov
 // zanulyaem v anchale, t.k. tam takje plohie tochki i ih kolichestvo
 // doljno byt' men'she zeroshift
 // adding normalizing also

 for (size_t i=0; i<size; ++i) {
  z[i][0] = ((x[i][0] * matrix[i][0] - matrix[i][1] * x[i][1]) * y[i][0] + (x[i][0] * matrix[i][1] + matrix[i][0] * x[i][1]) * y[i][1]) / norm;
  z[i][1] = ((x[i][0] * matrix[i][1] + matrix[i][0] * x[i][1]) * y[i][0] - (x[i][0] * matrix[i][0] - matrix[i][1] * x[i][1]) * y[i][1]) / norm;
 }

 // reverse
 fftwf_execute (pbackwardz);
}


// level calculation
void LevelCalc () {

 float sum1 = 0., sum2 = 0.;
 for (size_t i=0; i<size; ++i) { sum1 += x[i][0]; sum1 += x[i][1]; sum2 += y[i][0]; sum2 += y[i][1]; }
 Level1 = sum1/(2 * size);
 Level2 = sum2/(2 * size);
}

// level elimination
void NoLevel () { 
 for (size_t i=0; i<size; ++i) { x[i][0] -= Level1; x[i][1] -= Level1; y[i][0] -= Level2; y[i][1] -= Level2; }
}


// record to file
void Write (int fd, char * name, float * x, size_t size) {
 if ( (fd = open(name, O_TRUNC | O_CREAT | O_WRONLY, 0644)) == -1 )
  { printf("\nError: Couldn't open file %s for writing!\n", name);
    exit(1);
   } else { lseek(fd, 0, SEEK_SET);
            write(fd, x, size*sizeof(float));
            close(fd);
           }
}


// --------------- M A I N ------------------

int main (int argc, char* argv[]) {
 if (argc == 1) {
  printf ("Usage: %s <matrix> <infile1> <infile2> <baseoutname> <RM>\n", argv[0]);
  exit (0);
 }
 if (argc < 6) {
  printf ("Not all the parameters!\n");
  exit (1);
 }

 RM = atof (argv[5]);

 matrix = (fftwf_complex *) fftwf_malloc (size * sizeof(fftwf_complex));
 if (!matrix) { printf ("Not enough memory!\n"); exit(1); }

 int file_matrix;
 if ( (file_matrix = open(argv[1], O_RDONLY)) == -1 )
  { printf("\nError:\nCouldn't open matrix \"%s\" for reading!\n", argv[1]); exit(1);  }
  read(file_matrix, matrix, size * sizeof(fftwf_complex));
  close(file_matrix);

 x = (fftwf_complex *) fftwf_malloc (size * sizeof(fftwf_complex));
 y = (fftwf_complex *) fftwf_malloc (size * sizeof(fftwf_complex));
 z = (fftwf_complex *) fftwf_malloc (size * sizeof(fftwf_complex));
 if (!x || !y || !z) { printf ("Not enough memory!\n"); exit(1); }

 signed char* data;
 data = new signed char[2 * size];
 if (!data) { printf ("Not enough memory!\n"); exit(1); }

 struct stat info;
 if ( stat(argv[2], &info) != 0 ) { perror("stat"); exit (1); }

 int in1, in2;
 unsigned long counter = 0; 
 ssize_t bytesread;
 uintmax_t left, filesize = info.st_size;

 if ( stat(argv[3], &info) != 0 ) { perror("stat"); exit (1); }
 if (filesize != info.st_size) { printf("Size of input files is different!\n"); exit(1); }


 re = new float[2 * zeroshift];
 im = new float[2 * zeroshift];
 av = new float[2 * zeroshift];
 ph = new float[2 * zeroshift];
 if (!re || !im || !av || !ph) { printf ("Not enough memory!\n"); exit(1); }
 memset (re, 0, 2 * zeroshift * sizeof(float));
 memset (im, 0, 2 * zeroshift * sizeof(float));
 memset (av, 0, 2 * zeroshift * sizeof(float));
 memset (ph, 0, 2 * zeroshift * sizeof(float));

 /* calculate badpoints */
 BadPoints();

 /* output some information */
 printf ("\n");
 printf ("BETA1: %g\n", BETA1);
 printf ("BETA2: %g\n", BETA2);
 printf ("filesize: %.0lf\n", (double)filesize);
 printf ("size: %lu   badpoints: %lu ... zeroshift: %lu  [%s]\n", size, badpoints, zeroshift, zeroshift >= badpoints ? "OK" : "!!");
 printf ("freq: %g\nbandwidth: %g\ntimeres: %g\nRM = %g\n", freq, bandwidth, timeres, RM);
 printf ("\n");
 printf ("matrix: %s\n", argv[1]);
 printf ("input files: [1]: %s  [2]: %s -> baseoutname: %s  .real .imagine .module .phase\n", argv[2], argv[3], argv[4]);
 printf ("\nProcessing [%4lu] ... ", (unsigned long)(((double)filesize)/2/(size)));

 pforwardx  = fftwf_plan_dft_1d (size, x, x, FFTW_FORWARD, FFTW_ESTIMATE);
 pforwardy  = fftwf_plan_dft_1d (size, y, y, FFTW_FORWARD, FFTW_ESTIMATE);
 pbackwardz = fftwf_plan_dft_1d (size, z, z, FFTW_BACKWARD, FFTW_ESTIMATE);

  /* read input file 1 */
  if ( (in1 = open (argv[2], O_RDONLY | O_LARGEFILE)) < 0 ) { perror("open input file 1"); exit(1); }
  /* read input file 2 */
  if ( (in2 = open (argv[3], O_RDONLY | O_LARGEFILE)) < 0 ) { perror("open input file 2"); exit(1); }

 /* infinite loop */
 while (1) {
 
  printf (".%5lu", counter); fflush (NULL);
  left = ((uintmax_t)(size)) * ((uintmax_t)counter) * 2;
  if (filesize - left < (uintmax_t)(2 * size)) break;

   lseek (in1, (off_t)left, SEEK_SET);
   memset (data, 0, 2 * size);
   bytesread = read (in1, data, 2 * size);
   if (bytesread != 2 * size) break;
  for (size_t i=0, j=0; i<2*size; i+=2, ++j) { x[j][0] = (float)data[i]; x[j][1] = (float)data[i+1]; }

   lseek (in2, (off_t)left, SEEK_SET);
   memset (data, 0, 2 * size);
   bytesread = read (in2, data, 2 * size);
   if (bytesread != 2 * size) break;

  for (size_t j=0; j<zeroshift; ++j) { y[j][0] = 0.; y[j][1] = 0.; }
  for (size_t i=2*zeroshift, j=zeroshift; i<2*size; i+=2, ++j) { y[j][0] = (float)data[i]; y[j][1] = (float)data[i+1]; }

  if (counter == 0) { LevelCalc(); printf ("\b\b\b\b\b Level1 = %g  Level2 = %g      ", Level1, Level2); fflush (NULL); }
  NoLevel();
  DeDispersion();

  for (size_t i=size-zeroshift+1, j=0; i<size; ++i, ++j) {
   re[j] = z[i][0];
   im[j] = z[i][1];
   av[j] = (float)sqrt (z[i][0] * z[i][0] + z[i][1] * z[i][1]);
   ph[j] = (float)((180./M_PI) * atan2 ( (double)z[i][1], (double)z[i][0] ) );
  }
  for (size_t i=0, j=zeroshift-1; i<=zeroshift; ++i, ++j) {
   re[j] = z[i][0];
   im[j] = z[i][1];
   av[j] = (float)sqrt (z[i][0] * z[i][0] + z[i][1] * z[i][1]);
   ph[j] = (float)((180./M_PI) * atan2 ( (double)z[i][1], (double)z[i][0] ) );
  }

  printf ("\b\b\b\b\b"); fflush (NULL);
  ++counter;

 } /* end of infinite loop */

 close (in1);
 close (in2);

 for (size_t i=0; i<2*zeroshift; ++i) {
  re[i] /= counter;
  im[i] /= counter;
  av[i] /= counter;
  ph[i] /= counter;
 }

 int out_re, out_im, out_av, out_ph;
 char *out_re_name, *out_im_name, *out_av_name, *out_ph_name;
 out_re_name = new char[255];
 out_im_name = new char[255];
 out_av_name = new char[255];
 out_ph_name = new char[255];
 if (!out_re_name || !out_im_name || !out_av_name || !out_ph_name) { printf ("Not enough memory!\n"); exit(1); }
 memset (out_re_name, 0, 255);
 memset (out_im_name, 0, 255);
 memset (out_av_name, 0, 255);
 memset (out_ph_name, 0, 255);
 sprintf (out_re_name, "%s%s", argv[4], ".real");
 sprintf (out_im_name, "%s%s", argv[4], ".imagine");
 sprintf (out_av_name, "%s%s", argv[4], ".module");
 sprintf (out_ph_name, "%s%s", argv[4], ".phase");

 Write (out_re, out_re_name, re, 2*zeroshift);
 Write (out_im, out_im_name, im, 2*zeroshift);
 Write (out_av, out_av_name, av, 2*zeroshift);
 Write (out_ph, out_ph_name, ph, 2*zeroshift);

 fftwf_destroy_plan (pforwardx);
 fftwf_destroy_plan (pforwardy);
 fftwf_destroy_plan (pbackwardz);

 fftwf_free (matrix);
 fftwf_free (x);
 fftwf_free (y);
 fftwf_free (z);
 delete (data);

 delete (re);
 delete (im);
 delete (av);
 delete (ph);

 delete (out_re_name);
 delete (out_im_name);
 delete (out_av_name);
 delete (out_ph_name);

 return 0;
}
