#!/usr/bin/perl -w
#
# Fit CCF module by gaussian
#

if ($#ARGV == -1) { die "Usage: $0 <title string> file\n"; }

$title = $ARGV[0]; # Title for postscript file (source name, etc.)
$file = $ARGV[1];  # File with CCF module to process

system ("bd2t $file > /dev/null");
$txtfile = $file . ".ascii";
$gnufile = ".gnu";
$psfile = $title . ".ps";
$fitlog = $title . ".fit";

substr ($title, 1, 1) = "\\\\_";

open (GNU, ">$gnufile") || die "can't open temporary file \"$gnufile\" for writing!\n";
 print GNU "#!/usr/bin/gnuplot -persist\n";
 print GNU "FIT_LIMIT = 1e-7\n";
 print GNU "f(x) = a*exp(-((x-b)**2)/(2*(c**2))) + d\n";
 print GNU "set samples 1000\n";
 print GNU "fit [-17:17] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-15:15] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-10:10] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-5:5] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-3:3] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-2:2] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-3:3] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-5:5] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-10:10] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-15:15] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "fit [-17:17] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c,d\n";
 print GNU "set out \"fit.dat\"\n";
 print GNU "set term table\n";
 print GNU "plot [-17:17] f(x)\n";
close (GNU);

system ("gnuplot < $gnufile");
$status = `tail -n 10 fit.log | grep Undefined | wc -l`;
chomp $status;

$maxpoint = `dt-d -i -l 2037 -w 21 $file | grep max | grep -v xmax | awk '\{print \$3\}' -`;
chomp $maxpoint;

if ($status == 0) {
 @param = `tail -n 13 fit.log | head -n 4 | awk '{print \$3, \$4, \$5}' -`;
 map { chomp; } @param;

 system ("echo \"===================================================\" > $fitlog");
 system ("echo \"File: $file\" >> $fitlog");
 system ("echo \"---------------------------------------------------\" >> $fitlog");
 system ("echo \" ampl  = $param[0]\" >> $fitlog");
 system ("echo \" mean  = $param[1]\" >> $fitlog");
 system ("echo \" sigma = $param[2]\" >> $fitlog");
 system ("echo \" b     = $param[3]\" >> $fitlog");
 system ("echo \"===================================================\" >> $fitlog");

 map { $_ =~ s/^(.*)\+\/\-(.*)$/$1\{\/Symbol\ \\261\}$2/; } @param;
 @temp = split /\s+/, $param[0];
 $ampl = $temp[0];
 @temp = split /\s+/, $param[3];
 $bl = $temp[0];
 $maxvalue =  $ampl + $bl - 0.0001;
 if ($maxpoint > $maxvalue) { $maxvalue = $maxpoint; }

 open (GNU, ">$gnufile") || die "can't open temporary file \"$gnufile\" for writing!\n";
  print GNU "#!/usr/bin/gnuplot -persist\n";
  print GNU "set nokey\n";
  print GNU "set mxtics 5\n";
  print GNU "set mytics 2\n";
  print GNU "set title \"$title\" 0.000000,0.000000  \"\"\n";
  print GNU "set xlabel \"Lag (counts)\" 0.000000,0.000000  \"\"\n";
  print GNU "set ylabel \"CCF\" 0.000000,0.000000  \"\"\n";
  print GNU "set label 1 \"y = a\\264e\^\{-\{(x-\{\/Symbol m\})\}\^2\/2\{\/Symbol s\}\^2\} + b\\n\\na = $param[0]\\n\{\/Symbol m\} = $param[1]\\n\{\/Symbol s\} = $param[2]\\nb = $param[3]\" at 9,$maxvalue right font \"Helvetica, 12\"\n";
  print GNU "set out \"$psfile\"\n";
  print GNU "set term post land enh\n";
  print GNU "plot [-10:10] \"$txtfile\" using (\$1-2047):2 w l lt 1, \"fit.dat\" w l lt 2\n";
  print GNU "#    EOF\n";
 close (GNU);

 system ("gnuplot < $gnufile");
 system ("rm -f $gnufile $txtfile fit.dat fit.log");
}
 else {  # status == 1

 $blev1 = `dt-d -i -l 0 -w 1997 $file | grep \\<x\\> | awk '\{print \$4\}' -`;
 chomp $blev1;
 $blev2 = `dt-d -i -l 2098 $file | grep \\<x\\> | awk '\{print \$4\}' -`;
 chomp $blev2;
 $blev = ($blev1 + $blev2) / 2.;

 open (GNU, ">$gnufile") || die "can't open temporary file \"$gnufile\" for writing!\n";
  print GNU "#!/usr/bin/gnuplot -persist\n";
  print GNU "FIT_LIMIT = 1e-7\n";
  print GNU "f(x) = a*exp(-((x-b)**2)/(2*(c**2))) + $blev\n";
  print GNU "set samples 1000\n";
  print GNU "fit [-17:17] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-15:15] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-10:10] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-5:5] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-3:3] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-2:2] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-3:3] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-5:5] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-10:10] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-15:15] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "fit [-17:17] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,b,c\n";
  print GNU "set out \"fit.dat\"\n";
  print GNU "set term table\n";
  print GNU "plot [-17:17] f(x)\n";
 close (GNU);

 system ("gnuplot < $gnufile");
 $status = `tail -n 10 fit.log | grep Undefined | wc -l`;
 chomp $status;

 if ($status == 0) {
  @param = `tail -n 11 fit.log | head -n 3 | awk '{print \$3, \$4, \$5}' -`;
  map { chomp; } @param;
  
  system ("echo \"===================================================\" > $fitlog");
  system ("echo \"File: $file\" >> $fitlog");
  system ("echo \"---------------------------------------------------\" >> $fitlog");
  system ("echo \" ampl  = $param[0]\" >> $fitlog");
  system ("echo \" mean  = $param[1]\" >> $fitlog");
  system ("echo \" sigma = $param[2]\" >> $fitlog");
  system ("echo \" b     = $blev\" >> $fitlog");
  system ("echo \"===================================================\" >> $fitlog");

  map { $_ =~ s/^(.*)\+\/\-(.*)$/$1\{\/Symbol\ \\261\}$2/; } @param;
  @temp = split /\s+/, $param[0];
  $ampl = $temp[0];
  $maxvalue = $ampl + $blev - 0.0001;
  if ($maxpoint > $maxvalue) { $maxvalue = $maxpoint; }

  open (GNU, ">$gnufile") || die "can't open temporary file \"$gnufile\" for writing!\n";
   print GNU "#!/usr/bin/gnuplot -persist\n";
   print GNU "set nokey\n";
   print GNU "set mxtics 5\n";
   print GNU "set mytics 2\n";
   print GNU "set title \"$title\" 0.000000,0.000000  \"\"\n";
   print GNU "set xlabel \"Lag (counts)\" 0.000000,0.000000  \"\"\n";
   print GNU "set ylabel \"CCF\" 0.000000,0.000000  \"\"\n";
   print GNU "set label 1 \"y = a\\264e\^\{-\{(x-\{\/Symbol m\})\}\^2\/2\{\/Symbol s\}\^2\} + b\\n\\na = $param[0]\\n\{\/Symbol m\} = $param[1]\\n\{\/Symbol s\} = $param[2]\\nb = $blev\" at 9,$maxvalue right font \"Helvetica, 12\"\n";
   print GNU "set out \"$psfile\"\n";
   print GNU "set term post land enh\n";
   print GNU "plot [-10:10] \"$txtfile\" using (\$1-2047):2 w l lt 1, \"fit.dat\" w l lt 2\n";
   print GNU "#    EOF\n";
  close (GNU);

  system ("gnuplot < $gnufile");
  system ("rm -f $gnufile $txtfile fit.dat fit.log");
 }
 else { # status == 1   again

  open (GNU, ">$gnufile") || die "can't open temporary file \"$gnufile\" for writing!\n";
   print GNU "#!/usr/bin/gnuplot -persist\n";
   print GNU "FIT_LIMIT = 1e-7\n";
   print GNU "f(x) = a*exp(-((x)**2)/(2*(c**2))) + $blev\n";
   print GNU "set samples 1000\n";
   print GNU "fit [-10:10] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,c\n";
   print GNU "fit [-5:5] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,c\n";
   print GNU "fit [-3:3] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,c\n";
   print GNU "fit [-2:2] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,c\n";
   print GNU "fit [-3:3] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,c\n";
   print GNU "fit [-5:5] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,c\n";
   print GNU "fit [-10:10] [] f(x) \"$txtfile\" using (\$1-2047):2 via a,c\n";
   print GNU "set out \"fit.dat\"\n";
   print GNU "set term table\n";
   print GNU "plot [-10:10] f(x)\n";
  close (GNU);

  system ("gnuplot < $gnufile");
 
  @param = `tail -n 9 fit.log | head -n 2 | awk '{print \$3, \$4, \$5}' -`;
  map { chomp; } @param;

  system ("echo \"===================================================\" > $fitlog");
  system ("echo \"File: $file\" >> $fitlog");
  system ("echo \"---------------------------------------------------\" >> $fitlog");
  system ("echo \" ampl  = $param[0]\" >> $fitlog");
  system ("echo \" mean  = 0\" >> $fitlog");
  system ("echo \" sigma = $param[1]\" >> $fitlog");
  system ("echo \" b     = $blev\" >> $fitlog");
  system ("echo \"===================================================\" >> $fitlog");

  map { $_ =~ s/^(.*)\+\/\-(.*)$/$1\{\/Symbol\ \\261\}$2/; } @param;
  @temp = split /\s+/, $param[0];
  $ampl = $temp[0];
  $maxvalue = $ampl + $blev - 0.0001;
  if ($maxpoint > $maxvalue) { $maxvalue = $maxpoint; }

  open (GNU, ">$gnufile") || die "can't open temporary file \"$gnufile\" for writing!\n";
   print GNU "#!/usr/bin/gnuplot -persist\n";
   print GNU "set nokey\n";
   print GNU "set mxtics 5\n";
   print GNU "set mytics 2\n";
   print GNU "set title \"$title\" 0.000000,0.000000  \"\"\n";
   print GNU "set xlabel \"Lag (counts)\" 0.000000,0.000000  \"\"\n";
   print GNU "set ylabel \"CCF\" 0.000000,0.000000  \"\"\n";
   print GNU "set label 1 \"y = a\\264e\^\{-\{(x-\{\/Symbol m\})\}\^2\/2\{\/Symbol s\}\^2\} + b\\n\\na = $param[0]\\n\{\/Symbol m\} = 0\\n\{\/Symbol s\} = $param[1]\\nb = $blev\" at 9,$maxvalue right font \"Helvetica, 12\"\n";
   print GNU "set out \"$psfile\"\n";
   print GNU "set term post land enh\n";
   print GNU "plot [-10:10] \"$txtfile\" using (\$1-2047):2 w l lt 1, \"fit.dat\" w l lt 2\n";
   print GNU "#    EOF\n";
  close (GNU);
  
  system ("gnuplot < $gnufile");
  system ("rm -f $gnufile $txtfile fit.dat fit.log");
 }

}
