#include "unpack.h"

/******************************************************************************/
/*	unpack_pfs_4c4b							      */
/******************************************************************************/
void unpack_pfs_4c4b(char *buf, float *rcp, float *lcp, int bufsize)
{
  /*
    unpacks 4-channel, 4-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output arrays rcp and lcp each contain bufsize floats
    rcp and lcp must each have storage for at least bufsize*sizeof(float)
  */
  
  /* order is RCP-I, RCP-Q and LCP-I, LCP-Q */
  /* i.e. board 1 channel A, board 1 channel B */
  /* and  board 2 channel A, board 2 channel B */
  /* tested */
  
  char value;
  float lookup[16] = {+15,+13,+11,+9,+7,+5,+3,+1,-1,-3,-5,-7,-9,-11,-13,-15}; 
  int i,r,l;
  
  r = 0;
  l = 0;
  for (i = 0; i < bufsize; i += 4) 
    {
      value = buf[i+3];
      lcp[l++] = lookup[value & 15];
      value = value >> 4;
      lcp[l++] = lookup[value & 15];

      value = buf[i+2];
      rcp[r++] = lookup[value & 15];
      value = value >> 4;
      rcp[r++] = lookup[value & 15];

      value = buf[i+1];
      lcp[l++] = lookup[value & 15];
      value = value >> 4;
      lcp[l++] = lookup[value & 15];

      value = buf[i];
      rcp[r++] = lookup[value & 15];
      value = value >> 4;
      rcp[r++] = lookup[value & 15];
    }

  return;
}

/******************************************************************************/
/*	unpack_pfs_4c2b							      */
/******************************************************************************/
void unpack_pfs_4c2b(char *buf, float *rcp, float *lcp, int bufsize)
{
  /*
    unpacks 4-channel, 2-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output arrays rcp and lcp each contain 2*bufsize floats
    rcp and lcp must each have storage for at least 2*bufsize*sizeof(float)
  */
  
  /* order is RCP-I, RCP-Q and LCP-I, LCP-Q */
  /* i.e. board 1 channel A, board 1 channel B */
  /* and  board 2 channel A, board 2 channel B */
  /* tested */
  
  char value;
  float lookup[4] = {+3,+1,-1,-3};  
  int i,r,l;
  
  r = 0;
  l = 0;
  for (i = 0; i < bufsize; i += 4) 
    {
      value = buf[i+3];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];

      value = buf[i+2];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];

      value = buf[i+1];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];

      value = buf[i];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];
    }

  return;
}

/******************************************************************************/
/*	unpack_pfs_4c2b_rcp						      */
/******************************************************************************/
void unpack_pfs_4c2b_rcp(char *buf, float *rcp, int bufsize)
{
  /*
    unpacks 4-channel, 2-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output array rcp contains 2*bufsize floats
    rcp must have storage for at least 2*bufsize*sizeof(float)
  */
  
  /* order is RCP-I, RCP-Q and LCP-I, LCP-Q */
  /* i.e. board 1 channel A, board 1 channel B */
  /* and  board 2 channel A, board 2 channel B */
  /* tested */
  
  char value;
  float lookup[4] = {+3,+1,-1,-3};  
  int i,r;
  
  r = 0;
  for (i = 0; i < bufsize; i += 4) 
    {
      value = buf[i+3];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];

      value = buf[i+2];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];

      value = buf[i+1];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];

      value = buf[i];
      rcp[r++] = lookup[value & 3];
      value = value >> 2;
      rcp[r++] = lookup[value & 3];
    }

  return;
}

/******************************************************************************/
/* unpack_pfs_4c2b_lcp                                                        */
/******************************************************************************/
void unpack_pfs_4c2b_lcp(char *buf, float *lcp, int bufsize)
{
  /*
    unpacks 4-channel, 2-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output array lcp each contains 2*bufsize floats
    lcp must each have storage for at least 2*bufsize*sizeof(float)
  */
  
  char value;
  float lookup[4] = {+3,+1,-1,-3};  
  int i,l;
  
  l = 0;
  for (i = 0; i < bufsize; i += 4) 
    {
      value = buf[i+3];
      value = value >> 4;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];

      value = buf[i+2];
      value = value >> 4;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];

      value = buf[i+1];
      value = value >> 4;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];

      value = buf[i];
      value = value >> 4;
      lcp[l++] = lookup[value & 3];
      value = value >> 2;
      lcp[l++] = lookup[value & 3];
    }

  return;
}



/******************************************************************************/
/*	unpack_pfs_2c2b							      */
/******************************************************************************/
void unpack_pfs_2c2b(char *buf, float *outbuf, int bufsize)
{
  /*
    unpacks 2-channel, 2-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output array contains 4*bufsize floats
    outbuf must have storage for at least 4*bufsize*sizeof(float)
  */
  
  /* tested */

  char value;
  float lookup[4] = {+3,+1,-1,-3}; 
  int i,k;
  
  k = 0;
  for (i = 0; i < bufsize; i += 4)
    {
      value = buf[i+3];
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      k += 4;

      value = buf[i+2];
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      k += 4;

      value = buf[i+1];
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      k += 4;

      value = buf[i];
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      k += 4;
    }

  return;
}

/******************************************************************************/
/*	unpack_pfs_2c8b   						      */
/******************************************************************************/
void unpack_pfs_2c8b(char *buf, float *outbuf, int bufsize)
{
  /*
    unpacks 2-channel, 8-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output array outbuf contains bufsize floats
    output array must have been allocated for at least bufsize*sizeof(float)
  */

  /* order is RCP-I, RCP-Q */
  /* i.e. board 1 channel A, board 1 channel B */
  /* tested */

  int i,k;
  
  k = 0;
  for (i = 0; i < bufsize; i+=4)
    {
      outbuf[k++] = (float) (unsigned char) buf[i+2];
      outbuf[k++] = (float) (unsigned char) buf[i+3];
      outbuf[k++] = (float) (unsigned char) buf[i+0];
      outbuf[k++] = (float) (unsigned char) buf[i+1];
    }

  return;
}

/******************************************************************************/
/*	unpack_pfs_2c4b   						      */
/******************************************************************************/
void unpack_pfs_2c4b(char *buf, float *outbuf, int bufsize)
{
  /*
    unpacks 2-channel, 4-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output array outbuf contains 2*bufsize floats
    output array must have been allocated for at least 2*bufsize*sizeof(float)
  */

  /* order is RCP-I, RCP-Q */
  /* i.e. board 1 channel A, board 1 channel B */
  /* tested */

  char value;
  float lookup[16] = {+15,+13,+11,+9,+7,+5,+3,+1,-1,-3,-5,-7,-9,-11,-13,-15}; 
  int i,k;
  
  k = 0;
  for (i = 0; i < bufsize; i += 4)
    {
      value = buf[i+3];
      outbuf[k++] = lookup[value & 15];
      value = value >> 4;
      outbuf[k++] = lookup[value & 15];

      value = buf[i+2];
      outbuf[k++] = lookup[value & 15];
      value = value >> 4;
      outbuf[k++] = lookup[value & 15];

      value = buf[i+1];
      outbuf[k++] = lookup[value & 15];
      value = value >> 4;
      outbuf[k++] = lookup[value & 15];

      value = buf[i+0];
      outbuf[k++] = lookup[value & 15];
      value = value >> 4;
      outbuf[k++] = lookup[value & 15];
    }

  return;
}

/******************************************************************************/
/* unpack_pfs_signedbytes						      */
/******************************************************************************/
void unpack_pfs_signedbytes(char *buf, float *outbuf, int bufsize)
{
  /*
    unpacks signed bytes, eg stream obtained by digital filtering
    of data from the portable fast sampler
    input array buf is of size bufsize bytes
    output array outbuf contains bufsize floats
    output array must have been allocated for at least bufsize*sizeof(float)
  */

  int i;
  
  for (i = 0; i < bufsize; i++)
    outbuf[i] = (float) buf[i];
  
  return;
}


/* untested below */
/* unmodified for little endian */

/******************************************************************************/
/*	unpack_pfs_2c2b							      */
/******************************************************************************/
void iq_swap_unpack_pfs_2c2b(char *buf, float *outbuf, int bufsize)
{
  /* untested */

  /*
    unpacks 2-channel, 2-bit data from the portable fast sampler
    input array buf is of size bufsize bytes
    output array contains 4*bufsize floats
    outbuf must have storage for at least 4*bufsize*sizeof(float)
  */
  
  /* appropriate with first PLD program */

  char value;
  float lookup[4] = {+3,+1,-1,-3};  
  int i,k;
  
  k = 0;
  for (i = 0; i < bufsize; i += 4)
    {
      value = buf[i+3];
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      k += 4;

      value = buf[i+2];
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      k += 4;

      value = buf[i+1];
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      k += 4;

      value = buf[i+0];
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      k += 4;

    }

  return;
}

/******************************************************************************/
/*	unpack_pfs_2c4b_bal   						      */
/******************************************************************************/
void unpack_pfs_2c4b_bal(char *buf, float *outbuf, int bufsize,float dcoffi, float dcoffq, float rmsioq)
{
  /*
    unpacks 2-channel, 4-bit data from the portable fast sampler
    subtracts dc offsets and balances channels
    input array buf is of size bufsize bytes
    output array outbuf contains 2*bufsize floats
    output array must have been allocated for at least 2*bufsize*sizeof(float)
  */

  /* order is RCP-I, RCP-Q */
  /* i.e. board 1 channel A, board 1 channel B */
  /* appropriate with first PLD program */

  char value;
  float lookup[16] = {+15,+13,+11,+9,+7,+5,+3,+1,-1,-3,-5,-7,-9,-11,-13,-15}; 
  int i,k;
  
  k = 0;
  for (i = 0; i < bufsize; i += 4)
    {
      value = buf[i+1];
      outbuf[k++] = (lookup[value & 15] - dcoffi) / rmsioq;
      value = value >> 4;
      outbuf[k++] = (lookup[value & 15] - dcoffq);

      value = buf[i+0];
      outbuf[k++] = (lookup[value & 15] - dcoffi) / rmsioq;
      value = value >> 4;
      outbuf[k++] = (lookup[value & 15] - dcoffq);

      value = buf[i+3];
      outbuf[k++] = (lookup[value & 15] - dcoffi) / rmsioq;
      value = value >> 4;
      outbuf[k++] = (lookup[value & 15] - dcoffq);

      value = buf[i+2];
      outbuf[k++] = (lookup[value & 15] - dcoffi) / rmsioq;
      value = value >> 4;
      outbuf[k++] = (lookup[value & 15] - dcoffq);
    }

  return;
}

/******************************************************************************/
/*	unpack_pfs_xc2b							      */
/******************************************************************************/
void unpack_pfs_xc2b(char *buf, float *outbuf, int bufsize, int chan, float dcoffi, float dcoffq, float rmsioq)
{
  /*
    unpacks one half of 4-channel, 2-bit data from the portable fast sampler
    subtracts dc offsets and balances channels
    rcp or lcp is unpacked depending on the value of chan, which must be 1 or 2
    input array buf is of size bufsize bytes
    output array outbuf contains 2*bufsize floats
    outbuf must have storage for at least 2*bufsize*sizeof(float)
  */
  
  /* order is RCP-I, RCP-Q if chan = 1 */
  /* i.e. board 1 channel A, board 1 channel B */
  /* order is LCP-I, LCP-Q if chan = 2 */
  /* i.e. board 2 channel A, board 2 channel B */
  /* appropriate with first PLD program */
  
  char value;
  float lookup[4] = {+3,+1,-1,-3};  
  int i,r,l;
  
  r = 0;
  l = 0;

  if (chan == 1)
    for (i = 0; i < bufsize; i += 2) 
      {
	value = buf[i+1];
	outbuf[r++] = (lookup[value & 3] - dcoffi) / rmsioq;
	value = value >> 2;
	outbuf[r++] = (lookup[value & 3] - dcoffq);
	
	value = buf[i];
	outbuf[r++] = (lookup[value & 3] - dcoffi) / rmsioq;
	value = value >> 2;
	outbuf[r++] = (lookup[value & 3] - dcoffq);
      }
  else if (chan == 2)
    for (i = 0; i < bufsize; i += 2) 
      {
	value = buf[i+1] >> 4;
	outbuf[l++] = (lookup[value & 3] - dcoffi) / rmsioq;
	value = value >> 2;
	outbuf[l++] = (lookup[value & 3] - dcoffq);
	
	value = buf[i] >> 4;
	outbuf[l++] = (lookup[value & 3] - dcoffi) / rmsioq;
	value = value >> 2;
	outbuf[l++] = (lookup[value & 3] - dcoffq);
      }
  else 
    {
      perror("incorrect channel number in call to unpack_pfs_xc2b\n");
      exit(1);
    }
  return;
}

/*****************************************************************************/
/*	unpack_gsb							     */
/*****************************************************************************/
void unpack_gsb(char *buf, float *outbuf, int bufsize)
{
  /*
    unpacks 2-channel, 2-bit data from the Golevka Sampling Box 
    input array buf is of size bufsize bytes
    output array contains 4*bufsize floats
    outbuf must have storage for at least 4*bufsize*sizeof(float)
  */
  
  /* order is channel A then channel B */

  char value;
  float lookup[4] = {3,-1,+1,-3}; 
  int i,j,k;
  
  j = 0;
  for (i = 0, k = 0; i < bufsize; i++, k += 4)
    {
      value = buf[i];
      outbuf[k+2] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+3] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+0] = lookup[value & 3];
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3];
    }
  return;
}

/*****************************************************************************/
/*	unpack_gsb_bal							     */
/*****************************************************************************/
void unpack_gsb_bal(char *buf, float *outbuf, int bufsize, float dcoffi, float dcoffq, float rmsioq)
{
  /*
    unpacks 2-channel, 2-bit data from the Golevka Sampling Box 
    input array buf is of size bufsize bytes
    output array contains 4*bufsize floats
    outbuf must have storage for at least 4*bufsize*sizeof(float)
  */
  
  /* order is channel A then channel B */

  char value;
  float lookup[4] = {3,-1,+1,-3}; 
  int i,j,k;
  
  j = 0;
  for (i = 0, k = 0; i < bufsize; i++, k += 4)
    {
      value = buf[i];
      outbuf[k+2] = (lookup[value & 3] - dcoffi) / rmsioq;
      value = value >> 2;
      outbuf[k+3] = lookup[value & 3] - dcoffq;
      value = value >> 2;
      outbuf[k+0] = (lookup[value & 3] - dcoffi) / rmsioq;
      value = value >> 2;
      outbuf[k+1] = lookup[value & 3] - dcoffq;
    }
  return;
}

/*****************************************************************************/
/*	pack_gsb							     */
/*****************************************************************************/
void pack_gsb(unsigned char *in, unsigned long *out, int bufsize)
{
  /* 
     packs data from the Golevka Sampling Box
     incoming data is pair of digitized signals with 8-bit resolution
     the bits are arranged so that the two most significant bits 
     occupy the lowest order four bits of each pair of bytes
     this routine keeps only those four bits and reduces the overall 
     data rate to disk by a factor of four
  */

  unsigned long tmp;
  int words = bufsize / 2;
  int i;

  tmp = 0;
  for( i = 1; i <= words; i++ ) 
    {
      tmp = tmp << 4 | (0x0f & *in);
      in+=2;
      if( i%8 == 0 ) 
	{
	  *out++ = tmp;
	  tmp = 0;
	}
    }
}

