#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <getopt.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <sys/resource.h>

// dispersion constant
#define ALPHA1 2.607131e16
#define ALPHA2 (ALPHA1 / (2 * M_PI))

// type of input data
#define TYPE float

double Period; // pulsar period at pointed time in ns
double DM = 71.0398; // dispersion measure
// from Exp* log-file Fcentr = 2165 MHz  but constructing the scheme of the frequencies conversions
// with LO1 = 3665 MHz LO2 = 1750 MHz and baseband LO=300 MHz 
// Fcentr appeared to be 2140 MHz and so Flower = 2065 MHz (BW=150 MHz)
// So, I think that Fcentr=2165 MHz was just how we chose the sky radio-band but it's not really
// fcentr in how we digitize
double Fcentr = 2215.;  // the center frequency in MHz
double BW = 150.; // bandwidth in MHz
double Tres = 6.666667; // time resolution in ns
int ChannelNumber = 64;  // number of individual channels

// bandwidth of separate channel in MHz
double BWi;
// time resolution in separate channel in ns
double Tresi;
// center frequency of separate channel in MHz
double Fcentri;
double Frefi; // the reference frequency (to which we will dedisperse in separate channels)
              // we will choose it as lower edge for every channel
// band seems to be direct, rather than "inverse" (for 1937 data taken at 2005, Jan 6  with VA)
// // lower sideband had to be taken, rather than "Upper" as is written in Exp* log-file
bool is_band_direct = true;


// size of the input files (with detected signals of 1 period long)
off_t filesize;
// the delay in counts for the particular frequency channel
signed long delay;
// absolute delay (without sign) and taking into account the pulsar period
unsigned long absdelay;
// pulsar period in counts 
off_t cPeriod;

// output path
char *Path;






// give delay in counts between Fcentr and  Frefi
// if delay > 0 so we have to shift data in direction to the end
// if delay < 0 so we have to move data to the beginning
signed long showdelay (double Frefi) {

 double tbad;
 tbad = DM * ALPHA2 * ( 1./(Fcentr * Fcentr) - 1./(Frefi * Frefi) );
 tbad = (tbad * 0.001)/Tresi;

 return (signed long)rint(tbad);
}

// split the path and filename
// return filename
char *baseName (char * str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
}

// split the path and filename
// return path
char *getPath (char * dest, char * str) {
 strcpy (dest, str);
 char *s = dest;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 if (len != -1) { *s = '\0'; return dest; }
  else return ".";
}

/*===================Parsing the command line===========================*/

int ParseCmdLine (int argc, char *argv[]) {
  int op;
  struct option long_options[] = { {"direct", no_argument, 0, 10},
	                           {"inverse", no_argument, 0, 11},
	                           {"dm", required_argument, 0, 12},
				   {"fcentr", required_argument, 0, 13},
				   {"bw", required_argument, 0, 14},
				   {"numch", required_argument, 0, 15},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "p:t:d:", long_options, 0)) != EOF)
    switch(op){
      case 'p':
        Period = fabs(atof(optarg));
      break;
            
      case 't':
        Tres = fabs(atof(optarg));
      break;
      
      case 'd':
	  if (Path) { Path = 0; delete (Path); }
	  Path = new char[255];
	  if (!Path) { perror("new"); exit(1); }
	  memcpy(Path, optarg, (int)strlen(optarg));
      break;

      case 10:
          is_band_direct = true;
      break;

      case 11:
          is_band_direct = false;
      break;

      case 12:
        DM = fabs(atof(optarg));
      break;
      
      case 13:
        Fcentr = fabs(atof(optarg));
      break;

      case 14:
        BW = fabs(atof(optarg));
      break;
      
      case 15:
        ChannelNumber = atoi(optarg);
      break;

      case '?':
        printf ("Wrong option!\n");
	exit (1);
      break;
    }
  return(optind);
}

/*==================  m a i n  ========================*/

int main (int argc, char *argv[]) {

 if (argc == 1) { 
  printf ("Usage: %s [options] <one file from lin.[X|Y].*>\n"
	  "  [options]: -p <period>\n"
	  "             -t <timeres>\n"
	  "             -d <path>\n"
	  "             --dm <DM>\n"
	  "             --fcentr <Fcentr>\n"
	  "             --bw <BW>\n"
	  "             --direct (for direct band, default)\n"
	  "             --inverse (for inverse band)\n"
	  "             --numch <Nchannels>\n", baseName(argv[0])); 
  exit (0); 
 }

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;
 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE,&r);

 Path = new char[255];
 if (!Path) { perror("new"); exit(1); }
 memset (Path, 0, 255);
 sprintf (Path, "%s", "\0");

 // number of command line argument with name of the file
 int next = ParseCmdLine(argc, argv);
 if (argc < next + 1) { printf("Not point the file!\n"); exit(1); }
 int in;   // input file descriptor

 if (!strcmp(Path, "\0")) { memset (Path, 0, 255); sprintf (Path, "%s", getPath(Path, argv[next])); }

 struct stat info;
 if ((stat(argv[next], &info)) == -1) { perror("stat"); exit(1); }
 filesize = info.st_size / sizeof(float);

 unsigned long bytesread; // number of actually read bytes
 TYPE* data; // the array of the whole input data
 data = new TYPE[filesize];
 if (!data) { perror("new"); exit(1); }

 // array of aligned data
 float *aligned;
 aligned = new float[filesize];
 if (!aligned) { perror("new"); exit(1); }



 // bandwidth of separate channel in MHz
 BWi = BW / ChannelNumber;
 // time resolution in separate channel in ns
 Tresi = Tres * ChannelNumber;
 // calculating the pulsar period in counts
 cPeriod = (off_t)rint(Period/Tresi);

 // output filename
 char *output;
 output = new char[255];
 if (!output) { perror("new"); exit(1); }
 int out; // output file descriptor

 printf ("filesize: %ld\n", filesize);
 printf ("dm: %lf  fcentr: %lf  bw: %lf (%s)  tres: %lf\n", DM, Fcentr, BW, is_band_direct ? "direct" : "inverse", Tres);
 printf ("number of channels: %d\n\n", ChannelNumber);
 printf ("period: %lf\n", Period);

 float sign = (is_band_direct ? 1. : -1.);
 // loop on frequency channels
 for (int ch=0; ch<ChannelNumber; ++ch) {

  memset (data, 0, filesize * sizeof (TYPE));
  in = open (argv[next+ch], O_RDONLY | O_LARGEFILE);
  if (in == -1) { perror("open"); exit (1); }
  lseek (in, 0, SEEK_SET);
  bytesread = read (in, data, filesize * sizeof (TYPE));
  if (bytesread != filesize * sizeof (TYPE)) { perror("read"); exit(1); }
  close (in);
  memset (aligned, 0, filesize * sizeof (float));

  Fcentri = Fcentr + (-1.*sign)*(BW/2.) + sign*(BWi/2.) + sign * ch * BWi;
  Frefi = Fcentr + (-1.*sign)*(BW/2.) + sign * ch * BWi;  // make the lower (upper if band inverse) frequency of every channel as reference

  // get delay in counts
  delay = showdelay (Frefi);
  absdelay = labs(delay);
  absdelay = absdelay % cPeriod;
  if (delay == 0) for (off_t i=0; i<filesize; ++i) aligned[i] = data[i];
  if (delay > 0) {
   for (off_t i=0, j=absdelay; j<filesize; ++i, ++j) aligned[j] = data[i]; 
   for (off_t i=filesize-absdelay, j=0; j<absdelay; ++i, ++j) aligned[j] = data[i];
  }
  if (delay < 0) {
   for (off_t j=0, i=absdelay; j<filesize-absdelay; ++i, ++j) aligned[j] = data[i]; 
   for (off_t j=filesize-absdelay, i=0; j<filesize; ++i, ++j) aligned[j] = data[i];
  }

  printf ("[%02d]: Frefi = %lf  delay = %ld  cPeriod = %ld  absdelay = %ld   ", ch, Frefi, delay, cPeriod, absdelay);
  fflush (NULL);

  // saving into file
  memset (output, 0, 255);
  sprintf (output, "%s/%s.al", Path, baseName(argv[next+ch]));

  out = open (output, O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644);
  lseek (out, 0, SEEK_SET);
  write (out, aligned, sizeof(float)*filesize);
  close (out);
  printf ("aligned -> %s\n", output);

 } // end of the loop on frequency channels (ch)
 
 delete (data);
 delete (aligned);
 delete (output);
 delete (Path);
 return 0;
}
