#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <getopt.h>

#include <sys/resource.h>

// the first digit in 2-bit group supposed to be 'amplitude', i.e. a-s
// rather than s-a as usual
// now i am not sure about this ^
// signed char vlba[4] = { -3, 1, -1, 3 };
signed char vlba[4] = { -3, -1, 1, 3 };

// the size of the Exp-file
unsigned long filesize = 4194304 / sizeof (long);
// the size of the single channel
unsigned long channelsize = filesize / 4;

// output path
char *Path;



// split the path and filename
// return filename
char *baseName (char * str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
}

// split the path and filename
// return path
char *getPath (char * dest, char * str) {   
 strcpy (dest, str);
 char *s = dest;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 if (len != -1) { *s = '\0'; return dest; }
  else return ".";
}  

// split the extesion and rest of the filename
// return extension (divider is dot)
char *getExt (char * str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '.' && len--) ;
 return (++s);
}


/*===================Parsing the command line===========================*/

int ParseCmdLine (int argc, char *argv[]) {
  int op;
  struct option long_options[] = { {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "d:", long_options, 0)) != EOF)
    switch(op){
      case 'd':
	  if (Path) { Path = 0; delete (Path); }
	  Path = new char[255];
	  if (!Path) { perror("new"); exit(1); }
	  memcpy(Path, optarg, (int)strlen(optarg));
      break;
      
      case '?':
        printf ("Wrong option!\n");
	exit (1);
      break;
    }
  return(optind);
}


/* ===============  m a i n  ====================*/ 

int main (int argc, char *argv[]) {
 
 if (argc == 1) {
   printf ("Usage: %s [options] <Exp-files>\n"
           "  [options]: -d <path>\n", baseName(argv[0]));
   exit (0);
 }

 /* Making this process to be able to create files larger than 2GB */
 struct rlimit r;
 r.rlim_cur = r.rlim_max = RLIM_INFINITY;
 setrlimit(RLIMIT_FSIZE,&r);

 Path = new char[255];
 if (!Path) { perror("new"); exit(1); }
 memset (Path, 0, 255);
 sprintf (Path, "%s", "\0");

 // number of command line argument with name of the file
 int next = ParseCmdLine(argc, argv);
 if (argc < next + 1) { printf("Not point the file!\n"); exit(1); }
 
 if (!strcmp(Path, "\0")) { memset (Path, 0, 255); sprintf (Path, "%s", getPath(Path, argv[next])); }

 unsigned long bytesread; // number of actually read bytes
 unsigned long* data; // the array of the whole input data (in one file)
 data = new unsigned long[filesize];
 if (!data) { perror("new"); exit(1); }

 long output_size = 8 * filesize;  // the size of the output data (for one polarization)
 signed char* output; // the output data
 output = new signed char[output_size];
 if (!output) { perror("new"); exit(1); }

 // names of the output files
 char* outX, *outY;
 outX = new char[255];
 outY = new char[255];
 if (!outX || !outY) { perror("new"); exit(1); }
 
 int in;   // input file descriptor
 int out;  // output file descriptor
 // loop on input Exp-files
 for (int f=next; f<argc; ++f) {

  memset (data, 0, filesize * sizeof(long));
  in = open (argv[f], O_RDONLY | O_LARGEFILE);
  if (in == -1) { perror("open"); exit (1); }
  lseek (in, 0, SEEK_SET);
  bytesread = read (in, data, filesize*sizeof(long));
  if (bytesread != filesize*sizeof(long)) { perror("read"); exit(1); }
  close (in);

  memset (output, 0, output_size);

  // loop on elements for inphase & quad channels of LinX data
  for (long i=0, j=channelsize, k=0; i<channelsize; ++i, ++j, k+=32) {
   for (long m=k; m < k+32; m+=16) {
    output[m+12] = vlba[data[i] & 0x03]; 
    output[m+13] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+14] = vlba[data[i] & 0x03]; 
    output[m+15] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+8] = vlba[data[i] & 0x03]; 
    output[m+9] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+10] = vlba[data[i] & 0x03]; 
    output[m+11] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+4] = vlba[data[i] & 0x03]; 
    output[m+5] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+6] = vlba[data[i] & 0x03]; 
    output[m+7] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m]   = vlba[data[i] & 0x03]; 
    output[m+1] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+2] = vlba[data[i] & 0x03]; 
    output[m+3] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
   }
  } // end of the i-loop

  // writing the LinX to the file
  memset (outX, 0, 255);
  sprintf (outX, "%s/%s.%s", Path, "lin.X", getExt(argv[f]));
  out = open (outX, O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644);
  if (out == -1) { perror("open"); exit(1); }
  lseek (out, 0, SEEK_SET);
  write (out, output, output_size);
  close (out);

  memset (output, 0, output_size);

  // loop on elements for inphase & quad channels of LinY data
  for (long i=2*channelsize, j=3*channelsize, k=0; j<filesize; ++i, ++j, k+=32) {
   for (long m=k; m < k+32; m+=16) {
    output[m+12] = vlba[data[i] & 0x03]; 
    output[m+13] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+14] = vlba[data[i] & 0x03]; 
    output[m+15] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+8] = vlba[data[i] & 0x03]; 
    output[m+9] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+10] = vlba[data[i] & 0x03]; 
    output[m+11] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+4] = vlba[data[i] & 0x03]; 
    output[m+5] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+6] = vlba[data[i] & 0x03]; 
    output[m+7] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m]   = vlba[data[i] & 0x03]; 
    output[m+1] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
    output[m+2] = vlba[data[i] & 0x03]; 
    output[m+3] = vlba[data[j] & 0x03];
    data[i] >>= 2; 
    data[j] >>= 2;
   }
  } // end of the i-loop

  // writing the LinY to the file
  memset (outY, 0, 255);
  sprintf (outY, "%s/%s.%s", Path, "lin.Y", getExt(argv[f]));
  out = open (outY, O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644);
  if (out == -1) { perror("open"); exit(1); }
  lseek (out, 0, SEEK_SET);
  write (out, output, output_size);
  close (out);
 	 
 } // end of loop on Exp-files
 
 delete (data);
 delete (output);
 delete (outX);
 delete (outY);
 delete (Path);
 return 0;
}
