#define MAIN_
#include "bind2sm.hpp"


/*==========================Prompting===================================*/

void Help()
 {
  printf("b2t converts incoming binary files to ASCII format (ready for SM)\n"
	 "  Synopsis:\n"
	 "	b2t [options] [ file1 file2 ... ]\n"
	 "  Options:\n"
	 "	h	    this screen.\n"
	 "	b o_set	    start with offset o_set, if o_set is negative, then\n"
	 "		    start from begining of file\n"
	 "	e o_set	    end at offset o_set, if filesize is below, then\n"
	 "		    end at EOF\n"
	 "	s factor    produce smoothing of incoming data with value factor (>0)\n"
	 "	a factor    perform averaging with factor value (>0)\n");
  exit(0);
 }


/*==========================MAIN========================================*/

int main(int argc, char **argv)
 {
  char resName[MAXSTRSIZE], op;
  int start_offset = 0, end_offset = 0, smooth_points = 1, average_points = 1;
  
  if(argc < 2) Help();
  while((op = getopt(argc, argv, "hb:e:s:a:")) != EOF){
    switch(op){
      case 'h':
      case ':':
      case '?':
	Help();
      break;

      case 'b':
	start_offset = atoi(optarg);
	if(start_offset < 0) start_offset = 0;
      break;

      case 'e':
	end_offset = atoi(optarg);
	if(end_offset < 0) end_offset = 0;
      break;

      case 's':
	smooth_points = atoi(optarg);
	if(smooth_points <= 0) Help();
      break;

      case 'a':
	average_points = atoi(optarg);
	if(average_points <= 0) Help();
      break;
    }
  }

  unsigned count = 0;

  for(int index = optind; index < argc; index++){
    DataSize = GetBinFileLength(argv[index]);
    if(end_offset == 0 || end_offset > DataSize) end_offset = DataSize;
    DataSize = end_offset - start_offset;
    Profile = (ptrBinData) realloc(Profile, DataSize*sizeof(BinData));
    cout << count << '\t' << argv[index] << endl;
    LoadBinFile(argv[index], Profile, start_offset, DataSize);

    if(smooth_points != 1)
      DataSize = Smooth(Profile, DataSize, smooth_points);

    if(average_points != 1)
      DataSize = Average(Profile, DataSize, average_points);

    strcpy(resName, argv[index]);
    strcat(resName, ".ascii");

    int resFds = open(resName, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
    char buf[MAXSTRSIZE];
    for(int i = 0; i < DataSize; i++){
      sprintf(buf, "%d %lf\n", start_offset + i*average_points, Profile[i]);
      write(resFds, buf, strlen(buf));
    }
    close(resFds);
    ++count;
  }

  free(Profile);
  return(0);
 }
