#define MAIN_
#include "readme.hpp"
#include "bitstat.hpp"   

float * mass, sum;
long Pos = -1;
off_t sz = 0, left = 0, bitN = 0, wsize = 0;
unsigned char Save = 1, Statistics = 0;
int Header = 0;
char *AuxExt, *OutFile, *OutputPath;
bool is_quite = false;
bool is_voltage_optimized = false;


/*-- this function gives value of inverse error function -------*/
double inverf (double );


/*-------     (!) HOST  NETWORK   (!) 
            -------------------------------------------*/

TYPE nh2hn ( TYPE data, int razm1 )
{
  char tmp;
  const int razm = sizeof(TYPE);
  union { TYPE d;
          unsigned char uc[razm];
         } b;
  b.d = data;
  for (int i=0;i<razm/2;i++) { tmp = b.uc[i];
                               b.uc[i] = b.uc[razm-1-i];
			       b.uc[razm-1-i] = tmp;
                              }
  return b.d;           
 }
    
/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to correct 2-bit sampled data for bit statistics\n\n"
	 "[USAGE]: %s [options] <file(s) to process>\n\n"
	 "[OPTIONS]:\n"
	 "-d, --path  <path>           - path to the output file(s), default - the same\n"
	 "                               where are the source files\n"
	 "--stat                       - print number of different counts (1,-1,3,-3) in\n"
	 "                               percents\n"
	 "--ext       <string>         - auxiliary filename extension given to the output\n"	 
	 "                               file(s), default - .bst\n"
	 "--out       <filename>       - name of file for output data. If --ext is given\n"
	 "                               then full name is filename + extension\n"
	 "                               default - input file + .bst\n"
	 "--header    <size>           - header in bytes of input file(s), default = 0\n"
	 "-l, --left  <left boundary>  - left boundary of used window, default = 0\n"
	 "-s, --size  <size of window> - size of used window, default - equal to size of file\n"
	 "-n, --num   <value>          - number of points to obtain statistics of bits\n"
	 "                               default - one-twentieth of size of used window or\n"
	 "                               if less than 1, equal to 1\n"
	 "-p, --pos   <value>          - position of active value within window used to\n"
	 "                               obtain statistics of bits. The <value> must be >= 0\n"
	 "                               and < than <value> in -n option\n"
	 "                               default - in the middle of window\n"
	 "--nowrite                    - don't write marginal values, i.e. values with the\n"
	 "                               same bit statistics on the edges of data sample\n"
	 "                               default - write\n"
	 "-w, --wsize <value>          - size of block to record in case of large files.\n"
	 "                               If it's pointed then dest file(s) are forming by\n"
	 "                               appending consecutively the blocks of selected size;\n"
	 "                               default - equal to size of file (in points)\n"
	 "--voltage-optimized          - use voltage-optimized scheme, default - power-optimized\n"
	 "                               for more details, see Jenet & Anderson, 1998, PASP, 110, 1467\n"
	 "-q, --quite                  - turn on quite output, skip printing the iteration changes\n"
	 "-h, --help                   - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"left", required_argument, 0, 'l'},
				   {"size", required_argument, 0, 's'},
				   {"num", required_argument, 0, 'n'},
				   {"pos", required_argument, 0, 'p'},
				   {"path", required_argument, 0, 'd'},
				   {"wsize", required_argument, 0, 'w'},
				   {"ext", required_argument, 0, 10},
				   {"out", required_argument, 0, 11},
				   {"header", required_argument, 0, 12},
                                   {"nowrite", no_argument, 0, 13},
                                   {"stat", no_argument, 0, 14},
                                   {"quite", no_argument, 0, 'q'},
                                   {"voltage-optimized", no_argument, 0, 15},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "hl:s:d:n:p:w:q", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 'l':
        left = atol(optarg);
	if (left < 0) left = 0;
      break;
      
      case 's':
        sz = atol(optarg);
	if (sz <= 0) sz = -1;
      break;
      
      case 'n':
        bitN = atol(optarg);
	if (bitN < 1) bitN = 0;
      break;
      
      case 'p':
        Pos = atol(optarg);
	if (Pos < 0) Pos = -1;
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 'w':
        wsize = atol(optarg);
	if (wsize <= 0) wsize = -1;
      break;
      
      case 10:
	  if( AuxExt ) {
	    AuxExt = 0;
	    free(AuxExt);
	                 }
	  if( !( AuxExt = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(AuxExt, optarg, (int)strlen(optarg));
      break;
      
      case 11:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
      
      case 12:
       Header = atoi(optarg);
       if (Header < 0) Header = 0;
      break;
      
      case 13:
       Save = 0;
      break;
      
      case 14:
       Statistics = 1;
       bitN = 1;
      break;

      case 'q':
       is_quite = true;
      break;

      case 15:
       is_voltage_optimized = true;
      break;
      
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
 }

/*--- ,       ----*/

char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/*-----     ----*/

template <class tFormat>
void Write (tFormat m, off_t size, int razm, char *name, char *ext, int flag, int wri)
{
   int out;
   char *outfilename;
   outfilename = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);

   if (!strcmp(OutputPath, "\0")) { if (strcmp(OutFile, "\0") && !strcmp(ext, ".bst")) sprintf(outfilename, "%s", OutFile);
    else sprintf(outfilename, "%s%s", (!strcmp(OutFile, "\0") ? name : OutFile), ext);
    } else {
    if (strcmp(OutFile, "\0") && !strcmp(ext, ".bst")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
    else sprintf(outfilename, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), ext);
    }

   if (wri == 1) { printf(" writing... [%s]", outfilename); fflush(NULL); }
   
   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
	   write(out, m, size*razm);	                                           
           close(out);
        }
	
   memset(m, 0, razm*size);
   	
   delete(outfilename);
 }

/*------------      ----------------------*/

void ReadSrc (off_t l, off_t size, char *name)
{
  int file;
  TYPE *data;
  
     data = new TYPE[size];
     if (!data) { printf("Not enough memory!\n"); exit(1); }

     memset(data, 0, size*sizeof(TYPE));
     memset(mass, 0, size*sizeof(float));
       
       if ( (file = open(name, O_RDONLY | O_LARGEFILE)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
       if (lseek(file, (off_t)Header + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, data, size*sizeof(TYPE));
        close(file);

   for (off_t q=0;q<size;q++) {
	
  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
           mass[q] = (float)data[q];
	   
	                      }
    delete(data);	
 }

/*--------- ,    ----------------*/

float RecoveredValue (float value, float sum) {

 if (value == 0.) return 0.;

 double part = (double)(sum / (float)bitN); // percent of "1"-bits

 double x = inverf(part); // determine "x" (argument of erf function) = t/(sigma * sqrt(2))
 double ts = x*sqrt((double)2.); // t/sigma
 double st = ((double)1.)/ts; // sigma/t
 static double pi2 = sqrt((double)2./(double)M_PI); // sqrt(2/pi)

 double x2 = ((double)-1.)*x*x; // t^2/(2*sigma^2)
 double y;

 if (!is_voltage_optimized) {
/* Power optimized levels */
  y = 2. * (fabs(value) == 1. ? ((double)value)*st*sqrt( (double)1. - ts*pi2*(exp(x2)/part) ) : ((double)value/(double)3.)*st*sqrt( (double)1. + ts*pi2*(exp(x2)/((double)1. - part)) ) );
 } else {
/* Voltage optimized levels */
  y = 2. * (fabs(value) == 1. ? ((double)value)*st*pi2*(((double)1.-exp(x2)) / part) : ((double)value/(double)3.)*st*pi2*exp(x2)/((double)1. - part) );
 }

 return (float)y;
}  

/*-----------     - -------------------*/

off_t BitCorrection (float * x, off_t size, off_t sh, off_t t, off_t iter) {

 float * save, pVal, fSum;
 
 if (t == 0) {
 if (Pos != 0 && Save == 1) { save = new float[Pos];
                              if (!save) { printf("Not enough memory!\n"); exit(1); }

		              memset(save, 0, Pos*sizeof(float));
		              for (off_t i=0;i<(off_t)Pos;i++) save[i] = x[i];
                             }
               }
			     
 for (off_t i=0;i<=size-bitN-sh;i++) {
 
  if (i == 0) { if (t == 0) {
                  for (off_t j=0;j<bitN;j++) sum += (x[j] == 1. || x[j] == -1. ? 1. : 0.);
                  if (Save == 1) fSum = sum;
                             } else sum += (x[bitN-1+i] == 1. || x[bitN-1+i] == -1. ? 1. : 0.);
               }
	       
  pVal = x[i];
  x[i] = RecoveredValue(x[Pos+i], sum);
  if (i != size-bitN-sh) sum += (x[bitN+i] == 1. || x[bitN+i] == -1. ? 1. : 0.);
  if (t+1 == iter && i == size-bitN-sh) ; else sum -= (pVal == 1. || pVal == -1. ? 1. : 0.);
  
                                        }
 
 if (Save == 1) { if (t == 0) { if (Pos != 0) { for (off_t i=size-bitN-sh;i>=0;i--) x[i+Pos] = x[i];
		                                for (off_t i=0;i<(off_t)Pos;i++) x[i] = RecoveredValue(save[i], fSum);
		                                delete(save);
		                               }
                               }
                  if (t+1 == iter) for (off_t i=(off_t)Pos+size-(off_t)bitN-sh+1;i<size;i++) x[i-(t == 0 ? 0 : Pos)] = RecoveredValue(x[i], sum);
                 }

   return (t == 0 ? (t+1 == iter ? size-sh-(Save == 1 ? 0 : bitN) : size-bitN+1-sh+(Save == 1 ? Pos : 0)) : (t+1 == iter ? size-sh-(Save == 1 ? Pos : bitN) : size-bitN+1-sh));
}

/*====================== Main Function ===================================*/
  
int main ( int argc, char *argv[] )
 {
   struct stat info;
   int next;
   off_t Size, iter, le, win;

   if (argc == 1) Help(argv[0]);
   
   AuxExt     = new char[MaxString+1];
   OutFile    = new char[MaxString+1];
   OutputPath = new char[MaxString+1];
   if (!AuxExt || !OutFile || !OutputPath) { printf("Not enough memory!\n"); exit(1); }
   memset(AuxExt, 0, MaxString+1);
   memset(OutFile, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   sprintf(AuxExt, "%s", ".bst");
   sprintf(OutFile, "%s", "\0");
   sprintf(OutputPath, "%s", "\0");
   
   next = ParseCmdLine(argc, argv);
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }
   
   if ( stat(argv[next], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[next]); exit(1); }

   Size = (info.st_size - Header)/sizeof(TYPE);
   if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }
   if (sz == 0 || sz > Size) sz = Size;
   if (left >= Size) left = 0;
   if (left + sz > Size) sz = Size - left;

   if (bitN == 0 || bitN > sz) bitN = sz/20;
   if (bitN < 1) bitN = 1;

   if (Pos == -1 || Pos >= (long)bitN) Pos = (long)floor(((float)bitN)/2.);

   if (wsize == 0 || wsize > sz) wsize = sz;
   if (wsize < bitN) wsize = bitN;

   iter = (wsize == sz ? 1 : ((sz-bitN+1)%(wsize-bitN+1) == 0 ? (sz-bitN+1)/(wsize-bitN+1) : (sz-bitN+1)/(wsize-bitN+1) + 1));

   mass = new float[wsize];   
   if (!mass) { printf("Not enough memory!\n"); exit(1); }

   double bitPerc[4];  //  -3,-1,1,3

   

printf("\nProcessing...\n");

    for (long i=next;i<argc;i++)
     {

 printf(" [%3d]\t%s ... ", i-next, argv[i]); 
 fflush(NULL);

 sum = 0.;
 for (int l=0;l<4;++l) bitPerc[l] = 0.;

 for (off_t t=0; t<iter; t++) {

    if (!is_quite) { printf(".%5d", t+1); fflush(NULL); }

  if (Statistics == 0) {  
    le  = ( t*(wsize-bitN+1) + bitN >= sz ? left + sz - bitN : ( t*(wsize-bitN+1) + wsize >= sz ? left + sz - wsize : left + t*(wsize-bitN+1) ));
    ReadSrc (le, ( t*(wsize-bitN+1) + bitN >= sz ? bitN : wsize ), argv[i]);
    win = BitCorrection (mass, t*(wsize-bitN+1) + bitN >= sz ? bitN : wsize, ( t+1 < iter ? ( (t+1)*(wsize-bitN+1) + bitN >= sz ? (t+1)*(wsize-bitN+1) + bitN - sz : ( (t+1)*(wsize-bitN+1) + wsize >= sz ? (t+1)*(wsize-bitN+1) + wsize - sz : 0 )) : 0), t, iter);
    
    if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); }
    
    Write (mass, win, sizeof(float), argv[i], AuxExt, t+1, iter-t);
    
  } else { //   1,-1,3,-3
            le = left+ t*wsize;
            ReadSrc (le, (t+1)*wsize > sz ? sz - t*wsize : wsize, argv[i]);
	    for (off_t h=0;h<( (t+1)*wsize > sz ? sz - t*wsize : wsize );++h) {
	      if (mass[h] == -3.) bitPerc[0] += 1.;
	       else if (mass[h] == -1.) bitPerc[1] += 1.;
	        else if (mass[h] == 1.) bitPerc[2] += 1.;
		 else if (mass[h] == 3.) bitPerc[3] += 1.;
	     }
	    if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); }
          }
 }
 printf(" - done\n");
 
 if (Statistics == 1) {
  for (int l=0;l<4;++l) { bitPerc[l] /= sz; bitPerc[l] *= 100.; }
  printf(" [\033[4mBit Statistics (%)\033[0m]: \t");
  printf("\033[1m[-3]\033[0m %.2f\t\033[1m[-1]\033[0m %.2f\t\033[1m[ 1]\033[0m %.2f\t\033[1m[ 3]\033[0m %.2f\n\n", bitPerc[0], bitPerc[1], bitPerc[2], bitPerc[3]);
 }
	
       }
       
  delete(mass);
  delete(AuxExt);
  delete(OutFile);
  delete(OutputPath);

  return 0;
 }

