#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Programma perepisana s drugoj, c++'snoj */

/* fur'jovaja funkcija (template):
   float *x ili double *x --- massiv dannyh dliny 2*size, gde na chjotnyh
         nomerah --- veshchestvennye chasty, na nechjotnyh --- mnimye;
   long size --- razmer FFT, stepen' dvojki;
   signed char isign --- znak FFT (isign >= 0 --- prjamoe, isign < 0 --- obratnoe);
   bool precision --- true - togda schitaem v double, inache (false) vo float
   float scale --- normirovka FFT.
   bool use_scale --- ispol'zovat' (true) ili net (false) normirovku
*/

template <class T>
void FFT (T *x, long size, signed char isign, bool precision, float scale, bool use_scale = true) {

 long l1, k, l2, n2, j, npow, two_j;
 double hlpre, hlpim, phasere, phaseim, ampre, ampim;

 if (isign < 0) for (long i=1; i<2*size+1; i+=2) x[i] *= -1.;
   
 j = 1;
 n2 = size/2;
 npow = (long)(log(size)/log(2) + 0.5);

 for(long i=2; i<2*size; i+=2) {
 
  two_j = 2*j;
  if (i < two_j) {   /* perestanovki vhodnogo massiva */
     hlpre = x[two_j - 2];
     hlpim = x[two_j - 1];

     x[two_j - 2] = x[i-2];
     x[two_j - 1] = x[i-1];

     x[i-2] = hlpre;
     x[i-1] = hlpim;
  }
  
  k = n2;
  while (k < j) {
   j -= k;
   k /= 2;
  }
  j += k;
 }

/*-- A vot i samo FFT --*/

if (precision) {

 double * y;
 y = new double[2*size];
 if (!y) { printf("Not enough memory!\n"); exit(1); }
 for (long w=0; w<2*size; ++w) y[w] = x[w];

 for (long l=0; l<npow; ++l) {
 
  l1 = 2<<l;
  l2 = l1/2;
  phasere = cos(M_PI/(double)l2);
  phaseim = -sin(M_PI/(double)l2);
  ampre = 1;
  ampim = 0;

  for (j=0; j<l2; ++j) {
   two_j = 2*j;
   for (long i=two_j; i<2*size; i+=2*l1) {
     long i1 = i + 2*l2;
     hlpre = y[i1] * ampre - y[i1+1] * ampim;
     hlpim = y[i1] * ampim + y[i1+1] * ampre;

     y[i1]   = y[i] - hlpre;
     y[i1+1] = y[i+1] - hlpim;

     y[i]   += hlpre;
     y[i+1] += hlpim;
    }
    
    hlpre = ampre * phasere - ampim * phaseim;
    ampim = ampre * phaseim + ampim * phasere;
    ampre = hlpre;
   }
  }

 for (long w=0; w<2*size; ++w) x[w] = y[w];
 delete(y);
 
} else { /*-- single precision --*/

 for (long l=0; l<npow; ++l) {
 
  l1 = 2<<l;
  l2 = l1/2;
  phasere = cos(M_PI/(double)l2);
  phaseim = -sin(M_PI/(double)l2);
  ampre = 1;
  ampim = 0;

  for (j=0; j<l2; ++j) {
   two_j = 2*j;
   for (long i=two_j; i<2*size; i+=2*l1) {
     long i1 = i + 2*l2;
     hlpre = x[i1] * ampre - x[i1+1] * ampim;
     hlpim = x[i1] * ampim + x[i1+1] * ampre;

     x[i1]   = x[i] - hlpre;
     x[i1+1] = x[i+1] - hlpim;

     x[i]   += hlpre;
     x[i+1] += hlpim;
    }
    
    hlpre = ampre * phasere - ampim * phaseim;
    ampim = ampre * phaseim + ampim * phasere;
    ampre = hlpre;
   }
  }
} // else 

 if (isign < 0) for (long i=1; i<2*size+1; i+=2) x[i] *= -1.;
 
 /* normirovka */
 if (use_scale) for (long i=0; i<2*size; ++i) x[i] *= scale;
}

/* -------- For FFT -------------------*/

template <class T>
void NegHarmTransPos (T * massiv, long N, int sign) {

 double temp0, temp1, temp2, temp3;

 if (sign >= 0) {
                  for (long t=0; t<N/2-1; ++t) {
                   temp0 = massiv[2*t];     massiv[2*t]    = massiv[2*(t+N/2-1)];     massiv[2*(t+N/2-1)]    = temp0; 
                   temp1 = massiv[2*t +1];  massiv[2*t +1] = massiv[2*(t+N/2-1) +1];  massiv[2*(t+N/2-1) +1] = temp1; 
                  }
		  
                   temp0 = massiv[2*N-4];  temp1 = massiv[2*N-4 +1];
		   temp2 = massiv[2*N-2];  temp3 = massiv[2*N-2 +1];
		   
		   for (long t=N-3; t>=N/2-1; --t) {
		    massiv[2*(t+2)]    = massiv[2*t];
		    massiv[2*(t+2) +1] = massiv[2*t +1];
		   }
		   
		   massiv[N-2] = temp0;   massiv[N-2 +1] = temp1;
		   massiv[N]   = temp2;   massiv[N +1]   = temp3;
                  } // if

 if (sign < 0) {
                 for (long t=N/2+1; t<N; ++t) {
                  temp0 = massiv[2*t];     massiv[2*t]    = massiv[2*(t-N/2-1)];     massiv[2*(t-N/2-1)]    = temp0;
                  temp1 = massiv[2*t +1];  massiv[2*t +1] = massiv[2*(t-N/2-1) +1];  massiv[2*(t-N/2-1) +1] = temp1;
                 }
		 
                  temp0 = massiv[N-2];  temp1 = massiv[N-2 +1];					       
                  temp2 = massiv[N];    temp3 = massiv[N +1];
		  
	          for (long t=N/2+1; t<N; ++t) {
		   massiv[2*(t-2)]    = massiv[2*t];
		   massiv[2*(t-2) +1] = massiv[2*t +1];
		  }
		  
		  massiv[2*(N-2)] = temp0;  massiv[2*(N-2) +1] = temp1; 				
		  massiv[2*(N-1)] = temp2;  massiv[2*(N-1) +1] = temp3; 				
                } // if
}

