#define MAIN_
#include "readme.hpp"
#include "fav.hpp"

int HeaderSize = 0, Clipping;
signed char HeaderType = -1, WriteHeader = 1;
long wsize = -1, win = -1, left = 0;
char *OutputPath, *AuxExt, *OutFile, *TypeString, *ChanString;
int **chans, ChannelNumber, FullChanNum, TimeNumber = 1;
bool is_quite = false;

/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to decode words in fast mode (mainly S2-TCI words)\n\n"
	 "[USAGE]: %s [options] <file(s) to process>\n\n"
	 "[OPTIONS]:\n"
	 "-o, --source <filename>       - name of source file, default \"bit.info\"\n"
	 "--create                      - create source file with name and destination pointed\n"
	 "                                before in -o or --source option. Otherwise, source\n"
	 "                                file \"bit.info\" will be created in current directory\n"
	 "-d, --path   <path>           - path to the output file(s), default - the same\n"
	 "                                where are the source files\n"
	 "--ext        <string>         - auxiliary filename extension given to the output file(s),\n"	 
	 "                                default - .cv . You can use one symbol %% in order to have\n"
	 "                                opportunity to point number of channel in filename\n"
	 "--out        <filename>       - name of file for output data. If --ext is given then full\n"
	 "                                name is filename + extension, default - input file + .cv\n"
	 "--header     <value>          - Size of header (in bytes) of processed files, default - -1.\n"
	 "                                If <value> = -1 then size of header will be reading from\n"
	 "                                first 4 bytes as in TCI format and after whole header will\n"
	 "                                be read. In case of non-TCI header it won't be writing to\n"
	 "                                the output file(s), otherwise - writing if it's not --noheader\n"
	 "                                option in command line\n"
	 "--noheader                    - not write TCI format header to the output file(s), default - write\n"
	 "-t, --type   <string>         - set type of decoding procedure. If <string> = vlba then\n"
	 "                                programm uses the corresponding values for VLBA coding, \n"
	 "                                <string> = at   for AT coding. You can use your own coding\n"
	 "                                matrix if <string> = 1,5,7,8 (for example) where 1 - is value\n"
	 "                                if all bits in channel equals to zero and so on. Default - AT coding\n"
	 "-c, --channels <string>       - set what channels must be decoded, channel numbers have to \n"
	 "                                be dividing by commas. You can set <string> = all then all channels\n"
	 "                                from bit-info file will be decoded. Default - all channels\n"
	 "-l, --left   <value>          - set left position of processed window, default - 0.\n"
	 "                                The <value> is number of input words from file beginning\n"
	 "-s, --size   <value>          - set processed window of input file(s), default - whole file.\n"
	 "                                The <value> is number of input words\n"
	 "-w, --wsize  <size of block>  - size of block (in input words) to record in case of large files.\n"
	 "                                If it's pointed then dest file(s) are forming by appending\n"
	 "                                consecutively the blocks of selected size, default - equal to\n"
	 "                                size of file (in input words)\n"
	 "-q, --quite                   - turn on quite output, skip printing the iteration changes\n"
	 "-h, --help                    - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"source", required_argument, 0, 'o'},
				   {"path", required_argument, 0, 'd'},
				   {"size", required_argument, 0, 's'},
				   {"left", required_argument, 0, 'l'},
				   {"wsize", required_argument, 0, 'w'},
				   {"type", required_argument, 0, 't'},
				   {"channels", required_argument, 0, 'c'},
				   {"create", no_argument, 0, 10},
				   {"header", required_argument, 0, 11},
				   {"noheader", no_argument, 0, 12},
				   {"ext", required_argument, 0, 13},
				   {"out", required_argument, 0, 14},
				   {"quite", no_argument, 0, 'q'},
				   {0, 0, 0, 0}
                                  };

  optind = 0; optarg = 0;
  while((op = getopt_long(argc, argv, "ho:d:w:t:c:l:s:q", long_options, 0)) != EOF)
    switch(op){
      case 'o':
	memset(BitFile, 0, MaxString+1);
	sprintf(BitFile, "%s", optarg);
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 'h':
        Help(argv[0]);
      break;
      
      case 'w':
        wsize = atol(optarg);
	if (wsize <= 0) wsize = -1;
      break;
      
      case 'l':
        left = atol(optarg);
	if (left < 0) left = 0;
      break;
      
      case 's':
        win = atol(optarg);
	if (win <= 0) win = -1;
      break;
      
      case 't':
	  if( TypeString ) {
	    TypeString = 0;
	    free(TypeString);
	                 }
	  if( !( TypeString = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(TypeString, optarg, (int)strlen(optarg));
      break;
      
      case 'c':
	  if( ChanString ) {
	    ChanString = 0;
	    free(ChanString);
	                 }
	  if( !( ChanString = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(ChanString, optarg, (int)strlen(optarg));
      break;
      
      case 10:
        CreateBitFile();
        exit(0);
      break;
      
      case 11:
        HeaderSize = atoi(optarg);
	if (HeaderSize < 0) { HeaderType = -1; HeaderSize = 0; }
	 else HeaderType = 0;
      break;
      
      case 12:
        WriteHeader = 0;
      break;
      
      case 13:
	  if( AuxExt ) {
	    AuxExt = 0;
	    free(AuxExt);
	                 }
	  if( !( AuxExt = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(AuxExt, optarg, (int)strlen(optarg));
      break;
      
      case 14:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;

      case 'q':
       is_quite = true;
      break;
      
      case '?':
        Help(argv[0]);
      break;
    }

  return(optind);
 }

/*--- ,       ----*/

char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/*-----  -   S2-TCI  ----*/

void HeaderWrite (char *name, char *ext, int ch, int flag)
{
   int out, infile;
   char *outfilename, *base;
   outfilename = new char[MaxString+1];
   base = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);
   memset(base, 0, MaxString+1);
   
   if (!strcmp(OutputPath, "\0")) { if (strcmp(OutFile, "\0") && !strcmp(ext, ".out")) sprintf(outfilename, "%s", OutFile);
    else { sprintf(base, "%s%s", (!strcmp(OutFile, "\0") ? name : OutFile), ext);
	   sprintf(outfilename, base, ch);	          
          }
    } else {
    if (strcmp(OutFile, "\0") && !strcmp(ext, ".out")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
    else { sprintf(base, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), ext);
	   sprintf(outfilename, base, ch);	                    
          }
    }

   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
         if ( (infile = open(name, O_RDONLY)) == -1 )
            { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	    
            lseek(infile, 0, SEEK_SET);
            lseek(out, 0, SEEK_SET);
	    read(infile, &HeaderSize, sizeof(int));

	     #ifdef _FR_SUN
	      #ifdef _TO_INTELPC
	       HeaderSize = ntohl(HeaderSize);
	      #endif
	     #endif
	  
	     #ifdef _FR_INTELPC
	      #ifdef _TO_SUN
	       HeaderSize = h2nl(HeaderSize);
	      #endif
	     #endif

	  if (WriteHeader == 1) {
	  char src;
	  write(out, &HeaderSize, sizeof(int));
	  for (int w=0;w<HeaderSize-sizeof(int);w++) {
	  read(infile, &src, sizeof(char));
	  write(out, &src, sizeof(char));             }
	                         }
	  
	    close(infile);
            close(out);
        }
	
   delete(outfilename);
   delete(base);
 }

/*-----     ----*/

template <class tFormat>
void Write (tFormat m, long size, int razm, char *name, char *ext, int ch, int flag)
{
   int out;
   char *outfilename, *base;
   outfilename = new char[MaxString+1];
   base = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);
   memset(base, 0, MaxString+1);
   
   if (!strcmp(OutputPath, "\0")) { if (strcmp(OutFile, "\0") && !strcmp(ext, ".out")) sprintf(outfilename, "%s", OutFile);
    else { sprintf(base, "%s%s", (!strcmp(OutFile, "\0") ? name : OutFile), ext);
	   sprintf(outfilename, base, ch);	          
          }
    } else {
    if (strcmp(OutFile, "\0") && !strcmp(ext, ".out")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
    else { sprintf(base, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), ext);
	   sprintf(outfilename, base, ch);	                    
          }
    }

   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
	   write(out, m, size*razm);	                                           
           close(out);
        }
	
   memset(m, 0, razm*size);
   	
   delete(outfilename);
   delete(base);
 }

/*--------    ------------------------*/

template <class tFormat>
void ReadSrc (tFormat m, long l, long size, int razm, char *name)
{
  int file;
  
       memset(m, 0, size*razm);       
       if ( (file = open(name, O_RDONLY)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
        if (lseek(file, HeaderSize + l*razm, SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, m, size*razm);
        close(file);

   for (long q=0;q<size;q++) {
	
  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (razm != 1) m[q] = nh2hn(m[q]);
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (razm != 1) m[q] = nh2hn(m[q]);
   #endif
  #endif
	                      }		
 }

/*---------    ,        -----------------*/

template <class F>
int ChanSearch ( F ch, int *tm, int size, int check )
{
  for (int f=0;f<size;f++) if (ch == tm[f] && f != check) return 1;
  return 0;
}

/*---------         ---------------------*/

int ChannelsDefine ( char * ch )
{
  int num = 0;
  signed int *chtmp;
              chtmp = new signed int[1024];
  char *dest;
        dest = new char[MaxString+1];
        if (!chtmp || !dest) { printf("Not enough memory!\n"); exit(1); }
        memset(chtmp, -107, 1024*sizeof(signed int));
       
       if (!strcasecmp(ch, "all")) { for (int h=0;h<inNumber;h++) if (ChanSearch((sIn+h)->channel, chtmp, num, -1) == 0) { chtmp[num] = (sIn+h)->channel; num++; } 
                                    }
         else {
	        char *p = ch, *p1;
                int n, numer = 0, fl;
                while (*p) { if (*p == ',') numer++;
                             p++;
                            }
	        num = numer + 1;
                p = ch;
                numer = -1;
		
                  while (*p) {  p1 = p; n = 0;                                                                                                                                                      
		                memset(dest, 0, MaxString+1);                                                                                                                                 
		                while(*p1 && *p1 != ',') { if (!isdigit(*p1)) { printf("\nError:\nBad symbols in option --channels ! - %s\n", ch); exit(1); } else { p1++; n++; } }
		                strncat(dest, p, n);
				numer++;
		                chtmp[numer] = (signed int)atoi(dest);
		                p += ((int)strlen(dest) + 1);                                                                                                                                 
		              }                                                                                                                                                               

		 for (int d=0;d<num;d++) if (ChanSearch(chtmp[d], chtmp, num, d) == 1) { printf("\nError:\nThere are the same channels in option --channels ! - %s\n", ch); exit(1); }
		 for (int d=0;d<num;d++) { fl = 0;
		                           for (int h=0;h<inNumber;h++) if (chtmp[d] == (int)(sIn+h)->channel) { fl = 1; break; }
		                           if (fl == 0) { printf("\nError:\nThere is no channel %d in bit-info file !\n", chtmp[d]); exit(1); }
		                          }
	        }

       chans = new int*[num];
       for (int j=0;j<num;j++) chans[j] = new int[2];
       if (!chans) { printf("Not enough memory!\n"); exit(1); }
       for (int j=0;j<num;j++) memset(chans[j], 0, 2*sizeof(int));
       for (int b=0;b<num;b++) { chans[b][0] = chtmp[b];
                                 for (int u=0, k=0;u<Clipping*FullChanNum;u+=Clipping, k++)
				   if (chans[b][0] == (int)(sIn+u)->channel) { chans[b][1] = k; break; }
                                }
       delete(chtmp);
       delete(dest);
       return num;
}

/*----------          ----------------------*/

void MyType ( char * tip )
{
 char *p = tip, *dest, *p1;
 int numCom = 0, n;
 while (*p) { if (*p == ',') numCom++;
              p++;
             }
	     
 mycode = new signed char[numCom+1];
 dest   = new char[MaxString+1];
 if (!mycode || !dest) { printf("Not enough memory!\n"); exit(1); }
 memset(mycode, 0, (numCom+1)*sizeof(signed char));
 
 p = tip;
 numCom = -1;
  while (*p) {  p1 = p; n = 0;
                memset(dest, 0, MaxString+1);
                while(*p1 && *p1 != ',') { if (!isdigit(*p1) && *p1 != '-') { printf("\nError:\nBad symbols in option --type ! - %s\n", tip); exit(1); } else { p1++; n++; } }
                strncat(dest, p, n);
		if (atoi(dest) > 128 || atoi(dest) < -127) { printf("\nError:\nThe value in option --type is out of range! - %s\n", tip); exit(1); }
		numCom++;
		mycode[numCom] = (signed char)atoi(dest);
		p += ((int)strlen(dest) + 1);
              }

 delete(dest);
}

/*--------       ----------------*/

int ClippingDefine ( void )
{
 int cl = 0, fill[2] = { 0, 0 };
 char trafaret[2] = { 's', 'a' };

 for (int h=0;h<inNumber;h++)
  for (int f=0;f<2;f++)
   if ((sIn+h)->bit == trafaret[f] && fill[f] == 0) { fill[f] = 1; cl++; }

  return cl;
}

/*---------     ----------------*/

unsigned long recode ( TYPE i, TYPE *maskin, unsigned long *maskval, int Channel )
{
  unsigned long val, znachenie = 0;
  
  for (int t=0;t<TimeNumber;t++) { val = 0;
   for (int h=0;h<Clipping;h++) { if (i & maskin[(sIn+(Clipping*(Channel + t*FullChanNum)+h))->place])
                                    val = val | maskval[8*sizeof(unsigned long) - Clipping + h];
                                 }
      znachenie += accord[val]*accord[val] ;				 
   }   
  return (znachenie);
}

/*---------     ----------------*/

void MatrixFilling ( TYPE *maskin, unsigned long *maskval, unsigned long **matrix )
{
  for (int v=0;v<ChannelNumber;v++) for (unsigned long q=0;q<(1<<(8*sizeof(TYPE)));q++)
   matrix[v][q] = recode((TYPE)q, maskin, maskval, chans[v][1]);
}

/*============================ M A I N ============================================*/

int main ( int argc, char *argv[] )
{
 struct stat info;
 long size;
 int next, shift;
 unsigned long *maskval;
 TYPE *maskin, *indata;
 float **data;
 unsigned long **matrix;

   if (argc == 1) Help(argv[0]); 

   BitFile = new char[MaxString+1];
   if (!BitFile) { printf("Not enough memory!\n"); exit(1); }
   memset(BitFile, 0, MaxString+1);
   sprintf(BitFile, "%s", "bit.info");
   
   AuxExt = new char[MaxString+1];
   OutFile = new char[MaxString+1];
   OutputPath = new char[MaxString+1];
   TypeString = new char[MaxString+1];
   ChanString = new char[MaxString+1];
   if (!AuxExt || !OutFile || !OutputPath || !TypeString || !ChanString) { printf("Not enough memory!\n"); exit(1); }
   memset(AuxExt, 0, MaxString+1);
   memset(OutFile, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   memset(TypeString, 0, MaxString+1);
   memset(ChanString, 0, MaxString+1);
   sprintf(AuxExt, "%s", ".cv");
   sprintf(OutFile, "%s", "\0");
   sprintf(OutputPath, "%s", "\0");
   sprintf(TypeString, "%s", "at");
   sprintf(ChanString, "%s", "all");

   next = ParseCmdLine(argc, argv);
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }

   BitPlacementRead();                                //    

   Clipping = ClippingDefine();                       //    
   FullChanNum = ChannelsDefine("all");
   ChannelNumber = ChannelsDefine(ChanString);
   TimeNumber = (int)ceil((float)inNumber/(float)(Clipping*FullChanNum));                   

     maskin = new TYPE[entryNumber];
     if (!maskin) { printf("Not enough memory!\n"); exit(1); }
     for (int i=entryNumber-1;i>=0;i--) maskin[i] = 1<<(entryNumber-1-i);

 if ( stat(argv[next], &info) != 0 ) { printf("\nError:\nCouldn't open \"%s\"!\n", argv[next]); exit(1); }

 if (HeaderType == -1) for (int v=0;v<ChannelNumber;v++) HeaderWrite(argv[next], AuxExt, chans[v][0], 1);
 size = (info.st_size - HeaderSize)/sizeof(TYPE);
 if (size <= 0) { printf("\nError:\nSize of header is too much!\n"); exit(1); }

 if (win == -1 || win > size) win = size;
 if (left >= size) left = 0;
 if (left + win > size) win -= (left + win - size);
 
 wsize = (wsize == -1 ? win : (wsize >= win ? win : wsize));

 indata = new TYPE[wsize];
 if (!indata) { printf("Not enough memory!\n"); exit(1); }

 data = new float *[ChannelNumber];
 matrix = new unsigned long *[ChannelNumber];
 for (int v=0;v<ChannelNumber;v++) { data[v] = new float[wsize]; matrix[v] = new unsigned long[1<<(8*sizeof(TYPE))]; }
 accord = new signed char[1<<Clipping];
 maskval = new unsigned long[8*sizeof(unsigned long)];
 if (!data || !accord || !maskval) { printf("Not enough memory!\n"); exit(1); }
 
 for (int v=0;v<ChannelNumber;v++) { memset(data[v], 0, wsize*sizeof(float)); memset(matrix[v], 0, (1<<(8*sizeof(TYPE)))*sizeof(unsigned long)); }
 memset(accord, 0, (1<<Clipping)*sizeof(signed char));
 if (strcasecmp(TypeString, "vlba") && strcasecmp(TypeString, "at")) MyType(TypeString);
 for (int g=0;g<(1<<Clipping);g++) accord[g] = (!strcasecmp(TypeString, "vlba") ? (!(vlba[g]) ? 0 : vlba[g]) : (!strcasecmp(TypeString, "at") ? (!(at[g]) ? 0 : at[g]) : (!(mycode[g]) ? 0 : mycode[g]) ));
 for (int g=8*sizeof(unsigned long)-1;g>=0;g--) maskval[g] = 1<<(8*sizeof(unsigned long)-1-g);

 MatrixFilling (maskin, maskval, matrix);     //     

printf("\nProcessing...\n");

for (int z=next;z<argc;z++) {

 printf("  [%3d] %s ... ", z-next, argv[z]); fflush(NULL);

for (long t=0;t<(win%wsize == 0 ? win/wsize : win/wsize + 1);t++) {

  if (!is_quite) { printf(".%5d", t+1); fflush(NULL); }

  if (HeaderType == -1 && t == 0) for (int v=0;v<ChannelNumber;v++) HeaderWrite(argv[z], AuxExt, chans[v][0], 1);
  ReadSrc (indata, left + t*wsize, (t*wsize + wsize > win ? win - t*wsize : wsize), sizeof(TYPE), argv[z]);

  for (int v=0;v<ChannelNumber;v++) {  
   for (long q=0;q<(t*wsize + wsize > win ? win - t*wsize : wsize);q++)
     data[v][q] = (float)(matrix[v][indata[q]])/(float)TimeNumber;
   Write (data[v], (t*wsize + wsize > win ? win - t*wsize : wsize), sizeof(float), argv[z], AuxExt, chans[v][0], 0);   
  }
    if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); }
                                                                   }
     printf(" - done\n");
                             }

  delete(indata);
  delete(maskin);
  delete [] data;
  delete [] matrix;
  delete(sIn);
  delete(accord);
  delete(maskval);
  delete(AuxExt);
  delete(OutFile);
  delete(OutputPath);
  delete(BitFile);
  if (strcasecmp(TypeString, "vlba") && strcasecmp(TypeString, "at")) delete(mycode);
  delete(TypeString);
  delete(ChanString);
  delete [] chans;
  
  return 0;
}

