#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <math.h>
#include <time.h>

#define MaxString 255

unsigned long size = 1;
bool is_binary = false;
char *outname;  // name of the output file
bool is_two_bit = false;
float *data;   // data array (floats)
signed char *data2;  // data array (signed chars)

/* return random Gauss value */
double gauss_random () {
 long rn1, rn2;
 double yj, zj, pj , xi;

 do { rn1 = random(); rn2 = random();
      yj = 2.*(((double)rn1)/((double)(RAND_MAX+1.0))) - 1.;
      zj = 2.*(((double)rn2)/((double)(RAND_MAX+1.0))) - 1.;
      pj = yj*yj + zj*zj;
     } while ( pj >= 1. );

 return ( yj * sqrt ((-2.*log(pj))/pj) );
}

/*==========================Prompting===================================*/

void Help (char *s) {
  printf("\nPROGRAMM %s\n"
         "Return random Gauss distributed values\n\n"
	 "[USAGE]: %s [options]\n\n"
	 "[OPTIONS]:\n"
	 "-o, --out  <file>    - name of the output file, by default file is ascii\n"
	 "-s, --size <size>    - number of generated values, default - 1\n"
	 "-b, --binary         - the output file is binary, float or signed char\n"
	 "-2, --twobit         - the output random values will be 2-bit samples signed chars\n"
	 "-h, --help           - help\n\n", s, s);
  exit(0);
}

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
                                   {"out", required_argument, 0, 'o'},
                                   {"size", required_argument, 0, 's'},
				   {"binary", no_argument, 0, 'b'},
				   {"twobit", no_argument, 0, '2'},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "h2bs:o:", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 'o':
	  if (outname) { outname = 0; free (outname); }
	  if ( !( outname = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) ) { perror("calloc outname"); exit(1); }
	  memcpy (outname, optarg, (int)strlen(optarg));
      break;
            
      case 's':
	  size = atol(optarg);
      break;

      case 'b':
	  is_binary = true;
      break;
      
      case '2':
       is_two_bit = true;
      break;
            
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
}

/*--- m a i n ---*/  
int main (int argc, char *argv[]) {

 float temp;
 signed char temp2;
 int out;
 FILE *fout;

 outname = new char[MaxString+1];
 if (!outname) { perror("new outname"); exit(1); }
 memset(outname, 0, MaxString+1);
 sprintf(outname, "%s", "\0");
   
 ParseCmdLine(argc, argv);
 if (size < 1) size = 1;

 if (!strcmp(outname, "\0")) out = 1; else out = open (outname, O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644);
 if (out == -1) { perror("open outname"); exit(1); }

 srandom((unsigned long)time(NULL));
   
 // if is_binary = true, then will use array
 if (is_binary) { // binary

  if (is_two_bit) { //signed char
    data2 = new signed char[size];
    if (!data2) { perror ("new data2"); exit (1); }
    memset (data2, 0, size * sizeof(signed char));
  } else { // float 
    data = new float[size];
    if (!data) { perror ("new data"); exit (1); }
    memset (data, 0, size * sizeof(float));
  }
  for (unsigned long i=0; i<size; ++i) {
   temp = (float)gauss_random ();
   if (is_two_bit) {
    // we compare with 1 or -1 because Sigma = 1 (the generated distribution is for Mean = 0, Sigma = 1)
    if (temp >= 0) { if (temp >= 1) data2[i] = 3; else data2[i] = 1; }
     else { if (temp <= -1) data2[i] = -3; else data2[i] = -1; }
   } else data[i] = temp;
  }
  lseek (out, 0, SEEK_SET);
  if (is_two_bit) write (out, data2, size * sizeof(signed char)); else write (out, data, size*sizeof(float));
  close (out);

 } else { // ascii

  fout = fdopen (out, "wt");
  fseek (fout, 0, SEEK_SET);
  for (unsigned long i=0; i<size; ++i) {
   temp = (float)gauss_random ();
   if (is_two_bit) {
    if (temp >= 0) { if (temp >= 1) temp2 = 3; else temp2 = 1; }
     else { if (temp <= -1) temp2 = -3; else temp2 = -1; }
    fprintf (fout, "%d\n", temp2);
   } else fprintf (fout, "%f\n", temp);
  }

  fclose (fout);
 }

 delete (outname); 
 if (is_binary) {
  if (is_two_bit) delete (data2); else delete (data);
 }

 return 0;
}

