#define MAIN_
#include "readme.hpp"
#include "knife.hpp"   

unsigned char ReadAnSig = 0, Link = 0, NoSingle = 1, NoEnd = 0;
off_t shift = 0, bl = 0, window = -1, left, step, Counts = -1, sz;
off_t Begin = 0, Number = -1;
off_t szc;
off_t Size = -1, filesize;
double razmer, lc;
double Period = -1., TimeResolution = -1.;
int Header = 0;
char *OutFile, *Mask, *OutputPath, *ProfFile;
TYPE *data;
float *prof;
off_t schetchik = 0;
bool is_quite = false;

/*-------     (!) HOST  NETWORK   (!) 
            -------------------------------------------*/

TYPE nh2hn ( TYPE data, int razm1 )
{
  char tmp;
  const int razm = sizeof(TYPE);
  union { TYPE d;
          unsigned char uc[razm];
         } b;
  b.d = data;
  for (int i=0;i<razm/2;i++) { tmp = b.uc[i];
                               b.uc[i] = b.uc[razm-1-i];
			       b.uc[razm-1-i] = tmp;
                              }
  return b.d;           
 } 

   
/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to cut long record to the individual smaller records\n"
	 "with the period of pulsar\n\n"
	 "[USAGE]: %s [options] <file(s) to process>\n\n"
	 "[OPTIONS]:\n"
	 "-c, --counts  <value>       - period in counts\n"
	 "-p, --period  <value>       - period of pulsar in ns. If option -c is given then\n"
	 "                              period in counts will be used\n"
	 "-t, --timeres <value>       - time resolution in ns. Work only together with\n"
	 "                              -p option. In case of absence -p or -t AND -c\n"
	 "                              error will be registrated\n"
	 "--yessingle                 - write files with single pulses, default - not write\n"
	 "--noend                     - don't write file with not full period (at the end of\n"
	 "                              file), default - write (with name \"remainder\")\n"
	 "--number      <value>       - number of periods to write, default - all\n"
	 "-f, --begin   <value>       - number of first period to write, default - 0\n"
	 "-s, --shift   <shift>       - shift from the file beginning, default - 0\n"
	 "-l, --left    <left>        - left boundary of chosen window in points relative to\n"
	 "                              the shift, default - 0\n"
	 "-w, --window  <window>      - size of chosen window in points, default - from left\n"
	 "                              boundary to the end of period\n"
	 "-b, --size    <size>        - size in counts of the used sample, default - equal to size of\n"
	 "                              the file (in counts)\n"
	 "--link                      - to link periods of next input file(s) with previous,\n"
	 "                              default - no\n"
	 "--as                        - input file is the analytical signal, i.e. with the\n"
	 "                              interchange of real and imaginary parts\n"
	 "-d, --path    <path>        - path to the output file(s), default - the same\n"
	 "                              where are the source files\n"
	 "--prof        <profname>    - name of the average profile with destination directory\n"
	 "                              default - where are the source files with name = input file + .fld\n"
	 "--mask        <mask>        - mask for the extension of output file(s),\n"
	 "                              default - .r%%05ld  If it will be other '%%' symbols\n"
	 "                              except that for the output numbers in type of int,\n"
	 "                              long etc. (%%d for example) then the output file\n"
	 "                              extension will be non-predicted\n"
	 "--out         <filename>    - the base name of file for output data. The output\n"
	 "                              filename is base name + mask\n"
	 "                              default - input file + .r%%05ld (see --mask)\n"
	 "--header      <size>        - header in bytes of input file(s), default - 0\n"
	 "-q, --quite                 - turn on quite output, skip printing the iteration changes\n"
	 "-h, --help                  - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"shift", required_argument, 0, 's'},
				   {"counts", required_argument, 0, 'c'},
				   {"period", required_argument, 0, 'p'},
				   {"timeres", required_argument, 0, 't'},
				   {"left", required_argument, 0, 'l'},
				   {"window", required_argument, 0, 'w'},
				   {"begin", required_argument, 0, 'f'},
				   {"path", required_argument, 0, 'd'},
                                   {"as", no_argument, 0, 20},
				   {"header", required_argument, 0, 21},
				   {"out", required_argument, 0, 22},
				   {"mask", required_argument, 0, 23},
				   {"number", required_argument, 0, 24},
                                   {"link", no_argument, 0, 25},
                                   {"yessingle", no_argument, 0, 26},
                                   {"noend", no_argument, 0, 27},
				   {"quite", no_argument, 0, 'q'},
				   {"size", required_argument, 0, 'b'},
				   {"prof", required_argument, 0, 28},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "hp:t:c:s:l:w:f:d:qb:", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 's':
        shift = (off_t)atof(optarg);
	if (shift < 0) shift = 0;
      break;
            
      case 'c':
        Counts = (off_t)atof(optarg);
	if (Counts < 1) Counts = -1;
      break;
      
      case 'p':
        Period = atof(optarg);
	if (Period <= 0.) Period = -1.;
      break;
      
      case 't':
        TimeResolution = atof(optarg);
	if (TimeResolution <= 0.) TimeResolution = -1.;
      break;
      
      case 'l':
        bl = (off_t)atof(optarg);
	if (bl < 0) bl = 0;
      break;
      
      case 'w':
        window = (off_t)atof(optarg);
	if (window < 1) window = -1;
      break;
      
      case 'f':
        Begin = (off_t)atof(optarg);
	if (Begin < 0) Begin = 0;
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 20:
       ReadAnSig = 1;
      break;
      
      case 21:
       Header = atoi(optarg);
       if (Header < 0) Header = 0;
      break;
      
      case 22:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
      
      case 23:
	  if( Mask ) {
	    Mask = 0;
	    free(Mask);
	                 }
	  if( !( Mask = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(Mask, optarg, (int)strlen(optarg));
      break;
            
      case 24:
        Number = (off_t)atof(optarg);
	if (Number < 1) Number = -1;
      break;
      
      case 25:
        Link = 1;
      break;
      
      case 26:
        NoSingle = 0;
      break;
      
      case 27:
        NoEnd = 1;
      break;

      case 'q':
       is_quite = true;
      break;

      case 'b':
        Size = (off_t)atof(optarg);
	if (Size < 1) Size = -1;
      break;

      case 28:
        if (ProfFile) {
          ProfFile = 0;
          free(ProfFile);
        }
        if (!( ProfFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
          { printf("Not enough memory!\n"), exit(1); }
        memcpy (ProfFile, optarg, (int)strlen(optarg));
      break;
            
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
 }

/*--- ,       ----*/

char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/*-----     ----*/

template <class tFormat>
void Write (tFormat m, off_t size, int razm, char *name, off_t number, int flag)
{
   int out;
   char *outfilename, *base;
   outfilename = new char[MaxString+1];
          base = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);
   memset(base, 0, MaxString+1);

   if (!strcmp(OutputPath, "\0")) {
     sprintf(base, "%s%s", "%s", Mask);
     sprintf(outfilename, base, (!strcmp(OutFile, "\0") ? name : OutFile), number);
    } else {
             sprintf(base, "%s%s%s", "%s/", "%s", Mask);
             sprintf(outfilename, base, OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), number);
            }
	    
    if (flag == 3) {
     char *newname = new char[MaxString + 1];
     memset (newname, 0, MaxString +1);
     if (!strcmp(OutputPath, "\0")) sprintf (newname, "%s", "remainder");
      else sprintf (newname, "%s/%s", OutputPath, "remainder");
      
     rename (outfilename, newname);
     delete (newname);
     delete (outfilename);
     delete (base);
     return;
    }

    if (flag == 0) remove(outfilename);
     else
  {
      
   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
	   write(out, m, size*razm);	                                           
           close(out);
        }
  }
      
   memset(m, 0, razm*size);
   	
   delete(outfilename);
   delete(base);   
 }

void ReadSrc (off_t l, off_t size, char *name)
{
  int file;

     memset(data, 0, size*sizeof(TYPE));
       
       if ( (file = open(name, O_RDONLY | O_LARGEFILE)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
       if (lseek(file, (off_t)Header + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, data, size*sizeof(TYPE));
        close(file);

   for (long q=0;q<size;q++) {
	
  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
		              }		
 }
     
/*====================== Main Function ===================================*/
  
int main ( int argc, char *argv[] )
 {
   struct stat info;
   int next; 
   off_t countup;

   if (argc == 1) Help(argv[0]);
   
   OutFile     = new char[MaxString+1];
   Mask        = new char[MaxString+1];
   OutputPath  = new char[MaxString+1];
   ProfFile    = new char[MaxString+1];
   if (!OutFile || !Mask || !OutputPath || !ProfFile) { printf("Not enough memory!\n"); exit(1); }
   memset(OutFile, 0, MaxString+1);
   memset(Mask, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   memset(ProfFile, 0, MaxString+1);
   sprintf(OutFile, "%s", "\0");
   sprintf(Mask, "%s", ".r%05ld");
   sprintf(OutputPath, "%s", "\0");
   sprintf(ProfFile, "%s", "\0");
   
   next = ParseCmdLine(argc, argv);   
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }

   if (!strcmp(ProfFile, "\0")) {
    memset (ProfFile, 0, MaxString+1);
    sprintf (ProfFile, "%s.fld", argv[next]);
   }

   if ( stat(argv[next], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[next]); exit(1); }

   filesize = (info.st_size - Header)/((ReadAnSig == 0 ? 1 : 2) * sizeof(TYPE));
   if (filesize <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }
   if (Size == -1 || Size > filesize) Size = filesize;
   if (shift >= Size) shift = 0;
   if (bl >= Size || shift + bl >= Size) bl = 0;
   
   step = ( Counts == -1 ? ( Period == -1. || TimeResolution == -1. ? (off_t)-10 : (off_t)rint(Period/TimeResolution) ) : Counts );
   if ( step == (off_t)-10 ) { printf("\nError:\nYou need to point period in counts (--counts) or\npoint both period and time resolution (--period , --timeres)\n"); exit(1); }

//   if (step > Size) { printf("\nError:\nPulsar period is more then size of file !\n"); exit(1); }
   
   left = shift + bl;
   if (window > step) { window = step; printf("\n- warning ! - chosen window is more then pulsar period!\nWindow will be equaled to period\n"); }
   if (window == -1) sz = step; else sz = window;   
   if (left + sz > Size && Link == 0) printf("\n- warning ! - processed window is more then size between end of file\nand left position! Save only available counts.\n");

    data = new TYPE[(ReadAnSig == 0 ? sz : 2*sz)];
    if (!data) { printf("Not enough memory!\n"); exit(1); }

    prof = new float[(ReadAnSig == 0 ? sz : 2*sz)];
    if (!prof) { printf("Not enough memory!\n"); exit(1); }
    memset (prof, 0, (ReadAnSig == 0 ? sz : 2*sz)*sizeof(float));

printf("\nProcessing...\n");

 for (int i=next;i<argc;i++) {

 printf(" [%3d]\t%s ... ", i-next, argv[i]);  fflush(NULL);

 countup = (Link == 0 ? 0 : (i == next ? 0 : countup) );
 razmer = (Link == 0 ? 0 : (i == next ? 0. : razmer + Size) );
 lc = (Link == 0 ? left : (i == next ? left : lc - razmer) );
 szc = (Link == 0 ? sz : (i == next ? sz : sz - szc) );

 if ( stat(argv[i], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[i]); exit(1); }
     
 Size = (info.st_size - Header)/((ReadAnSig == 0 ? 1 : 2) * sizeof(TYPE));
 if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }

   if (szc < sz ) {
     if (szc > Size) szc = Size;
     if (countup-1 >= Begin && (Number == -1 ? 1 : (countup-1 < Begin + Number)) ) {
     if (!is_quite) { printf(" + "); fflush(NULL); }
     ReadSrc(0, (ReadAnSig == 0 ? szc : 2*szc), argv[i]);   
     if (NoEnd == 1 && szc == Size) ; else {
      schetchik++;
      for (off_t m=0; m<(ReadAnSig == 0 ? szc : 2*szc); ++m) prof[m] += (float)data[m];
     }
     if (NoSingle == 1) ;
       else {
             if (NoEnd == 1 && szc == Size) Write (data, szc, sizeof(TYPE), argv[i-1], countup-1, 0);
	       else Write (data, szc, sizeof(TYPE), argv[i-1], countup-1, 2);
             }
     }
     szc = sz;
   }

  if (i != argc-1 && (lc >= Size/* || step > Size*/) ) { printf("\nError:\nPulsar period is more then size of current file!\n"); exit(1); }
   
 while (lc < Size) {
   
   if (Number != -1) if (countup >= Begin + Number) { countup++; break; }
   
   if (lc + szc > Size) szc -= ((off_t)rint(lc) - Size) + szc;  

  if (countup >= Begin) {
    if (!is_quite) { 
     printf(".%5d", countup+1); 
     fflush(NULL);
     if (szc < sz) printf(" + ");
    }
    ReadSrc((ReadAnSig == 0 ? (off_t)rint(lc) : 2*(off_t)rint(lc)), (ReadAnSig == 0 ? szc : 2*szc), argv[i]);
    if (NoEnd == 1 && Link == 0 && szc < sz) ; else {
     schetchik++;
     for (off_t m=0; m<(ReadAnSig == 0 ? szc : 2*szc); ++m) prof[m] += (float)data[m];
    }
    if (NoSingle == 1 || (NoEnd == 1 && Link == 0 && szc < sz)) ; 
     else Write (data, szc, sizeof(TYPE), argv[i], countup, 1);
  }      
    ++countup;
    lc = ( Counts == -1 ? (double)left + rint((countup*Period)/TimeResolution) : ((double)left) + (double)(countup*step));
    if (lc < razmer + Size) { lc -= razmer; if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); } }
     else { if (!is_quite) fflush(NULL); }
                       }

  if (countup-1 >= Begin && (Number == -1 ? 1 : (countup-1 < Begin + Number))) printf("  - done -\n");
    else printf("  - skip -\n");
	
 } //for     	     		         										   

  if (NoEnd == 1 && Link == 1 && szc < sz && NoSingle == 0) Write (data, szc, sizeof(TYPE), argv[argc-1], countup-1, 0);
  // rename file for "remainder"
  if (NoEnd == 0 && szc < sz && NoSingle == 0) Write (data, szc, sizeof(TYPE), argv[argc-1], countup-1, 3);
  for (off_t m=0; m<(ReadAnSig == 0 ? sz : 2*sz); ++m) prof[m] /= schetchik;

  /* writing the profile */
  int outdescr;
  if ( (outdescr = open(ProfFile, O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", ProfFile);
        exit (1);
      }
    else {
     lseek(outdescr, 0, SEEK_SET);
     write(outdescr, prof, (ReadAnSig == 0 ? sz : 2*sz)*sizeof(float));
     close(outdescr);
    }

  delete (OutFile);
  delete (Mask);
  delete (OutputPath);
  delete (ProfFile);
  delete (data);
  delete (prof);

  return 0;
}

