/* these preprocessor' defines need to support Large Files */

# ifdef LARGEFILE
#   define _GNU_SOURCE 1
#   define _FILE_OFFSET_BITS 64
# endif

#define MaxString 255

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>

/*+++++++++++++++++++ erase path part from filename +++++++++++++++++*/

char *SelectFileName (char *str) {
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
}
 
/*++++++++++++++++++++ main +++++++++++++++++++++++++++++++++++++*/

int main ( int argc, char *argv[] )
{
 struct stat info1, info2;
 int fdinput1, fdinput2, fdoutput;
 long int bufsize = 10000000;
 char *outname;
 float *in1, *in2, *out;
 long int bytesread;
 int i, j;

 outname = (char *)calloc(MaxString, sizeof(char));
 if (!outname) {
  perror("calloc outname: ");
  exit(1);
 }
 memset(outname, 0, MaxString);
 sprintf(outname, "%s+%s", argv[1], SelectFileName(argv[2]));

 if (argc == 1) {
  printf("Usage: argv[0] <file1> <file2>\n\t<file1> - real part of analytical signal\n\t<file2> - imagine part of analytical signal\n");
  exit(0);
 }
 if (argc < 3) {
  printf("Error: you have to select two input files!\n");
  exit(1);
 }

  /* open file input */   
#ifdef LARGEFILE
  if((fdinput1 = open(argv[1], O_RDONLY | O_LARGEFILE)) < 0 ) { perror("open input file"); exit(1); }
  if((fdinput2 = open(argv[2], O_RDONLY | O_LARGEFILE)) < 0 ) { perror("open input file"); exit(1); }
#else
  if((fdinput1 = open(argv[1], O_RDONLY)) < 0 ) { perror("open input file"); exit(1); }
  if((fdinput2 = open(argv[2], O_RDONLY)) < 0 ) { perror("open input file"); exit(1); }
#endif

  /* open file output */ 
#ifdef LARGEFILE
  if((fdoutput = open(outname, O_WRONLY | O_CREAT | O_TRUNC | O_LARGEFILE, 0644)) < 0 ) { 
   perror("open output file: ");
   exit(1);
  }
#else
  if((fdoutput = open(outname, O_WRONLY | O_CREAT | O_TRUNC, 0644)) < 0 ) {
   perror("open output file: ");
   exit(1);
  }
#endif

  /* check file size */
  if (stat (argv[1], &info1) < 0) {
   perror("input file status: ");
   exit(1);
  }
  if (stat (argv[2], &info2) < 0) {
   perror("input file status: ");
   exit(1);
  }
  if (info1.st_size != info2.st_size) {
   printf("Error: size of files are not equal!\n");
   exit(1);
  }
  if (info1.st_size % 4 != 0) {
   printf("Warning: file size %d is not a multiple of 4\n", info1.st_size);
  }

 /* allocate storage */
 in1 = (float *)calloc(bufsize, sizeof(float));
 in2 = (float *)calloc(bufsize, sizeof(float));
 out = (float *)calloc(2*bufsize, sizeof(float));
 if (!in1 || !in2 || !out) {
  perror("calloc in1, in2, out: ");
  exit(1);
 }

  /* infinite loop */ 
  while (1)
    {
      /* read first buffer */
      bytesread = read(fdinput1, in1, bufsize);
      /* check for end of file */
      if (bytesread == 0) break;
      /* read second buffer */
      bytesread = read(fdinput2, in2, bufsize);
      /* check for end of file */
      if (bytesread == 0) break;
      /* handle small buffers */
      if (bytesread != bufsize*sizeof(float)) {
       bufsize = bytesread/sizeof(float);
      }

      for (i=0,j=0; i<bufsize; ++i, j+=2) {
       out[j]   = in1[i];
       out[j+1] = in2[i];
      }

      /* write data to output file */
       if (2 * bufsize != write(fdoutput, out, 2*bufsize*sizeof(float))) {
        perror("write: ");
        exit(1);
       }

    } /* end of infinite loop */   

close(fdinput1);
close(fdinput2);
close(fdoutput);

free(outname);
free(in1);
free(in2);
free(out);
		    
return 0;
}

