#define MAIN_
#include "readme.hpp"
#include "modulate.hpp"

float *mass, *mod;
unsigned char ModSource = 0;
int Header = 0;
long wsize = -1;
char *OutFile, *OutputPath, *ModFile, *Delimiter, *AuxExt;
bool is_quite = false;

/*-------     (!) HOST  NETWORK   (!) 
            -------------------------------------------*/

TYPE nh2hn ( TYPE data, int razm1 )
{
  char tmp;
  const int razm = sizeof(TYPE);
  union { TYPE d;
          unsigned char uc[razm];
         } b;
  b.d = data;
  for (int i=0;i<razm/2;i++) { tmp = b.uc[i];
                               b.uc[i] = b.uc[razm-1-i];
			       b.uc[razm-1-i] = tmp;
                              }
  return b.d;           
 }

/*---      ---*/

float NoiseLevel ( float * x, long sz )
 {   
  float Sum = 0.;
    for (long i=0;i<sz;i++) Sum += x[i];
    return Sum/sz;
  }

/*-----      -----------*/

void LevelEliminate ( float * x, float level, long sz )
{
    for (long i=0;i<sz;i++) x[i] -= level; 
}

/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to modulate one of the processed file by another\n\n"
	 "[USAGE]: %s [options] <file(s) for modulating> [<delimiter> <file(s) as modulated>]\n\n"
	 "[OPTIONS]:\n"
	 "--delimit   <string>          - set delimiter string between file(s) for modulating and\n"
	 "                                file(s) as modulated. The number of files for modulating have\n"
	 "                                to equal the number of files as modulated. If delimiter\n"
	 "                                doesn't set then option -m must be set, otherwise - error\n"
	 "                                default - :\n"
	 "-m, --mod   <filename>        - name of file as modulated, the only file for all files\n"
	 "                                for modulating. If delimiter and file(s) as modulated is set\n"
	 "                                then this option is ignored\n"
	 "-d, --path  <path>            - path to the output difference file(s), default - current directory\n"
	 "-w, --wsize <value>           - size of block to record in case of large files.\n"
	 "                                If it's pointed then dest file(s) are forming by\n"
	 "                                appending consecutively the blocks of selected size;\n"
	 "                                default - equal to size of file (in points)\n"
	 "--ext       <string>          - auxiliary filename extension given to the output\n"	 
	 "                                file(s), default - .mod\n"
	 "--out       <filename>        - name of file for output data. If --ext is given then full name is \n"
	 "                                filename + extension, default - input file + .mod\n"
	 "--header    <size>            - header in bytes of input file(s), default = 0\n"
	 "-q, --quite                   - turn on quite output, skip printing the iteration changes\n"
	 "-h, --help                    - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"mod", required_argument, 0, 'm'},
				   {"path", required_argument, 0, 'd'},
				   {"wsize", required_argument, 0, 'w'},
				   {"delimit", required_argument, 0, 10},
				   {"out", required_argument, 0, 11},
				   {"ext", required_argument, 0, 12},
				   {"header", required_argument, 0, 13},
				   {"quite", no_argument, 0, 'q'},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "hd:m:w:q", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 'm':
          ModSource = 1;
	  if( ModFile ) {
	    ModFile = 0;
	    free(ModFile);
	                 }
	  if( !( ModFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(ModFile, optarg, (int)strlen(optarg));
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 'w':
        wsize = atol(optarg);
	if (wsize <= 0) wsize = -1;
      break;
      
      case 10:
	  if( Delimiter ) {
	    Delimiter = 0;
	    free(Delimiter);
	                 }
	  if( !( Delimiter = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(Delimiter, optarg, (int)strlen(optarg));
      break;

      case 11:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
      
      case 12:
	  if( AuxExt ) {
	    AuxExt = 0;
	    free(AuxExt);
	                 }
	  if( !( AuxExt = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(AuxExt, optarg, (int)strlen(optarg));
      break;
      
      case 13:
       Header = atoi(optarg);
       if (Header < 0) Header = 0;
      break;

      case 'q':
       is_quite = true;
      break;
      
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
 }

/*--- ,       ----*/

char *SelectFileName ( char * str )
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }
 
/*-----     ----*/

template <class tFormat>
void Write (tFormat m, long size, int razm, char *name, char *ext, int flag, int wri)
{
   int out;
   char *outfilename;
   outfilename = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);

   if (!strcmp(OutputPath, "\0")) { if (strcmp(OutFile, "\0") && !strcmp(ext, ".diff")) sprintf(outfilename, "%s", OutFile);
    else sprintf(outfilename, "%s%s", (!strcmp(OutFile, "\0") ? name : OutFile), ext);
    } else {
    if (strcmp(OutFile, "\0") && !strcmp(ext, ".out")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
    else sprintf(outfilename, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), ext);
    }

   if (wri == 1) { printf(" writing... [%s]", outfilename); fflush(NULL); }
   
   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
	   write(out, m, size*razm);	                                           
           close(out);
        }
	
   memset(m, 0, razm*size);
   	
   delete(outfilename);
 }

/*-----------      -----------------------*/

void ReadSrc (float *x, long l, long size, char *name)
{
  int file;
  TYPE *data;

     data = new TYPE[size];
     if (!data) { printf("Not enough memory!\n"); exit(1); }

     memset(data, 0, size*sizeof(TYPE));
     memset(x, 0, size*sizeof(float));
       
       if ( (file = open(name, O_RDONLY)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
       if (lseek(file, Header + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, data, size*sizeof(TYPE));
        close(file);

   for (long q=0;q<size;q++) {
	
  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
             x[q] = (float)(data[q]);	     
	                                  }		
    delete(data);	
 }
 
/*--------         ----------------*/

int SplitterSearch ( int argc, char *argv[], int next, char *Delimiter )
{
  for(int i=next;i<argc;i++) if (!strcmp(argv[i], Delimiter)) return i;
  return -1;
}
     
/*====================== Main Function ===================================*/
  
int main ( int argc, char *argv[] )
 {
   struct stat info;
   int next, spl;
   long Size, iter, le, win;
   float Level;

   if (argc == 1) Help(argv[0]);
   
   OutFile    = new char[MaxString+1];
   AuxExt = new char[MaxString+1];
   OutputPath = new char[MaxString+1];
   Delimiter  = new char[MaxString+1];
   if (!OutFile || !OutputPath || !Delimiter || !AuxExt) { printf("Not enough memory!\n"); exit(1); }
   memset(OutFile, 0, MaxString+1);
   memset(AuxExt, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   memset(Delimiter, 0, MaxString+1);
   sprintf(OutFile, "%s", "\0");
   sprintf(AuxExt, "%s", ".mod");
   sprintf(OutputPath, "%s", "\0");
   sprintf(Delimiter, "%s", ":");

   next = ParseCmdLine(argc, argv);
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }

   spl = SplitterSearch(argc, argv, next, Delimiter);
   if (spl == next) { printf("\nError:\nWrong using of delimiter!\nYou must point file(s) for modulating and/or file(s) as modulated\n"); exit(1); }
   if (spl == -1) spl = argc; else { if (spl - next != argc - 1 - spl) { printf("\nError:\nNumber of files for modulating doesn't equal\nto number of files as modulated !\n"); exit(1); }
                                     ModSource = 2;
                                    }

   if ( stat(argv[next], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[next]); exit(1); }
   
   Size = (info.st_size - Header)/sizeof(TYPE);
   if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }

   if (ModSource != 0) {
   if ( stat((ModSource == 1 ? ModFile : argv[spl+1]), &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", (ModSource == 1 ? ModFile : argv[spl+1])); exit(1); }
     
   long s = (info.st_size - Header)/sizeof(TYPE);
   if (s <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }
   if (s !=  Size) { printf("\nError:\nSize of file(s) for modulating doesn't equal to size of file(s) as modulated !\n"); exit(1); }
   } else { printf("\nError:\nYou must point out the file(s) as modulated !\n"); exit(1); }
					   
   if (wsize == -1 || wsize > Size) wsize = Size;   
   iter = (Size%wsize == 0 ? Size/wsize : Size/wsize + 1);
   
   mass = new float[wsize];
   mod = new float[wsize];
   if (!mass || !mod) { printf("Not enough memory!\n"); exit(1); }

printf("\nProcessing...\n");

    for (int i=next;i<spl;i++)
     {

 printf(" [%3d]\t%s ... ", i-next, argv[i]);
 fflush(NULL);

 for (long t=0; t<iter; t++) {
        
    le = wsize*t;
    win = (le + wsize > Size ? Size - le : wsize);
    ReadSrc(mass, le, win, argv[i]);
    
    if (t == 0) { printf("%s ... ", (ModSource == 1 ? ModFile : argv[spl+1+i-next]) ); fflush(NULL); }
    if (!is_quite) { printf(".%5d", t+1); fflush(NULL); }
    
    ReadSrc(mod, le, win, (ModSource == 1 ? ModFile : argv[spl+1+i-next]) );

    if (t == 0) Level = NoiseLevel(mass, win);
    LevelEliminate(mass, Level, win);
    for (long h=0;h<win;h++) mass[h] *= mod[h];
    
    if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); }
    
    Write (mass, win, sizeof(float), argv[i], AuxExt, t+1, iter-t);
 }
    
 printf(" - done\n");
	
       }

  delete(mass);
  delete(mod);
  delete(AuxExt);
  delete(OutFile);
  delete(OutputPath);
  if (ModSource == 1) free(ModFile);
  delete(Delimiter);

  return 0;
 }

