#define MAIN_
#include "readme.hpp"
#include "ndzap.hpp"   

TYPE * mass;
off_t sz = 0, left = 0, Num = 1000, wsize = 0;
int Header = 0;
char *AuxExt, *OutFile, *OutputPath;
bool is_quite = false;
bool use_zeros = false;


/*-- this function gives value of inverse error function -------*/
double inverf (double );

/* return random Gauss value */
double gauss_random () {
 long rn1, rn2;
 double yj, zj, pj , xi;

 do { rn1 = random(); rn2 = random();
      yj = 2.*(((double)rn1)/((double)(RAND_MAX+1.0))) - 1.;
      zj = 2.*(((double)rn2)/((double)(RAND_MAX+1.0))) - 1.;
      pj = yj*yj + zj*zj;
     } while ( pj >= 1. );

 return ( yj * sqrt ((-2.*log(pj))/pj) );
}

/* return 2-bit sampled random Gauss value */
signed char gauss_random_2bit (double s) {
 double value = gauss_random();
 if (value >= 0) { if (value >= s) return 3; else return 1; }
  else { if (value <= -s) return -3; else return -1; }
}

/*-------     (!) HOST  NETWORK   (!) 
            -------------------------------------------*/

TYPE nh2hn ( TYPE data, int razm1 )
{
  char tmp;
  const int razm = sizeof(TYPE);
  union { TYPE d;
          unsigned char uc[razm];
         } b;
  b.d = data;
  for (int i=0;i<razm/2;i++) { tmp = b.uc[i];
                               b.uc[i] = b.uc[razm-1-i];
			       b.uc[razm-1-i] = tmp;
                              }
  return b.d;           
 }

/*==========================Prompting===================================*/

void Help (char *s) {
  printf("\nPROGRAMM %s\n"
         "It is used to zap noise-diode signal from 2-bit sampled data\n\n"
         "[USAGE]: %s [options] <file(s) to process>\n\n"
         "[OPTIONS]:\n"
         "-d, --path  <path>           - path to the output file(s), default - the same\n"
         "                               where are the source files\n"
         "--ext       <string>         - auxiliary filename extension given to the output\n"
         "                               file(s), default - .ndz\n"
         "--out       <filename>       - name of file for output data. If --ext is given\n"
         "                               then full name is filename + extension\n"
         "                               default - input file + .ndz\n"
         "--header    <size>           - header in bytes of input file(s), default = 0\n"
         "-l, --left  <left boundary>  - left boundary of used window, default = 0\n"
         "-s, --size  <size of window> - size of used window, default - equal to size of file\n"
         "-n, --num   <value>          - number of continuous +/-1 samples to be recognized as\n"
         "                               noise-diode signal, default - 1000\n"
         "-w, --wsize <value>          - size of block to record in case of large files.\n"
         "                               If it's pointed then dest file(s) are forming by\n"
         "                               appending consecutively the blocks of selected size;\n"
         "                               default - equal to size of file (in points)\n"
	 "--zero                       - substitute noise-diode with zeros rather than by gaussian noise\n"
         "-q, --quite                  - turn on quite output, skip printing the iteration changes\n"
         "-h, --help                   - help\n\n", s, s);
 exit(0);
}

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
   int op;
   struct option long_options[] = { {"help", no_argument, 0, 'h'},
                                    {"left", required_argument, 0, 'l'},
                                    {"size", required_argument, 0, 's'},
                                    {"num", required_argument, 0, 'n'},
                                    {"path", required_argument, 0, 'd'},
                                    {"wsize", required_argument, 0, 'w'},
                                    {"ext", required_argument, 0, 10},
                                    {"out", required_argument, 0, 11},
                                    {"header", required_argument, 0, 12},
                                    {"quite", no_argument, 0, 'q'},
                                    {"zero", no_argument, 0, 13},
                                    {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "hl:s:d:n:w:q", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 'l':
        left = atol(optarg);
	if (left < 0) left = 0;
      break;
      
      case 's':
        sz = atol(optarg);
	if (sz <= 0) sz = -1;
      break;
      
      case 'n':
        Num = atol(optarg);
	if (Num < 1) Num = 1000;
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 'w':
        wsize = atol(optarg);
	if (wsize <= 0) wsize = -1;
      break;
      
      case 10:
	  if( AuxExt ) {
	    AuxExt = 0;
	    free(AuxExt);
	                 }
	  if( !( AuxExt = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(AuxExt, optarg, (int)strlen(optarg));
      break;
      
      case 11:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
      
      case 12:
       Header = atoi(optarg);
       if (Header < 0) Header = 0;
      break;
      
      case 'q':
       is_quite = true;
      break;

      case 13:
       use_zeros = true;
      break;
      
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
}
    
/*--- ,       ----*/

char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/*-----     ----*/

template <class tFormat>
void Write (tFormat m, off_t size, int razm, char *name, char *ext, int flag, int wri)
{
   int out;
   char *outfilename;
   outfilename = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);

   if (!strcmp(OutputPath, "\0")) { if (strcmp(OutFile, "\0") && !strcmp(ext, ".ndz")) sprintf(outfilename, "%s", OutFile);
    else sprintf(outfilename, "%s%s", (!strcmp(OutFile, "\0") ? name : OutFile), ext);
    } else {
    if (strcmp(OutFile, "\0") && !strcmp(ext, ".ndz")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
    else sprintf(outfilename, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), ext);
    }

   if (wri == 1) { printf(" writing... [%s]", outfilename); fflush(NULL); }
   
   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
	   write(out, m, size*razm);	                                           
           close(out);
        }
	
   memset(m, 0, razm*size);
   	
   delete(outfilename);
 }

/*------------      ----------------------*/

void ReadSrc (off_t l, off_t size, char *name)
{
  int file;

     memset(mass, 0, size*sizeof(TYPE));
       
       if ( (file = open(name, O_RDONLY | O_LARGEFILE)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
       if (lseek(file, (off_t)Header + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, mass, size*sizeof(TYPE));
        close(file);


  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    for (off_t q=0; q<size; ++q) if (sizeof(TYPE) != 1) mass[q] = nh2hn(mass[q], sizeof(TYPE));
   #endif
  #endif

  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    for (off_t q=0; q<size; ++q) if (sizeof(TYPE) != 1) mass[q] = nh2hn(mass[q], sizeof(TYPE));
   #endif
  #endif

}

/*--- m a i n ---*/
int main (int argc, char *argv[]) {
   struct stat info;
   int next;
   off_t Size, iter, le, win;
   off_t prev, last; // previous and last +/-3 values
   double sigma = 1.;
   off_t n_summed = 0, one_summed = 0;
   bool trigger = false;

   if (argc == 1) Help(argv[0]);
   
   AuxExt     = new char[MaxString+1];
   OutFile    = new char[MaxString+1];
   OutputPath = new char[MaxString+1];
   if (!AuxExt || !OutFile || !OutputPath) { printf("Not enough memory!\n"); exit(1); }
   memset(AuxExt, 0, MaxString+1);
   memset(OutFile, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   sprintf(AuxExt, "%s", ".ndz");
   sprintf(OutFile, "%s", "\0");
   sprintf(OutputPath, "%s", "\0");
   
   next = ParseCmdLine(argc, argv);
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }

   srandom((unsigned long)time(NULL));
   
   if ( stat(argv[next], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[next]); exit(1); }

   Size = (info.st_size - Header)/sizeof(TYPE);
   if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }
   if (sz == 0 || sz > Size) sz = Size;
   if (left >= Size) left = 0;
   if (left + sz > Size) sz = Size - left;

   if (wsize == 0 || wsize > sz) wsize = sz;
   if (Num > wsize) Num = wsize;

   iter = (wsize == sz ? 1 : (sz%wsize == 0 ? sz/wsize : sz/wsize + 1));

   mass = new TYPE[wsize];   
   if (!mass) { printf("Not enough memory!\n"); exit(1); }

 printf("\nProcessing...\n");
 for (long i=next;i<argc;i++) {

 printf(" [%3d]\t%s ... ", i-next, argv[i]); fflush(NULL);
 for (off_t t=0; t<iter; t++) {

  if (!is_quite) { printf(".%5d", t+1); fflush(NULL); }

  le  = (t*wsize >= sz ? left + sz - wsize : left + t*wsize);
  win = (le + wsize > sz ? sz - le : wsize);
  ReadSrc (le, win, argv[i]);

  // checking for previous and next +/-3 values
  prev = 0;
  last = 0;
  for (off_t j=0; j<win; ++j) {
   if (mass[j] == -3 || mass[j] == 3) { trigger = true; last = j; }
   if (last - prev < Num) { 
     if (trigger && !use_zeros) {
      for (off_t w=prev; w<last; ++w) {
       n_summed++;
       if (mass[w] == 1 || mass[w] == -1) { one_summed++; }
      }
      trigger = false;
     }
    prev = last; 
   } else { if (!use_zeros) { 
              sigma = sqrt((double)2.) * inverf ( ((double)one_summed)/((double)n_summed) );
              for (off_t k=prev+1; k<last; ++k) mass[k] = gauss_random_2bit(sigma); 
	      n_summed = 0;
	      one_summed = 0;
	    } else { for (off_t k=prev+1; k<last; ++k) mass[k] = 0; }
           prev = last;
    }
  }
    
  if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); }
    
  Write (mass, win, sizeof(TYPE), argv[i], AuxExt, t+1, iter-t);
 }
 printf(" - done\n");
}
       
delete(mass);
delete(AuxExt);
delete(OutFile);
delete(OutputPath);

return 0;
}
