#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

/*-- this function gives value of inverse error function -------*/
double inverf (double);

/*--------- get recovered value ----------------*/
double get_recovered_value (double part, bool unit=true) {

 // part - percent of "1"-bits
 // unit - true if value = 1, false if value = 3

 double x = inverf(part); // determine "x" (argument of erf function) = t/(sigma * sqrt(2))
 double ts = x*sqrt((double)2.); // t/sigma
 double st = ((double)1.)/ts; // sigma/t
 static double pi2 = sqrt((double)2./(double)M_PI); // sqrt(2/pi)

 double x2 = ((double)-1.)*x*x; // t^2/(2*sigma^2)

/* Power optimized levels */
// double y = 2. * (unit ? st*sqrt( (double)1. - ts*pi2*(exp(x2)/part) ) : st*sqrt( (double)1. + ts*pi2*(exp(x2)/((double)1. - part)) ) );

/* Voltage optimized levels */
 double y = 2. * (unit ? st*pi2*(((double)1.-exp(x2)) / part) : st*pi2*exp(x2)/((double)1. - part) );

 return y;
}  

/*------------ m a i n -----------------*/  
int main (int argc, char *argv[]) {

  printf ("recovered values\n-----------------------\n");

  // loop on "1"-bits percents
  for (double i=0.; i<1.; i=i+0.01) {
   printf ("%.2f   %.3lf   %.3lf\n", i*100., get_recovered_value (i, true), get_recovered_value (i, false));	
  }
	
  return 0;
}
