#define MAIN_
#include "readme.hpp"
#include "spinw.hpp"   

float *mass;
char *AuxExt, *OutFile, *OutputPath;

/*---- ,   (size)  ,
                    2 ------------------------*/

long resize (long sz)
 {
   long n = 0;
   sz += 1;
   while ( (1<<(++n)) < sz) ;
   return (1<<(--n));
  }

/*---      ---*/

float NoiseLevel ( float * Massive, long left, long sz )
 {   
  float Sum = 0.;
    for (long i=left;i<left+sz;i++) Sum += Massive[i];
    return Sum/sz;
  }

/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to eliminate level in chosen spectrum window\n"
	 "Some options repeat the fieldnames of source file, in case when\n"
	 "some value is pointed in both source file and command line, value\n"
	 "from command line is used\n\n"
	 "[USAGE]: %s [options] <file(s) with power spectra>\n\n"
	 "[OPTIONS]:\n"
	 "-o, --source <filename>       - name of source file, default \"winsp.set\"\n"
	 "                                If filename = no then source file won't be used and\n"
	 "                                all parameters are inputed from command line\n"
	 "--create                      - create source file with name and destination pointed\n"
	 "                                before in -o or --source option. Otherwise, source\n"
	 "                                file \"winsp.set\" will be created in current directory\n"
	 "-l, --left   <value>          - set left boundary of spectrum window, default = 0\n"
	 "-w, --size   <value>          - set size of spectrum window, default = size of file(s)\n"
	 "                                with power spectra\n"
	 "-u, --used   <value>          - set size of file which was used to determine left\n"
	 "                                boundary and size of spectrum window, default = the\n"
	 "                                same that processed file(s) if there is no information\n"
	 "                                about this in source file\n"
	 "-d, --path   <path>           - path to the output file(s), default - the same\n"
	 "                                where are the source file(s)\n"
	 "--ext        <string>         - auxiliary filename extension given to the output\n"	 
	 "                                file(s), default - .nolevel\n"
	 "--out        <filename>       - name of file for output data. If --ext is given\n"
	 "                                then full name is filename + extension\n"
	 "                                default - input file + .nolevel\n"
	 "--header     <value>          - size of header (in bytes) of file(s) with power spectra\n"
	 "                                default - 0\n"
	 "-h, --help                    - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op, tm;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"source", required_argument, 0, 'o'},
				   {"left", required_argument, 0, 'l'},
				   {"size", required_argument, 0, 'w'},
				   {"used", required_argument, 0, 'u'},
				   {"path", required_argument, 0, 'd'},
				   {"create", no_argument, 0, 10},
				   {"header", required_argument, 0, 11},
				   {"ext", required_argument, 0, 12},
				   {"out", required_argument, 0, 13},
				   {0, 0, 0, 0}
                                  };

  optind = 0; optarg = 0;
  while((op = getopt_long(argc, argv, "ho:l:w:u:d:", long_options, 0)) != EOF)
    switch(op){
      case 'o':
	memset(ResourceFileName, 0, MaxString+1);
	sprintf(ResourceFileName, "%s", optarg);
      break;
      
      case 'h':
        Help(argv[0]);
      break;
      
      case 'l':
        sRes.SpWindowLeft = atol(optarg);
	if (sRes.SpWindowLeft < 0) sRes.SpWindowLeft = 0;
      break;
      
      case 'w':
        sRes.SpWindowSize = atol(optarg);
	if (sRes.SpWindowSize < 1) sRes.SpWindowSize = -1;
      break;
      
      case 'u':
        sRes.UsedFileSize = atol(optarg);
	if (sRes.UsedFileSize < 1) sRes.UsedFileSize = -1;
      break;
      
      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;
      
      case 10:
        CreateSourceFile();
        exit(0);
      break;
      
      case 11:
        sRes.HeaderSize = atoi(optarg);
	if (sRes.HeaderSize < 0) sRes.HeaderSize = 0;
      break;
      
      case 12:
	  if( AuxExt ) {
	    AuxExt = 0;
	    free(AuxExt);
	                 }
	  if( !( AuxExt = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(AuxExt, optarg, (int)strlen(optarg));
      break;
      
      case 13:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
      
      case '?':
        Help(argv[0]);
      break;
    }

  return(optind);
 }
 
/*--- ,       ----*/

char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/*-----     ----*/

template <class tFormat>
void Write (tFormat m, long size, int razm, char *name, char *ext, int flag)
{
   int out;
   char *outfilename;
   outfilename = new char[MaxString+1];

   memset(outfilename, 0, MaxString+1);

   if (!strcmp(OutputPath, "\0")) { if (strcmp(OutFile, "\0") && !strcmp(ext, ".nolevel")) sprintf(outfilename, "%s", OutFile);
    else sprintf(outfilename, "%s%s", (!strcmp(OutFile, "\0") ? name : OutFile), ext);
    } else {
    if (strcmp(OutFile, "\0") && !strcmp(ext, ".nolevel")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
    else sprintf(outfilename, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), ext);
    }

   printf("writing... [%s] - ", outfilename);
   fflush(NULL);
   
   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
	   write(out, m, size*razm);	                                           
           close(out);
        }
	
   delete(outfilename);
 }

void ReadSrc (long l, long size, char *name)
{
  int file;
  TYPE *data;

     data = new TYPE[size];
     if (!data) { printf("Not enough memory!\n"); exit(1); }

     memset(data, 0, size*sizeof(TYPE));
     memset(mass, 0, size*sizeof(float));
       
       if ( (file = open(name, O_RDONLY)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
        if (lseek(file, sRes.HeaderSize + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, data, size*sizeof(TYPE));
        close(file);

   for (long q=0;q<size;q++) {
	
  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
             mass[q] = (float)data[q];
	     
	                                  }		
    delete(data);	
 }
    
/*====================== Main Function ===================================*/
  
int main ( int argc, char *argv[] )
 {	    
   struct stat info;
   long Size, left, sz;
   int next;
   float ratio, level;

   if (argc == 1) Help(argv[0]); 

   AuxExt = new char[MaxString+1];
   OutFile = new char[MaxString+1];
   OutputPath = new char[MaxString+1];
   if (!AuxExt || !OutFile || !OutputPath) { printf("Not enough memory!\n"); exit(1); }
   memset(AuxExt, 0, MaxString+1);
   memset(OutFile, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   sprintf(AuxExt, "%s", ".nolevel");
   sprintf(OutFile, "%s", "\0");
   sprintf(OutputPath, "%s", "\0");

   ResourceFileName = new char[MaxString+1];
   if (!ResourceFileName) { printf("Not enough memory!\n"); exit(1); }
   memset(ResourceFileName, 0, MaxString+1);
   sprintf(ResourceFileName, "%s", "winsp.set");

   ParseCmdLine(argc, argv);

   if (strcmp(ResourceFileName, "no")) ReadFromRes();   
   
   next = ParseCmdLine(argc, argv);
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) with power spectra!\n"); exit(1); }

   if ( stat(argv[next], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[next]); exit(1); }

  Size = (info.st_size - sRes.HeaderSize)/sizeof(TYPE);
  if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }
  
  if (2*(Size - 1) != resize(2*(Size - 1))) { printf("\nError:\nYour file(s) with power spectra doesn't contain N/2 + 1 positive harmonics!\n"); exit(1); }

  if (sRes.UsedFileSize == -1) sRes.UsedFileSize = Size;
  
  if (2*(sRes.UsedFileSize - 1) != resize(2*(sRes.UsedFileSize - 1))) { printf("\nError:\nSize of file used to determine spectrum window\ndoesn't equal to N/2 + 1 positive harmonics!\n"); exit(1); }

  ratio = ((float)(Size - 1))/((float)(sRes.UsedFileSize - 1));

  if (sRes.SpWindowLeft >= sRes.UsedFileSize) sRes.SpWindowLeft = 0; 
  if (sRes.SpWindowSize == -1 || sRes.SpWindowSize > sRes.UsedFileSize) sRes.SpWindowSize = sRes.UsedFileSize;
  if (sRes.SpWindowLeft + sRes.SpWindowSize > sRes.UsedFileSize) sRes.SpWindowSize -= (sRes.SpWindowLeft + sRes.SpWindowSize - sRes.UsedFileSize);

  left = (long)(ratio*sRes.SpWindowLeft);
  sz   = (long)(ratio*sRes.SpWindowSize);

  if (left >= Size) left = 0; 
  if (sz > Size) sz = Size;
  if (left + sz > Size) sz -= (left + sz - Size);
  
  mass = new float[Size];
  if (!mass) { printf("Not enough memory!\n"); exit(1); }

 printf("\nProcessing...\n");

    for (int i=next;i<argc;i++)
     {

 printf(" [%3d]\t%s ... ", i-next, argv[i]);
 fflush(NULL);

    ReadSrc(0, Size, argv[i]);   
    level = NoiseLevel(mass, left, sz);
    for (long t=0;t<Size;t++) mass[t] = (t<left || t>=left+sz ? 0. : mass[t] - level);
    Write (mass, Size, sizeof(float), argv[i], AuxExt, 1);

 printf("done\n");
	
       }
       
  delete(mass);
  delete(ResourceFileName);
  delete(AuxExt);
  delete(OutFile);
  delete(OutputPath);

  return 0;
 }

