#define UTIL_
#include "readme.hpp"
#include "spinw.hpp"

/*---------------------------------------------------------------------------
                  
-----------------------------------------------------------------------------*/

enum{
  CHAR,
  UNSIGNED,
  INT,
  LONG,
  FLOAT,
  ALLOCSTR,
  DOUBLE,
  NEWSECTION = 64};   /*----- NEWSECTION    ,  
                                   -------------*/

struct sResStrucEntry {
                    const char *fieldname;
                    char vartype;
                    void *field;
		                  };

/*----    sResStrucEntry ,
         -- const ( - cfg),    (void *field)
             sRes; 
          ,     --------------*/

static const struct sResStrucEntry cfg[] = {
 { "TimeResolution",              DOUBLE,                &sRes.TimeResolution },      

 { "SpWindowLeft",                LONG + NEWSECTION,     &sRes.SpWindowLeft },      
 { "SpWindowSize",                LONG,                  &sRes.SpWindowSize },      
 { "UsedFileSize",                LONG,                  &sRes.UsedFileSize },      
 { "BandSmooth",                  LONG,                  &sRes.BandSmooth },      
 
 { "HeaderSize",                  INT,                   &sRes.HeaderSize }
                                             };					    

/*----     -----*/

void CreateSourceFile ( void )		     
 {
  struct stat inf; 
  char *p;

  if ( stat(ResourceFileName, &inf) == 0 ) {
      printf("\nWarning!\nSource file \"%s\" already exists!\n", ResourceFileName); exit(1); }
	
  FILE * resource;
  if ( (resource = fopen(ResourceFileName, "wt")) == NULL ) {
	printf("\nError:\nCouldn't open source file \"%s\" for writing!\n", ResourceFileName);
	exit(1);
	                                                    }
                   fseek(resource, 0, SEEK_SET);
		    
  char curline[MaxString+1];
  int cfgLength = sizeof(cfg)/sizeof(struct sResStrucEntry);
  struct sResStrucEntry *tmp = (struct sResStrucEntry *)cfg;

  fprintf(resource, "#\n# PROGRAMM spinw  Source File\n#   You can read some information by:  spinw --help\n#\n\n");
	       
  do{							    
     if(tmp->vartype & NEWSECTION) fprintf(resource, "\n"); 

	switch(tmp->vartype & ~NEWSECTION){
	  case CHAR :
		sprintf(curline, "%-33s=   %d\n", tmp->fieldname, *(char *)tmp->field);
	  break;
	  
	  case UNSIGNED :
		sprintf(curline, "%-33s=   %u\n", tmp->fieldname, *(unsigned *)tmp->field);
	  break;
	  
	  case INT :
		sprintf(curline, "%-33s=   %d\n", tmp->fieldname, *(int *)tmp->field);
	  break;
	  
	  case LONG :
		sprintf(curline, "%-33s=   %ld\n", tmp->fieldname, *(long *)tmp->field);
	  break;
	  
	  case FLOAT :
		sprintf(curline, "%-33s=   %g\n", tmp->fieldname, *(float *)tmp->field);
	  break;
	  
	  case DOUBLE :
		sprintf(curline, "%-33s=   %g\n", tmp->fieldname, *(double *)tmp->field);
	  break;
	  
	  case ALLOCSTR :
		sprintf(curline, "%-33s=   %s\n", tmp->fieldname, *(char **)tmp->field);
	  break;
	                                   }
        fputs(curline, resource); 
	++tmp;
       } while(--cfgLength);
       
     fclose(resource);     
 }

/*-----------------      ---------------------------

      ,    (#), 
   -- .   ,    ,
      (".source.set"),    
    ("source.set").       
   ,        , 
     (     ),    
        !!!!
-----------------------------------------------------------------------------*/

void Rec2Res ( void )		     
 {
  int err = 0, newl;
  char *p;
  char *tmpname;
        tmpname = new char[MaxString+1];
	if (!tmpname) printf("Not enough memory!\n"), exit(1);

	memset(tmpname, 0, MaxString+1);
	strcat(tmpname, ".");
	strcat(tmpname, ResourceFileName);
	
  FILE * resourceread;
         if ( (resourceread = fopen(ResourceFileName, "r+t")) == NULL ) { CreateSourceFile();
	                                                                  return;
	                                                                 }
                        fseek(resourceread, 0, SEEK_SET);
  FILE * resource;
         if ( (resource = fopen(tmpname, "w+t")) == NULL ) {
	    printf("\nError:\nCouldn't open file \"%s\" for writing!\n", tmpname);
	    exit(1);
	                                                    }
                    fseek(resource, 0, SEEK_SET);
  char curline[MaxString+1];
  char curlineread[MaxString+1];
  int cfgLength = sizeof(cfg)/sizeof(struct sResStrucEntry);
  struct sResStrucEntry *tmp = (struct sResStrucEntry *)cfg;

  while (1)  {
               if ( (fgets(curlineread, MaxString+1, resourceread)) != NULL ) {  
	            p = curlineread;
	            while(*p && isascii(*p) && isspace(*p)) ++p;
	                                                                        } else err = 1;
               if (*p == '#') break;
	       if (*p == '\0') continue;
	       if ((*p != '#' && *p != '\0') || err == 1) {
	        fprintf(resource, "#\n# PROGRAMM spinw  Source File\n#   You can read some information by:  spinw --help\n#\n\n");
		break;
	                                                   }
              } 
	
       fseek(resourceread, 0, SEEK_SET);
       
  do{
     newl = 0;
     while ( fgets(curlineread, MaxString+1, resourceread) ) {  
	     p = curlineread;
	     while(*p && isascii(*p) && isspace(*p)) ++p;
	     newl = (*p == '\0' ? 1 : (*p == '#' ? 0 : newl));
	     if(*p == '#' || *p == '\0') fputs(curlineread, resource); else break;
                                                              }
							    
     if(tmp->vartype & NEWSECTION) { if (newl == 0) fprintf(resource, "\n"); }

	switch(tmp->vartype & ~NEWSECTION){
	  case CHAR :
		sprintf(curline, "%-33s=   %d\n", tmp->fieldname, *(char *)tmp->field);
	  break;
	  
	  case UNSIGNED :
		sprintf(curline, "%-33s=   %u\n", tmp->fieldname, *(unsigned *)tmp->field);
	  break;
	  
	  case INT :
		sprintf(curline, "%-33s=   %d\n", tmp->fieldname, *(int *)tmp->field);
	  break;
	  
	  case LONG :
		sprintf(curline, "%-33s=   %ld\n", tmp->fieldname, *(long *)tmp->field);
	  break;
	  
	  case FLOAT :
		sprintf(curline, "%-33s=   %g\n", tmp->fieldname, *(float *)tmp->field);
	  break;
	  
	  case DOUBLE :
		sprintf(curline, "%-33s=   %g\n", tmp->fieldname, *(double *)tmp->field);
	  break;
	  
	  case ALLOCSTR :
		sprintf(curline, "%-33s=   %s\n", tmp->fieldname, *(char **)tmp->field);
	  break;
	                                   }

	 fputs(curline, resource); 
	++tmp;
       } while(--cfgLength);
     fclose(resource);
     fclose(resourceread);
     remove(ResourceFileName);
     rename(tmpname, ResourceFileName);
     delete(tmpname);
 }

/*---------       -----------*/

void CfgItemRead( char *p, struct sResStrucEntry *tmp )
 {
  int l; 
 
  switch(tmp->vartype & ~NEWSECTION){
    case CHAR: *(char *)tmp->field = (char)atoi(p); break;

    case UNSIGNED: *(unsigned *)tmp->field = (unsigned)atoi(p); break;
    
    case INT: *(int *)tmp->field = atoi(p); break;
    
    case LONG: *(long *)tmp->field = atol(p); break;

    case FLOAT: *(float *)tmp->field = (float)atof(p); break;
    
    case DOUBLE: *(double*)tmp->field = atof(p); break;

    case ALLOCSTR :
          l = (int)strlen(p);
	  if( *(char **)tmp->field ) {
	    *(char **)tmp->field = 0;
	    free(*(char **)tmp->field);
	                               }
	  if( !( *(char **)tmp->field = (char *)calloc(l, sizeof(char *)) ) ) { printf("Not enough memory!\n"), exit(1); }
	  memcpy(*(char **)tmp->field, p, l);
	break;}
 }

/*-------------        
        .  
     ,     . , 
   
----------------------------------------------------------------------------------*/

char *ItemFit( char *p, const char *cfgvar )
 {
  int l = (int)strlen(cfgvar);
  if ( !strncasecmp(p, cfgvar, l) )
   {
    p += l;
    while(*p && isascii(*p) && isspace(*p)) p++;
    if (*p != '=') return 0;
    p++;
    while(*p && isascii(*p) && isspace(*p)) p++;
    return p;
   }
  return 0;
 }

/*------------     ----------------*/

void ProcessLine( char *p, struct sResStrucEntry * tmp, int cfgLength )
 {
  char *p1;
  while( cfgLength-- ) {
	  p1 = ItemFit(p, tmp->fieldname);
	  if (p1) {
            CfgItemRead(p1, tmp);
            return;
                   }
          tmp++;
	                }
    printf("Error:\nBad line in resource file \"%s\" !\n", ResourceFileName);
    exit(1);
 }

/*----------------     ------------*/
 
void ReadFromRes( void )
 {
  FILE * resource;
         if ( (resource = fopen(ResourceFileName, "r+t")) == NULL ) {
	    printf("\nError:\nCouldn't open source file \"%s\" !\n", ResourceFileName);
	    exit(1);
	                                                              }
                    fseek(resource, 0, SEEK_SET);
  char *p, curline[MaxString+1];
  int cfgLength = sizeof(cfg)/sizeof(struct sResStrucEntry);
  struct sResStrucEntry * tmp = (struct sResStrucEntry *)cfg;

  while( fgets(curline, MaxString+1, resource)  ) {
	p = curline;
	curline[(int)strlen(curline) -1] = 0;
	while(*p && isascii(*p) && isspace(*p)) ++p;
	if(!*p || *p == '#') continue;
	ProcessLine(p, tmp, cfgLength);
	                                               }
      fclose(resource);						       
 }


/*-------------------------------------------------------------------------------------

                           
	    
---------------------------------------------------------------------------------------*/






/*---------------------------------------------------------------

                

-----------------------------------------------------------------*/




/*----      HOST (  i80x86 ,
    Least Significant Byte first -     )  
     NETWORK (  Internet,  SUN SPARC; Most Significant Byte first -
        )     unsigned long. 
     ,       (htonl, htons)  
     SUN-       .    
    NETWORK  HOST  i80x86  (ntohl, ntohs),     
      -------*/

unsigned long h2nl( unsigned long hostlong )
{
  union{ unsigned long ul;
         unsigned char uc[4];
        } b;
  b.ul = hostlong;
  char tmp = b.uc[0]; b.uc[0] = b.uc[3]; b.uc[3] = tmp;
  tmp = b.uc[1]; b.uc[1] = b.uc[2]; b.uc[2] = tmp; 
 return b.ul;
}


/*----      
       unsigned short ------*/

unsigned short h2ns( unsigned short hostshort )
{
  union{ unsigned short us;
         unsigned char uc[2];
        } b;
  b.us = hostshort;
  char tmp = b.uc[0]; b.uc[0] = b.uc[1]; b.uc[1] = tmp;
 return b.us;
}

/*-------     (!) HOST  NETWORK   (!) 
            -------------------------------------------*/

TYPE nh2hn ( TYPE data, int razm1 )
{
  char tmp;
  const int razm = sizeof(TYPE);
  union { TYPE d;
          unsigned char uc[razm];
         } b;
  b.d = data;
  for (int i=0;i<razm/2;i++) { tmp = b.uc[i];
                               b.uc[i] = b.uc[razm-1-i];
			       b.uc[razm-1-i] = tmp;
                              }
  return b.d;           
 }





/*-------------------------------------------------------------------

                     

---------------------------------------------------------------------*/

