/*-----------------------
prg <   > <Q> <U>
------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <getopt.h>
#include <math.h>

#define MaxString 255

long bufsize = 2000000;
char *TemplateFilename;

/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to calculate degree of linear polarization\n\n"
	 "[USAGE]: %s [options] <Q> <U>\n\n"
	 "[OPTIONS]:\n"
	 "-o, --out    <template>       - filename template of output files, default - out.\n"
	 "-b, --buffer <size>           - specifies the size of read buffer in points, default = 2,000,000\n"
	 "                                it must not be < 1 and > 50,000,000\n"
	 "-h, --help                    - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op, tm;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
                                   {"out", required_argument, 0, 'o'},
                                   {"buffer", required_argument, 0, 'b'},
				   {0, 0, 0, 0}
                                  };

  optind = 0; optarg = 0;
  while((op = getopt_long(argc, argv, "ho:b:", long_options, 0)) != EOF)
    switch(op){
      case 'o':
	  if( TemplateFilename ) {
	    TemplateFilename = 0;
	    free(TemplateFilename);
	                          }
	  if( !( TemplateFilename = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(TemplateFilename, optarg, (int)strlen(optarg));
      break;
      
      case 'b':
       bufsize = atol(optarg);
       if (bufsize < 1 || bufsize > 50000000) {
        printf("Error: bufsize %ld is very large!\n", bufsize);
	exit(1);
       }
      break;
      
      case 'h':
        Help(argv[0]);
      break;

      case '?':
        Help(argv[0]);
      break;
    }

  return(optind);
 }

int main ( int argc, char *argv[] )
{
 struct stat info, *info1 = NULL;
 int next;
 const long Nf = 2, Of = 1;
 long bytesread;
 int inf[Nf], outf[Of];
 char symb[Of] = { 'P' };

 if (argc == 1) Help(argv[0]);

 TemplateFilename = new char[MaxString+1];
 if (!TemplateFilename) { printf("Not enough memory!\n"); exit(1); }
 memset(TemplateFilename, 0, MaxString+1);
 sprintf(TemplateFilename, "%s", "out.");

 next = ParseCmdLine(argc, argv);
 if (argc < next + Nf) { printf("You need point %d input files!\n", Nf); exit(1); }

 for (long i=next; i<next+Nf; ++i) { 		      
  if ( stat(argv[i], &info) != 0 )
   { printf("Error:\nCouldn't open \"%s\"!\n", argv[i]); exit(1); }
  if (info1 == NULL) info1 = &info;
   else { if (info.st_size != (*info1).st_size) { printf("Size of files is different!\n"); exit(1); }
   }
 }

 char ** outname;
 outname = new char*[Of];
 if (!outname) { printf("Not enough memory!\n"); exit(1); }
 for (long i=0; i<Of; ++i) {
  outname[i] = new char[MaxString];
  if (!outname[i]) { printf("Not enough memory!\n"); exit(1); }
  memset(outname[i], 0, MaxString);
  sprintf(outname[i], "%s%c", TemplateFilename, symb[i]);
 }
 

 float** in;
 in = new float*[Nf];
 if (!in) { printf("Not enough memory!\n"); exit(1); }
 for (long i=0; i<Nf; ++i) {
  in[i] = new float[bufsize];
  if (!in[i]) { printf("Not enough memory!\n"); exit(1); }
  memset(in[i], 0, bufsize*sizeof(float));
 }
 float* out;
 out = new float[bufsize];
 if (!out) { printf("Not enough memory!\n"); exit(1); }
 memset(out, 0, (bufsize)*sizeof(float));



 for (long i=0; i<Nf; ++i) {
  if ( (inf[i] = open(argv[next+i], O_RDONLY | O_LARGEFILE)) == -1 ) {
   printf("Error:\nCouldn't open \"%s\" for reading!\n", argv[next+i]); exit(1);
  }
 }
 for (long i=0; i<Of; ++i) {
  if ( (outf[i] = open(outname[i], O_TRUNC | O_CREAT | O_WRONLY | O_LARGEFILE, 0644 )) == -1 ) {
   printf("Error:\nCouldn't open \"%s\" for writing!\n", outname[i]); exit(1);
  }
 }

 while (1) {  /*-- infinite loop --*/
  
  for (long i=0; i<Nf; ++i) {
   bytesread = read(inf[i], in[i], bufsize*sizeof(float));
  }
  if (bytesread == 0) break;
  if (bytesread != bufsize*sizeof(float)) bufsize = bytesread/sizeof(float);

  /*--- calculate P  --*/
    for (long k=0; k<bufsize; ++k) out[k] = sqrt ( in[0][k]*in[0][k] + in[1][k]*in[1][k] );
    write(outf[0], out, (bufsize)*sizeof(float));
    memset(out, 0, (bufsize)*sizeof(float));
  /*-----------------------------------------------*/

  for (long i=0; i<Nf; ++i) memset(in[i], 0, bufsize*sizeof(float));
 } /* loop */

 for (long i=0; i<Nf; ++i) close(inf[i]);
 for (long i=0; i<Of; ++i) close(outf[i]);

 delete [] in;
 delete out;
 delete [] outname;
 delete TemplateFilename;

 return 0;
}

