/*-----------------------
prg <   > <LCP> <RCP>
------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <getopt.h>
#include <math.h>

#define MaxString 255

long bufsize = 2000000;
char *TemplateFilename;
bool StraighOfPhase = false; // true if method of phase straightening is used

/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to calculate Stoks parameters\n\n"
	 "[USAGE]: %s [options] <LCP> <RCP>\n\n"
	 "[OPTIONS]:\n"
	 "-o, --out    <template>       - filename template of output files, default - out.\n"
	 "-b, --buffer <size>           - specifies the size of read buffer in points, default = 2,000,000"
	 "                                it must not be < 1 and > 10,000,000\n"
	 "--phase_straightening         - use method of phase straightening to compute polarization angle\n"
	 "-h, --help                    - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op, tm;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
                                   {"out", required_argument, 0, 'o'},
                                   {"buffer", required_argument, 0, 'b'},
                                   {"phase_straightening", no_argument, 0, 11},
				   {0, 0, 0, 0}
                                  };

  optind = 0; optarg = 0;
  while((op = getopt_long(argc, argv, "ho:b:", long_options, 0)) != EOF)
    switch(op){
      case 'o':
	  if( TemplateFilename ) {
	    TemplateFilename = 0;
	    free(TemplateFilename);
	                          }
	  if( !( TemplateFilename = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(TemplateFilename, optarg, (int)strlen(optarg));
      break;
      
      case 'b':
       bufsize = atol(optarg);
       if (bufsize < 1 || bufsize > 10000000) {
        printf("Error: bufsize %ld is very large!\n", bufsize);
	exit(1);
       }
      break;
      
      case 'h':
        Help(argv[0]);
      break;

      case 11:
        StraighOfPhase = true;
      break;
	    
      case '?':
        Help(argv[0]);
      break;
    }

  return(optind);
 }

int main ( int argc, char *argv[] )
{
 struct stat info;
 int next;
 const long Nf = 2, Of = 1;
 long size = 0, bytesread;
 int inf[Nf], outf[Of];
 char symb[Of] = { 'A' };

 if (argc == 1) Help(argv[0]);

 TemplateFilename = new char[MaxString+1];
 if (!TemplateFilename) { printf("Not enough memory!\n"); exit(1); }
 memset(TemplateFilename, 0, MaxString+1);
 sprintf(TemplateFilename, "%s", "out.");

 next = ParseCmdLine(argc, argv);
 if (argc < next + Nf) { printf("You need point %d input files!\n", Nf); exit(1); }

 for (long i=next; i<next+Nf; ++i) { 		      
  if ( stat(argv[i], &info) != 0 )
   { printf("Error:\nCouldn't open \"%s\"!\n", argv[i]); exit(1); }
  if (size == 0) size = info.st_size/sizeof(float);
   else { if (info.st_size/sizeof(float) != size) { printf("Size of files is different!\n"); exit(1); }
   }
 }

 char ** outname;
 outname = new char*[Of];
 if (!outname) { printf("Not enough memory!\n"); exit(1); }
 for (long i=0; i<Of; ++i) {
  outname[i] = new char[MaxString];
  if (!outname[i]) { printf("Not enough memory!\n"); exit(1); }
  memset(outname[i], 0, MaxString);
  sprintf(outname[i], "%s%c", TemplateFilename, symb[i]);
 }
 

 float** in;
 in = new float*[Nf];
 if (!in) { printf("Not enough memory!\n"); exit(1); }
 for (long i=0; i<Nf; ++i) {
  in[i] = new float[bufsize];
  if (!in[i]) { printf("Not enough memory!\n"); exit(1); }
  memset(in[i], 0, bufsize*sizeof(float));
 }
 float* out;
 out = new float[bufsize/2];
 if (!out) { printf("Not enough memory!\n"); exit(1); }
 memset(out, 0, (bufsize/2)*sizeof(float));



 for (long i=0; i<Nf; ++i) {
  if ( (inf[i] = open(argv[next+i], O_RDONLY)) == -1 ) {
   printf("Error:\nCouldn't open \"%s\" for reading!\n", argv[next+i]); exit(1);
  }
 }
 for (long i=0; i<Of; ++i) {
  if ( (outf[i] = open(outname[i], O_TRUNC | O_CREAT | O_WRONLY, 0644 )) == -1 ) {
   printf("Error:\nCouldn't open \"%s\" for writing!\n", outname[i]); exit(1);
  }
 }

 while (1) {  /*-- infinite loop --*/
  
  for (long i=0; i<Nf; ++i) {
   bytesread = read(inf[i], in[i], bufsize*sizeof(float));
  }
  if (bytesread == 0) break;
  if (bytesread != bufsize*sizeof(float)) bufsize = bytesread/sizeof(float);

  /*--- calculate I  --*/
//    for (long j=0,k=0; j<bufsize; j+=2,++k) out[k] = in[1][j]*in[1][j] + in[1][j+1]*in[1][j+1] + in[0][j]*in[0][j] + in[0][j+1]*in[0][j+1];
//    write(outf[0], out, (bufsize/2)*sizeof(float));
//    memset(out, 0, (bufsize/2)*sizeof(float));
  /*-----------------------------------------------*/
  
  /*--- calculate V  --*/
//    for (long j=0,k=0; j<bufsize; j+=2,++k) out[k] = in[1][j]*in[1][j] + in[1][j+1]*in[1][j+1] - in[0][j]*in[0][j] - in[0][j+1]*in[0][j+1];
//    write(outf[1], out, (bufsize/2)*sizeof(float));
//    memset(out, 0, (bufsize/2)*sizeof(float));
  /*-----------------------------------------------*/
  
  /*--- calculate Q  --*/
//    for (long j=0,k=0; j<bufsize; j+=2,++k) out[k] = 2*(in[1][j]*in[0][j] + in[1][j+1]*in[0][j+1]);
//    write(outf[2], out, (bufsize/2)*sizeof(float));
//    memset(out, 0, (bufsize/2)*sizeof(float));
  /*-----------------------------------------------*/
  
  /*--- calculate U  --*/
//    for (long j=0,k=0; j<bufsize; j+=2,++k) out[k] = 2*(in[1][j+1]*in[0][j] - in[1][j]*in[0][j+1]);
//    write(outf[3], out, (bufsize/2)*sizeof(float));
//    memset(out, 0, (bufsize/2)*sizeof(float));
  /*-----------------------------------------------*/

   /*--- calculate P  --*/
//    for (long j=0,k=0; j<bufsize; j+=2,++k) out[k] = 2*sqrt( (in[1][j]*in[1][j] + in[1][j+1]*in[1][j+1]) * (in[0][j]*in[0][j] + in[0][j+1]*in[0][j+1]) );
//    write(outf[4], out, (bufsize/2)*sizeof(float));
//    memset(out, 0, (bufsize/2)*sizeof(float));
  /*-----------------------------------------------*/

  /*--- calculate A    --*/ 
    if (!StraighOfPhase) { for (long j=0,k=0; j<bufsize; j+=2,++k) out[k] = (180./M_PI)*atan2( in[1][j+1]*in[0][j] - in[1][j]*in[0][j+1], in[1][j]*in[0][j] + in[1][j+1]*in[0][j+1]); }
     else { //      
      double add = 0., prev = 0., cur, test;
      for (long j=0,k=0; j<bufsize; j+=2,++k) {
       cur = add + (180./M_PI)*atan2( in[1][j+1]*in[0][j] - in[1][j]*in[0][j+1], in[1][j]*in[0][j] + in[1][j+1]*in[0][j+1]);
       test = cur - prev;
       if (fabs(test) < 180.) ; else {
        if (test < 0.) { add += 360.; cur += 360.; } else { add -= 360.; cur -= 360.; }
       }
       prev = cur;
       out[k] = (float)cur;
     } // for     
    } // else
//    write(outf[5], out, (bufsize/2)*sizeof(float));
    write(outf[0], out, (bufsize/2)*sizeof(float));
    memset(out, 0, (bufsize/2)*sizeof(float));
  /*-----------------------------------------------*/

  for (long i=0; i<Nf; ++i) memset(in[i], 0, bufsize*sizeof(float));
 } /* loop */

 for (long i=0; i<Nf; ++i) close(inf[i]);
 for (long i=0; i<Of; ++i) close(outf[i]);

 delete [] in;
 delete out;
 delete [] outname;
 delete TemplateFilename;

 return 0;
}

