#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <getopt.h>
#include <math.h>

#include "readme.hpp"

#define MaxString 255

#ifndef TYPE
 #define TYPE float
#endif		

double *src;
int Header = 0;
unsigned char Fluc = 0, Error = 0;
char *AuxExt, *OutFile, *OutputPath;
double divider = 1.;
bool IsAcc = false, Precise = false;
bool is_quite = false;
long block = -1;


/*-------     (!) HOST  NETWORK   (!) 
            -------------------------------------------*/

TYPE nh2hn ( TYPE data, int razm1 )
{
  char tmp;
  const int razm = sizeof(TYPE);
  union { TYPE d;
          unsigned char uc[razm];
         } b;
  b.d = data;
  for (int i=0;i<razm/2;i++) { tmp = b.uc[i];
                               b.uc[i] = b.uc[razm-1-i];
			       b.uc[razm-1-i] = tmp;
                              }
  return b.d;           
 }
 
/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to summing select files\n\n"
	 "[USAGE]: %s [options] <file(s) to process>\n\n"
	 "[OPTIONS]:\n"
	 "-f, --fluc                    - this key is used to calculate not sum of all files\n"
	 "                                but the level-value (max-mean)/sigma, some kind of\n"
	 "                                fluctuation spectrum. This option can't be used together\n"
	 "                                with -e option\n"
	 "-e, --error                   - this key is used to calculate a root-mean-square deviation\n"
	 "                                from mean level. This option can't be used together\n"
	 "                                with -f option\n"
	 "-d, --path       <path>       - path to the output file, default - the same\n"
	 "                                where are the source files\n"
	 "--ext            <string>     - auxiliary filename extension given to the output\n"	 
	 "                                file, default - .mean\n"
	 "--out            <filename>   - name of file for output data. If --ext is given\n"
	 "                                then full name is filename + extension\n"
	 "                                default - input file + .mean\n"
	 "--header         <size>       - header in bytes of input file(s), default = 0\n"
	 "-a, --accumulate              - this key is used to accumulate output data usually\n"
	 "                                when there a lot of input files in the command line.\n"
	 "                                Often this option used together with --divider option.\n"
	 "                                This option can't be used together with -f or -e options.\n"
	 "                                Also one need to use option --out to set output file\n"
	 "--divider        <value>      - value of divider for the sum of input files. This option\n"
	 "                                have sense only together with -a option. The type of\n"
	 "                                value is double, default = 1.\n"
	 "--precisely                   - to write output data in double format, default - float\n"
	 "-b, --block  <size of block>  - size of block to record in case of large files.\n"
	 "                                If it's pointed then dest file(s) are forming by\n"
	 "                                appending consecutively the blocks of selected size;\n"
	 "                                default - equal to size of file (in points).\n"
	 "-q, --quite                   - turn on quite output, skip printing the summed files\n"
	 "-h, --help                    - help\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"path", required_argument, 0, 'd'},
				   {"fluc", no_argument, 0, 'f'},
				   {"error", no_argument, 0, 'e'},
				   {"block", required_argument, 0, 'b'},
				   {"ext", required_argument, 0, 10},
				   {"out", required_argument, 0, 11},
				   {"header", required_argument, 0, 12},
				   {"accumulate", no_argument, 0, 'a'},
				   {"divider", required_argument, 0, 13},
				   {"precisely", no_argument, 0, 14},
				   {"quite", no_argument, 0, 'q'},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "hfed:aqb:", long_options, 0)) != EOF)
    switch(op){
      case 'h':
        Help(argv[0]);
      break;
      
      case 'f':
        Fluc = 1;
      break;
      
      case 'e':
        Error = 1;
      break;

      case 'd':
	  if( OutputPath ) {
	    OutputPath = 0;
	    free(OutputPath);
	                          }
	  if( !( OutputPath = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutputPath, optarg, (int)strlen(optarg));
      break;

      case 'b':
        block = atol(optarg);
        if (block <= 0) block = -1;
      break;
      
      case 10:
	  if( AuxExt ) {
	    AuxExt = 0;
	    free(AuxExt);
	                 }
	  if( !( AuxExt = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(AuxExt, optarg, (int)strlen(optarg));
      break;
      
      case 11:
	  if( OutFile ) {
	    OutFile = 0;
	    free(OutFile);
	                 }
	  if( !( OutFile = (char *)calloc((int)strlen(optarg), sizeof(char *)) ) )
	    { printf("Not enough memory!\n"), exit(1); }
	  memcpy(OutFile, optarg, (int)strlen(optarg));
      break;
            
      case 12:
       Header = atoi(optarg);
       if (Header < 0) Header = 0;
      break;
      
      case 'a':
	  IsAcc = true;
      break;
      
      case 13:
        divider = atof(optarg);
        if (divider == 0.) { printf("\nError:\nDivider iz zero!\n"); exit(1); }
      break;
      
      case 14:
	  Precise = true;
      break;

      case 'q':
       is_quite = true;
      break;
            
      case '?':
        Help(argv[0]);
      break;
    }
  return(optind);
 }

/*--- ,       ----*/

char *SelectFileName (char * str)
{
 char *s = str;
 int len = (int)strlen(s);
 s += len;
 while (*(--s) != '/' && len--) ;
 return (++s);
 }

/*-----     ----*/

template <class tFormat>
void Write (tFormat m, off_t size, int razm, char *name, char *ext, int flag, int wri, long it)
{
   int out;
   char *outfilename;
   outfilename = new char[MaxString+1];
   if (!outfilename) { printf("Not enough memory!\n"); exit(1); }

   memset(outfilename, 0, MaxString+1);

    if (!strcmp(OutputPath, "\0")) { 
     if (strcmp(OutFile, "\0") && !strcmp(ext, ".mean")) sprintf(outfilename, "%s", OutFile);
      else sprintf(outfilename, "%s%s", (!strcmp(OutFile, "\0") ? name : OutFile), ext);
    } 
     else { 
      if (strcmp(OutFile, "\0") && !strcmp(ext, ".mean")) sprintf(outfilename, "%s/%s", OutputPath, OutFile);
       else sprintf(outfilename, "%s/%s%s", OutputPath, (!strcmp(OutFile, "\0") ? SelectFileName(name) : OutFile), ext);
     }
    
   if (it == 1) {
     printf("\nWriting... [%s] - ", outfilename); fflush(NULL);
   } else {
    if (wri == 1) { printf(" writing... [%s]", outfilename); fflush(NULL); }
   }
  
   if ( (out = open(outfilename, (flag == 1 ? O_TRUNC : O_APPEND) | O_CREAT | O_WRONLY | O_LARGEFILE, 0644)) == -1 )
      { printf("\nError: Couldn't open file %s for writing!\n", outfilename);
        exit(1);
       }
     else
       {
           lseek(out, 0, SEEK_SET);
           if (Precise) write(out, m, size*razm);
            else { float *raw;
                   raw = new float[size];
	           if (!raw) { printf("Not enough memory!\n"); exit(1); }
                   memset(raw, 0, size*sizeof(float));
	           for (off_t u=0; u<size; ++u) raw[u] = (float)m[u];
		   write(out, raw, size*sizeof(float));
	           delete(raw);
                 }	   
           close(out);
        }
	
   memset(m, 0, razm*size);
   	
   delete(outfilename);
 }

void ReadSrc (off_t l, off_t size, char *name)
{
  int file;
  TYPE *data;

     data = new TYPE[size];
     if (!data) { printf("Not enough memory!\n"); exit(1); }

     memset(data, 0, size*sizeof(TYPE));
     memset(src, 0, size*sizeof(double));
       
       if ( (file = open(name, O_RDONLY | O_LARGEFILE)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
       if (lseek(file, (off_t)Header + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, data, size*sizeof(TYPE));
        close(file);

   for (off_t q=0;q<size;q++) {
	
  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
             src[q] = (double)(data[q]);
	     
	                                  }		
    delete(data);	
 }
     
/*====================== Main Function ===================================*/
  
int main ( int argc, char *argv[] )
 {
   struct stat info;
   int next;
   off_t Size = 0, si, le, win;
   char *acc_name;
   long iter;

   if (argc == 1) Help(argv[0]);
   
   AuxExt = new char[MaxString+1];
   OutFile = new char[MaxString+1];
   OutputPath = new char[MaxString+1];
   if (!AuxExt || !OutFile || !OutputPath) { printf("Not enough memory!\n"); exit(1); }
   memset(AuxExt, 0, MaxString+1);
   memset(OutFile, 0, MaxString+1);
   memset(OutputPath, 0, MaxString+1);
   sprintf(AuxExt, "%s", ".mean");
   sprintf(OutFile, "%s", "\0");
   sprintf(OutputPath, "%s", "\0");
   
   next = ParseCmdLine(argc, argv);
   if (argc < next + 1) { printf("\nError:\nNot point the file(s) to process!\n"); exit(1); }
   
   if (Fluc == 1 && Error == 1) { printf("\nError:\nOptions -f and -e can not be used together!\n"); exit(1); }
   if (IsAcc && (Fluc == 1 || Error == 1)) { printf("\nError:\nOption -a can not be used together with -f or -e options!\n"); exit(1); }
   if (IsAcc && !strcmp(OutFile, "\0")) { printf("\nError:\nOption -a can not be used without --out option!\n"); exit(1); }
   
   if (IsAcc) {
   
     acc_name = new char[MaxString+1];
     if (!acc_name) { printf("Not enough memory!\n"); exit(1); }

     memset(acc_name, 0, MaxString+1);
     if (!strcmp(OutputPath, "\0")) { 
      if (strcmp(OutFile, "\0") && !strcmp(AuxExt, ".mean")) sprintf(acc_name, "%s", OutFile);
       else sprintf(acc_name, "%s%s", OutFile, AuxExt);
     } 
      else { 
       if (strcmp(OutFile, "\0") && !strcmp(AuxExt, ".mean")) sprintf(acc_name, "%s/%s", OutputPath, OutFile);
        else sprintf(acc_name, "%s/%s%s", OutputPath, OutFile, AuxExt);
      }

     if (stat(acc_name, &info) == 0) {
      Size = info.st_size / (Precise ? sizeof(double) : sizeof(float));
      if (Size == 0) { printf("\nError:\nSize of accumulate file \"%s\" is zero!\n", acc_name); exit(1); }
     }
   }
   
 for (int k=next; k<argc; ++k) {

   if ( stat(argv[k], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", argv[k]); exit(1); }

   si = (info.st_size - Header)/sizeof(TYPE);
   if (si <= 0) { printf("\nError:\nSize of header is to much! - \"%s\"\n", argv[k]); exit(1); }

   if ((k==next && (!IsAcc || stat(acc_name, &info) != 0)) || si < Size) Size = si;

 }

   if (block == -1 || block > Size) block = Size;
   iter = (Size == block ? 1 : (Size%block == 0 ? Size/block : Size/block + 1));

   double *sum, *sum2, *max, *acc;
   src = new double[block];
   sum = new double[block];
   if (!src || !sum) { printf("Not enough memory!\n"); exit(1); }

   memset(sum, 0, block*sizeof(double));

   if (Fluc == 1 || Error == 1) {
    sum2 = new double[block];
    if (!sum2) { printf("Not enough memory!\n"); exit(1); }
   
    memset(sum2, 0, block*sizeof(double));
   }
   
   if (Fluc == 1) {
    max  = new double[block];
    if (!max) { printf("Not enough memory!\n"); exit(1); }
   
    memset(max, 0, block*sizeof(double));
   }
   
   if (IsAcc) { 
    acc = new double[block]; 
    if (!acc) { printf("Not enough memory!\n"); exit(1); }
    
    memset(acc, 0, block*sizeof(double));
        
    if (stat(acc_name, &info) == 0) {
     int acc_file;
          
     if ( (acc_file = open(acc_name, O_RDONLY)) == -1 )
      { printf("\nError:\nCouldn't open accumulate file \"%s\" for reading!\n", acc_name); exit(1);  }

     if (Precise) read(acc_file, acc, block*sizeof(double));
      else { float *raw;
             raw = new float[block];
	     if (!raw) { printf("Not enough memory!\n"); exit(1); }
             memset(raw, 0, block*sizeof(float));
	     read(acc_file, raw, block*sizeof(float));
	     for (off_t u=0; u<block; ++u) acc[u] = raw[u];
	     delete(raw);
            }
     close(acc_file);
    }
   }

printf("\nProcessing...\n");

// loop on iterations
for (unsigned long z=0; z<iter; z++) {

 if (!is_quite) { printf(".%5d", z+1); fflush(NULL); }

 le = block * z;
 win = (le + block > Size ? Size - le : block);

    for (int i=next; i<argc; ++i) {
     
 if (iter == 1) { if (!is_quite) { printf("[%3d]\t%s summing ... \n", i-next, argv[i]); fflush(NULL); } }

 ReadSrc(le, win, argv[i]);
 for(off_t t=0; t<win; ++t) sum[t] += src[t];
 if (Fluc == 1) {
  for(off_t t=0; t<win; ++t) sum2[t] += src[t]*src[t];
  if (i == next) for(off_t t=0; t<win; ++t) max[t] = src[t];
   else for(off_t t=0; t<win; ++t) max[t] = (src[t] > max[t] ? src[t] : max[t]);
 }
       }
   
  if (IsAcc) {
   for(off_t t=0; t<win; ++t) sum[t] = (sum[t] / divider) + acc[t];
  } else for(off_t t=0; t<win; ++t) sum[t] /= (argc-next);

  if (Error == 1) {   if (iter == 1) { if (!is_quite) { printf("\n"); } }
                      for (int i=next; i<argc; ++i) {
                        if (iter == 1) { if (!is_quite) { printf("[%3d]\t%s calculating error ... \n", i-next, argv[i]); fflush(NULL); } }
                        ReadSrc(le, win, argv[i]);
                        for(off_t t=0; t<win; ++t) sum2[t] += (src[t] - sum[t])*(src[t] - sum[t]);
                       }
                   } // Error == 1
       
  if (Fluc == 1) {
   for(off_t t=0; t<win; ++t) sum2[t] = sqrt( sum2[t]/(argc-next) - sum[t]*sum[t] );
   for(off_t t=0; t<win; ++t) sum[t] = (max[t] - sum[t])/sum2[t];
  }

  if (Error == 1) for(off_t t=0; t<win; ++t) sum[t] = sqrt(sum2[t] / ((argc-next)*(argc-next-1)) );

  if (!is_quite) { printf("\b\b\b\b\b"); fflush(NULL); }
  
  Write (sum, win, sizeof(double), argv[argc-1], AuxExt, z+1, iter - z, iter);

  } // iterations

  if (iter == 1) printf("done\n\n"); else printf(" - done\n");

  delete(src);
  delete(sum);
  if (Fluc == 1 || Error == 1) delete(sum2);
  if (Fluc == 1) delete(max);
  if (IsAcc) delete(acc);
  delete(AuxExt);
  delete(OutFile);
  delete(OutputPath);
  if (IsAcc) delete(acc_name);

  return 0;
}

