/*===========================
      
.    
 .    ,  
.
============================*/

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <getopt.h>
#include <unistd.h>
#include <fcntl.h>

using namespace std;

//      
template<class T>void write2file (const char* s, long size, T* m);

#define WRITE(s,t,r,file, func) { s t *m1; \
                                  m1 = new s t[r.size()]; \
				  if ( !m1 ) { cout << "Not enough memory!\n"; exit(1); } \
				  memset(m1, 0, sizeof(s t)*r.size()); \
				  vector<string>::iterator it = r.begin(); \
				  for (long i=0;it!=r.end();++it, ++i) m1[i] = (s t)func((*it).c_str()); \
				  write2file(file, r.size(), m1); \
				  delete (m1); }



//  
string outfile, format = "f";
long column = 1;


//  
void help(const char* s) {
 cout << "PROGRAMM: " << s << endl;
 cout << "Converts one column of the ASCII incoming file to binary format" << endl;
 cout << "Usage: " << s << " <file> [-o <output> [-n <column> [-f <format>]]]" << endl;
 cout << "  <format> - one of the  uc, sc, us, ss, ui, si, ul, sl, f, d, ld" << endl;
 cout << "             where uc - unsigned char, etc." << endl;
 cout << "             default format is f, i.e. float" << endl;
 cout << "  <column> - number of column in ASCII <file>, must be from 1, default - 1" << endl;
 cout << "  <file>   - incoming ASCII file" << endl;
 cout << "  <output> - name of destination binary file, default - source name + \".bin\"" << endl;
 cout << "             Instead of regular <file> you can use - to read from standard input" << endl << endl;
 exit(0);
}

//           
void read_stdin (vector<string>& res) {
 if (!cin) { cout << "\nError:\nCouldn't open standard input for reading!\n" ; exit(1); }
 
 string tmp, str;
 while (!cin.eof()) {
  for (int i=1;i<column;++i) cin >> tmp;
  cin >> str; res.push_back(str);
  getline(cin, tmp);
 }
 
 res.pop_back();
}

//          
void read (const char* s, vector<string>& res) {
 ifstream in(s);
 if (!in) { cout << "\nError:\nCouldn't open file " << s << " for reading!\n" ; exit(1); }
 
 string tmp, str;
 while (!in.eof()) {
  for (int i=1;i<column;++i) in >> tmp;
  in >> str; res.push_back(str);
  getline(in, tmp);
 }
 
 in.close();
 res.pop_back();
}


void rec(const char* s, vector<string>& res) {
 if (format == "uc") WRITE(unsigned, char, res, s, atoi);
 if (format == "sc") WRITE(signed, char, res, s, atoi);
 if (format == "us") WRITE(unsigned, short, res, s, atoi);
 if (format == "ss") WRITE(signed, short, res, s, atoi);
 if (format == "ui") WRITE(unsigned, int, res, s, atoi);
 if (format == "si") WRITE(signed, int, res, s, atoi);
 if (format == "ul") WRITE(unsigned, long, res, s, atol);
 if (format == "sl") WRITE(signed, long, res, s, atol);
 if (format == "f")  WRITE(float, , res, s, atof);
 if (format == "d")  WRITE(double, , res, s, atof);
 if (format == "ld") WRITE(long, double, res, s, atof);
}


//      
template<class T>
void write2file (const char* s, long size, T* m) {
 int out;
 if (outfile.empty()) outfile = (string)s + ".bin";
 if ( (out = open(outfile.c_str(), O_TRUNC | O_CREAT | O_WRONLY, 0644)) == -1 )
   { cout << "\nError: Couldn't open file " << outfile << " for writing!\n"; exit(1); }
  else { lseek(out, 0, SEEK_SET);
	 write(out, m, size*sizeof(T));	                                           
         close(out);
        }
}


/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op;
  struct option long_options[] = { {"column", required_argument, 0, 'n'},
				   {"output", required_argument, 0, 'o'},
				   {"format", required_argument, 0, 'f'},
				   {0, 0, 0, 0}
                                  };

  while((op = getopt_long(argc, argv, "n:o:f:", long_options, 0)) != EOF)
    switch(op){
      case 'o':
       outfile = optarg;
      break;
            
      case 'n':
       column = atoi(optarg);
       if (column < 1) column = 1;
      break;

      case 'f':
       format = optarg;
       if (format != "uc" && format != "sc" && format != "us" && format != "ss" && format != "ui" &&
           format != "si" && format != "ul" && format != "sl" && format != "f" && format != "d" &&
	   format != "ld") format = "f";
      break;
            
      case '?':
        help(argv[0]);
      break;
    }
  return(optind);
 }


int main(int argc, char* argv[]) {
 int next;
 if (argc == 1) help(argv[0]);
 
 next = ParseCmdLine(argc, argv);
 if (argc < next + 1) { cout << "\nError:\nNot point the file to process!\n"; exit(1); }

 vector<string> res;
 if (!strcmp(argv[next], "-")) read_stdin(res);
  else read(argv[next], res);
 rec(strcmp(argv[next], "-") ? argv[next] : "stdin", res);
 
 return 0;
}


