#define MAIN_
#include "readme.hpp"
#include "winsp.hpp"   

/*---------     
              -----------------*/

long smoothing ( long sz )
{
 float sum = 0., tmp;
 sRes.BandSmooth = (sRes.BandSmooth > sz-1 ? sz - 1 : sRes.BandSmooth);
 if (shift < 0) shift = -1;
 if (shift > sRes.BandSmooth - 1) shift = -1;
 shift = ( shift == -1 ? (long)floor((double)((sRes.BandSmooth-1)/2.)) : shift);

  for(long r=0;r<sRes.BandSmooth;r++) sum += mass[r];
   tmp     = mass[0];
   mass[0] = sum/sRes.BandSmooth; 
  for(long t=0;t<sz-sRes.BandSmooth;t++) {
   sum = sum+mass[sRes.BandSmooth+t]-tmp;
   tmp = mass[t+1];
   mass[t+1] = sum/sRes.BandSmooth;
                                          }
 for (long t=sz-sRes.BandSmooth;t>=0;t--) mass[t+shift] = mass[t];
 for (long t=sz-sRes.BandSmooth+shift+1;t<sz;t++) mass[t] = mass[sz-sRes.BandSmooth+shift];
 for (long t=shift-1;t>=0;t--) mass[t] = mass[shift];
 
 return (sz - sRes.BandSmooth + 1);  
}

/*---- ,   (size)  ,
                    2 ------------------------*/

long resize (long sz)
 {
   long n = 0;
   sz += 1;
   while ( (1<<(++n)) < sz) ;
   return (1<<(--n));
  }

/*==========================Prompting===================================*/

void Help ( char *s )
 {
  printf("\nPROGRAMM %s\n"
         "It is used to define spectrum window\n"
	 "Some options repeat the fieldnames of source file, in case when\n"
	 "some value is pointed in both source file and command line, value\n"
	 "from command line is used\n\n"
	 "[USAGE]: %s [options] <file with band>\n\n"
	 "[OPTIONS]:\n"
	 "-o, --source <filename>       - name of source file, default \"winsp.set\"\n"
	 "                                If filename = no then source file won't be used and\n"
	 "                                all parameters are inputed from command line\n"
	 "-c, --corr   <filename>       - name of file with correction function. With this option\n"
	 "                                band will be corrected before drawing, default - no\n"
	 "--create                      - create source file with name and destination pointed\n"
	 "                                before in -o or --source option. Otherwise, source\n"
	 "                                file \"winsp.set\" will be created in current directory\n"
	 "-s, --smooth <value>          - set number of smooth points, default = 1024\n"
	 "--shift      <value>          - set shift from left of smoothed band in total window\n"
	 "                                default = (long)(smooth points - 1)/2\n"
	 "-l, --left   <value>          - set left boundary of spectrum window, default = 0\n"
	 "-w, --size   <value>          - set size of spectrum window, default = size of file with band\n"
	 "--header     <value>          - size of header (in bytes) of file with band\n"
	 "                                default - 0\n"
	 "--timeres    <value>          - set time resolution (in nanosec), default - 400.\n"
	 "-h, --help                    - help\n\n"
	 "[SPECIAL OPTIONS]:\n"
	 "-bg, -background <color>      - set the background color of full window ignoring the\n"
	 "                                X resource file .Winsp\n"
	 "-fg, -foreground <color>      - set the foreground color in full window ignoring the\n"
	 "                                X resource file .Winsp\n"
	 "-display <display>            - set the display to show window, for example odessa:0 ,\n"
	 "                                where odessa is a hostname, 0 - display number\n"
	 "-fn, -font <font>             - set the font in full window\n"
	 "-xrm <\"resource:value\">       - this option allows to set some value for some resource.\n"
	 "                                For example, option -xrm \"Winsp.height:300\" allows to set\n"
	 "                                the height of full window to the value of 300\n\n"
	 "[KEYS]:\n"
	 "<q>            - quit without saving\n"
	 "<Enter>        - view band with eliminated level and one more time - quit with saving\n"
	 "<Esc>          - return again to choosing spectrum window\n"
	 "<Tab>          - change active boundary\n"
	 "<Left / Right> - move active boundary to left/right\n"
	 "<PgUp / PgDn>  - increase/decrease the rate to move of boundaries\n"
	 "<a>            - switch on/off the auto-refresh of level pointer\n"
	 "<f>            - refresh level pointer\n\n"
	 "[SPECIAL KEYS]:\n"
	 "<LCtrl | RCtrl> - redraw the full window\n"
	 "<LCtrl + RCtrl> - switch on/off the mode of auto redrawing of full window\n"
	 "                  if BackingStore mode of Window Manager doesn't work\n\n"
	 "[MOUSE]:\n"
	 "<Button1>    - move left boundary\n"
	 "<Button3>    - move right boundary\n"
	 "<Button2>    - fresh level pointer\n\n", s, s);
  exit(0);
 }

/*===================Parsing the command line===========================*/

int ParseCmdLine( int argc, char *argv[] )
 {
  int op, tm;
  struct option long_options[] = { {"help", no_argument, 0, 'h'},
				   {"source", required_argument, 0, 'o'},
				   {"smooth", required_argument, 0, 's'},
				   {"left", required_argument, 0, 'l'},
				   {"size", required_argument, 0, 'w'},
				   {"corr", required_argument, 0, 'c'},
				   {"shift", required_argument, 0, 11},
				   {"timeres", required_argument, 0, 12},
				   {"create", no_argument, 0, 13},
				   {"header", required_argument, 0, 14},
				   {0, 0, 0, 0}
                                  };

  optind = 0; optarg = 0;
  while((op = getopt_long(argc, argv, "hs:o:l:w:c:", long_options, 0)) != EOF)
    switch(op){
      case 's':
	sRes.BandSmooth = atol(optarg);
	if (sRes.BandSmooth < 1) sRes.BandSmooth = 1; 
      break;
      
      case 'o':
	memset(ResourceFileName, 0, MaxString+1);
	sprintf(ResourceFileName, "%s", optarg);
      break;
      
      case 'c':
        CorrectionFunc = new char[MaxString+1];
	if (!CorrectionFunc) { printf("Not enough memory!\n"); exit(1); }
	memset(CorrectionFunc, 0, MaxString+1);
	sprintf(CorrectionFunc, "%s", optarg);
	UseCorrect = 1;
      break;
      
      case 'h':
        Help(argv[0]);
      break;
      
      case 'l':
        sRes.SpWindowLeft = atol(optarg);
	if (sRes.SpWindowLeft < 0) sRes.SpWindowLeft = 0;
      break;
      
      case 'w':
        sRes.SpWindowSize = atol(optarg);
	if (sRes.SpWindowSize < 1) sRes.SpWindowSize = -1;
      break;
      
      case 11:
        shift = atol(optarg);
      break;

      case 12:
        sRes.TimeResolution = atof(optarg);
      break;
            
      case 13:
        CreateSourceFile();
        exit(0);
      break;
      
      case 14:
        sRes.HeaderSize = atoi(optarg);
	if (sRes.HeaderSize < 0) sRes.HeaderSize = 0;
      break;
      
      case '?':
        Help(argv[0]);
      break;
    }

  return(optind);
 }

/*------------      -------------------------*/

void ReadSrc (float * x, long l, long size, char *name)
{
  int file;
  TYPE *data;

     data = new TYPE[size];
     if (!data) { printf("Not enough memory!\n"); exit(1); }

     memset(data, 0, size*sizeof(TYPE));
     memset(x, 0, size*sizeof(float));
       
       if ( (file = open(name, O_RDONLY)) == -1 )
         { printf("\nError:\nCouldn't open \"%s\" for reading!\n", name); exit(1);  }
	 
        if (lseek(file, sRes.HeaderSize + l*sizeof(TYPE), SEEK_SET) == -1 )
	 { printf("\nError:\nWrong execution of lseek() in file \"%s\"!\n", name); exit(1); }
	read(file, data, size*sizeof(TYPE));
        close(file);

   for (long q=0;q<size;q++) {
	
  #ifdef _FR_SUN
   #ifdef _TO_INTELPC
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
  
  #ifdef _FR_INTELPC
   #ifdef _TO_SUN
    if (sizeof(TYPE) != 1) data[q] = nh2hn(data[q], sizeof(TYPE));
   #endif
  #endif
             x[q] = (float)data[q];
	     
	                                  }		
    delete(data);	
 }
    
/*====================== Main Function ===================================*/
  
int main ( int argc, char *argv[] )
 {	    
   struct stat info;

   if (argc == 1) Help(argv[0]); 

   ResourceFileName   = new char[MaxString+1];
   XBaseResourcesFile = new char[MaxString+1];
   XResourceFile      = new char[MaxString+1];
   if (!ResourceFileName || !XBaseResourcesFile || !XResourceFile) { printf("Not enough memory!\n"); exit(1); }
   memset(ResourceFileName, 0, MaxString+1);
   memset(XBaseResourcesFile, 0, MaxString+1);
   memset(XResourceFile, 0, MaxString+1);
   sprintf(ResourceFileName, "%s", "winsp.set");
   sprintf(XBaseResourcesFile, "%s", ".BaseGraphics");
   sprintf(XResourceFile, "%s", ".Winsp");


   // ,     
   char *options[] = { };
   int opt_kind[] = { }; //   : 1 -  , 0 -  
   CommandLineOptions clo(argc, argv, options, opt_kind, 0);
   av = clo.remove();
   ac = clo.getN();

   ReadResources(ac, av);
   clo.update(ac, av);
   av = clo.add();
   ac = clo.getN();

   ParseCmdLine(ac, av);

   if (strcmp(ResourceFileName, "no")) ReadFromRes();
   
   next = ParseCmdLine(ac, av);
   if (ac < next + 1) { printf("\nError:\nNot point the file with band!\n"); exit(1); }

   if (!strcmp(ResourceFileName, "no")) { memset(ResourceFileName, 0, MaxString+1);
                                          sprintf(ResourceFileName, "%s", "winsp.set");
                                         }
   
   if ( stat(av[next], &info) != 0 ) {
     printf("\nError:\nCouldn't open \"%s\"!\n", av[next]); exit(1); }

  Size = (info.st_size - sRes.HeaderSize)/sizeof(TYPE);
  if (Size <= 0) { printf("\nError:\nSize of header is to much!\n"); exit(1); }
  
  if (2*(Size - 1) != resize(2*(Size - 1))) { printf("\nError:\nYour file with band doesn't contain N/2 + 1 positive harmonics!\n"); exit(1); }

  if (sRes.SpWindowLeft >= Size) sRes.SpWindowLeft = 0; 
  if (sRes.SpWindowSize == -1 || sRes.SpWindowSize > Size) sRes.SpWindowSize = Size;
  if (sRes.SpWindowLeft + sRes.SpWindowSize > Size) sRes.SpWindowSize -= (sRes.SpWindowLeft + sRes.SpWindowSize - Size);
  sRes.UsedFileSize = Size;
  
  mass = new float[Size];
  if (!mass) { printf("Not enough memory!\n"); exit(1); }

  ReadSrc(mass, 0, Size, av[next]);
  if (UseCorrect == 1) { float * cf;
                         cf = new float[Size];
			 if (!cf) { printf("Not enough memory!\n"); exit(1); }
			 ReadSrc(cf, 0, Size, CorrectionFunc);
			 for (long t=0;t<Size;t++) mass[t] = mass[t]*cf[t]*cf[t];
			 delete(cf);
                        }
  smoothing(Size);  
  for (long i=0;i<Size;i++) {
   gmax = (i == 0 ? mass[i] : (mass[i] > gmax ? mass[i] : gmax));
   gmin = (i == 0 ? mass[i] : (mass[i] < gmin ? mass[i] : gmin));
  }

  Plotting();

  delete(mass);
  delete(XBaseResourcesFile);
  delete(XResourceFile);
  delete(ResourceFileName);
  if (UseCorrect == 1) delete(CorrectionFunc);

  return 0;
 }

