/* ---------------------------------------------------------
 * Computes the FFTW wisdom for FLOAT (SINGLE) Precision
 * Wisdom file will be called "fftw_wisdom.txt"
 * ---------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "lagtiming.hpp"

#include <fftw3.h>

int main ( void ) {

  FILE *wisdomfile, *logfile;
  fftwf_plan plan;
  fftwf_complex *inout, *in, *out;
  int fftlen;

  logfile = fopen("wisdomf.log", "wt");
  if (logfile == NULL) {
    printf("Error opening \"wisdomf.log\"\n");
    printf("Exiting.\n");
    exit(1);
  }

  /* Generate the wisdom... */

  printf("\nAttempting to read the system wisdom file (float) ...\n");
  fprintf(logfile, "\nAttempting to read the system wisdom file (float) ...\n");
  if (!fftwf_import_system_wisdom()) {
    printf("  failed.  The file probably does not exist.\n\n");
    fprintf(logfile, "  failed.  The file probably does not exist.\n\n");
  } else {
     printf("  succeded.  Good.  We'll use it.\n\n");
     fprintf(logfile, "  succeded.  Good.  We'll use it.\n\n");
    }

   printf ("Reading float wisdom file \"/etc/fftw/wisdomf\" ...\n");
   fprintf (logfile, "Reading float wisdom file \"/etc/fftw/wisdomf\" ...\n");
   wisdomfile = fopen("wisdomf", "r");
   if (wisdomfile == NULL) {
     printf("Error opening \"/etc/fftw/wisdomf\"\n");
     fprintf(logfile, "Error opening \"/etc/fftw/wisdomf\"\n");
     printf("Probably dir \"/etc/fftw\" and wisdom file do not exist.\n");
     fprintf(logfile, "Probably dir \"/etc/fftw\" and wisdom file do not exist.\n");
     printf("Create directory... "); 
     fprintf(logfile, "Create directory... "); 
     fflush (NULL);
     mkdir ("/etc/fftw", 0755);
     printf("done\n");
     fprintf(logfile, "done\n");
   } else { 
            printf ("   done.\n");
            fprintf (logfile, "   done.\n");
            fclose(wisdomfile);
	   }

  LagTiming mark; // to measure the time
  
  printf("Creating Wisdom for FFTW (float).\n");
  printf("This may take a while...\n\n");
  printf("Generating plans for FFTs of length:\n");

  // logging
  fprintf(logfile, "Creating Wisdom for FFTW (float).\n");
  fprintf(logfile, "This may take a while...\n\n");
  fprintf(logfile, "Generating plans for FFTs of length:\n");

  fflush (NULL);

  fftlen = 2;

  while (fftlen <= 8388608) {

    printf(" EXHAUSTIVE\n");
    fprintf(logfile, " EXHAUSTIVE\n");

    printf("  out-of-place transform\n");
    fprintf(logfile, "  out-of-place transform\n");
    fflush (NULL);

    in  = (fftwf_complex *) fftwf_malloc(fftlen * sizeof(fftwf_complex));
    out = (fftwf_complex *) fftwf_malloc(fftlen * sizeof(fftwf_complex));

    printf("   %d forward    ", fftlen);
    fprintf(logfile, "   %d forward    ", fftlen);
    fflush (NULL);
    mark.start();
    plan = fftwf_plan_dft_1d(fftlen, in, out, FFTW_FORWARD, FFTW_EXHAUSTIVE);
    fftwf_destroy_plan(plan);
    mark.stop();
    printf ("[%s]\n", mark.lag());
    fprintf(logfile, "[%s]\n", mark.lag());

    printf("   %d backward   ", fftlen);
    fprintf(logfile, "   %d backward   ", fftlen);
    fflush (NULL);
    mark.start();
    plan = fftwf_plan_dft_1d(fftlen, in, out, FFTW_BACKWARD, FFTW_EXHAUSTIVE);
    fftwf_destroy_plan(plan);
    mark.stop();
    printf ("[%s]\n", mark.lag());
    fprintf(logfile, "[%s]\n", mark.lag());

    fftwf_free(in);
    fftwf_free(out);
    
    printf("  in-place transform\n");
    fprintf(logfile, "  in-place transform\n");
    fflush (NULL);
    
    inout = (fftwf_complex *) fftwf_malloc(fftlen * sizeof(fftwf_complex));
    printf("   %d forward    ", fftlen);
    fprintf(logfile, "   %d forward    ", fftlen);
    fflush (NULL);
    mark.start();
    plan = fftwf_plan_dft_1d(fftlen, inout, inout, FFTW_FORWARD, FFTW_EXHAUSTIVE);
    fftwf_destroy_plan(plan);
    mark.stop();
    printf ("[%s]\n", mark.lag());
    fprintf(logfile, "[%s]\n", mark.lag());

    printf("   %d backward   ", fftlen);
    fprintf(logfile, "   %d backward   ", fftlen);
    fflush (NULL);
    mark.start();
    plan = fftwf_plan_dft_1d(fftlen, inout, inout, FFTW_BACKWARD, FFTW_EXHAUSTIVE);
    fftwf_destroy_plan(plan);
    mark.stop();
    printf ("[%s]\n", mark.lag());
    fprintf(logfile, "[%s]\n", mark.lag());

    fftwf_free(inout);

    printf("Exporting wisdom to '/etc/fftw/wisdomf' ...\n");
    fprintf(logfile, "Exporting wisdom to '/etc/fftw/wisdomf' ...\n");
    fflush (NULL);

    /* Open wisdom file for writing... */

    wisdomfile = fopen("/etc/fftw/wisdomf", "w");
    if (wisdomfile == NULL) {
     printf ("\nError opening \"/etc/fftw/wisdomf\"\n");
     fprintf (logfile, "\nError opening \"/etc/fftw/wisdomf\"\n");
     printf ("Do you have permissions to write to \"/etc/fftw\"?\n");
     fprintf (logfile, "Do you have permissions to write to \"/etc/fftw\"?\n");
     exit (1);
    }

    /* Write the wisdom... */

    fftwf_export_wisdom_to_file(wisdomfile);

    /* Cleanup... */

    fclose(wisdomfile);
    printf("Done.\n\n");
    fprintf(logfile, "Done.\n\n");
    fflush (NULL);

    fftlen <<= 1;
  }

  fprintf (logfile, "\n\n=== END ===\n");
  fclose (logfile);
  return (0);

}
